/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.attributes.OverviewSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.attributes.AttributeGroupsCache");
dojo.require("com.ibm.rdm.web.attributes.AttributeGroupSection");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.parsers.DoorsLinksParser");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.requireLocalization("com.ibm.rdm.web.attributes", "OverviewSection");

var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;


// Here's an example of how attribute group values are specified for an normal artifact
// (i.e. NOT a collection) containing attributes from two attribute groups.
// <?xml version="1.0" encoding="UTF-8"?>
// <rte:text
//     xmlns:attribute="http://schema.ibm.com/rdm/2008/Attribute" xmlns:dc="http://purl.org/dc/terms/"
//     xmlns:h="http://www.w3.org/TR/REC-html40" xmlns:rte="http://jazz.net/xmlns/alm/rm/RichText/v0.1">
//   <attribute:attributeGroup key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup"/>
//   <attribute:stringAttribute key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#MyString_ID"
//       value="aaa"/>
//   <attribute:enumerationAttribute key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#MyList_ID"
//       value="good_ID"/>
//   <attribute:dateAttribute key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#MyDate_ID"
//       value="2009-10-30"/>
//   <attribute:integerAttribute key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#MyNumber_ID"
//       value="123"/>
//   <attribute:booleanAttribute key="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#MyBoolean_ID"
//       value="true"/>
//   <attribute:attributeGroup key="http://com.ibm.rdm.attribute.group.MySingleAttributeGroup"/>
//   <attribute:integerAttribute key="http://com.ibm.rdm.attribute.group.MySingleAttributeGroup#Myattribute_ID"
//       value="123"/>
//   <dc:title>Doc10</dc:title>
//   <h:body id="_58Bd8bD6Ed6jlNS-CYJcdA">
//     <h:p id="_58Bd8rD6Ed6jlNS-CYJcdA">doc 10</h:p>
//   </h:body>
// </rte:text>

// Here's an example of how attribute group values are specified for a collection containing a single attribute ...
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//     xmlns:dc="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
//     xmlns:rrmCollection="http://jazz.net/xmlns/alm/rm/Collection/v1.0/">
//   <rrmCollection:Collection rdf:about="">
//     <dc:title>col2</dc:title>
//     <dc:description/>
//     <rrmCollection:artifact rdf:resource="rrc109"/>
//     <attribute:key xmlns:attribute="http://schema.ibm.com/rdm/2008/Attribute#">http://com.ibm.rdm.attribute.group.MySingleAttributeGroup</attribute:key>
//     <MySingleAttributeGroup:Myattribute_ID xmlns:MySingleAttributeGroup="http://com.ibm.rdm.attribute.group.MySingleAttributeGroup#"
//         rdf:datatype="http://www.w3.org/2001/XMLSchema#integer">123</MySingleAttributeGroup:Myattribute_ID>
//   </rrmCollection:Collection>
// </rdf:RDF>

// Here's an example of how attribute group values are specified for a collection containing multiple attributes ...
// <?xml version="1.0" encoding="UTF-8"?>
// <rdf:RDF
//     xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#" xmlns:attribute="http://schema.ibm.com/rdm/2008/Attribute#"
//     xmlns:dc="http://purl.org/dc/terms/" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
//     xmlns:rrmCollection="http://jazz.net/xmlns/alm/rm/Collection/v1.0/">
//   <rrmCollection:Collection rdf:about="">
//     <dc:title>col1</dc:title>
//     <dc:description/>
//     <rrmCollection:artifact rdf:resource="rrc99"/>
//     <rrmCollection:artifact rdf:resource="rrc100"/>
//     <rrmCollection:artifact rdf:resource="rrc101"/>
//     <attribute:key xmlns:attribute="http://schema.ibm.com/rdm/2008/Attribute#">http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup</attribute:key>
//     <attribute:key xmlns:attribute="http://schema.ibm.com/rdm/2008/Attribute#">http://com.ibm.rdm.attribute.group.MySingleAttributeGroup</attribute:key>
//     <MyMultipleAttributeGroup:MyString_ID xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#">aaa</MyMultipleAttributeGroup:MyString_ID>
//     <MyMultipleAttributeGroup:MyList_ID xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#">good_ID</MyMultipleAttributeGroup:MyList_ID>
//     <MyMultipleAttributeGroup:MyDate_ID xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#">2009-10-30</MyMultipleAttributeGroup:MyDate_ID>
//     <MyMultipleAttributeGroup:MyNumber_ID xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#"
//         rdf:datatype="http://www.w3.org/2001/XMLSchema#integer">123</MyMultipleAttributeGroup:MyNumber_ID>
//     <MyMultipleAttributeGroup:MyBoolean_ID xmlns:MyMultipleAttributeGroup="http://com.ibm.rdm.attribute.group.MyMultipleAttributeGroup#"
//         rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean">true</MyMultipleAttributeGroup:MyBoolean_ID>
//     <MySingleAttributeGroup:Myattribute_ID xmlns:MySingleAttributeGroup="http://com.ibm.rdm.attribute.group.MySingleAttributeGroup#"
//         rdf:datatype="http://www.w3.org/2001/XMLSchema#integer">123</MySingleAttributeGroup:Myattribute_ID>
//   </rrmCollection:Collection>
// </rdf:RDF>

// Here's the representation for a wrapped resource containing two attribute
// groups with some fields set:
// 1) "Market" -- containing a "User" attribute "Power" value and a "Segment"
//    attribute "Canada" value.
// 2) "Origin" -- containing a "Target" attribute "Support" value.
// and another attribute group, "http://com.ibm.rdm.attribute.group.mygroup",
// with NO fields set ...
//
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:j.0="http://com.ibm.rdm.attribute.group.MARKET#"
//     xmlns:j.1="http://schema.ibm.com/rdm/2008/Attribute#"
//     xmlns:j.2="http://com.ibm.rdm.attribute.group.ORIGIN#"
//     xmlns:dc="http://purl.org/dc/terms/"
//     xmlns:reference="http://jazz.net/xmlns/alm/rm/Reference/v0.1#"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <reference:ReferenceElement rdf:about="../resources/rrc80">
//     <j.0:Segment_ID>Canada_ID</j.0:Segment_ID>
//     <dc:title>Classics_Vision.doc</dc:title>
//     <j.1:key>http://com.ibm.rdm.attribute.group.mygroup</j.1:key>
//     <j.0:User_ID>Power_ID</j.0:User_ID>
//     <dc:identifier>ReferenceElement__QmFpQLy7Ed6cv5f_aTHjkw</dc:identifier>
//     <dc:description></dc:description>
//     <j.1:key>http://com.ibm.rdm.attribute.group.ORIGIN</j.1:key>
//     <j.1:key>http://com.ibm.rdm.attribute.group.MARKET</j.1:key>
//     <j.2:Target_ID>Support_ID</j.2:Target_ID>
//     <rrm:hasLink>
//       <rrm:link>
//         <rrm:title>Link to End Story</rrm:title>
//         <rrm:href>https://localhost:9443/rdm/resources/rrc18</rrm:href>
//       </rrm:link>
//     </rrm:hasLink>
//   </reference:ReferenceElement>
//   <rdf:Description rdf:about="">
//     <reference:reference rdf:resource="../resources/rrc80"/>
//   </rdf:Description>
// </rdf:RDF>


dojo.declare(
	"com.ibm.rdm.web.attributes.OverviewSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    This class is responsible for the Overview section that appears
	//    in the sidebar.
	// description:
	//    This section displays high level artifact information such as the
	//    name, creation date, modified by, etc. as well as all attribute
	//    group settings for this artifact.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "attributes/templates/OverviewSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields
	resourceURI: null,
	
	XSI_NAMESPACE: "http://www.w3.org/2001/XMLSchema-instance",
	XSI_QUALIFIER: "xsi",

	ATTRIBUTE_NAMESPACE: "http://schema.ibm.com/rdm/2008/Attribute",
	ATTRIBUTE_QUALIFIER: "attribute",
	
	ATTRIBUTE_RRM_NAMESPACE: "http://schema.ibm.com/rdm/2008/Attribute#",
	ATTRIBUTE_RRM_QUALIFIER: "attribute",

	REFERENCE_NAMESPACE: "http://jazz.net/xmlns/alm/rm/Reference/v0.1#",
	REFERENCE_QUALIFIER: "reference",

	// ATTRIBUTE_GROUP_ID_PREFIX: string
	//    This string is for string comparison purposes when attempting to see
	//    whether a particular namespace represents an attribute group ID
	//    namespace (for example, "http://com.ibm.rdm.attribute.group.MARKET#"
	//    is the namespace that represents our MARKET attribute group).
	ATTRIBUTE_GROUP_ID_PREFIX: "http://com.ibm.rdm.attribute.group",

	///////////////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation
	project: null,
	createdOn: null,
	modifiedOn: null,
	modifiedBy: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields
	_messages: null,
	_document: null,
	_mapNamespaceToFieldValueProperties: null,
	_mapNamespaceToAttributeGroupSection: null,
	_userHover: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points
	_contentPane: null,
	_documentNameSpan: null,
	_projectSpan: null,
	_createdOnSpan: null,
	_modifiedOnSpan: null,
	_modifiedBySpan: null,
	_attributeGroupsDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._mapNamespaceToFieldValueProperties = new Object;
		this._mapNamespaceToAttributeGroupSection = new Object;
	},

	initialize: function() {
		this._contentPane.startup();
		if (this._document) {
			this._showAttributeGroupSections(this._projectURI, this._document);
		}
		this._addDoorsLink(this.resourceURI, this.contentType);
	},
	
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.attributes", "OverviewSection");
		dojo.forEach(["project", "createdOn", "modifiedOn", "modifiedBy", "doors"], function(prop){
			this[prop] = this._messages[prop];
		}, this);
	},
	
	destroy: function() {
		this.inherited(arguments);
		com.ibm.rdm.web.attributes.AttributeGroupsCache.cancelLastFetch();
		this._document = null;
	},

	handleRefresh: function() {
		// Note: 'updateProjectField' and 'updateDocumentFields' are called
		// directly to update all fields unrelated to attribute groups.
		// As for attribute groups, 'updateAttributeGroupSections' is called
		// to update the attribute groups from a newly retrieved document.
	},
	
	updateProjectField: function(projectName) {
		this._updateField(this._projectSpan, projectName);
	},

	updateDocumentFields: function(docName, createdDate, modifiedDate, modifiedBy) {
		this._updateField(this._documentNameSpan, docName);
		var artifactId = com.ibm.rdm.web.common.UriHelper.getArtifactId(this.resourceURI);
		var artifactIdText = null;
		if (artifactId != null) {
			artifactIdText = "(" + artifactId + ")";
		} else {
			artifactIdText = "";
		}
		this._updateField(this._documentIdSpan, artifactIdText);
		this._updateField(this._createdOnSpan, this._composeShortDate(createdDate));
		this._updateField(this._modifiedOnSpan, this._composeShortDate(modifiedDate));
		this._updateField(this._modifiedBySpan, modifiedBy);
	},

	updateCreatedDateField: function(createdDate) {
		this._updateField(this._createdOnSpan, this._composeShortDate(createdDate));
	},

	updateNameField: function(docName) {
		this._updateField(this._documentNameSpan, docName);
		var artifactId = com.ibm.rdm.web.common.UriHelper.getArtifactId(this.resourceURI);
		var artifactIdText = null;
		if (artifactId != null) {
			artifactIdText = "(" + artifactId + ")";
		} else {
			artifactIdText = "";
		}
		this._updateField(this._documentIdSpan, artifactIdText);
	},

	updateModifiedDateField: function(modifiedDate) {
		this._updateField(this._modifiedOnSpan, this._composeShortDate(modifiedDate));
	},

	updateModifiedByField: function(userName, userId) {
		this._updateField(this._modifiedBySpan, userName);
		this._removeUserHover();
		this._createUserHover(userId, this._modifiedBySpan);
	},

	// TODO The following functions, updateAttributeGroupSections and setProjectURI,
	// can't assume that they're called in any particular order.  Consider changing
	// the calling code to assure that setProjectURI is called before
	// updateAttributeGroupSections.

	updateAttributeGroupSections: function(doc) {
		this._document = doc;
		if (this._projectURI) {
			this._destroyStoredAttributes();
			this._showAttributeGroupSections(this._projectURI, this._document);
		}
	},

	setProjectURI: function(projectURI) {
		if (! this._projectURI) {
			this._projectURI = projectURI;
			this.initialize();
		}
	},

	containerDimensionsHaveChanged: function() {
		var height = this.domNode.parentNode.clientHeight;
		var width = this.domNode.parentNode.clientWidth;
		var contentPaneTop = (
			this._contentPane.domNode.offsetTop
			- this._contentPane.domNode.parentNode.offsetTop);
		var scrollHeightToUse = (height - contentPaneTop);
		this._contentPane.resize({ h: scrollHeightToUse, w: width });
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_showAttributeGroupSections: function(projectURI, doc) {
		if (this.contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION) {
			this._showCollectionAttributeGroupSections(projectURI, doc);
		}
		else if (com.ibm.rdm.web.common.MimeTypeRegistry.isWrapperResource(this.contentType)) {
			this._showWrappedResourceAttributeGroupSections(projectURI, doc);
		}
		else {
			this._showResourceAttributeGroupSections(projectURI, doc);
		}
	},

	_showWrappedResourceAttributeGroupSections: function(projectURI, doc) {
		// summary:
		//    Show attribute group information for a wrapped resource.
		var self = this;
		var rdfNode = doc.firstChild;
		var attributeGroupNamespaces = [];
		var nodes = domutils.getElementsByTagNameNS(
			rdfNode, "ReferenceElement",
			this.REFERENCE_NAMESPACE, this.REFERENCE_QUALIFIER
		);
		var referenceNode = nodes[0];
		for (var node = referenceNode.firstChild;
			 node != null;
			 node = node.nextSibling
			 ) {
			if (node.nodeType === 1) { // Only consider element nodes
				var namespaceURI = node.namespaceURI;
				if (namespaceURI && namespaceURI.indexOf(self.ATTRIBUTE_GROUP_ID_PREFIX) === 0) {
					// The element has a format similar to the following:
					//     <j.0:Segment_ID>Canada_ID</j.0:Segment_ID>
					var elementName = dojo.isIE ? node.baseName : node.localName;
					var textNodeValue = dojo.isIE ? node.text : node.textContent;
					var attributeGroupNamespace = namespaceURI.substring(0, namespaceURI.length-1);
					var attributeGroupFieldId = elementName;
					var attributeGroupFieldValue = textNodeValue;
					this._storeAttributeGroupFieldValue(
						attributeGroupNamespace,
						attributeGroupFieldId,
						attributeGroupFieldValue);
				}
				else if (namespaceURI && namespaceURI === this.ATTRIBUTE_RRM_NAMESPACE) {
					var elementName = dojo.isIE ? node.baseName : node.localName;
					var textNodeValue = dojo.isIE ? node.text : node.textContent;
					if (elementName === "key") {
						// The element has a format similar to the following:
						//    <j.1:key>http://com.ibm.rdm.attribute.group.MARKET</j.1:key>
						attributeGroupNamespaces.push(textNodeValue);
					}
				}
			}
		}
		// Fetch [only] the attribute groups that are found in this resource's
		// document.
		com.ibm.rdm.web.attributes.AttributeGroupsCache.fetchAttributeGroups(
			projectURI,
			attributeGroupNamespaces,
			this._showAttributeGroup,
			this
		);
	},

	_showResourceAttributeGroupSections: function(projectURI, doc) {
		// summary:
		//    Show attribute group information for "normal" resources (i.e. not
		//    a Collection or a wrapped resource).
		var attributeGroupNamespaces = [];
		// Fill in 'attributeGroupNamespaces'.
		var attributeGroupNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
			doc,
			"attributeGroup",
			this.ATTRIBUTE_NAMESPACE,
			this.ATTRIBUTE_QUALIFIER);
		for (var i = 0; i < attributeGroupNodes.length; i++) {
			var namespace = attributeGroupNodes[i].getAttribute("key");
			attributeGroupNamespaces.push(namespace);
		}
		var self = this;
		var gatherAndStoreTypedAttributeFieldValues = function(attributeTypeTagName) {
			var attributeTypeNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
				doc,
				attributeTypeTagName,
				self.ATTRIBUTE_NAMESPACE,
				self.ATTRIBUTE_QUALIFIER);
			for (var i = 0; i < attributeTypeNodes.length; i++) {
				var attributeTypeNode = attributeTypeNodes[i];
				var key = attributeTypeNode.getAttribute("key");
				var hashIndex = key.indexOf("#");
				var namespace = key.substring(0, hashIndex);
				var fieldId = key.substring(hashIndex+1);
				var value = attributeTypeNode.getAttribute("value");
				self._storeAttributeGroupFieldValue(namespace, fieldId, value);
			}
		};
		// Store away the field values that are found in this resource.
		gatherAndStoreTypedAttributeFieldValues("booleanAttribute");
		gatherAndStoreTypedAttributeFieldValues("dateAttribute");
		gatherAndStoreTypedAttributeFieldValues("decimalAttribute");
		gatherAndStoreTypedAttributeFieldValues("enumerationAttribute");
		gatherAndStoreTypedAttributeFieldValues("integerAttribute");
		gatherAndStoreTypedAttributeFieldValues("stringAttribute");
		gatherAndStoreTypedAttributeFieldValues("URIAttribute");
		// Fetch [only] the attribute groups that are found in this resource's
		// document.
		com.ibm.rdm.web.attributes.AttributeGroupsCache.fetchAttributeGroups(
			projectURI,
			attributeGroupNamespaces,
			this._showAttributeGroup,
			this
		);
	},
	
	_showCollectionAttributeGroupSections: function(projectURI, doc) {
		// summary:
		//    Show attribute group information for a Collection.
		var attributeGroupNamespaces = [];
		var mapAttributeGroupNamespaceToBoolean = new Object();
		// Fill in 'attributeGroupNamespaces' and 'mapAttributeGroupNamespaceToBoolean'.
		var collectionNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
			doc,
			"Collection",
			queryutils.COLLECTION_NAMESPACE,
			queryutils.COLLECTION_QUALIFIER)[0];
		var attributeGroupNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
			collectionNode,
			"key",
			this.ATTRIBUTE_RRM_NAMESPACE,
			this.ATTRIBUTE_RRM_QUALIFIER);
		for (var i = 0; i < attributeGroupNodes.length; i++) {
			var attributeGroupNamespace = queryutils.getTextNodeValue(attributeGroupNodes[i]);
			attributeGroupNamespaces.push(attributeGroupNamespace);
			mapAttributeGroupNamespaceToBoolean[attributeGroupNamespace] = true;
		}
		// Store away the field values that are found in this resource.
		var collectionChildNodes = collectionNode.childNodes;
		for (var i = 0; i < collectionChildNodes.length; i++) {
			var node = collectionChildNodes[i];
			if (node.attributes
				&& (node.attributes.length == 1 || node.attributes.length == 2)
				) {
				var firstAttributeValue = queryutils.getTextNodeValue(node.attributes[0]);
				var textThatMightBeNamespace = firstAttributeValue.substring(0, firstAttributeValue.length-1);
				if (mapAttributeGroupNamespaceToBoolean[textThatMightBeNamespace]) {
					var namespace = textThatMightBeNamespace;
					var fieldId = dojo.isIE ? (node.baseName) : (node.localName);
					var attributeValue = queryutils.getTextNodeValue(node);
					this._storeAttributeGroupFieldValue(namespace, fieldId, attributeValue);
				}
			}
		}
		// Fetch [only] the attribute groups that are found in this resource's
		// document.
		com.ibm.rdm.web.attributes.AttributeGroupsCache.fetchAttributeGroups(
			projectURI,
			attributeGroupNamespaces,
			this._showAttributeGroup,
			this
		);
	},

	_storeAttributeGroupFieldValue: function(attributeGroupStyleNamespace, fieldId, value) {
		 var array = this._mapNamespaceToFieldValueProperties[attributeGroupStyleNamespace];
		 if (!array) {
			 array = [];
			 this._mapNamespaceToFieldValueProperties[attributeGroupStyleNamespace] = array;
		 }
		 array.push({
			 id: fieldId,
			 value: value
		 });
	},

	_showAttributeGroup: function(namespace, id, displayName, fieldProperties) {
		var section = this._mapNamespaceToAttributeGroupSection[namespace];
		if (!section) {
			section = new com.ibm.rdm.web.attributes.AttributeGroupSection({
				sectionTitle: displayName
			});
			section.startup();
			this._mapNamespaceToAttributeGroupSection[namespace] = section;
		}
		section.update(fieldProperties, this._mapNamespaceToFieldValueProperties[namespace]);
		var child = this._attributeGroupsDiv.firstChild;
		while (child != null && displayName > dijit.byNode(child).sectionTitle) {
			child = child.nextSibling;
		}
		if (child) {
			dojo.place(section.domNode, child, "before");
		} else {
			this._attributeGroupsDiv.appendChild(section.domNode);
		}
	},

	_updateField: function(parentNode, textToAdd) {
		if (textToAdd) {
			var textNode = document.createTextNode(textToAdd);
			var firstChild = parentNode.firstChild;
			if (firstChild) {
				parentNode.replaceChild(textNode, firstChild);
			} else {
				parentNode.appendChild(textNode);
			}
		}
	},

	_composeShortDate: function(date) {
		if (date)
			return dojo.date.locale.format(date, { formatLength: "short" });
		else
			return "";
	},
	
	_destroyStoredAttributes: function() {
		for (var attributeNamespace in this._mapNamespaceToAttributeGroupSection) {
			this._mapNamespaceToAttributeGroupSection[attributeNamespace].destroyRecursive(false);
		}
		this._mapNamespaceToFieldValueProperties = new Object;
		this._mapNamespaceToAttributeGroupSection = new Object;
	},
	
	_addDoorsLink: function(resourceURI, contentType) {
		
		if(contentType === com.ibm.rdm.web.common.MimeTypes.RICH_TEXT.contentType){
			var postData = com.ibm.rdm.web.FRONTING_SERVICE_LINKS + encodeURI("?resourceURI=" + resourceURI);
			var deferred = RDMClient.doGet({uri:postData});    	
			deferred.addCallback(this, this._handleQuerySuccessResponse);
		}
		else{
			return;
		}
	},
	
	_handleQuerySuccessResponse: function(response) {
		if(response.doc){
			var descriptionNodes = domutils.getElementsByTagNameNS(
						response.doc, "Description",
						queryutils.RDF_NAMESPACE, 
						queryutils.RDF_QUALIFIER);
			for(var i=0; i<descriptionNodes.length; i++) {
				var hasLinkTypeNode = domutils.getElementsByTagNameNS(
						descriptionNodes[i], "hasLinkType",
						queryutils.RRM_NAMESPACE, 
						queryutils.RRM_QUALIFIER);
				if(hasLinkTypeNode){
					var linkSourceNode = domutils.getElementsByTagNameNS(
						descriptionNodes[i], "linkSource",
						queryutils.RRM_NAMESPACE, 
						queryutils.RRM_QUALIFIER);
					var linkSourceAttribute = domutils.getAttributeNS(
						linkSourceNode[0], "resource", 
						queryutils.RDF_NAMESPACE, 
						queryutils.RDF_QUALIFIER);
					//check for Doors link type
					var hasLinkTypeAttribute = domutils.getAttributeNS(
						hasLinkTypeNode[0], "resource", 
						queryutils.RDF_NAMESPACE, 
						queryutils.RDF_QUALIFIER);
					if((hasLinkTypeAttribute == com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES + 
						queryutils.DOORSRRCLINKTYPE)&&(linkSourceAttribute == this.resourceURI)){
						var DoorsLinksParser = new com.ibm.rdm.web.parsers.DoorsLinksParser({
						"descNode": descriptionNodes[i]});
						this._createDoorsDisplayInfo(DoorsLinksParser);
					}    				   					
				}
			}
		}
	},
	
	_createDoorsDisplayInfo: function(DoorsLinksParser){
		var div = document.createElement("DIV");
		dojo.addClass(div, "doors-document");
		div.appendChild(document.createTextNode(this._messages.doors));
		this._attributeGroupsDiv.appendChild(div);
		var link = ResourceLinksHandler.getResourceLink(DoorsLinksParser.getDoorsLinkTarget(), DoorsLinksParser.getDocumentName(), false);
		dojo.addClass(link, "doors-link");
		div.appendChild(link);
		this._attributeGroupsDiv.appendChild(div);
	},
	
	_createUserHover: function(userId, node) {
		var self = this;
		var onProjectMembersComplete = function(userPropertyInfos) {
			// Note: The userPropertyInfos is key'd on userURI, not userId, so we need to
			// iterate over them to find the appropriate one ...
			for (var userUri in userPropertyInfos){	
				if (userId == userPropertyInfos[userUri].userId){
					userInfo = userPropertyInfos[userUri];
					var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : this._projectURI}, null);
					userHover.createHoverPopup(node, userInfo);
					self._userHover = userHover;
					break;
				}
			}
		}
		if (this._projectURI){
			var deferred = ProjectCache.getProjectUsersWithRoles(this._projectURI);
			deferred.addCallback(this, onProjectMembersComplete);
		}	
	},

	_removeUserHover: function() {
		if (this._userHover) {
			this._userHover.destroy();
			this._userHover = null;
		}
	}
});
