/* 
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.CommentPane");
dojo.require("dijit.TitlePane");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.commenting.Comment");
dojo.require("com.ibm.rdm.web.commenting.CommentingController");
dojo.require("com.ibm.rdm.web.common.IconSelectWidget");
dojo.require("com.ibm.rdm.web.common.UsersSelectWidget");
dojo.require("com.ibm.rdm.web.commenting.utils")
dojo.require("dojo.date.locale");
dojo.require("dijit.Menu");
dojo.require("dijit.Dialog");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ProjectCache");

var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var CommentingUtils = com.ibm.rdm.web.commenting.utils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;

/*
   All implementers must extend _Widget, and have a property called commentingWidget, which
   is an instance of com.ibm.rdm.web.commenting.CommentingWidget.
*/
dojo.declare("com.ibm.rdm.web.commenting.WidgetMixin", null, {
    // Contract
    getController: function() {
        return this.commentingWidget._controller;
    },
    hide: function() {
    	// TODO (LPK) workaround for sequencing error on edit reply comments
    	if (this.domNode == null) return;
    	this.domNode.style.display = "none";
    },
    isHidden: function() {
    	// TODO (LPK) workaround for sequencing error on edit reply comments
    	if (this.domNode == null) return true;
    	return this.domNode.style.display == "none";
    },
    unhide: function() {
    	// TODO (LPK) workaround for sequencing error on edit reply comments
    	if (this.domNode == null) return;
        this.domNode.style.display = "block";
    }

});
 

dojo.declare("com.ibm.rdm.web.commenting.CommentPane", [dijit._Widget, dijit._Templated], {
	annotation: null,
	resourceLocationPane: null, 
	widgetsInTemplate: true,
	commentingWidget: null,  // required for WidgetMixin
	_commentTable: null,
	_messages: null,
	replyPanes: null,
	parentPane: null,
	isExpanded: null,
	_lengthForTruncation: 300,
	_isTruncated: false,
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "commenting/templates/CommentPane.html"),
    // Internals
    postCreate: function() {
        this.inherited("postCreate", arguments);
        this.setComment(this.annotation);
        //attach context menu from the commenting widget to current commentpane
        this.commentingWidget._commentingToolbarOptionsMenu.bindDomNode(this.domNode);
		this.connect(this._commentTable, "onkeypress", this._onKeyPressComment); 
		this.replyPanes = new Array();
		this.isExpanded = false;
		if (this.annotation.isResolved) {
			this._resolveAnchor.setAttribute("title", this._messages.unresolveTooltip);
		} else {
			this._resolveAnchor.setAttribute("title", this._messages.resolveTooltip);
		}
		this._hideHoverItems();
    },
   	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "commenting");
   	},
    destroy: function() {
        this.inherited(arguments);
    },
    expand: function() {
    	if (!this.isRootPane()) return;
    	if (this.annotation.isShowForFilter()) {
        	this.isExpanded = true;
        	this._commentTextRow.style.display = "";
        	dojo.forEach(this.replyPanes, function(replyPane) {
        		replyPane.unhide();
        	});
    	}
    },
    collapse: function() {
    	if (!this.isRootPane()) return;
        this.isExpanded = false;
        this._commentTextRow.style.display = "none";
        dojo.forEach(this.replyPanes, function(replyPane) {
        	replyPane.hide();
        });
    },
    expandCollapse: function() {
    	if (!this.isRootPane()) return;
    	if (this.isExpanded) {
        	this.collapse();
    	} else {
        	this.expand();
    	}
    },
    /**
        Update the state of this pane the provided comment. Can be called
        after creation to update what the widget displays.
    */
    setComment: function(comment /*Comment*/) {
        this.annotation = comment;

        if (!this.annotation.priority) {
            this.annotation.priority = com.ibm.rdm.web.commenting.PRIORITY_LOW;
        }
        
        /*
        var iconClass = "priority" + this.annotation.priority + (this.annotation.isResolved?"Resolved":"");
        if (this._priorityIconDDWidget) {
            this._priorityIconDDWidget.destroyRecursive();
        }
        this._priorityIconDDWidget = new com.ibm.rdm.web.commenting.IconDropdown({menu: self._priorityMenu, iconClass: iconClass});
        this._priorityIconDDNode.appendChild(this._priorityIconDDWidget.domNode);
        
        this.colorClass = this._calculateColorScheme(this.annotation); 
        dojo.addClass(this._prioritySelectTdNode, this.colorClass);
        */
        if(this.commentingWidget){
        	this.annotation.projectURI = this.commentingWidget.getProjectURI();
        }
        
        // Reply node
        if (!this.annotation.isRoot) {
        	dojo.addClass(this._commentPaneNode, "rdm-annotation-replyCommentPane");
        }
        else {
        	dojo.addClass(this._commentPaneNode, "rdm-annotation-rootCommentPane");
        }
        
        // id & text
        this._commentIndexNode.innerHTML = this.annotation.commentNumber + ".&nbsp;";
        
        this._isTruncated = CommentingUtils.setCommentText(this._commentTextNode, this.annotation);
        
        if (!this.annotation.isRoot) {
        	CommentingUtils.createTitleNode(this._commentHeadlineNode, this.annotation, this.annotation.parent, this.commentingWidget);
        } else {
        	CommentingUtils.createTitleNode(this._commentHeadlineNode, this.annotation);
        }

        // TEMP
        /*
        function m(s) {
            return s.substring(s.indexOf("comment")); 
        }
        this._tempNode.innerHTML = 
            "Self: " 
            + m(this.annotation.linkSelfUri)
            + ", Par: " + m(this.annotation.parentComment)
            + ", Root: " + m(this.annotation.rootComment)
            + ", isRoot: " + this.annotation.isRoot() 
            + ", isResolved: " + this.annotation.isResolved
            ;
        */    
        // END TEMP
        
        
        // date & author details
        if (!this.annotation.isRoot) {
        	this._authorDateNode.appendChild(
            	com.ibm.rdm.web.commenting.utils.createCommentByline(this.annotation)
        	);
        } else {
        	this._authorDateNode.appendChild(
            	com.ibm.rdm.web.commenting.utils.createCommentByline(this.annotation)
        	);
        }
        this.updateReplyDetails();
        //dojo.addClass(this._authorDateNode, "resolved-comment-text");
    },
    
    addMore: function() {
	   	if (this._isTruncated) {
       		this._commentMoreLink.innerHTML = this._messages.more;
            CommentingUtils.hookupCommentMoreLink(this._commentMoreLink, this.annotation, this.commentingWidget);
		} else {
       		this._commentMoreLink.innerHTML = "";
       	}
    },
    
    // Contract
    getResourceLocationId: function() {
        return this.annotation.locationId;
    },
    select: function() {
        dojo.removeClass(this._commentPaneNode, "rdm-annotation-commentpane-hover");
        dojo.removeClass(this._commentSubjectRow, "rdm-annotation-commentpane-hover");
        dojo.removeClass(this._commentAuthorRow, "rdm-annotation-commentpane-hover");
        dojo.addClass(this._commentPaneNode, "rdm-annotation-commentpane-highlighted");
        this._setStyleOfFrontPanes("transparent none repeat scroll 0%");
        var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
        this.commentingWidget.publish(topic, [this.annotation.locationId, false]);
    },
    unselect: function() {
        dojo.removeClass(this._commentPaneNode, "rdm-annotation-commentpane-highlighted");
        this._setStyleOfFrontPanes("");
    },
    getRootPane: function() {
        if (this.parentPane == null) return this;
        else return this.parentPane.getRootPane();
    },
    isRootPane: function() {
    	return this.annotation.parentComment == null || this.annotation.parentComment == "";
    },
    getController: function() {
        return this.commentingWidget._controller;
    },
    hide: function() {
    	if (this.domNode == null) return;
    	this.domNode.style.display = "none";
    },
    isHidden: function() {
    	return this.domNode.style.display == "none";
    },
    unhide: function() {
    	// if the comment pane is not showing due to filtering, don't unhide the pane
    	if (!this.annotation.isShowForFilter()) return;
    	// if the parent comment pane is collapsed, don't unhide the pane
    	if (this.parentPane != null && !this.parentPane.isExpanded) return;
    	if (this.domNode == null) return;
        this.domNode.style.display = "block";
    },
    _resolveClicked: function(event) {
		if (dojo.hasClass(this._resolveAnchor, "enabled")) {
    		event.preventDefault();
			event.stopPropagation();
			this.commentingWidget.resolveButtonClick(this.annotation);
		}
    },    
    _editClicked: function(event) {
		if (dojo.hasClass(this._editAnchor, "enabled")) {
    		event.preventDefault();
			event.stopPropagation();
			this.commentingWidget.editButtonClick(this.annotation);
		}
    },    
    _replyToClicked: function(event) {
		if (dojo.hasClass(this._replyToAnchor, "enabled")) {
    		event.preventDefault();
			event.stopPropagation();
			this.commentingWidget.replyToButtonClick(this.annotation);
		}
    },    
    _onPaneClicked: function(event) {
    	if (this.isRootPane()) return;
        this.commentingWidget._selectCommentPane(this.annotation.linkSelfUri, false);
    },
    _onHeaderClicked: function(event) {
    	if (!this.isRootPane()) return;
        this.expandCollapse();
        this.commentingWidget._selectCommentPane(this.annotation.linkSelfUri, false);
        this._hoverCommentHeader();
    },
    _onTextClicked: function(event) {
    	if (!this.isRootPane()) return;
        this.commentingWidget._selectCommentPane(this.annotation.linkSelfUri, false);
    },
    _onPaneRightClicked: function(event) {
        this.commentingWidget._selectCommentPane(this.annotation.linkSelfUri, false);
        //this.commentingWidget._commentingToolbarOptionsMenu._openMyself(event);
    },
    _onHeaderMouseOver: function(event) {
    	if (this.isRootPane()) this._hoverCommentHeader();
    	else this._hoverCommentPane();
    },
    _onHeaderMouseOut: function(event) {
    	if (!this.isRootPane()) return;
        this._unhoverCommentHeader();
    },
    _onTextMouseOver: function(event) {
        this._hoverCommentPane();
    },
    _onTextMouseOut: function(event) {
        this._unhoverCommentPane();
    },
    _onPaneMouseOver: function(event) {
    	this._showHoverItems();
        //event.preventDefault();
        //event.stopPropagation();
    	if (this.isRootPane()) return;
        this._hoverCommentPane();
        
    },
    _onPaneMouseOut: function(event) {
    	if (!this.isRootPane() || this.isExpanded) {
    		this._hideHoverItems();
    	}
    	if (this.isRootPane()) return;
        //event.preventDefault();
        //event.stopPropagation();
        this._unhoverCommentPane();
    },
    _hideHoverItems: function() {
        dojo.removeClass(this._replyToAnchor, "showElement");
        dojo.addClass(this._replyToAnchor, "hideElement");   
        dojo.removeClass(this._editAnchor, "showElement");
        dojo.addClass(this._editAnchor, "hideElement");   
        dojo.removeClass(this._resolveAnchor, "showElement");
        dojo.addClass(this._resolveAnchor, "hideElement");   
    },
    _showHoverItems: function() {
        dojo.addClass(this._replyToAnchor, "showElement");
        dojo.removeClass(this._replyToAnchor, "hideElement"); 
        dojo.addClass(this._editAnchor, "showElement");
        dojo.removeClass(this._editAnchor, "hideElement");   
        dojo.addClass(this._resolveAnchor, "showElement");
        dojo.removeClass(this._resolveAnchor, "hideElement");  
        this.commentingWidget.setButtonStates(this.annotation, this._replyToAnchor, this._editAnchor, this._resolveAnchor); 
    },
    
    _unhoverCommentHeader: function() {
        dojo.removeClass(this._commentSubjectRow, "rdm-annotation-commentpane-hover");
        dojo.removeClass(this._commentAuthorRow, "rdm-annotation-commentpane-hover");
        dojo.removeClass(this._commentSubjectRow, "rdm-annotation-commentpane-selected-hover");
        dojo.removeClass(this._commentAuthorRow, "rdm-annotation-commentpane-selected-hover");
        var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
        this.commentingWidget.publish(topic, [this.annotation.locationId, false]);
    },
    _hoverCommentHeader: function() {
        if (!dojo.hasClass(this._commentPaneNode, "rdm-annotation-commentpane-highlighted")) {
            dojo.addClass(this._commentSubjectRow, "rdm-annotation-commentpane-hover");
            dojo.addClass(this._commentAuthorRow, "rdm-annotation-commentpane-hover");
            var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
            this.commentingWidget.publish(topic, [this.annotation.locationId, true]);
        } else {
            dojo.addClass(this._commentSubjectRow, "rdm-annotation-commentpane-selected-hover");
            dojo.addClass(this._commentAuthorRow, "rdm-annotation-commentpane-selected-hover");
        }
    },
    _unhoverCommentPane: function() {
        dojo.removeClass(this._commentPaneNode, "rdm-annotation-commentpane-hover");
        this._setStyleOfFrontPanes("none");
        var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
        this.commentingWidget.publish(topic, [this.annotation.locationId, false]);
    },
    _hoverCommentPane: function() {
        if (!dojo.hasClass(this._commentPaneNode, "rdm-annotation-commentpane-highlighted")) {
            // Hover style, only if the element has not been selected
	        this._setStyleOfFrontPanes("transparent none repeat scroll 0%");
	        dojo.addClass(this._commentPaneNode, "rdm-annotation-commentpane-hover");
	        var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
	        this.commentingWidget.publish(topic, [this.annotation.locationId, true]);
        }
    },
    _onKeyPressComment: function(event) {
    	if(event.charCode == dojo.keys.SPACE) { 
    		this.commentingWidget._selectCommentPane(this.annotation.linkSelfUri, false);
    	}
    	if(event.keyCode == dojo.keys.ENTER) { 
    		this.expandCollapse();
    	}
    },
     
    updateReplyDetails: function() {
    	var message = null;
    	if (this.annotation.replies == null || this.annotation.replies.length == 0) {
    		message = "";
    	} else {
    		if (this.annotation.isRoot) {
    			var allReplies = this.annotation.getAllReplies();
        		var replyCount = allReplies.length;
        		var showCount = 0;
        		dojo.forEach(allReplies, function(reply) {
        			if (reply.isShowForFilter()) showCount = showCount + 1;
        		});
        		if (replyCount > 1) {
        			if (replyCount == showCount) {
        				message = dojo.string.substitute(this.commentingWidget._messages.nReplies, [replyCount]);
        			} else {
        				message = dojo.string.substitute(this.commentingWidget._messages.nOfMReplies, [showCount, replyCount]);
        			}
        		} else {
        			if (replyCount == showCount) {
        				message = this.commentingWidget._messages.oneReply;
        			} else {
        				message = dojo.string.substitute(this.commentingWidget._messages.nOfMReply, [showCount, replyCount]);
        			}
        		}
        		message = "&nbsp;" + message;
    		} else {
        		message = "";
    		}
    	}
    	this._repliesCountNode.innerHTML = message;
    },
    
    /*
        Hack to make sure highlight and hover background colors are not
        blocked by the nodes on top.
        val = "transparent" allows setting new background on the whole pane
        val = "" restores to normal state
        TODO: Change this
        
    */
    _setStyleOfFrontPanes: function(val) {
        if (!val)
            val = "none";
        //this._authorDateSectionContainerNode.style.background = val;
        this._commentTextNode.style.background = val;
        //this._authorDateSectionNode.style.background = val;
    
    }
});

dojo.declare("com.ibm.rdm.web.commenting.IconDropdown", [dijit._Widget, dijit._Templated], {
    templateString: "<div dojoAttachPoint=\"_node\" class=\"iconDropdown\" ><div dojoAttachPoint=\"_iconNode\" dojoAttachEvent=\"onclick:_onIconClick, onmouseover:_onMouseOver, onmouseout:_onMouseOut\"></div></div>" ,
    iconClass: null,
    menu: null,
    toggleOnClick: null,
    onClick: function() {
        // user defined
    },
    postCreate: function() {
        this.inherited("postCreate", arguments);
        dojo.addClass(this._iconNode, this.iconClass);
    },
    isActive: function() {
        return dojo.hasClass(this._iconNode, (this.iconClass + "Enabled"));
    },
    activate: function() {
        if (!this.isActive()) {
	        dojo.removeClass(this._iconNode, this.iconClass);
	        dojo.addClass(this._iconNode, this.iconClass + "Enabled");
	    }
    },
    deactivate: function() {
        if (this.isActive()) {
	        dojo.removeClass(this._iconNode, this.iconClass + "Enabled");
	        dojo.addClass(this._iconNode, this.iconClass);
        }
    },
    toggle: function() {
        if (this.isActive()) {
            this.deactivate();
        } else {
            this.activate();
        }
            
    },
    destroy: function() {
        this.inherited(arguments);
    },
    _onIconClick: function(event) {
        if (this.toggleOnClick) {
            this.toggle();
        }
        if (this.menu!=null && this.isActive()) {
            this.menu._openMyself(event);
        }
        // user defined
        this.onClick();
    },
    _onMouseOver: function(event) {
        if (!this.isActive()) {  // CSS selector for sibling classes do not work in IE. For now, prevent hover style if active.
            dojo.addClass(this._iconNode, this.iconClass + "Hover");
        }
    },
    _onMouseOut: function(event) {
        dojo.removeClass(this._iconNode, this.iconClass + "Hover");
    }
    
    
    
});

dojo.declare("com.ibm.rdm.web.commenting.ResourceLocationPane", [dijit.TitlePane, com.ibm.rdm.web.commenting.WidgetMixin], {
	/*
		To add a child CommentPane, you must use either addChildCommentPane() method or specify
		the CommentPane in ctor's param.commentPanes array, so that CommentPane.resourceLocationPane
		can be set.
	*/

	// Array of CommentPane. Must be set in ctor.
	commentPanes: null,
	// Reference to containing CommentingWidget. Must be set in ctor.
	commentingWidget: null,
	paneId: null, 
    // location id in the resource. Must be passed in ctor
    _subscriptions: null,
    resourceLocationId: null,
    widgetsInTemplate: true,
    editCommentForm: null,
    // Override template of TitlePane to remove dijitInlineBox class, which inserts display=-moz-inline-stack, preventing rich html being displayed in the title in FF
    templateString:"<div class=\"dijitTitlePane\" >\n\t<div  dojoAttachEvent=\"onmouseenter:_onTitleMouseOver, onmouseleave:_onTitleMouseOut, ondblclick:_titleDblClick,onclick:_titleClick, onkeypress: _onPaneTitleClick\" \n\t\t\twaiRole=\"button\" class=\"dijitTitlePaneTitle\" dojoAttachPoint=\"focusNode\">\n\t\t<span dojoAttachPoint=\"arrowNode\" class=\"dijitInline dijitArrowNode\"><span dojoAttachPoint=\"arrowNodeInner\" class=\"dijit_a11y dijitArrowNodeInner\"></span></span>\n\t\t<span dojoAttachPoint=\"titleNode\" class=\" dijitTitleNode rdm-annotation-widgetpane-title\"><span dojoAttachPoint=\"_titleCommentDescriptionNode\"></span><span dojoAttachPoint=\"_titleCommentCountNode\"></span></span>\n\t</div>\n\t<div class=\"dijitTitlePaneContentOuter\" dojoAttachPoint=\"hideNode\">\n\t\t<div class=\"dijitReset\" dojoAttachPoint=\"wipeNode\">\n\t\t\t<div class=\"dijitTitlePaneContentInner\" dojoAttachPoint=\"containerNode\" waiRole=\"region\">\n\t\t\t\t<!-- nested divs because wipeIn()/wipeOut() doesn't work right on node w/padding etc.  Put padding on inner div. -->\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n",
    postCreate: function() {
        this.inherited("postCreate", arguments);
    	this.paneId = this.resourceLocationId;
    	this.initializeCommentPanes(this.commentPanes);
        // Override default styles 
        this.containerNode.style.padding = "0px 0px 0px 0px";
        //this.updateCount(); update after filtering
        this._subscriptions = [];
        this._initSubscriptions();
    },
    subscribe: function(topic, method) {
        this._subscriptions.push(
        		dojo.subscribe(
        				EventTopics.getTopic(topic, this.commentingWidget.resourceURI), 
        				this, 
        				method));
    },
    _initSubscriptions: function() {
    	//this.subscribe(EventTopics.RESOURCE_LOADED_IN_VIEWER_TOPIC, "updateTitleDescription");
    	//this.subscribe(EventTopics.REFRESH_COMMENT_TITLES_TOPIC, "updateTitleDescription");
    },
    initializeCommentPanes: function(commentPanes) {
    	this.commentPanes = commentPanes;
    	for(i=0; (this.commentPanes && i<this.commentPanes.length);  i++) {
    		this.commentPanes[i].resourceLocationPane = this;
    		this.containerNode.appendChild(this.commentPanes[i].domNode);
        }
		for (var i=0; i<this.commentPanes.length; i++) {
    		var commentPane = this.commentPanes[i];
    		if (commentPane.isRootPane()) {
    			if (commentPane.isExpanded) {
    				commentPane.expand();
    			} else {
    				commentPane.collapse();
    			}
    		}
		}
    },
    addMoreButton: function() {
    	var commentPanes = this.commentPanes;
    	if (commentPanes) {
    		for (var i=0; i< commentPanes.length; i++) {
    			commentPanes[i].addMore();
    		}
    	}
    },
    destroy: function() {
        for (var i = 0; i < this._subscriptions.length; i++) {
	            dojo.unsubscribe(this._subscriptions[i]);
	            this._subscriptions[i] = null;
	        }
        this._subscriptions = [];
        this.inherited(arguments);
    },
    destroyCommentPanes: function() {
        for (var i=0; this.commentPanes && i<this.commentPanes.length; i++) {
            var p = this.commentPanes[0];
            p.destroyRecursive();
        }
    },
    // Contract 
    getResourceLocationId: function() {
        return this.resourceLocationId;
    },
    updateCount:function() {
        if (!this.isHidden()) {
	    	var t = ""; 
	        var total = (this.commentPanes)?(this.commentPanes.length):0;
	        var unfiltered = 0;
			for (var i=0; i<this.commentPanes.length; i++) {
	    		var commentPane = this.commentPanes[i];
	    		if (commentPane.annotation.isShowForFilter()) unfiltered++;
			}
	        var countText = dojo.string.substitute(this.commentingWidget._messages.showCount, [unfiltered, total]);    
	        dojo.isIE?(this._titleCommentCountNode.innerText = " " + countText):(this._titleCommentCountNode.textContent = " " + countText);
        }
    },
    setTitle: function(title) {
    	dojo.isIE?(this._titleCommentDescriptionNode.innerText = title):(this._titleCommentDescriptionNode.textContent = title);
    },
    select: function() {
        dojo.addClass(this.domNode, "rdmAnnotationHighlighted");
        dojo.addClass(this.focusNode, "rdm-annotation-dijitTitlePaneTitleHighlighted");
    },
    unselect: function() {
        dojo.removeClass(this.domNode, "rdmAnnotationHighlighted");
        dojo.removeClass(this.focusNode, "rdm-annotation-dijitTitlePaneTitleHighlighted");
    },
    _isForm: function() {
        var cs = this.containerNode.childNodes;
        for (var i=0; cs && i<cs.length; i++) {
            var c = cs.item(i);
            var w;
            try {
                w = dijit.byNode(c);
            } catch(e) {
            	// don't handle the error, false will be returned in this instance
            }
            if (w && w.declaredClass=="com.ibm.rdm.web.commenting.EditCommentForm" ) {
                return true;             
            } 
        }
        return false;
    },
    _onPaneTitleClick: function(e) {
        //dojo.stopEvent(e);
        if(e.keyCode == dojo.keys.ENTER || e.charCode == dojo.keys.SPACE){
            this.toggle();
        }
        
    },
    _titleClick: function(event) {
    	//event.preventDefault();
        //event.stopPropagation();
        var stayPut = true;
        if (!dojo.hasClass(this.focusNode, "rdm-annotation-dijitTitlePaneTitleHighlighted") && 
        		this.commentingWidget._groupCommentsBy == com.ibm.rdm.web.commenting.CommentingWidget.Ordering.GROUPBY_ARTIFACT) {
        	this.commentingWidget._unselectHighlightedCommentPane();
        	if (!this.open)
            	this.toggle();
        }
        else
        	this.toggle();
        this.commentingWidget._highlightPane(this, stayPut);
    },
    _titleDblClick: function(event) {
        /*event.preventDefault();
        event.stopPropagation();
        this.toggle();*/
    },
    _onTitleMouseOver: function(event) {
    	//event.preventDefault();
        //event.stopPropagation();
    	dojo.addClass(this.focusNode, "titlePaneHover");
    	var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
    	this.commentingWidget.publish(topic, [this.resourceLocationId, true]);
    },
    _onTitleMouseOut: function(event) {
    	//event.preventDefault();
        //event.stopPropagation();
    	dojo.removeClass(this.focusNode, "titlePaneHover");
    	var topic = com.ibm.rdm.web.commenting.EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC;
    	this.commentingWidget.publish(topic, [this.resourceLocationId, false]);
    },
    hideIfAllChildrenAreHidden: function() {
        // TODO: Inefficient. Keep track of hidden children.
        // 6454: QuickFix: There can be more than one edit form under for RLPane.
        var hideMe = true;
        if (this._isForm()) {
            hideMe = false;
        }
        for (var i=0; hideMe&& this.commentPanes && i<this.commentPanes.length; i++) {
            var cPane = this.commentPanes[i];
            if (!cPane.isHidden()) {
                hideMe = false;
                break;
            }
        }
        
        if (hideMe) {
            this.hide();
        }
    },
    toggle: function(){
		(this.open)?this.wipeNode.style.display="none":this.wipeNode.style.display="";
		(this.open)?this.hideNode.style.display="none":this.hideNode.style.display="";
		this.open =! this.open;

		// load content (if this is the first time we are opening the TitlePane
		// and content is specified as an href, or we have setHref when hidden)
		this._loadCheck();

		this._setCss();
	}
});

