/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.commenting.FilterWidget");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.requireLocalization("com.ibm.rdm.web.commenting", "commenting");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

dojo.declare("com.ibm.rdm.web.commenting.FilterWidget", [dijit._Widget, dijit._Templated], {
    /**
        By default, comments updated before UPDATED_DAYS_BEFORE_DEFAULT days before
        today are filtered. Must be negative.
    */
    UPDATED_DAYS_BEFORE_DEFAULT: -7,
    // Main CommentingWidget. Must be passed in ctor. 
    commentingWidget: null,
    // UNORDERED list of hidden comment panes
    _filteredCommentPanes: null,
    _isCommentCountDisplayed: null,
    _priorityLabelNode: null,
    _forLabelNode: null,
    _authorLabelNode: null,
    _howRecentLabelNode: null,
    _messages: null,
    _projectURI: null,
    _hideResolvedCheckNode: null,
    _hideDeletedCheckNode: null,
    _showOnlyReviewCheckNode: null,
    _showAllCheckboxNode: null,
    _showAllLabel: null,
    _showOnlyReviewLabel: null,
    howRecentSelectWidget: null,
    priorityWidget: null,
    authorWidget: null,
    directedToWidget: null,
    
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "commenting/templates/FilterWidget.html"),
    postCreate: function() {
        this._filteredCommentPanes = new Array();
        // Priority field
        this.priorityWidget = new com.ibm.rdm.web.common.IconSelectWidget({
            widgetmode: "full", 
            iconSelectOptions: com.ibm.rdm.web.commenting.utils.getFilterPriorityOptions()
        }, this._prioritySelectNode);
        // accessibility, match up label with the widget
		this.priorityWidget._htmlSelectNode.setAttribute("id", this.priorityWidget.id);
		this._priorityLabelNode.setAttribute("for", this.priorityWidget._htmlSelectNode.id);

        // How Recent dropdown
        this.howRecentSelectWidget = new com.ibm.rdm.web.common.IconSelectWidget({
            widgetmode: "full", 
            iconSelectOptions: com.ibm.rdm.web.commenting.utils.getUpdatedOptions()
        }, this._howRecentSelectNode);        
        // accessibility, match up label with the widget
		this.howRecentSelectWidget._htmlSelectNode.setAttribute("id", this.howRecentSelectWidget.id);
		this._howRecentLabelNode.setAttribute("for", this.howRecentSelectWidget._htmlSelectNode.id);
		if (this.commentingWidget.isReviewArtifact()) {
        	this._showOnlyReviewCheckNode.checked = false;
        	this._showAllCheckboxNode.checked = true;
        	this._disableShowAll();
			this._disableShowOnlyReview();
		} else if (this.commentingWidget.viewer.isInReviewMode()) {
        	this._showOnlyReviewCheckNode.checked = true;
        	this._showAllCheckboxNode.checked = false;
        	this._enableShowAll();
        } else {
        	this._showOnlyReviewCheckNode.checked = false;
        	this._showAllCheckboxNode.checked = true;
        	this._disableShowAll();
        }
    },    
    postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.commenting", "commenting");
        if (this.commentingWidget.viewer.isInReviewMode()) {
            dojo.forEach(["showOnly"], function(prop){
                this[prop] = this._messages["showCurrentReview"];
            }, this);
        } else {
            dojo.forEach(["showOnly"], function(prop){
                this[prop] = this._messages["showOnlyReview"];
            }, this);
        }
    },
    _connectFormEvents: function() {
    	var formSelectFieleds = this.domNode.getElementsByTagName("SELECT");
    	for (var i=0; i<formSelectFieleds.length; i++) {
    		dojo.connect(formSelectFieleds[i], "onchange", this, "_onStartFilteringClicked");
    	}
    	dojo.connect(this._hideResolvedCheckNode, "onclick", this, "_onStartFilteringClicked");
    	dojo.connect(this._hideDeletedCheckNode, "onclick", this, "_onStartFilteringClicked");
    	dojo.connect(this._showOnlyReviewCheckNode, "onclick", this, "_onStartFilteringClicked");
    },
    /**
        Complete destruction
    */
    destroy: function() {
        this.priorityWidget.destroyRecursive();
        if (this.directedToWidget != null) {
        	this.directedToWidget.destroyRecursive();
        }
        if (this.authorWidget != null) {
        	this.authorWidget.destroyRecursive();
        }
        // this.updatedWidget.destroyRecursive();
        this.howRecentSelectWidget.destroyRecursive();
        if (this._tooltips) {
            dojo.forEach(this._tooltips, function(tt) {
                tt.destroyRecursive();
            });
            this._tooltips = null;
        }
        this.inherited(arguments);
    },
    getCommentCount: function(callbackFn, callbackScope) {
    	this._onHeaderCountComplete = callbackFn;
    	this._onHeaderCountScope = callbackScope;
   		if (this._isCommentCountDisplayed) {
   			this._onHeaderCountComplete.call(this._onHeaderCountScope, this.commentingWidget._allCommentPanes.count);
   		}
    },
    hide: function() {
        this._filterFormNode.style.display = "none";
        //this._showAllNode.style.display = "none";
    },
    unhide: function() {
        this._filterFormNode.style.display = "block";
        //this._showAllNode.style.display = "inline";
        
    },
    showMessage: function(message) {
        this.commentingWidget.showMessage(message);
    },
    updateCurrentFilterMessage: function(){
    	this._isCommentCountDisplayed = true; 
        var nTot = this.commentingWidget._allCommentPanes.count;
        var nFil = this._filteredCommentPanes.length;
        var nShow = nTot - nFil;
        var self = this;
        
        function trim(userId) {
           var i = userId.lastIndexOf("/");
           return userId.substring(i+1);
        }
        
        if (nShow == nTot) {
        	var msg = dojo.string.substitute(this._messages.filterTotal, [nTot]);
        }
        else {
        	var msg = dojo.string.substitute(this._messages.filterXOfTotal, [nShow, nTot]);
        }
        this.showMessage(msg);
        
    	if (this._onHeaderCountComplete) {
    		this._onHeaderCountComplete.call(this._onHeaderCountScope, nTot);
    	}
    },
    /**
        Returns false if comment should be filtered out, true if the comment
        matches the search criteria from the filter form.
    */
    isShowComment: function(c /*Comment*/) {
        var sself = this;
        
        if (sself._showOnlyReviewCheckNode.checked) {
        	if (sself.commentingWidget.viewer.isInReviewMode()) {
        		if (!c.isReviewComment() || c.reviewId != sself.commentingWidget.viewer.reviewId) return false;
        	} else {
        		if (!c.isReviewComment()) return false;
        		var uri = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + c.reviewId;
        		var review = sself.commentingWidget.getReviewDescription(uri);
        		if (review == null) return false;
        		var state = review.reviewStatus;
        		// only show if review is active (not draft or finalized)
        		if ((state == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Draft) ||
        				(state == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Finalized)) {
        			return false;
        		}
        	}
        }
        if (sself._hideDeletedCheckNode.checked) {
			if (c.isElementDeleted) return false;
        }
        
        // TEMP: userIds have been changed again
        // lp = lastPartOf
        function lpo(userId) {
            // old style userId
            if (userId.indexOf("/")==-1) {
                return userId;
            }
            else {
                // Get the last fragment
                var i = userId.lastIndexOf("/");
                var u = userId.substring(i+1);
                return u;
            }
        } 
        
        function priorityMatch(c) {
            return c.priority==sself.priorityWidget.selected || sself.priorityWidget.selected==com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED; 
        }
        function directedToMatch(c) {
        	if (!sself.directedToWidget.selected) return true;
        	if (sself.directedToWidget.selected==sself.directedToWidget.UNSPECIFIED) return true;
        	return c.isDirectedTo(sself.directedToWidget.selected);
        }
        function authorMatch(c) {
            return !sself.authorWidget.selected || lpo(c.author)==lpo(sself.authorWidget.selected) || sself.authorWidget.selected==sself.authorWidget.UNSPECIFIED;
        }
        function resolutionMatch(c) {
            return (!sself._hideResolvedCheckNode.checked)?true:!c.isResolved;
        }
        function howRecentMatch(c) {
            var range = sself.howRecentSelectWidget.selected;
            if (range==com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED) {
                return true;
            }
            return range == c.getDateRange();            
        }
        return priorityMatch(c)&&directedToMatch(c)&&authorMatch(c)&&resolutionMatch(c)&&howRecentMatch(c);
    },
    /**
        Un/hides a CommentPane based on the current filter
    */
    applyFilter: function(commentPanes /*Array[CommentPane]*/) {
    	if (!this._projectURI) //project URI hasn't been set, can't filter
    		return;
    	var timer = (new Date()).getTime();
        this.showMessage(this._messages.filtering);
        for(var i=0; i<commentPanes.length; i++) {
            var cPane = commentPanes[i];
            var comment = cPane.annotation;
            comment.isShow = this.isShowComment(comment);
        }
        for(var i=0; i<commentPanes.length; i++) {
            var cPane = commentPanes[i];
            var comment = cPane.annotation;
            if (comment.isShowForFilter()) {
                cPane.unhide();
                if (comment.isRoot) cPane.updateReplyDetails();
            } else {
                cPane.hide();
                this._filteredCommentPanes.push(cPane);
            } 
        }
        
        this.updateCurrentFilterMessage();
        ErrorHandler.debug("filtered comments widgets in : " + ((new Date()).getTime() - timer) + "ms");
    
    },
    setProjectURI: function(projectURI) {
    	var firstTime = this._projectURI == null;
    	this._projectURI = null;
    	var selectedAuthor = null;
    	var selectedDirectedTo = null;
    	// recreate the user widgets so that current user data is picked up
    	if (this.authorWidget) {
    		selectedAuthor = this.authorWidget.selected;
    		this.authorWidget.destroyRecursive();
    	}
    	if (this.directedToWidget) {
    		selectedDirectedTo = this.directedToWidget.selected;
    		this.directedToWidget.destroyRecursive();
    	}
    	
    	this.directedToWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
    		widgetmode: "full", 
    		forFilter: true,
    		projectURI: projectURI,
    		selectValue: selectedDirectedTo
    		}, 
    		null);
        // accessibility, match up label with the widget
		this.directedToWidget._htmlSelectNode.setAttribute("id", this.directedToWidget.id);
		this._directedToLabelNode.setAttribute("for", this.directedToWidget._htmlSelectNode.id);
    	this._directedToSelectNode.innerHTML = "";   
    	this._directedToSelectNode.appendChild(this.directedToWidget.domNode);   
   	
    	this.authorWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
    		widgetmode: "full", 
    		forFilter: true,
    		projectURI: projectURI,
    		selectValue: selectedAuthor
    		},
    		null);
        // accessibility, match up label with the widget
		this.authorWidget._htmlSelectNode.setAttribute("id", this.authorWidget.id);
		this._authorLabelNode.setAttribute("for", this.authorWidget._htmlSelectNode.id);
    	this._authorSelectNode.innerHTML = "";   
    	this._authorSelectNode.appendChild(this.authorWidget.domNode);  
    	
    	// set the project URI after the widgets are created, otherwise an ordering error occurs 
    	// (i.e., filtering occurs before the user widgets are created), then force filtering to 
    	// workaround the ordering issue (necessary when widgets are recreated during a refresh)
    	this._projectURI = projectURI;
    	if (!firstTime) {
    		// don't filter if comments not yet loaded, doing so 
    		// will indicate (0) comments
    		this._filterCommentPanes();
    	}
    	
    	this._connectFormEvents();
    },
    _onShowAllClicked: function(event) {
        if (!this._showAllCheckboxNode.checked)
        	return;
        this.priorityWidget.select(com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED);
        this.directedToWidget.select(this.directedToWidget.UNSPECIFIED);
        this.authorWidget.select(this.authorWidget.UNSPECIFIED);
        this.howRecentSelectWidget.select(com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED);  
        this._hideResolvedCheckNode.checked = false;
        this._hideDeletedCheckNode.checked = false;
        this._showOnlyReviewCheckNode.checked = false;
        this._filterCommentPanes(event);
        this._disableShowAll();
    },
    _filterCommentPanes: function() {
    	 this._filteredCommentPanes = new Array();       
    	 this.applyFilter(this.commentingWidget._allCommentPanes.getValueList());
    	 // Notify the editor if needed
    	 if (this.commentingWidget._commentsLocated) {
    		 this.commentingWidget._locateCommentsInDocument();
    	 }
    	 dojo.forEach(this.commentingWidget._allLocationPanes.getValueList(), function(locationPane) {
    		 locationPane.updateCount();
    	 });
        
        this.commentingWidget.locateInDoc();
    },
    _isFilterShowAll: function() {
    	if (this._showOnlyReviewCheckNode.checked) return false;
    	if (this._hideDeletedCheckNode.checked) return false;
    	if (this._hideResolvedCheckNode.checked) return false;
    	if (this.directedToWidget.selected != this.directedToWidget.UNSPECIFIED) return false;    	
    	if (this.authorWidget.selected != this.directedToWidget.UNSPECIFIED) return false;    	
    	if (this.howRecentSelectWidget.selected != com.ibm.rdm.web.commenting.UPDATED_UNSPECIFIED) return false;    	
    	if (this.priorityWidget.selected != com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED) return false;
    	return true;
    },
    _onStartFilteringClicked: function(event) {
    	if (this._isFilterShowAll()) {
        	this._showAllCheckboxNode.checked = true;
            this._disableShowAll();
    	} else {
        	this._showAllCheckboxNode.checked = false;
            this._enableShowAll();
    	}
        this._filterCommentPanes();
        this.commentingWidget._setHeightOfScrollingPaneElement();
    },
    _onCloseClicked: function(event) {
        this.hide();
        this.commentingWidget._setHeightOfScrollingPaneElement();
    },
    _onCloseLinkClicked: function(event) {
    	if( event ){
	        dojo.stopEvent(event);
    	}
        this.commentingWidget.closeFilter();
    },
    _disableShowOnlyReview: function() {
		this._showOnlyReviewCheckNode.disabled = true;
		this._showOnlyReviewLabel.className = "show-label-disabled";
    },  
    _disableShowAll: function() {
		this._showAllCheckboxNode.disabled = true;
		this._showAllLabel.className = "show-label-disabled";
    },  
    _enableShowAll: function() {
		this._showAllCheckboxNode.disabled = false;
		this._showAllLabel.className = "showall-label-enabled";
    }
 });
