/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.common.IconSelectWidget");


dojo.declare("com.ibm.rdm.web.common.IconSelectWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "common/templates/IconSelectWidget.html"),
    /**
        Use the example in CommentingWidget for the spec of iconSelectOptions
        First one is always the default.
        Each entry corresponding to an "option" will be assigned an index property, correspoding to indes the select will assign to them.
    */
    iconSelectOptions: null,
    /*
        String value of
        minimal: Default. Only icon visible. When icon is clicked until it is clicked, dropdown is shown
        full: icon and dropdown always visible 
    */
    widgetmode: null,
    /**
        Is widget editable (enabled) or display value only (disabled).
    */
    disabled: null,
    /*
        value of option, corresponding to the desired default value. 
        select() function can also be used. Always reflects the value
        of the current selection, but select() function must be used
        to change the current selection.
    */
    selected: null,
    postCreate: function() {
        this.inherited("postCreate", arguments);
        if (this.iconSelectOptions) {
            this._init();
        }
    },
    /**
        If need to wait for AJAX call, etc to get iconSelectOptions,
        this method can be used by extenders of this class.
    */
    initialize: function(iconSelectOptions) {
        this.iconSelectOptions = iconSelectOptions;
        this._init();
    },
    /**
        Selects a given value from this.iconSelectOptions
    */
    select: function(selectedValue) {
		if (this.iconSelectOptions[selectedValue] == undefined) return;
        var newInd = this.iconSelectOptions[selectedValue].index;
        this._htmlSelectNode.selectedIndex = newInd;
        this._setIconForSelectedValue();
        this.selected = selectedValue; 
    },
    isMinimalMode: function() {
        return (!this.widgetmode || this.widgetmode=="minimal" );
    },
    isFullMode: function() {
        return this.widgetmode=="full";
    },
    isDisabled: function() {
        return this.disabled;
    },
    isEnabled: function() {
        return !this.isDisabled();
    },
    _init: function() {
        this._createSelect();
    
        if (!this.widgetmode) {
            this.widgetmode = "minimal";
        }
        
        this._setIconForSelectedValue();
        if (this.widgetmode == "minimal") {
            this._htmlSelectNode.style.display = "none";
        }
        if (this.selected) {
            this.select(this.selected);
        }
        else {
            this.selected = this.getSelectedValue();
        }
    },
    _isSelectVisible: function() {
        return (this._htmlSelectNode.style.display)!="none";
    },
    getSelectedValue: function() {
        var sInd = this._htmlSelectNode.selectedIndex;
        var v = this._htmlSelectNode.options[sInd==-1?0:this._htmlSelectNode.selectedIndex].value;
        return v; 
    },
    getSelectedIndex: function() {
       return this._htmlSelectNode.selectedIndex;
    },
    _setIconForSelectedValue: function() {
        var selectedValue = this.getSelectedValue();
        var iconUri = this.iconSelectOptions[selectedValue].iconUri;
        var iconNode = null;
        if (iconUri!=null) {
	        iconNode = this._createImgNode(iconUri);
        }
        else {// no icon
            iconNode = document.createElement("span");
            // TODO: Make it optional to have an empty spot for the image. 
            //iconNode.style.marginLeft = "1px";
            dojo.addClass(iconNode, "author-select-icon-node");
             
        }
        if (this._iconLocationNode.firstChild) {
            this._iconLocationNode.replaceChild(iconNode, this._iconLocationNode.firstChild);
        }
        else {
            this._iconLocationNode.appendChild(iconNode);
        }
        
    },
    _onSelectBlur: function(event) {
        if (this.widgetmode == "minimal" && this._isSelectVisible() ) {
            this._htmlSelectNode.style.display = "none";
        }

    },
    _onOptionSelected: function(event) {
        this._setIconForSelectedValue();
        this.selected = this.getSelectedValue();
    },
    _onIconClicked: function(event) {
        if (this.widgetmode == "minimal" && !this._isSelectVisible() && this.isEnabled() ) {
            this._htmlSelectNode.style.display = "inline";
        }
    },
    _onIconMouseover: function(event) {
        if (this.widgetmode == "minimal" && !this._isSelectVisible() && this.isEnabled() ) {
            dojo.removeClass(this._iconLocationNode, "rdm-select-icon-plain");
            dojo.addClass(this._iconLocationNode, "rdm-select-icon-hoverover");
        }
    },
    _onIconMouseout: function(event) {
        if (this.widgetmode == "minimal") {
            dojo.removeClass(this._iconLocationNode, "rdm-select-icon-hoverover");
            dojo.addClass(this._iconLocationNode, "rdm-select-icon-plain");
        }
    },
    _createSelect: function() {
        var index = 0;
        for (pValue in this.iconSelectOptions) {
            var priority =  this.iconSelectOptions[pValue];
            if (priority) {
	            var option = this._createOption(priority.text, pValue, false, priority.optionClass);
	            priority.index = index++;
	            this._htmlSelectNode.appendChild(option);
            }
        }
    },
    _createOption: function(label, value, selected, optionClass) {
          var option = document.createElement("option");
          option.value = value;
          option.appendChild(document.createTextNode(label));
          if (selected) {
            option.setAttribute("selected", "");
          }
          if (optionClass) {
            dojo.addClass(option, optionClass);
          }
          return option;
    }, 
    _createImgNode: function(src) {
        var imgNode = document.createElement("img");
        imgNode.setAttribute("src", src);
        if (!dojo.isIE) {
            imgNode.style.verticalAlign = "bottom";
        }
        return imgNode; 
    }
    
    
});

