 /* Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
  *
  * US Government Users Restricted Rights - Use, duplication or disclosure
  * restricted by GSA ADP Schedule Contract with IBM Corp.
  */
(function() {
dojo.provide("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.parsers.xml2json");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("jazz.ui.Dialog");
dojo.require("dojo.DeferredList");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("dojox.data.dom");

var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var UpdateContext = com.ibm.rdm.web.common.UpdateContext;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;

var FOLDER = "folder";
var TAG = "tag";
var PUBLIC = "public";
var ROOT = "root";
var SELF = "self";

var QUERY_PAGE_SIZE = 512;

com.ibm.rdm.web.common.ProjectCache = {
	// summary:
	//    This static class is used to obtain information about Project
	//    It caches this information so that subsequent requests
	//    for the same information do not needlessly tax the server (and
	//    client as we avoid needlessly reparsing the same document
	//    information).


	///////////////////////////////////////////////////////////////////////////////////
	// Private fields
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common"),


	///////////////////////////////////////////////////////////////////////////////////
	// API
 	_init: function(projectURI) {
		this._deferredsCacheMap = {};		
 	},

	getProject: function(projectURL) {
		var fakeDeferred = new dojo.Deferred();
		var deferred = this.getUserMemberProjects();
		deferred.addCallback(this, 
			function(projects) {
				var project = projects[projectURL];
				if (!project) {
					var errorMessage = this._messages.noProject + projectURL;
					ErrorHandler.getErrorSectionWidget().showErrorMessage(errorMessage, errorMessage);
					fakeDeferred.errback(errorMessage);
				}
				else {
					fakeDeferred.callback(projects[projectURL]);
				}
			}
		);
		deferred.addErrback(this, function(errorResponse){
			fakeDeferred.errback(errorResponse);
		});
		return fakeDeferred;
	},
	
	/**
	 * PROJECTS
	 */
	getUserMemberProjects: function() {	
		var projectMembersQuery = com.ibm.rdm.web.FRONTING_SERVICE_JFS_PROJECT_AREAS + '?hasMember=' + com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		return RDMClient.doGet({
				uri: projectMembersQuery, 
				parser: this.projectsCallbackParser
			});
	},

	projectsCallbackParser: function(response) {
		var projectsMap = {};
		var projectsParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
		var parsedProjects  = projectsParser.parse()["project-areas"];
		for(parsedProject in parsedProjects) { 
			if (parsedProject == "xmlAttributes")
				continue;
			var project = parsedProjects[parsedProject];
			project["name"] = projectsParser.getProperty(project, 'xmlAttributes.name');
			project["url"] = projectsParser.getProperty(project, 'url.value');
			project["rolesUrl"] = projectsParser.getProperty(project, 'roles-url.value');
			project["linksUrl"] = projectsParser.getProperty(project, 'links-url.value');
			project["membersUrl"] = projectsParser.getProperty(project, 'members-url.value');
			project["archived"] = projectsParser.getProperty(project, 'archived.value');
			projectsMap[project["url"]] = project;
		}
		return projectsMap;
	},
	
	/**
	 * USERS
	 */
	getCurrentUser: function() {
	 	var userDeferred = new dojo.Deferred();
		var whoamiDeferred = RDMClient.doGet({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_JFS_CURRENT_USER,
			xhrArgs: {
				handleAs: "text"
			}
		});
		whoamiDeferred.addCallback(this, function(whoamiResponse){
			var currentUserDeferred = RDMClient.doGet({
				uri: whoamiResponse.doc + '?format=' + encodeURIComponent('application/rdf+xml'),				
				parser: function(response) {
						var user = {};
						var userParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
						var userDescription = userParser.getProperty(userParser.parse(), "RDF.Description");
						user.name = userParser.getProperty(userDescription, 'name.value');
						user.img = userParser.getProperty(userDescription, 'img.xmlAttributes.resource');
						user.userId = userParser.getProperty(userDescription, 'nick.value');
						user.email = userParser.getProperty(userDescription, 'mbox.xmlAttributes.resource');
						user.uri = userParser.getProperty(userDescription, 'xmlAttributes.about');
						return user;
				}
			});
			currentUserDeferred.addCallback(this, function(user){
				userDeferred.callback(user);
			});
			currentUserDeferred.addErrback(this, function(errorUserResponse){
				userDeferred.errback(errorUserResponse);
			});
		});
		whoamiDeferred.addErrback(this, function(errorWhoamiResponse){
			userDeferred.errback(errorWhoamiResponse);
		});
		return userDeferred;
	},

	getAllUsers: function(projectURL) {
		return RDMClient.doGet({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + '?query=' + encodeURIComponent('foaf:name="*"'),
			parser:	function(response) {
				var users = {};
				var usersParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
				var parsedUsers = usersParser.parse().feed;
				for(parsedUser in parsedUsers) { 
					if (parsedUser.indexOf("entry") != 0)
						continue;
					var user = {};
					var userDescription = usersParser.getProperty(parsedUsers[parsedUser], "content.RDF.Description");
					user.name = usersParser.getProperty(userDescription, 'name.value');
					user.photoUri = usersParser.getProperty(userDescription, 'img.xmlAttributes.resource');
					if (!user.photoUri)
						user["photoUri"]= "com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png";
					user.userId = usersParser.getProperty(userDescription, 'nick.value');
					user.email = usersParser.getProperty(userDescription, 'mbox.xmlAttributes.resource');
					user.url = usersParser.getProperty(userDescription, 'xmlAttributes.about');

					users[user.url] = user;
				}
				return users;
			}
		});
	},

	getProjectUsers: function(projectURL) {
		var fakeDeferred = new dojo.Deferred();

		var membersDeferred = RDMClient.doGet({
				uri: projectURL + "/members",  
				parser:	function(response) {
						var users = {};
						var usersParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
						var parsedUsers = usersParser.parse().members;
						for(parsedUser in parsedUsers) { 
							if (parsedUser == "xmlAttributes")
								continue;
							user = parsedUsers[parsedUser];

							user["url"] = usersParser.getProperty(user, 'user-url.value');

							//get role-assgnments uris
							var roles = user["role-assignments"];
							user.userRoles = [];
							for (role in roles) {
								var rolesUri = usersParser.getProperty(roles[role], 'role-url.value')
								user.userRoles.push(roles[role]["role-url"].value);
							}
							users[user.url] = user;
						}
						return users;
					}
			}
		);
		
		var allUserDescriptionsDeferred = this.getAllUsers();

		var usersDeferredList = new dojo.DeferredList([membersDeferred,allUserDescriptionsDeferred]);
		usersDeferredList.addCallback(this, function(results){
			//create a new "intersection" of resultsets
			var users = {};
			var membersDeferredResult = results[0][1];
			var allUsersDeferredResult = results[1][1];
			
			for (userUri in membersDeferredResult) {
				users[userUri] = allUsersDeferredResult[userUri]; //user description
			}
			fakeDeferred.callback(users);
		});
		usersDeferredList.addErrback(this, function(errorResponse){
			fakeDeferred.errback(errorResponse);
		});
		return fakeDeferred;
	},

	getProjectUsersWithRoles: function(projectURL) {
		var fakeDeferred = new dojo.Deferred();

		var membersDeferred = RDMClient.doGet({
				uri: projectURL + "/members",  
				parser:	function(response) {
						var users = {};
						var usersParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
						var parsedUsers = usersParser.parse().members;
						for(parsedUser in parsedUsers) { 
							if (parsedUser == "xmlAttributes")
								continue;
							user = parsedUsers[parsedUser];

							user["url"] = usersParser.getProperty(user, 'user-url.value');

							//get role-assgnments uris
							var roles = user["role-assignments"];
							user.userRoles = [];
							for (role in roles) {
								var rolesUri = usersParser.getProperty(roles[role], 'role-url.value')
								user.userRoles.push(roles[role]["role-url"].value);
							}
							users[user.url] = user;
						}
						return users;
					}
			}
		);
		var rolesDeferred = RDMClient.doGet({
				uri: projectURL + "/roles",  
				parser:	function(response) {
						var roles = {};
						var rolesParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
						var parsedRoles = rolesParser.parse().roles;
						for(roleEntry in parsedRoles) { 
							if (roleEntry == "xmlAttributes")
								continue;
							var roleObject = parsedRoles[roleEntry];
							var role = [];

							role["id"] = rolesParser.getProperty(roleObject, 'id.value');
							role["label"] = rolesParser.getProperty(roleObject, 'label.value');
							role["description"] = rolesParser.getProperty(roleObject, 'description.value');
							role["url"] = rolesParser.getProperty(roleObject, 'url.value');
							roles[role.url] = role;
						}
						return roles;
					}
			}
		);
		var allUserDescriptionsDeferred = this.getAllUsers();

		var usersDeferredList = new dojo.DeferredList([membersDeferred,rolesDeferred,allUserDescriptionsDeferred]);
		usersDeferredList.addCallback(this, function(results){
			//create a new "intersection" of resultsets
			var usersAndRoles = {};
			var membersDeferredResult = results[0][1];
			var rolesDeferredResult = results[1][1];
			var allUsersDeferredResult = results[2][1];
			
			for (userUri in membersDeferredResult) {
				usersAndRoles[userUri] = allUsersDeferredResult[userUri]; //user description
				//process roles
				usersAndRoles[userUri]["userRoles"] = [];
				for (var i=0; i<membersDeferredResult[userUri].userRoles.length; i++) {
					var roleUri = membersDeferredResult[userUri].userRoles[i];
					usersAndRoles[userUri]["userRoles"].push(rolesDeferredResult[roleUri]);
				}
			}
			fakeDeferred.callback(usersAndRoles);
		});
		usersDeferredList.addErrback(this, function(errorResponse){
			fakeDeferred.errback(errorResponse);
		});
		return fakeDeferred;
	},

	/**
	 * PERMISSIONS/OPERATIONS
	 */	
	getOperations: function() {	
		var operationsURI = com.ibm.rdm.web.FRONTING_SERVICE_OPERATIONS;		
		var deferred = RDMClient.doGet(
			{
				uri: operationsURI,
				parser:	function(response) {
					var operations = {};
					var operationsParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
					var parsedOperations = operationsParser.parse().operations;
					for (parsedOperation in parsedOperations) { 
						if (parsedOperation == "xmlAttributes")
							continue;
						if (parsedOperation.indexOf("operation") != 0)
							continue;
						var operationObject = parsedOperations[parsedOperation];
						var operation = {};
						operation.url = operationObject.xmlAttributes["url"];
						operation.name = operationObject.xmlAttributes["name"];
						operations[operation.url] = operation;
					}
					return operations;
				}
			}
		);
		return deferred;
	},

	getOperationPermission: function(operationURL, projectURL) {
		var permissionURL = operationURL + "?" 
			+ 'resourceContext=' + encodeURIComponent(projectURL) 
			+ "&action=modify";
		var deferred = RDMClient.doGet(
			{
				uri: permissionURL,
				parser: function(response) {
					var permissionInfo = {};
					var permissionParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
					var parsedPermission = permissionParser.parse();
					var reportNode = permissionParser.getProperty(parsedPermission, 'operation-report');
					var status = reportNode.xmlAttributes["overallStatus"];
					var info = new Object;
					info.projectURL = projectURL;
					info.operationURL = operationURL;
					if (status != "OK") {
						info.isAllowed = false;
					} else {
						var actionsNode = permissionParser.getProperty(reportNode, 'action-permissions');
						var actionNode = permissionParser.getProperty(actionsNode, 'action');
						var allowed = actionNode.xmlAttributes["allowed"];
						info.isAllowed = allowed == "true";
					}
					return info;
				}
			}
		);
		return deferred;
	},
	
	/**
	 * FOLDERS
	 */
	getProjectFolders: function(projectURL) {
		var projectFoldersQuery = com.ibm.rdm.web.FRONTING_SERVICE_FOLDERS + '?resourceContext=' + encodeURIComponent(projectURL);
		return RDMClient.doGet({
				uri: projectFoldersQuery,
				parser:	function(response) {
						var folders = [];
						var foldersParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
						var parsedFolders = foldersParser.parse().RDF;
						for(parsedFolder in parsedFolders) { 
							if (parsedFolder == "xmlAttributes")
								continue;
							var folder = parsedFolders[parsedFolder];	    		
							folder["title"] = folder["title"].value;
							folder["resourceURI"] = folder.xmlAttributes["about"];
							var parentURI = folder["parent"].xmlAttributes["resource"];
							folder["parentURI"] =  (parentURI.indexOf('..') == 0)?
											queryutils.getFullyQualifiedUriFromRelativeLink(folder["resourceURI"], parentURI):
											queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(folder["resourceURI"], parentURI);
							folders.push(folder);    
						}
						
						var foldersTreeMap = {};
						foldersTreeMap[ROOT] = [];
						if (folders.length > 0) {
							
							//find root folder
							var topId = null;
							var rootId = null;
							for (var i = 0; i < folders.length; i++) {
								var topId = folders[0].parentURI;
								var rootId = folders[0].resourceURI;
								if (topId == projectURL)
									break;
							}
							
							//construct a tree map
							for (var i = 0; i < folders.length; i++) {
								var key;
								if (folders[i].parentURI == topId) {
									key = "top"; //ROOT;
								}
								else if (folders[i].parentURI == rootId) {
									key = ROOT;
								} else {
									key = folders[i].parentURI;
								}
		
								if (!foldersTreeMap[key]) {
									foldersTreeMap[key] = [];
								}
								if (!foldersTreeMap[folders[i].resourceURI]) {
									foldersTreeMap[folders[i].resourceURI] = [];
								}
								foldersTreeMap[key].push(folders[i]);
								foldersTreeMap[folders[i].resourceURI][SELF] = folders[i]; 
							}
						}
						return foldersTreeMap;
					}
			}
		);
	},
	
	/**
	 * TAGS
	 */
	getProjectTags: function(projectURL) {
		var projectTagsQuery = com.ibm.rdm.web.FRONTING_SERVICE_TAGS + '?resourceContext=' + encodeURIComponent(projectURL);
		return RDMClient.doGet({
				uri: projectTagsQuery, 
				parser: this._tagsCallbackParser
			});
	},

	getResourceTags: function(resourceURL) {
		var resourceTagsQuery = com.ibm.rdm.web.FRONTING_SERVICE_TAGS + '?taggedResource=' + encodeURIComponent(resourceURL);
		return RDMClient.doGet({
				uri: resourceTagsQuery, 
				parser: this._tagsCallbackParser
			});
	},

	_tagsCallbackParser: function(response) {
		var tags = {};
		try {
			var tagNodes = domutils.getElementsByTagNameNS(response.doc, "tag", "http://com.ibm.rdm/navigation#", "ns");
			for (var i = 0; i < tagNodes.length; i++) {
				var tag = {};
				var titleNode = domutils.getElementsByTagNameNS(tagNodes[i], "title", queryutils.DCTERMS_NAMESPACE, queryutils.NAV_QUALIFIER)[0];
				var title = (titleNode.firstChild ? titleNode.firstChild.nodeValue : "");
				var descriptionNode = domutils.getElementsByTagNameNS(tagNodes[i], "description", queryutils.DCTERMS_NAMESPACE, queryutils.NAV_QUALIFIER)[0];
				var description = (descriptionNode.firstChild ? descriptionNode.firstChild.nodeValue : "");		
				var scopeNode = domutils.getElementsByTagNameNS(
						tagNodes[i], "scope", queryutils.NAV_NAMESPACE, queryutils.NAV_QUALIFIER)[0];
				var scope = (scopeNode.firstChild ? scopeNode.firstChild.nodeValue : "");
				var resourceURI = domutils.getAttributeNS(
						tagNodes[i], "about", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER);
				
				tag["title"] = title;
				tag["description"] = description;
				tag["scope"] = scope;
				tag["resourceURI"] = resourceURI;
				
				tags[resourceURI] = tag;    
			}
		}
		catch (e) {
			console.error(e)
		}
		return tags;
	},
			
	getFrontingServerServiceDocument: function() {
		ErrorHandler.warn('TODO: by making this request we are assuming that fronting and webUI servers run collocated');
		return RDMClient.doGet({uri: com.ibm.rdm.web.FRONTING_SERVICE_DOCUMENT});
	},
	
	/**
	 * QUERY
	 */
	QUERY_TYPE_DEFAULT: "query",
	QUERY_TYPE_DESCRIBE: "describe",
	QUERY_TYPE_SEARCH: "search",
	executeQuery: function(queryArgs) {
		var deferred = new dojo.Deferred();

		if (!queryArgs.queryType)
			queryArgs.queryType = this.QUERY_TYPE_DEFAULT;
			
		var query = queryArgs.queryType + "=" + queryArgs.query;
		var properties = (queryArgs.properties)?"&properties=" + queryArgs.properties:"";
		var namespaces = (queryArgs.namespaces)?"&namespaces=" + queryArgs.namespaces:"";
		
		if (queryArgs.projectSnapshotURI) {
			query += " and inBaseline=<" + queryArgs.projectSnapshotURI + ">";
		}

		var fullQuery = query;
		fullQuery += " /limit=-1";
		fullQuery += properties;
		fullQuery += namespaces;
		
		var queryDeferred = RDMClient.doPost({
				uri: com.ibm.rdm.web.FRONTING_SERVICE_QUERY + "?size=" + QUERY_PAGE_SIZE,
				projectURI: queryArgs.projectURI,
				postData: fullQuery,
				//parser: queryArgs.parser,
				skipCache: queryArgs.skipCache,
				cancelDeferreds: queryArgs.cancelDeferreds
			});
		queryDeferred.addCallback(this, function(xmlQueryRespose){
			var nextPageURI = queryutils.getNextPageURI(xmlQueryRespose.doc);
			if (nextPageURI) {	
				nextPageURI = queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(com.ibm.rdm.web.FRONTING_SERVICE_MULTI_FETCH, nextPageURI);		
				var paginationDeferred = this._paginateThroughQueryResults([queryArgs.parser(xmlQueryRespose)], nextPageURI, queryArgs.parser);
				paginationDeferred.addCallback(this, function(paginationResponse){
					deferred.callback(paginationResponse);
				});
				paginationDeferred.addErrback(this, function(errorPaginationResponse){
					deferred.errback(errorPaginationResponse);
				});
			}
			else {
				deferred.callback(queryArgs.parser(xmlQueryRespose));
			}
		});
		queryDeferred.addErrback(this, function(xmlErrorQueryRespose){
			deferred.errback(xmlErrorQueryRespose);
		});
		
		return deferred;
	},

	_paginateThroughQueryResults: function(resultPagesArray, nextPageURI, parser, deferred) {
		if (!deferred) {
			deferred = new dojo.Deferred();
		}
		if (nextPageURI) {
			var _nextPageDeferred = RDMClient.doGet({
				uri: nextPageURI,
				skipCache: true
			});
			_nextPageDeferred.addCallback(this, function(response){
				var _nextPageURI = queryutils.getNextPageURI(response.doc);
				if (_nextPageURI) {
					_nextPageURI = queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(com.ibm.rdm.web.FRONTING_SERVICE_MULTI_FETCH, _nextPageURI);
				}
				resultPagesArray.push(parser(response));
				this._paginateThroughQueryResults(resultPagesArray, _nextPageURI, parser, deferred);
			});
			_nextPageDeferred.addErrback(this, function(errorResponse){
				deferred.errback(errorResponse);
			});
		}
		else {
			if (resultPagesArray && resultPagesArray[0] && dojo.isArray(resultPagesArray[0])) {
				var mergedPagesResultsArray = [];
				for (var i=0; i<resultPagesArray.length; i++) {
					for (var j=0; j<resultPagesArray[i].length; j++) {
						mergedPagesResultsArray.push(resultPagesArray[i][j]);
					}
				}
				deferred.callback(mergedPagesResultsArray);
			}
			else if (resultPagesArray && resultPagesArray[0] && !dojo.isArray(resultPagesArray[0])) {
				var mergedPagesResultsMap = {};
				for (var i=0; i<resultPagesArray.length; i++) {
					var pageMap = resultPagesArray[i];
					for (mapKey in pageMap) {
						mergedPagesResultsMap[mapKey] = pageMap[mapKey];
					}
				}
				deferred.callback(mergedPagesResultsMap);
			}
			else {
				deferred.callback(null);
			}
		}
		return deferred;
	},
	
	/**
	 * MULTI-FETCH
	 * queryArgs {
	 * 		query
	 * 		search
	 * 		projectURI
	 * 		namespaces
	 * 		projectSnapshotURI
	 * 		cancelDeferreds
	 * 		skiptCache
	 * 		parser
	 * 		disablePagination
	 * 		extendedQueryMode
	 * }
	 */
	executeMultiFetch: function(queryArgs) {
		var deferred = new dojo.Deferred();
		
		//sanity check
		if (!(queryArgs.query || queryArgs.search)) {
			ErrorHandler.warn("empty query");
			deferred.callback([]);
			return deferred;
		}
		
		queryArgs.disablePagination = (queryArgs.disablePagination)?true:false;
		
		var fullQuery = "";
		// Note: The 'search' and 'namespaces' properties are optional but the
		// 'query' property is required.
		if (queryArgs.search) {
			fullQuery = "search=" + queryArgs.search + "&";
		}

		var snapshotQueryParameter = "";
		if (queryArgs.projectSnapshotURI) {
			snapshotQueryParameter += "inBaseline=<" + queryArgs.projectSnapshotURI + "> and ";
		}
		
		fullQuery += "query=" + snapshotQueryParameter + queryArgs.query;
		
		var queryLimitParameter = "";
		if (!queryArgs.disablePagination) {
			fullQuery += " /limit=-1";
		}
		
		if (queryArgs.namespaces) {
			fullQuery += "&namespaces=" + queryArgs.namespaces;
		}
		
		if (queryArgs.extendedQueryMode) {
			fullQuery += "&mode=EXTENDED&fConstraint=MUST&wConstraint=MUST";
		}
		else {
			fullQuery += "&mode=BASIC&fConstraint=MAY&wConstraint=MAY";
		}
		
		var multiFetchQueryDeferred = RDMClient.doPost({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_MULTI_FETCH + "?size=" + QUERY_PAGE_SIZE,
			projectURI: queryArgs.projectURI,
			postData: fullQuery,
			skipCache: queryArgs.skipCache,
			cancelDeferreds: queryArgs.cancelDeferreds,
			parser: (queryArgs.parser)?queryArgs.parser:this.multiFetchQueryParser
		});
		multiFetchQueryDeferred.addCallback(this, function(parsedResponse){
			//check if there is a link to the next page
			var nextPageURI = parsedResponse.nextPageURI;

			if (nextPageURI && queryArgs.disablePagination) {
				var line1 = document.createElement("DIV");
				line1.appendChild(document.createTextNode(this._messages.showingMaxResult1));
				var line2 = document.createElement("DIV");
				line2.appendChild(document.createTextNode(this._messages.showingMaxResult2));
				var contentNode = document.createElement("DIV");
				contentNode.appendChild(line1);
				contentNode.appendChild(line2);
				var dialog = new jazz.ui.Dialog({
					contentNode: contentNode,
					primaryTitle: this._messages.queryLimitation
				});
			}
						
			if (nextPageURI && !queryArgs.disablePagination) {
				nextPageURI = queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(com.ibm.rdm.web.FRONTING_SERVICE_MULTI_FETCH, nextPageURI);
				var paginationDeferred = this._paginateThroughMultiFetchQueryResults(
													[parsedResponse.parsedQueryResult],
													nextPageURI,
													this.multiFetchQueryParser);
				paginationDeferred.addCallback(this, function(parsedPaginationResponse){
					deferred.callback(parsedPaginationResponse);
				});
				paginationDeferred.addErrback(this, function(errorPaginationResponse){
					deferred.errback(errorPaginationResponse);
				});
			}
			else {
				deferred.callback(parsedResponse.parsedQueryResult);
			}
		});
		multiFetchQueryDeferred.addErrback(this, function(errorResponse){
			deferred.errback(errorResponse);
		});
		return deferred;
	},
	
	_paginateThroughMultiFetchQueryResults: function(resultPagesArray, nextPageURI, parser, deferred) {
		if (!deferred) {
			deferred = new dojo.Deferred();
		}
		if (nextPageURI) {
			var _nextPageDeferred = RDMClient.doGet({
				uri: nextPageURI,
				skipCache: true,
				parser: parser
			});
			_nextPageDeferred.addCallback(this, function(response){
				var _nextPageURI = response.nextPageURI;
				if (_nextPageURI) {
					_nextPageURI = queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(com.ibm.rdm.web.FRONTING_SERVICE_MULTI_FETCH, _nextPageURI);
				}
				resultPagesArray.push(response.parsedQueryResult)
				this._paginateThroughMultiFetchQueryResults(resultPagesArray, _nextPageURI, parser, deferred);
			});
			_nextPageDeferred.addErrback(this, function(errorResponse){
				deferred.errback(errorResponse);
			});
		}
		else {
			var mergedPagesResultsArray = [];
			for (var i=0; i<resultPagesArray.length; i++) {
				for (var j=0; j<resultPagesArray[i].length; j++) {
					mergedPagesResultsArray.push(resultPagesArray[i][j]);
				}
			}
			deferred.callback(mergedPagesResultsArray);
		}
		return deferred;
	},
	
	multiFetchQueryParser: function(response) {
		var resourceInfos = [];
		var xmlparser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
		var parsedQuery  = xmlparser.parse(true);
		var resourceDescriptions = xmlparser.getObjectsByNS("Description", queryutils.RDF_NAMESPACE);
		for (var i=0; i<resourceDescriptions.length; i++) {
			try {
				var descriptionNode = resourceDescriptions[i];
				var resourceURI = 		xmlparser.getProperty(descriptionNode, 'xmlAttributes.about');
				var title = 			xmlparser.getProperty(descriptionNode["title:" + queryutils.DCTERMS_NAMESPACE], 'value');
				var parentResourceURI = xmlparser.getProperty(descriptionNode["parent:" + queryutils.NAV_NAMESPACE], 'xmlAttributes.resource');
				var etag = 				xmlparser.getProperty(descriptionNode["etag:" + queryutils.JFS_NAMESPACE], 'value');
				etag = etag.substring(1, etag.length -1);
				var contentType = 		xmlparser.getProperty(descriptionNode["format:" + queryutils.DCTERMS_NAMESPACE], 'value');
				var lastModifiedBy = 	xmlparser.getProperty(descriptionNode["contributor:" + queryutils.DCTERMS_NAMESPACE], 'xmlAttributes.resource');
				var lastModifiedDate = 	dojo.date.stamp.fromISOString(xmlparser.getProperty(descriptionNode["modified:" + queryutils.DCTERMS_NAMESPACE], 'value'));
				var createdDate = 		dojo.date.stamp.fromISOString(xmlparser.getProperty(descriptionNode["created:" + queryutils.DCTERMS_NAMESPACE], 'value'));
				var projectURI = 		xmlparser.getProperty(descriptionNode["resourceContext:" + queryutils.JFS_NAMESPACE], 'xmlAttributes.resource');
				var wrapperReference = 	xmlparser.getProperty(descriptionNode["reference:" + queryutils.REFERENCE_NAMESPACE], 'xmlAttributes.resource');
				
				// if an artifact is deleted, we get an empty xml entry in the feed
				// so if one of the system properties is missing after it is parsed,
				// that means that the artifact is deleted and we should skip it
				// 20868: Comments for deleted artifacts are not being tagged
				if (!contentType) {
					continue;
				}
				
				//process tags
				var tags = [];
				var tag = xmlparser.getProperty(descriptionNode["tag:" + queryutils.NAV_NAMESPACE], 'xmlAttributes.resource');
				var tagIndex = 1;
				while(tag) {
					tags.push(tag);
					var tag = xmlparser.getProperty(descriptionNode["tag:" + queryutils.NAV_NAMESPACE + tagIndex], 'xmlAttributes.resource');
					tagIndex++;
				}

				switch(contentType) {
					case MimeTypes.CONTENT_TYPE_REVIEW:
						//TODO: FIXME
						var reviewStatus = xmlparser.getProperty(descriptionNode["reviewStatus:" + queryutils.REVIEW_NAMESPACE], 'value');
						var reviewTimestamp = xmlparser.getProperty(descriptionNode["reviewTimestamp:" + queryutils.REVIEW_NAMESPACE], 'value');
						title = xmlparser.getProperty(descriptionNode["title:" + queryutils.DCTERMS_NAMESPACE], 'value');
						break;
					case MimeTypes.CONTENT_TYPE_REQUIREMENT:
						var linkToReqPro = xmlparser.getProperty(descriptionNode["linkToReqPro:" + queryutils.REQUIREMENT_NAMESPACE_WITH_POUND], 'value');
						var reqProTag = xmlparser.getProperty(descriptionNode["reqProTag:" + queryutils.REQUIREMENT_NAMESPACE_WITH_POUND], 'value');
						break;
					case MimeTypes.CONTENT_TYPE_UPLOADED:
						//TODO: FIXME
						contentType = xmlparser.getProperty(descriptionNode["contentType:" + queryutils.REFERENCE_NAMESPACE], 'value');
						break;
				}

				/*var projectURI = resourcesParser.getProperty(resourceDescription, 'resource-context.xmlAttributes.resource');
				//some resources use "title" field instead of "name"
				if (!title) {
					title = resourcesParser.getProperty(resourceDescription, 'name.value');
				}
				var etag = resourcesParser.getProperty(resourceDescription, 'etag.value');
				// Strip the leading/trailing double quotes off the etag
				etag = etag.substring(1, etag.length-1);
				var contentType = resourcesParser.getProperty(resourceDescription, 'contentType.value');
				if (!contentType)
					contentType = resourcesParser.getProperty(resourceDescription, 'format.value');
				var lastModifiedBy = resourcesParser.getProperty(resourceDescription, 'contributor.xmlAttributes.resource');
				var lastModifiedDate = dojo.date.stamp.fromISOString(resourcesParser.getProperty(resourceDescription, 'modified.value'));
				var linkToReqPro = resourcesParser.getProperty(resourceDescription, 'linkToReqPro.value');
				var reqProTag = resourcesParser.getProperty(resourceDescription, 'reqProTag.value');
				var createdDate = dojo.date.stamp.fromISOString(resourcesParser.getProperty(resourceDescription, 'created.value'));
				var parentResourceURI = resourcesParser.getProperty(resourceDescription, 'parent.xmlAttributes.resource');
				var wrapperReference = resourcesParser.getProperty(resourceDescription, 'reference.xmlAttributes.resource')
				var reviewStatus = resourcesParser.getProperty(resourceDescription, 'reviewStatus.value')
				var reviewTimestamp = resourcesParser.getProperty(resourceDescription, 'reviewTimestamp.value')
				// Note by SPC: We're constructing a property list here that
				// is supposed to hold the same values we'd see in if we
				// were to use the ResourceInfo class.  For the time being
				// I've commented out some of the fields that used to be
				// set when using ResourceInfo.  In addition, I've added some
				// FIXME's for:
				//   'description'
				//      ... as I don't see any info in the multi-fetch
				//          results that enable obtaining the description
				//          (perhaps index rules need updating).
				 * 
				 */
				resourceInfos.push({
					contentType: contentType,
					displayName: title,
					name: title,
					etag: etag,
					lastModifiedBy: lastModifiedBy,
					lastModifiedDate: lastModifiedDate,
					linkToReqPro: linkToReqPro,
					reqProTag: reqProTag,
					resourceURI: resourceURI,
					wrappedResourceURI: resourceURI,
					projectURI: projectURI,
					createdDate: createdDate,
					parentResourceURI: parentResourceURI,
					reviewStatus: reviewStatus,
					reviewTimestamp: reviewTimestamp,
					wrapperReference: wrapperReference,
					tags: tags
				});
			}
			catch(e) {
				console.error(e);
			}
		}
		return {
			"nextPageURI": xmlparser.getProperty(parsedQuery['RDF:http://www.w3.org/1999/02/22-rdf-syntax-ns#'], "link.xmlAttributes.href"),
			"parsedQueryResult": resourceInfos
		};
	},
	
	/**
	 * uses multi-fetch service
	 */
	getBatchResourceDescriptionsMultiFetch: function(resourceURIs, projectSnapshotURI, extendedQueryMode/*BASIC|EXTENDED*/) {
		var batchDeferred = new dojo.Deferred();
		var query = "resource in [";
		for (var i=0; i<resourceURIs.length; i++)
			query += "<" + resourceURIs[i] + ">" + ((i < resourceURIs.length -1)?",":"");		
		query += "]";
		
		var descriptionsDeferred = this.executeMultiFetch({
				query:query,
				namespaces: 'nav=http://com.ibm.rdm/navigation#, dc=http://purl.org/dc/terms/, jazz=http://jazz.net/xmlns/foundation/1.0/, att=http://schema.ibm.com/rdm/2008/Attribute#, wrapper=http://jazz.net/xmlns/alm/rm/Reference/v0.1#',
				projectSnapshotURI: projectSnapshotURI,
				extendedQueryMode: extendedQueryMode,
				skipCache: false
		});
		descriptionsDeferred.addCallback(this, function(descriptions){
			var descriptionsMap = {};
			for (var i=0; i<descriptions.length; i++) {
				var resourceURI = descriptions[i].resourceURI;
				descriptionsMap[resourceURI] = descriptions[i];
			}
			//convert all deleted/missing links to NULL
			for (var i=0; i<resourceURIs.length; i++) {
				if (!descriptionsMap[resourceURIs[i]]) {
					descriptionsMap[resourceURIs[i]] = null;
				}
			}
			batchDeferred.callback(descriptionsMap);
		});
		descriptionsDeferred.addErrback(this, function(response){
			batchDeferred.errback(response);
		});
		return batchDeferred;
	},
	
	getResourceDescriptionMultiFetch: function(resourceURI, projectSnapshotURI, extendedQueryMode/*true|false:BASIC|EXTENDED*/) {
		// summary:
		//    Gets indexed properties for a resource.
		// description:
		//    Wraps getBatchResourceDescriptions to get indexed properties for a
		//    particular resource.  Add a addCallback to the returned value in
		//    order to be informed of the indexed property information when it
		//    arrives.  Your callback will be called with a single argument;
		//    an object whose properties represent all the indexed properties
		//    for the passed resourceURI (and the value for any of these
		//    properties will give you the appropriate indexed propety value).
		// resourceURI: String
		//    A resource URI such as "https://localhost:9443/rdm/resources/_lckswDjhEd67TdjgK4pk9g".
		// returns:
		//    A deferred.
		var deferred = this.getBatchResourceDescriptionsMultiFetch([resourceURI], projectSnapshotURI, extendedQueryMode/*BASIC|EXTENDED*/);
		deferred.addCallback(
			function(mapResourceURIToIndexedProperties) {
				return mapResourceURIToIndexedProperties[resourceURI];
			}
		);
		return deferred;
	},
	
	getProjectSnapshot: function(projectSnapshotURI) {
		return this.getResourceDescriptionMultiFetch(projectSnapshotURI);
	},
	
	getProjectSnapshots: function(projectURI) {
		return this.executeQuery({
			query: "type=<http://jazz.net/xmlns/foundation/1.0/Baseline> and resourceContext=<" + projectURI + ">", 
			properties: "dc:title, OPTIONAL dc:description, dc:issued, jazz:resourceContextId",
			skipCache: false,
			parser: function(response) {
				var snapshots = {};
				var snapshotsParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
				var parsedSnapshots = snapshotsParser.parse().feed;
				for (snapshotEntry in parsedSnapshots) {
					if (snapshotEntry.indexOf("entry") == 0) {
						var snapshot = parsedSnapshots[snapshotEntry].content.result;
						
						var snapshotBindingProperties = {};
						for (snapshotBindingEntry in snapshot) {
							if (snapshotBindingEntry == "xmlAttributes") {
								continue;
							}
							var snapshotBinding = snapshot[snapshotBindingEntry];
							snapshotBindingProperties[snapshotBinding.xmlAttributes.name] = (snapshotBinding.literal)?snapshotBinding.literal.value:snapshotBinding.uri.value;
						}
						
						var snapshotDate = new Date(snapshotBindingProperties.issued);
						var dateStr = dojo.date.locale.format(snapshotDate, {formatLength:'long',selector:'date'});
						var timeStr = dojo.date.locale.format(snapshotDate, {formatLength:'short',selector:'time'});
						
						var dateRange = DateTimeHelper.getDateRange(snapshotDate);
						snapshotBindingProperties.dateRange = dateRange;
						snapshotBindingProperties.snapshotDateLabel = dateStr + " " + timeStr;
						snapshotBindingProperties.snapshotDate = snapshotDate;
						
						snapshots[snapshotBindingProperties.resource] = snapshotBindingProperties;
					}
				}
				return snapshots;
			}
		}); 
	},
	
	_getProjectSnapshot: function(snapshotURI) {
		return RDMClient.doGet({
			uri: snapshotURI, 
			parser: function(response) {
				var snapshot = {};
				var snapshotParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
				var parsedSnapshot = snapshotParser.parse().Baseline;
				
				snapshot.description = snapshotParser.getProperty(parsedSnapshot, 'description.value');
				snapshot.issued = snapshotParser.getProperty(parsedSnapshot, 'issued.value');
				snapshot.title = snapshotParser.getProperty(parsedSnapshot, 'title.value');
				
				for (snapshotResourceEntry in parsedSnapshot) {
					if (snapshotResourceEntry.indexOf("references") == 0) {
						var snapshotResourceURI = snapshotParser.getProperty(parsedSnapshot[snapshotResourceEntry], 'xmlAttributes.resource');
					}
				}
				return snapshot;
			}
		}); 
	},	
	
	sortUserPropertiesArrayByTitle: function(users) {
		// summary:
		//    Convert a user properties array (passed by the getProjectUsers/-
		//    getAllUsers completion callbacks) into a sorted list of user names.
		//    For example, given this array:
		//	    [ { id: "kim@company.com",
		//          title: "Kim"
		//          email: "kim@company.com",
		//          uri: "/jazz/users/kim@company.com",
		//          photoURI: "/converter/jazz/web/com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
		//        },
		//	      { id: "bob@company.com",
		//          title: "Bob",
		//          email: "bob@company.com",
		//          uri: "/jazz/users/bob@company.com",
		//          photoURI: "/converter/jazz/web/com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
		//        }
		//      ]
		//    the following would be returned:
		//	    [ { id: "bob@company.com",
		//          title: "Bob",
		//          email: "bob@company.com",
		//          uri: "/jazz/users/bob@company.com",
		//          photoURI: "/converter/jazz/web/com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
		//        },
		//        { id: "kim@company.com",
		//          title: "Kim"
		//          email: "kim@company.com",
		//          uri: "/jazz/users/kim@company.com",
		//          photoURI: "/converter/jazz/web/com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
		//        }
		//      ]
	
		var usersArray = [];
		for (userURI in users)
			usersArray.push(users[userURI]);
	
		var compareTitle = function(userProperties1, userProperties2) {
			var name1 = userProperties1.name.toLowerCase();
			var name2 = userProperties2.name.toLowerCase();
			return ((name1 < name2) ? -1 : 1);
		};
		usersArray.sort(compareTitle);
		return usersArray;
	},
	
	
	
	
	
	
	
	
	
	multiGet: function(argsArray) {
		var deferred = new dojo.Deferred();
		var multiGetDeferred = RDMClient.doMultiGet(argsArray);
		multiGetDeferred.addCallback(this, function(multiGetResponse){
			var parsedResponses = [];
			var multiGetResponsesNodes = domutils.getElementsByTagNameNS(multiGetResponse.doc, "MultiResponse", 'http://com.ibm.rdm/multi-request#', 'rrmMulti');
			for (var i=0; i<multiGetResponsesNodes.length; i++) {
				var responseContentNode = domutils.getElementsByTagNameNS(multiGetResponsesNodes[i], "responseContent", 'http://com.ibm.rdm/multi-request#', 'rrmMulti')[0];
				var responseXML = dojo.isIE?responseContentNode.childNodes[0].text:responseContentNode.childNodes[0].textContent;
				responseXML = dojo.trim(responseXML);
				var xmlDoc = dojox.data.dom.createDocument(responseXML);
				if (argsArray[i].parser) {
					parsedResponses.push(argsArray[i].parser({doc:xmlDoc}));
				}
				else {
					parsedResponses.push(xmlDoc);
				}
			}
			deferred.callback(parsedResponses);
		});
		multiGetDeferred.addErrback(this, function(multiGetErrorResponse){
			deferred.callback(multiGetErrorResponse);
		});
		return deferred;
	},
	
	
	
	_getCollectionArtifacts: function(collection, collectionQuery, projectSnapshotURI) {
		var deferred = new dojo.Deferred();
		var queryInfoCollection = dojo.clone(collectionQuery);
		var queryDeferred = this.executeMultiFetch(collectionQuery);
		
		queryDeferred.addCallback(this, function(resourceInfos){
			//we will be changing the response objec, so we need to clone it
			var _resourceInfos = dojo.clone(resourceInfos);

			if (collectionQuery.projectSnapshotURI) {
				//run a query to get all collection artifacts including the ones outside of baseline
				queryInfoCollection.projectSnapshotURI = projectSnapshotURI;
				var collectionArtifactsDeferred = this.executeMultiFetch(queryInfoCollection);
				collectionArtifactsDeferred.addCallback(this, function(collectionArtifacts){
					//we will be changing the response object, so we need to clone it
					var _collectionArtifacts = dojo.clone(collectionArtifacts);
		
					//merge to map, which can contain artifacts that in snapshot, not in snapshot or deleted	
					var allCollectionResourcesMap = {};
					for (var i=0; i<_resourceInfos.length; i++){
						var snapshotResource = _resourceInfos[i];
						snapshotResource.inSnapshot = true;
						snapshotResource.projectSnapshotURI = collectionQuery.projectSnapshotURI;
						snapshotResource.isCollectionArtifactAssociatedWithSnapshot = true;
						allCollectionResourcesMap[snapshotResource.resourceURI] = snapshotResource;
					}
					for (var j=0; j<_collectionArtifacts.length; j++){
						var collectionResource = _collectionArtifacts[j];
						collectionResource.isCollectionArtifactAssociatedWithSnapshot = true;
						if (!allCollectionResourcesMap[collectionResource.resourceURI]) {
							allCollectionResourcesMap[collectionResource.resourceURI] = collectionResource;
						}
					}
					var collectionArtifactsArray = [];
					for (collectionResourceURI in allCollectionResourcesMap) {
						collectionArtifactsArray.push(allCollectionResourcesMap[collectionResourceURI]);
					}
					deferred.callback(collectionArtifactsArray);
				});
			}
			else {
				//if no snapshot show head version
				for (i=0; i<_resourceInfos.length; i++){
					_resourceInfos[i].isCollectionArtifactAssociatedWithSnapshot = false;
				}
				deferred.callback(_resourceInfos);
			}
		});
		queryDeferred.addErrback(this, function(errorResponse) {
			deferred.errback(errorResponse);
		});
		return deferred;

	},
	
	/*
	 * Retrieves URIs of resources in collection together with the associated project snapshot URI
	 */
	getCollection: function(collectionURI, revisionID, projectSnapshotURI, collectionQuery /*optional*/) {
		var deferred = new dojo.Deferred();
		//get collection content
		var collectionDeferred = RDMClient.doGet({
				uri: collectionURI,
				revision: revisionID,
		        projectSnapshotURI: projectSnapshotURI
			});
		collectionDeferred.addCallback(this, function(collectionContentResponse) {
			var collectionArtifactURIs = [];
			var associatedSnapshotURI = null;
			
			//parse collection to get artifacts URIs together with the associated snapshot URI
			var doc = collectionContentResponse.doc;
			var artifactNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
					 doc, "artifact",
					 queryutils.COLLECTION_NAMESPACE, queryutils.COLLECTION_QUALIFIER);

			for (var i=0; i<artifactNodes.length; i++) {
				collectionArtifactURIs.push(
					queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(
						collectionURI,
						com.ibm.rdm.web.util.domutils.getAttributeNS(artifactNodes[i], "resource", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER)
					)
				);
			}	
			
			//get associated snapshot URI
			var projectSnapshotNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(
					 doc, "snapshot",
					 queryutils.COLLECTION_NAMESPACE, queryutils.COLLECTION_QUALIFIER);
			
			if (projectSnapshotNodes.length > 0) {
				var relativeProjectSnapshotURI = com.ibm.rdm.web.util.domutils.getAttributeNS(projectSnapshotNodes[0], "resource", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER);
				associatedSnapshotURI = queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(collectionURI, relativeProjectSnapshotURI);
			}
			
			if (!collectionQuery) {
				var namespaces = 'nav=http://com.ibm.rdm/navigation#, dc=http://purl.org/dc/terms/, jazz=http://jazz.net/xmlns/foundation/1.0/, att=http://schema.ibm.com/rdm/2008/Attribute#, wrapper=http://jazz.net/xmlns/alm/rm/Reference/v0.1#';
				collectionQuery = {
					query: this._composeURIsClause(collectionArtifactURIs),
					namespaces: namespaces,
					projectSnapshotURI: associatedSnapshotURI
				};
			}
			else {
				collectionQuery.query += ((collectionQuery.query)?" and ":"") + this._composeURIsClause(collectionArtifactURIs);
				collectionQuery.projectSnapshotURI = associatedSnapshotURI;
			}
			
			var collection = {
				collectionArtifactURIs: collectionArtifactURIs,
				associatedSnapshotURI: associatedSnapshotURI,
				collectionArtifacts: []
			};
			
			if (collectionArtifactURIs.length > 0) {
				var collectionArtifactsDeferred = this._getCollectionArtifacts(collection, collectionQuery, projectSnapshotURI);
				collectionArtifactsDeferred.addCallback(this, function(collectionResources) {
					collection.collectionArtifacts = collectionResources;
					deferred.callback(collection);
				});
			}
			else {
				deferred.callback(collection);
			}
			
		});
		collectionDeferred.addErrback(this, function(errorResponse) {
			deferred.errback(errorResponse);
		});
		return deferred;
	},
	_composeURIsClause: function(URIs) {
    	var URIsClause = "";
		if (URIs && URIs.length > 0) {
			URIsClause = 'resource in [';
			for (var i=0; i<URIs.length; i++) {
				URIsClause += (i==0?'':',') + '<' + URIs[i] + '>';
			}
			URIsClause += ']';
		}
        return URIsClause;
    }

};

	// Operation constants
	com.ibm.rdm.web.common.ProjectCache.MODIFY_RESOURCE = "com.ibm.rrs.saveResource"; 
	com.ibm.rdm.web.common.ProjectCache.MODIFY_TEMPLATE = "com.ibm.rrs.saveTemplate";  
	com.ibm.rdm.web.common.ProjectCache.MODIFY_ATT_GROUP = "com.ibm.rrs.saveAttributeGroup";  
	com.ibm.rdm.web.common.ProjectCache.MODIFY_PRIVATE_TAG = "com.ibm.rrs.savePrivateTag"; 
	com.ibm.rdm.web.common.ProjectCache.MODIFY_REVIEW = "com.ibm.rrs.saveReview"; 
	com.ibm.rdm.web.common.ProjectCache.MODIFY_REVIEW_APPROVAL = "com.ibm.rrs.saveReviewApproval"; 
	com.ibm.rdm.web.common.ProjectCache.MODIFY_COMMENT = "com.ibm.rrs.saveComment";  
	com.ibm.rdm.web.common.ProjectCache.MODIFY_REQPRO_CON = "com.ibm.rrs.saveRequisiteProIntegration";
	com.ibm.rdm.web.common.ProjectCache.MODIFY_DOORS_CON = "com.ibm.rrs.saveDOORSIntegration"; 
	com.ibm.rdm.web.common.ProjectCache.MODIFY_PROJECT_USERS = "com.ibm.team.process.server.saveProjectArea/report"; 

})();
