/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.rdm.web.common.RoleAssignment");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

dojo.declare("com.ibm.rdm.web.common.RoleAssignment", null, {
    constructor: function(params) {
        // summary:
        //    Used by to represent role assignments of a user. Projects are saved
        //    in a hash (this.roles), where key is the 
        //    short role  (Reader, Writer, etc), value is a project object 
        //    {"projectURI": <project uri>, "projectName": <project name>}.
        // description:
        //    Note that Composer exposes 3 roles: Admin, Author, Reviewer. These roles map to the 
        //    following JRS roles: Admin, Writer, Commented. This object uses JRS roles. There
        //    is additional JRS role, called Reader, which should be ignored by Composer.
        //    This class ignores the Reader role, i.e. if Reader is the only role returned for a user
        //    the user is consider a non-member. 
        this.userId = params.userId;
        this.roles = {};
        this.allMemberProjects = new Array();
    },
    addProjectRole: function(roleUri, projectObj) {
        // roleUri: Full URI of the role.
        // projectObj: {"projectURI": <project uri>, "projectName": <project name>}
        var simpleRole = this._getSimpleRole(roleUri);
        
        if (simpleRole=="Reader") {
            ErrorHandler.log("RoleAssignment.addProjectRole()  has Reader as a role for project " + projectObj.projectURI + ", for user " + this.userId + ". Reader is not an allowed role. Ignoring");
            return;
        }
        
        var l = this.roles[simpleRole];
        if (!l)
            this.roles[simpleRole] = new Array();
            
        this.roles[simpleRole].push(projectObj);
        if (this._findProject(projectObj.projectURI, this.allMemberProjects)==-1) {
            this.allMemberProjects.push(projectObj);
            this.allMemberProjects.sort(function(a,b) {
                var p = a.projectURI;
                var q = b.projectURI;
                return p==q?0:(p<q?-1:1);
            });
        }
    },
    isWriterIn: function(projectURI) {
        return this._findProject(projectURI, this.roles["Writer"])!=-1;
    },
    isCommenterIn: function(projectURI) {
        return this._findProject(projectURI, this.roles["Commenter"])!=-1;
    },
    /*
    isReaderIn: function(projectURI) {
        return this._findProject(projectURI, this.roles["Reader"])!=-1;
    },
    */
    isAdminIn: function(projectURI) {
        return this._findProject(projectURI, this.roles["Admin"])!=-1;
    },
    _getSimpleRole: function(roleUri) {
        return roleUri.substring(roleUri.lastIndexOf("/")+1, roleUri.length);
    },
    _findProject: function(projectURI, projectList) {
        if (!projectList)
            return -1;
        for (var i=0; i<projectList.length; i++) {
            if (projectList[i].projectURI==projectURI) {
                return i;
            }
        }
        return -1;
    }
    
});




}());
