/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
 
 // Note: DO NOT USE  (function(){ around this module. It breaks with debug=true for unknown reasons!!!!
  
 
dojo.provide("com.ibm.rdm.web.common.UriHelper");

dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.util.queryutils");


var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var queryutils = com.ibm.rdm.web.util.queryutils;


com.ibm.rdm.web.common.UriHelper = {

    WEB_APP_CONVERTER: "/converter",


    getGefDiagramConversionUri: function() {
        return this.WEB_APP_CONVERTER + "/htmlgen";
    },

	getOAuthQueryParams: function() {
		// Summary:
		//    Returns an OAuth query params string which would be appropriate
		//    for appending to an exisitng URI string.
		var oauthToken = dojo.cookie("jfs-oauth-access-token0");
		var requestTokenSecret = dojo.cookie("jfs-oauth-access_token-secret0");
		return (
			"oauthToken=" + encodeURIComponent(oauthToken)
			+ "&requestTokenSecret=" + encodeURIComponent(requestTokenSecret)
		);
	},

	getReviewQueryParams: function(reviewParser) {
		// summary:
		//    Returns a string of the review related query params that are
		//    appropriate for passing in a GET for an artifact that is currently
		//    being shown in the context of a formal review.
		var queryParams = "";
		if (reviewParser && reviewParser.isFormal()
			&& reviewParser.getReviewState() != reviewTypes.REVIEW_STATE_TYPES.Draft
			) {
			// Note: It's expensive, converter-wise, to deal with formal
			// reviews, as it's got to use a ReviewURLRedirector when loading
			// the EMF model associated with the GEF based artifact.  Therefore
			// we only pass these params when it's a "formal" review -- we take
			// care to NOT to pass them in the case of an "informal" review.
			// Another slight optimization is that if it's formal but is still
			// in the Draft state, this means we show the artifacts at the HEAD
			// revision and therefore don't need to tell the converter that
			// they're associated with a review.
			queryParams = (
				"&reviewId=" + reviewParser.getReviewId()
				+ "&reviewRevision=" + reviewParser.getRevision()
				+ "&projectURL=" + encodeURIComponent(reviewParser.getProjectUri())
			);
		}
		return queryParams;
	},

	getBaselineQueryParams: function(projectSnapshotURI) {
		// summary:
		//    Returns a string of the baseline related query params that are
		//    appropriate for passing in a GET for an artifact that is currently
		//    being shown in the context of a snapshot found in the History viewer.
		var queryParams = "";
		if (projectSnapshotURI) {
			var baselineId = queryutils.getLastPathSegment(projectSnapshotURI);
			queryParams += "&baselineId=" + baselineId;
		}
		return queryParams;
	},

	getReviewParticipantResultsUri: function(reviewURI) {
		return reviewURI + "/results";
	},
    
    getResourceRevisionQueryAttribute: function(revision, /*Boolean?*/isFirst) {
    	return (
			revision ?
			(isFirst ? "?" : "&") + "revision=" + revision :
			""
		);
    },

    ////////////////////////// JAF Helpers ////////////////////////////////////////

    getResourceHistoryJafLink: function(resourceURI, revision) {
    	return "#action=com.ibm.rdm.web.pages.showArtifactHistory&artifactURI=" +
				encodeURIComponent(resourceURI) +
				this.getResourceRevisionQueryAttribute(revision);
    },

    getResourceInReviewContextJafLink: function(resourceURI, revision, reviewId) {
    	return "#action=com.ibm.rdm.web.pages.showArtifactInReviewContext&artifactURI=" +
				encodeURIComponent(resourceURI) +
				this.getResourceRevisionQueryAttribute(revision) +
				"&reviewId=" + reviewId;
    },
    
    getResourceJafLink: function(resourceURI) {
    	return "#action=com.ibm.rdm.web.pages.showArtifact&artifactURI=" +
				encodeURIComponent(resourceURI);
    },
    
    getArtifactId: function(resourceURI) {
    	// only return the id if it starts with rrc, this test
    	// will bypass ids that are UUID-based
    	var i = resourceURI.lastIndexOf("/");
        var artifactId = resourceURI.substring(i + 1);
        if (artifactId.indexOf("rrc") == 0) {
        	return artifactId.substring(3);
        } else {
        	return null;
        }
    }
    
};

