<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:h="http://www.w3.org/TR/REC-html40"
 xmlns:rte="http://jazz.net/xmlns/alm/rm/RichText/v0.1"
 xmlns:base="http://jazz.net/xmlns/alm/rm/Base/v0.1"
 xmlns:dc="http://purl.org/dc/terms/"
>
 
<xsl:output method="html" omit-xml-declaration="yes" indent="no"/>
 
 
<xsl:template match="/|comment()|processing-instruction()">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
</xsl:template>

<!-- remove "body" tag from richtext documents but keep its children -->
<xsl:template match="h:body">
    <xsl:apply-templates select="node()"/>
</xsl:template>

<!-- remove "h" namespaces -->
<xsl:template match="h:*">
    <xsl:element name="{local-name()}">
      <xsl:apply-templates select="@*|node()"/>
    </xsl:element>
</xsl:template>

<!-- remove "rpc" namespaces and enclose them into DIV 
<xsl:template match="rte:*">
    <xsl:element name="div">
	    <xsl:element name="{local-name()}">
	      <xsl:apply-templates select="@*|node()"/>
	    </xsl:element>
    </xsl:element>
</xsl:template>-->

<!-- Deleting the description, title and identifier nodes which we do not care about -->
<xsl:template match="dc:title | dc:description | dc:identifier">
</xsl:template>

<!-- normalize attributes -->
<xsl:template match="@*">
    <xsl:attribute name="{local-name()}">
      <xsl:value-of select="."/>
    </xsl:attribute>
</xsl:template>

<xsl:template match="*[local-name()='richtext' or local-name()='RICHTEXT' or local-name()='collection' or local-name()='COLLECTION']">
	<xsl:element name="div">
			<xsl:attribute name="rdmtype">richtext</xsl:attribute>
			<xsl:copy-of select="@*" /> 
			<xsl:attribute name="xmlsrc">
				<xsl:value-of select="@src"/>
			</xsl:attribute>
			<xsl:apply-templates/>
	</xsl:element> 
</xsl:template> 

<xsl:template match="*[local-name()='diagram' or local-name()='story']">
	<xsl:element name="span">
		<xsl:attribute name="rdmtype">diagram</xsl:attribute>
		<xsl:copy-of select="@*" /> 
		<xsl:attribute name="xmlsrc">
				<xsl:value-of select="@src"/>
			</xsl:attribute>
		<xsl:apply-templates/>
	</xsl:element> 
</xsl:template>

<xsl:template match="*[local-name()='a' or local-name()='img']">
	<xsl:element name="{local-name()}">
		<xsl:copy-of select="@*" /> 
		<xsl:attribute name="xmlsrc">
			<xsl:choose>
				<xsl:when test="local-name()='a'">
					<xsl:value-of select="@href"/>
			    </xsl:when>
				<xsl:when test="local-name()='img'">
					<xsl:value-of select="@src"/>
				</xsl:when>
			 </xsl:choose>
		</xsl:attribute>
		<xsl:apply-templates/>
	</xsl:element> 
</xsl:template>


<!-- 14067: Rich Text document badly rendered in Web UI -->
<!-- normalize-space function will make sure that only textual nodes that are not empty will be processed -->
<xsl:template match="text()[contains(.,'&#xD;') or normalize-space(.)!='']">
	<xsl:call-template name="replaceLineBreak">
		<xsl:with-param name="string">
			<xsl:call-template name="replaceAll">
				<xsl:with-param name="string" select="."/>
				<xsl:with-param name="substring" select="'  '"/>
				<xsl:with-param name="replacement" select="'&#160; '"/>
			</xsl:call-template>
		</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<!-- Recursively replaces all ocurrences of a special XML symbol &#xD; with <br/>  -->
<xsl:template name="replaceLineBreak">
	<xsl:param name="string"/>
	<xsl:choose>
		<xsl:when test="contains($string, '&#xD;')">
			<xsl:value-of select="substring-before($string, '&#xD;')"/>
			<br/>
			<xsl:call-template name="replaceLineBreak">
				<xsl:with-param name="string" select="substring-after($string, '&#xD;')"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$string"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- Helper function that recursively replaces all ocurrences of a replacement substring in a given string  -->
<xsl:template name="replaceAll">
	<xsl:param name="string"/>
	<xsl:param name="substring"/>
	<xsl:param name="replacement"/>
	<xsl:choose>
		<xsl:when test="contains($string, $substring)">
			<xsl:value-of select="substring-before($string, $substring)"/>
			<xsl:value-of select="$replacement" disable-output-escaping="yes"/>
			<xsl:call-template name="replaceAll">
				<xsl:with-param name="string" select="substring-after($string, $substring)"/>
				<xsl:with-param name="substring" select="$substring"/>
				<xsl:with-param name="replacement" select="$replacement"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$string"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>
 
</xsl:stylesheet>