/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.glossary.GlossaryWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.glossary.TermWidget");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.util.autoscroller");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.glossary", "GlossaryWidget");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var queryutils = com.ibm.rdm.web.util.queryutils;

var TOOLBAR_HEIGHT = 23;

/**
 * Main Glossary widget. 
*/
dojo.declare("com.ibm.rdm.web.glossary.GlossaryWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "glossary/templates/GlossaryWidget.html"),

    widgetsInTemplate: true,
    
	///////////////////////////////////////////////////////////////////////////
    // Attach points

	_glossaryWidgetNode: null,
	_loadingDiv: null,
	_contentDiv: null,
	_glossaryMenu: null,
	_dropdownDisplayNode: null,
	_dropdownSortNode: null,
	_contentPane: null,
	_termWidgetListNode: null,

	///////////////////////////////////////////////////////////////////////////
    // Public fields

	idNodesMap: {},
    
	///////////////////////////////////////////////////////////////////////////
    // Private fields

    _widthToUse: null,
    _heightToUse: null,
    _termWidgets: null,
    _termsLoadedCount: null,
    _mapURIToTerm: null,
    _sortDescending: null,
	_documentHasBeenLoaded: null,
	_displayMenuWidget: null,
	_sortMenuWidget: null,
	_etag: null,
	_selectedNode: null,
	_connectHandles: null,
	_subscriptions: null,
	_locationsMarkedForComments: null,
	
	_DISPLAY_TYPE: {
		LOADING_MESSAGE: 0,
		ARTIFACT_CONTENT: 1
	},

	///////////////////////////////////////////////////////////////////////////
    // API

    postCreate: function() {
		this._connectHandles = new Array();
		this._locationsMarkedForComments = new Array();

        this._initSubscriptions();
		
		this._mapURIToTerm = {};
		this._documentHasBeenLoaded = false;
		this._sortDescending = false;
		this._displayMenuWidget = new com.ibm.rdm.web.links.ToolbarDropdownMenu(
				{
				_imgURI: "com.ibm.rdm.web/common/images/show_details.gif",
				_options: [ this._messages.definition, this._messages.details, this._messages.status ],
				_scope: this,
				_callback: this.handleDisplay,
				_type: "CHECKBOX"
				});
		this._dropdownDisplayNode.appendChild( this._displayMenuWidget.domNode );
		
		this._sortMenuWidget = new com.ibm.rdm.web.links.ToolbarDropdownMenu(
				{
				_imgURI: "com.ibm.rdm.web/common/images/sort.gif",
				_options: [ this._messages.sortAscending, this._messages.sortDescending ],
				_scope: this,
				_callback: this.handleSort,
				_type: "RADIO"
				});
		this._dropdownSortNode.appendChild( this._sortMenuWidget.domNode );
		this.inherited("postCreate", arguments);
    },

    startup: function() {
    	this.inherited("startup", arguments);
    },
    
    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.glossary", "GlossaryWidget");
    },
	
    destroy: function() {
    	this._displayMenuWidget.destroyRecursive();
    	this._sortMenuWidget.destroyRecursive();
    	for( var uri in this._mapURIToTerm ) {
    		this._mapURIToTerm[uri].destroyRecursive();
    	}
        for (var i = 0; i < this._connectHandles.length; i++) {
            this.disconnect(this._connectHandles[i]);
            this._connectHandles[i] = null;
        }
        for (var i = 0; i < this._subscriptions.length; i++) {
            dojo.unsubscribe(this._subscriptions[i]);
            this._subscriptions[i] = null;
        }
		this.inherited("destroy",arguments);
    },
    
	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.
        this._loadingDiv.innerHTML = htmlContent;
		this._setDisplayType(this._DISPLAY_TYPE.LOADING_MESSAGE);
	},

    subscribe: function(topic, method) {
    	var topic = EventTopics.getTopic(topic, this.resourceURI);
    	ErrorHandler.debug("GlossaryWidget: subscribe to event -> " + topic);
        this._subscriptions.push(
        		dojo.subscribe(
        				topic, 
        				this, 
        				method));
    },

    publish: function(topic, params) {
    	var topic = EventTopics.getTopic(topic, this.resourceURI);
    	ErrorHandler.debug("GlossaryWidget: fire event -> " + topic);
		dojo.publish(
				topic, 
				params);
    },

    _initSubscriptions: function() {
    	this._subscriptions = [];
    	this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, "_onLocationPaneSelected");
    	this.subscribe(EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE, "_onHighlightComments");
    	this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC, "_onLocationPaneHover");
    },
    
    setDimensions: function(width, height) {
        this._setDimensions(width, height);
    },
    
    loadResource: function(contentType, revision) {
    	this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [null, null]);
        this._selectedNode = null;
        for (var i = 0; i < this._connectHandles.length; i++) {
            this.disconnect(this._connectHandles[i]);
        }
        this._connectHandles = new Array();

		var glossaryDeferred = RDMClient.doGet({
				uri: this.resourceURI,
				revision: revision,
		        projectSnapshotURI: this.resourceViewer.projectSnapshotURI
			});
    	glossaryDeferred.addCallback(this, this.handleLoadResponse);
		return glossaryDeferred;
    },
    
    handleLoadResponse: function() {
    	    	
    	if(!(com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES.substring(0,5) == this.resourceURI.substring(0,5))){
    		var resourceUri = queryutils.getFullyQualifiedUriFromRelativeLink(com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES, this.resourceURI);
    	}
    	else{
    		var resourceUri = this.resourceURI;
    	}
		this._setDisplayType(this._DISPLAY_TYPE.ARTIFACT_CONTENT);
    	var query = "links:hasLinkType=<" + com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES + 
    	"/GlossaryTerms> and links:linkTarget=<" + resourceUri + 
    	">";
    	var properties = "links:linkSource&namespaces=links=http://www.ibm.com/xmlns/rrm/1.0/";
    	ProjectCache.getUserMemberProjects().addBoth(this, function(projects){
    		//terms must be filtered by the projects, which users is a member of
    		var projectURIs = "";
    		for (projectURI in projects) {
    			projectURIs += projectURI + ",";
    		}
    		if (projectURIs) {
    			//remove last comma
    			projectURIs = projectURIs.substring(0, projectURIs.length-1);
    		}
			var glossaryLinksDeferred = ProjectCache.executeQuery({
				query:query,
				properties:properties,
				skipCache: false,
				projectURI: projectURIs,
				projectSnapshotURI: this.resourceViewer.projectSnapshotURI,
				parser: function(response) {
					var termLinks = [];
					var entries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "entry");
					for(var i=0; i<entries.length; i++){
						termLinks.push(com.ibm.rdm.web.util.domutils.getEntryBindingValueByName(entries[i], "linkSource"));
					}
					return termLinks;
				}
			});
			if (this.resourceViewer.projectSnapshotURI) {
				glossaryLinksDeferred.addCallback(this, function(terms) {
					var termsInBaselineDeferred = ProjectCache.getBatchResourceDescriptionsMultiFetch(terms, this.resourceViewer.projectSnapshotURI);
					termsInBaselineDeferred.addBoth(this, function(termsInBaseline){
						var termLinksInBaseline = [];
						for (termURI in termsInBaseline) {
							var termInBaseline = termsInBaseline[termURI];
							if (termInBaseline) { //check if it is deleted
								termLinksInBaseline.push(termURI + "?revision=" + termInBaseline.etag + "&amp;rrc:baseline=" + queryutils.getLastPathSegment(this.resourceViewer.projectSnapshotURI));
							}
						}
						this._constructTermWidgets(termLinksInBaseline);
					});
				});
			}
			else {
				glossaryLinksDeferred.addCallback(this, this._constructTermWidgets);
			}
		});
    },
    
    handleSort: function( options, index ) {
    	this.idNodesMap = {};
    	this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [null, null]);
    	if( options[index] == this._messages.sortAscending ) {
    		if( this._sortDescending ) { 
    			this._sortDescending = false;
    			this._termWidgets.reverse();
    		}
    	} else {
    		if( !this._sortDescending ) {
    			this._sortDescending = true;
    			this._termWidgets.reverse();
    		}    		
    	}
        for (var i = 0; i < this._connectHandles.length; i++) {
            this.disconnect(this._connectHandles[i]);
            this._connectHandles[i] = null;
        }
    	com.ibm.rdm.web.util.domutils.removeChildNodes( this._termWidgetListNode );
    	var self = this;
		dojo.forEach( this._termWidgets, function( term, i ) {
			var termNode = document.createElement( "div" );
			termNode.setAttribute("id", term.resourceURI);
			termNode.style.padding = "2px";
			termNode.appendChild( term.domNode );
			if( i % 2 == 0 ) {
				termNode.className = "odd-term";
			} else {
				termNode.className = "even-term";
			}
			self._termWidgetListNode.appendChild( termNode );
			self._connectHandles.push(
					self.connect(termNode, "onclick", self._onElementClicked));
			self.idNodesMap[termNode.id] = termNode;
		});
		
    },
    
    handleDisplay: function( options, optionsSelectedArray, index ) {
    	if( optionsSelectedArray[index] ) {
    		if( options[index] == this._messages.definition ) {
    			this._showDefinition();
    		} else if( options[index] == this._messages.details ) {
    			this._showDetails();
    		} else if( options[index] == this._messages.status ) {
    			this._showStatus();
    		}
    	} else {
    		if( options[index] == this._messages.definition ) {
    			this._hideDefinition();
    		} else if( options[index] == this._messages.details ) {
    			this._hideDetails();
    		} else if( options[index] == this._messages.status ) {
    			this._hideStatus();
    		}
    	}
    },
    
    select: function(node /*Node*/) {
    	if (this.isSelected(node))
            return;
    	
    	if (node != this._getRootArtifactNode()) {
    		dojo.addClass(node, "selected-viewer-element");
    	}
        //node.style.border = SELECTED_ELEMENT_BORDER;
        //if (node.nodeName != "BODY") //<body> doesn't need margin
        //	node.style.margin = SELECTED_ELEMENT_MARGIN;
        this.unselect(this._selectedNode);
        this._selectedNode = node;        
    },
    
    unselect: function(node /*Node*/) {
        if (!this.isSelected(node))
            return;
        dojo.removeClass(node, "selected-viewer-element");
        this._selectedNode = null;
    },
    
    toggleSelection: function(node) {
    	if (this.isSelected(node))
            this.unselect(node);
        else
            this.select(node);
    },
    
    isSelected: function(node /*Node */) {
        return node && this._selectedNode && this._selectedNode == node;
    },
    
	_setDisplayType: function(displayType) {
		// summary:
	    //    Reset whether we're viewing a Loading... message or the actual
		//    artifact content.
		// displayType:
		//    Must be one of the values found in _DISPLAY_TYPE field.
		if (displayType === this._DISPLAY_TYPE.LOADING_MESSAGE) {
			this._loadingDiv.style.display = "";
			this._contentDiv.style.display = "none";
		}
		else if (displayType === this._DISPLAY_TYPE.ARTIFACT_CONTENT) {
			this._loadingDiv.style.display = "none";
			this._contentDiv.style.display = "";
		}
	},
    
    _getRootArtifactNode: function() {
        return this._termWidgetListNode;
    },
    
    _onElementClicked: function(event) {
    	//dojo.stopEvent(event);
        var node = event.currentTarget;
        var unselectedNodeId = null;
        var selectedNodeId = null;
        if (this._selectedNode)
        	unselectedNodeId = this._selectedNode.id;
        this.toggleSelection(node);
        if (this._selectedNode)
        	selectedNodeId = this._selectedNode.id;
        this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [selectedNodeId, unselectedNodeId]);
    },
    
    _showDefinition: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.showDefinition();
    	});
    },

    _hideDefinition: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.hideDefinition();
    	});
    },

    _showStatus: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.showStatus();
    	});
    },
    
    _hideStatus: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.hideStatus();
    	});
    },

    _showDetails: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.showDetails();
    	});
    },
    
    _hideDetails: function() {
    	dojo.forEach( this._termWidgets, function( termWidget ){
    		termWidget.hideDetails();
    	});
    },
    
    _constructTermWidgets: function(termLinks) {
    	this._termWidgets = [];
    	var multiGetParams = [];
    	for (var i = 0; i < termLinks.length; i++) {
    		multiGetParams.push({
				uri: termLinks[i]
    		});
    	}
    	var multiGetDeferred = ProjectCache.multiGet(multiGetParams);
    	multiGetDeferred.addCallback(this, function(multiGetResponse){
    		for (var i=0; i<termLinks.length; i++) {
	    		var termWidget = new com.ibm.rdm.web.glossary.TermWidget({
						resourceURI: termLinks[i],
						resourceViewer: this.resourceViewer
					});
				termWidget.handleLoadResponse(multiGetResponse[i]);
				this._termWidgets.push(termWidget);
	    	}
	    	this._onTermLoadComplete();
    	});
    },
    
    _onTermLoadComplete: function() {
    	com.ibm.rdm.web.util.domutils.removeChildNodes(this._termWidgetListNode);
		this._termWidgets.sort(function(term1, term2) {
			return (term1.getTitle().toLowerCase()>term2.getTitle().toLowerCase())?
					1 : (term1.getTitle().toLowerCase()<term2.getTitle().toLowerCase())?
							-1 : 0;
		});
		if (this._sortDescending) { 
			this._termWidgets.reverse();
		}
		for (var i = 0; i < this._termWidgets.length; i++) {
			var term = this._termWidgets[i];
			var termNode = document.createElement( "div" );
			termNode.setAttribute("id", term.resourceURI);
			termNode.appendChild(term.domNode);
			if( i % 2 == 0 ) {
				termNode.className = "odd-term";
			} else {
				termNode.className = "even-term";
			}
			this._termWidgetListNode.appendChild(termNode);
			this._connectHandles.push(
					this.connect(termNode, "onclick", this._onElementClicked));
			this.idNodesMap[termNode.id] = termNode;
		}
		
		this._documentHasBeenLoaded = true;
		this._setDimensions();
    },
    
    _setDimensions: function(width, height) {
        // Note: width and height are optional.
        if (width && height) {
            this._widthToUse = width;
            this._heightToUse = height;
        }
        if( this._documentHasBeenLoaded) {
            var widthToUse = this._widthToUse;
            var heightToUse = this._heightToUse - TOOLBAR_HEIGHT;
            this._contentPane.style.height = heightToUse + "px";
            this._contentPane.style.width = widthToUse + "px";
            var contentPaneSize = {
                    h: heightToUse,
                    w: widthToUse
            };
            this._contentPane.resize(contentPaneSize);
        }
    },
    
    highlightComments: function(locationCommentDetails) {      

    },
    
    _onLocationPaneSelected: function(resourceLocationId, resourceUri) {
        var f = this._getRootArtifactNode();
        var node = null;
        if (resourceLocationId=="")
        	node = f;
        else {
        	node = dojo.query("[id='" + resourceLocationId + "']", f);
        	if(node.length<1)
        		node = null;
        	else
        		node = node[0];
        }
        if (node) {
            this.select(node);
           	com.ibm.rdm.web.util.autoscroller.scrollToNode(node, this._contentPane.domNode);
        }
        else {
            this.unselect(this._selectedNode);
        }
    },
    
    _onLocationPaneHover: function(resourceLocationId, isHoverOver) {
    	var f = this._getRootArtifactNode();
        var node = null;
        if (resourceLocationId=="") {
        	node = f;
        } else {
        	node = dojo.byId(resourceLocationId);
        }
        if (node) {
            if (isHoverOver) {
            	dojo.addClass(node, "hover-over-highlight");
            } else { 
	            dojo.removeClass(node, "hover-over-highlight");
            }
        }
    },
    
    _onHighlightComments: function(commentCounts) {
        for (var i = 0; i < this._locationsMarkedForComments.length; i++) {
            var node = this._locationsMarkedForComments[i];
            dojo.removeClass(node, "highlight-commented-element");
            this._removeCommentsCountFromUnselectedElement(node);
        }
        this._locationsMarkedForComments = new Array();
        
        for (resourceLocationId in commentCounts) {
        	var node = dojo.byId(resourceLocationId);
        	if (node) {
        		dojo.addClass(node, "highlight-commented-element");
        		this._locationsMarkedForComments.push(node);
        		this._addCommentsCountToSelectedElement(node, commentCounts[resourceLocationId]);
        	}
        }
    },
    
    _addCommentsCountToSelectedElement: function(oSelectedElement, iCommentsCount) {
		try {
	    	// var oCommentCountElement = this._iframeDoc().createElement("div");
	    	var oCommentCountElement = document.createElement("div");
			oCommentCountElement.id = oSelectedElement.id + "_commentsCount";
			oCommentCountElement.innerHTML = 
				'<table style="position:absolute;">' +
					'<tr>' +
						'<td>' +
							'<img style="vertical-align: middle;" src="com.ibm.rdm.web/commenting/images/comment.gif"/>' +
						'</td>' +
						'<td style="font-family:sans-serif;font-size:9pt;font-weight:normal;color:#033">' +
							iCommentsCount + 
						'</td>' +
					'</tr>' +
				'</table>';
	
			oCommentCountElement.style.styleFloat = "right"; //IE
			oCommentCountElement.style.cssFloat = "right"; //Mozilla
			oCommentCountElement.style.position = "relative";
			oCommentCountElement.style.top = "-20px";
			oCommentCountElement.style.left = "-50px";
			oCommentCountElement.style.height = "0px";
			oCommentCountElement.style.width = "0px";
			oSelectedElement.insertBefore(oCommentCountElement, oSelectedElement.firstChild);
		} catch (e) {
			console.error(e);
		}
    },
    
    _removeCommentsCountFromUnselectedElement: function (oUnselectedElement) {
        try {
        	var oCommentCountElement = dojo.query("[id='" + oUnselectedElement.id + "_commentsCount']", this._getRootArtifactNode());
	        oCommentCountElement[0].parentNode.removeChild(oCommentCountElement[0]);
        } catch(e) {
        	console.error(e);
	    }
    }

});



})();
