/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.glossary.TermWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.requireLocalization("com.ibm.rdm.web.glossary", "TermWidget");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var EventTopics = com.ibm.rdm.web.commenting.EventTopics;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

/**
 * Main Term widget. 
*/
dojo.declare("com.ibm.rdm.web.glossary.TermWidget", [dijit._Widget, dijit._Templated], {
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "glossary/templates/TermWidget.html"),

	///////////////////////////////////////////////////////////////////////////
    // Attach points

	_termWidgetNode: null,
	_loadingDiv: null,
	_contentDiv: null,
	_termTitleNode: null,
	_statusNode: null,
	_requirementsNode: null,
	_commentNode: null,
	_linksNode: null,
	_termDefinitionNode: null,
	_termDetailNode: null,
	_relatedTermsNode: null,
	_synonymsNode: null,
	_acronymsNode: null,
	_selectedNode: null,

	///////////////////////////////////////////////////////////////////////////
    // Public fields

	resourceURI: null,
    errorLoadingTerm: null,
    idNodesMap: [],

	///////////////////////////////////////////////////////////////////////////
	// NLS fields

	relatedTerm: null,
	synonyms: null,
	acronyms: null,
	
	///////////////////////////////////////////////////////////////////////////
    // Private fields

    _widthToUse: null,
    _heightToUse: null,
    _status: null,
    _synonyms: null,
    _relatedTerms: null,
    _definition: null,
    _title: null,
    _acronym: null,
    _linkCount: null,
    _linkedRequirementCount: null,
    _etag: null,
    _refreshDeferred: null,
    _documentHasBeenLoaded: null,

	_DISPLAY_TYPE: {
		LOADING_MESSAGE: 0,
		ARTIFACT_CONTENT: 1
	},
    
	///////////////////////////////////////////////////////////////////////////
	// API

    postCreate: function() {
    	this.inherited("postCreate", arguments);
/*    	var reqIcon = document.createElement("img");
    	var commentsIcon = document.createElement("img");
    	var linksIcon = document.createElement("img");
        reqIcon.setAttribute("src", "com.ibm.rdm.web/common/images/show_requirements.gif");
        commentsIcon.setAttribute("src", "com.ibm.rdm.web/common/images/show_comments.gif");
    	linksIcon.setAttribute("src", "com.ibm.rdm.web/common/images/show_links.gif");
    	reqIcon.className = "term-icon-image";
    	commentsIcon.className = "term-icon-image";
    	linksIcon.className = "term-icon-image";
    	this._requirementsNode.appendChild( reqIcon );
    	this._commentNode.appendChild( commentsIcon );
    	this._linksNode.appendChild( linksIcon );
*/    	//this.connect( this._linksWidget, "updateLinkCountHeader", this._displayTermProperties );
    	this._initSubscriptions(); 
    },

    startup: function() {
    	this.inherited("startup", arguments);
    },
    
    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.glossary", 
        												"TermWidget");
        dojo.forEach(["relatedTerm", "synonyms", "acronyms", "tags"], function(prop){
            this[prop] = this._messages[prop];
        }, this);
    },
	
    destroy: function() {
        for (var i = 0; i < this._subscriptions.length; i++) {
            dojo.unsubscribe(this._subscriptions[i]);
            this._subscriptions[i] = null;
        }
        this._subscriptions = [];
        this._documentHasBeenLoaded = false;
        this.inherited("destroy", arguments);
    },
    
	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.
        this._loadingDiv.innerHTML = htmlContent;
		this._setDisplayType(this._DISPLAY_TYPE.LOADING_MESSAGE);
	},

    loadResource: function(contentType, revision, skipErrors) {
    	var deferred = RDMClient.doGet({
					uri: this.resourceURI,
					revision: revision,
					skipErrors: skipErrors,
		        	projectSnapshotURI: this.resourceViewer.projectSnapshotURI
    			});
    	var self = this;
    	deferred.addCallback(this, this.handleLoadResponse);
    	deferred.addErrback( function( response ) { 
    		ErrorHandler.log( "Error occured in GET for resource URI: " + self.resourceURI );
    		return response;
    	});
		return deferred;
    },
    
    select: function(node /*Node*/) {
    	dojo.addClass(node, "term-widget-on-select");
        this.unselect(this._selectedNode);
        this._selectedNode = node;        
    },
    
    unselect: function(node /*Node*/) {
        if (!this.isSelected(node))
            return;
    	dojo.removeClass(node, "term-widget-on-select");
        this._selectedNode = null;
    },
    
    isSelected: function(node /*Node */) {
        return node && this._selectedNode && this._selectedNode == node;
    },
    
    //glossaries call this method directly 
    handleLoadResponse: function(response) {
		this._setDisplayType(this._DISPLAY_TYPE.ARTIFACT_CONTENT);
		// Populate the _contentDiv.
    	try {
	    	var doc = (response.doc)?response.doc:response;
	    	
	    	var definitionNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			doc, 
	    			"definition", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	var termNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			doc, 
	    			"term", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	
	    	var definitionNodes = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			doc, 
	    			"definition", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	
	    	var definitionText = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			definitionNodes[0], 
	    			"p", 
	    			com.ibm.rdm.web.util.queryutils.H_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.H_QUALIFIER 
	    	);
	    	this._definition = definitionText[0].firstChild.nodeValue;
	    	
	    	this._synonyms = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			doc, 
	    			"synonym", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE,
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	this._relatedTerms = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS( 
	    			doc, 
	    			"relatedTerm", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	this._title = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(doc);
	    	this._status =  com.ibm.rdm.web.util.domutils.getAttributeNS( 
	    			termNodes[0], 
	    			"status", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	this._acronym = com.ibm.rdm.web.util.domutils.getAttributeNS( 
	    			termNodes[0], 
	    			"abbreviation", 
	    			com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE, 
	    			com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER 
	    	);
	    	
    	} catch (e) {
    		console.error(e);
	    	this._definition = "";
	    	this._synonyms = "";
	    	this._relatedTerms = "";
	    	this._title = "";
	    	this._status =  "";
	    	this._acronym = "";
	    	this.errorLoadingTerm = true;
    	} finally {
	    	this._displayTermProperties();
	    	if(this._refreshDeferred && this._refreshDeferred.fired != 0) {
	    		this._refreshDeferred.callback();
	    	}
	    	this._documentHasBeenLoaded = true;
    	}
    	return response;
    },
    
    hideDefinition: function() {
    	this._termDefinitionNode.style.display = "none";
    },
    
    showDefinition: function() {
    	this._termDefinitionNode.style.display = "";
    },
    
    hideDetails: function() {
    	this._termDetailNode.style.display = "none";
    },
    
    showDetails: function() {
    	this._termDetailNode.style.display = "";
    },
    
    hideStatus: function() {
    	this._statusNode.style.display = "none";
    },
    
    showStatus: function() {
    	this._statusNode.style.display = "";
    },

	_setDisplayType: function(displayType) {
		// summary:
	    //    Reset whether we're viewing a Loading... message or the actual
		//    artifact content.
		// displayType:
		//    Must be one of the values found in _DISPLAY_TYPE field.
		if (displayType === this._DISPLAY_TYPE.LOADING_MESSAGE) {
			this._loadingDiv.style.display = "";
			this._contentDiv.style.display = "none";
		}
		else if (displayType === this._DISPLAY_TYPE.ARTIFACT_CONTENT) {
			this._loadingDiv.style.display = "none";
			this._contentDiv.style.display = "";
		}
	},
    
	_cleanUpTerm: function() {
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._acronymsNode );
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._termTitleNode );
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._statusNode );
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._termDefinitionNode );
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._synonymsNode );
		com.ibm.rdm.web.util.domutils.removeChildNodes( this._relatedTermsNode );	
	},
	
    _displayTermProperties: function() {

    	//this._linkCount = this._linksWidget.getLinkCount();
    	//this._linkedRequirementCount = this._linksWidget.getRequirementLinkCount();
		this._cleanUpTerm();
		
    	if( this._acronym ) {
    		this._acronymsNode.appendChild(document.createTextNode( this._acronym ) );
    	} else {
    		this._acronymsNode.appendChild(document.createTextNode( this._messages.none ) );
    	}
    	this._termTitleNode.appendChild(document.createTextNode( this._title ) );
    	this._statusNode.appendChild( this._getStatusImgTag( this._status ) );
    	this._statusNode.appendChild(document.createTextNode( this._status ) );
    	//this._requirementsNode.appendChild( document.createTextNode( "(" + this._linkedRequirementCount + ")" ) );
    	//this._commentNode.appendChild( document.createTextNode( "(" + ")" ) );
    	//this._linksNode.appendChild( document.createTextNode( "(" + this._linkCount + ")" ) );
/*    	if (dojo.isIE) {
    		this._termDefinitionNode.innerHTML = this._definition;
    	} else {*/
    	this._termDefinitionNode.appendChild(document.createTextNode(this._definition));
    	var termProperties = [ this._synonyms, this._synonymsNode, this._relatedTerms, this._relatedTermsNode ];
    	for( var i=0; (i+1) < termProperties.length; i+=2 ) {
    		var property = termProperties[i];
    		var node = termProperties[i+1]; 		
	    	if( property.length > 0 ) {
		    	for( var j=0; j < property.length; j++ ) {
		            var href = property[j].attributes.getNamedItem("href");
		            var resourceURI = com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + "/" + href.nodeValue;
		            var link = ResourceLinksHandler.getResourceLink(resourceURI, property[j].getAttribute("title"), true);
		            node.appendChild(link);
		    		if( j < property.length - 1 ) {
		    			node.appendChild(document.createTextNode( ", " ));
		    		}
		    	}
	    	} else {
	    		node.appendChild(document.createTextNode( this._messages.none ));
	    	}
    	}
    },
    
    _handleResourceContextClicked: function(event) {
    	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
    },
    
    _handleResourceClicked: function(event) {
		ResourceLinksHandler.handleAnchorClicked(event);
    },
    
    getTitle: function() {
    	return this._title;
    },
    
    _getStatusImgTag: function( status ) {
    	var statusImg = document.createElement("img");
    	if( status == "Published" ) {
    		statusImg.setAttribute("src", "com.ibm.rdm.web/common/images/published.gif");
    	} else if( status == "Draft" ) {
    		statusImg.setAttribute("src", "com.ibm.rdm.web/common/images/draft.gif");
    	} else if( status == "Deprecated" ) {
    		statusImg.setAttribute("src", "com.ibm.rdm.web/common/images/deprecated.gif");
    	}
    	statusImg.className = "term-icon-image";
    	return statusImg;
    },
    
    setDimensions: function(width, height) {
        this._setDimensions(width, height);
    },
    
    _setDimensions: function(width, height) {
        // Note: width and height are optional.
        if (width && height) {
            this._widthToUse = width;
            this._heightToUse = height;
        }
        // Note: leave some space for border highlighting
		this._termWidgetNode.style.height = (this._heightToUse - 4) + "px";
		this._termWidgetNode.style.width = (this._widthToUse - 4) + "px";
    },
    
    highlightComments: function() {
    	
    },
    
    handleRefresh: function(skipErrors) {
    	var deferred = RDMClient.doGet({
					uri: this.resourceURI,
					skipErrors: skipErrors,
		        	projectSnapshotURI: this.resourceViewer.projectSnapshotURI
    			});
    	deferred.addCallback(this, this.handleLoadResponse);
    	
    	return deferred;
    },
    
    subscribe: function(topic, method) {
    	var topic = EventTopics.getTopic(topic, this.resourceURI);
    	ErrorHandler.debug("TermWidget: subscribe to event -> " + topic);
        this._subscriptions.push(
        		dojo.subscribe(
        				topic, 
        				this, 
        				method));
    },
    
    _initSubscriptions: function() {
    	this._subscriptions = [];
    	this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC, "_onLocationPaneHover");
    	this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, "_onLocationPaneSelected");
    	ErrorHandler.info(EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC+ " is subscribed to _onLocationPaneHover");
    },
    
    _onLocationPaneSelected: function(resourceLocationId, resourceUri) {
        var node = null;
        if (resourceLocationId=="")
        	node = this._getRootArtifactNode();
        else 
        	node = this.idNodesMap[resourceLocationId];

        if (node) {
            this.select(node);
           	com.ibm.rdm.web.util.autoscroller.scrollToNode(node, this._getRootArtifactNode());
        }
        else {
            this.unselect(this._selectedNode);
        }
    },
    
    _onLocationPaneHover: function(resourceLocationId, isHoverOver) {
//    	console.info("TermWidget._onLocationPaneHover is invoked");
	  var node = null;
	  if (resourceLocationId=="")
	  	node = this._getRootArtifactNode();
	  else 
	  	node = this.idNodesMap[resourceLocationId];
	  
	  if (node) {
	      if (isHoverOver)
	      	this.hoverOver(node);
	     else 
	        this.hoverOut(node);
      }
    },
    
    hoverOver: function(node /*Node*/) { 
    	dojo.addClass(node, "term-widget-on-hover");
//    	console.info("term-widget-on-hover class is added");
    },
    
    hoverOut: function(node /*Node*/) {
    	dojo.removeClass(node, "term-widget-on-hover");
    },
    
    _getRootArtifactNode: function() {
        var r = this._termWidgetNode;
        return r;
    }    
});
})();
