/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.history.revisionsHelper");
dojo.require("com.ibm.rdm.web.history.RevisionsCache");
dojo.require("com.ibm.rdm.web.history.RevisionsParser");


var RevisionsCache = com.ibm.rdm.web.history.RevisionsCache;
var RevisionsParser = com.ibm.rdm.web.history.RevisionsParser;


com.ibm.rdm.web.history.revisionsHelper = {

	///////////////////////////////////////////////////////////////////////////
	// API

	getNewestPastRevisionByDate: function(resourceURI, /*Date*/date) {
		// summary:
		//    Determines the "appropriate" embedded artifact revision that
		//    should be loaded based on a particular timestamp.
		// description:
		//    Looks at the artifact history for 'resourceURI' to determine which
		//    revision has a date that is closest to the passed 'date'.  By
		//    "closest to" we mean a revision whose revision date is either equal
		//    to 'date' or older to this date but closest to this date among all
		//    of the revisions.
		// returns: Deferred
		//    Returns a deferred that will be called back with a property list
		//    in the following format:
		//        {
		//            revisionToUse: <revisionToUse>,
		//            headRevision: <headRevision>
		//        }
		//    both of these revision-based properties represent the appropriate
		//    "etag" values for the passed 'resourceURI'.
		var onResourceRevisionsComplete = function(response) {
			var revisionsParser = response.doc;
			var revisionInfos = revisionsParser.getRevisionInfos();
			var time = date.getTime();
			var revisionInfoToUse = null;
			for (var i = 0; i < revisionInfos.length; i++) {
				if (revisionInfos[i].revisionDate.getTime() <= time) {
					revisionInfoToUse = revisionInfos[i];
					break;
				}
			}

			// FIXME As it stands on 07/08/2009, the native client is storing
			// a value that is too course for comparison purposes.  For example
			//     <rrmReview:reviewTimestamp>Jul 8, 2009</rrmReview:reviewTimestamp>
			// which means revisionInfoToUse may be set to null.
			// As a workaround we'll just return the latest revision when this occurs.
			// Once native DOES store a more fine grained value we'll need to remove
			// this workaround.
			if (! revisionInfoToUse) {
				revisionInfoToUse = revisionInfos[0];
			}

			newestPastRevisionDeferred.callback({
				revisionToUse: revisionInfoToUse.revisionId
			});
		};
		var newestPastRevisionDeferred = new dojo.Deferred();
		RevisionsCache.getRevisions(resourceURI).addCallback(this, onResourceRevisionsComplete);
		return newestPastRevisionDeferred;
	},
	
	getNewestPastRevisionByDate2: function(revisionsParser, date) {
		var revisionInfos = revisionsParser.getRevisionInfos();
		var time = date.getTime();
		var revisionInfoToUse = null;
		for (var i = 0; i < revisionInfos.length; i++) {
			if (revisionInfos[i].revisionDate.getTime() <= time) {
				revisionInfoToUse = revisionInfos[i];
				break;
			}
		}
		if (! revisionInfoToUse) {
			revisionInfoToUse = revisionInfos[0];
		}
		return revisionInfoToUse;
	} 
};

})();
