/*
 * Copyright (c) 2007, 2008. IBM Corporation
 * All rights reserved.
 */
(function(){

dojo.provide("com.ibm.rdm.web.links.GraphicalLinksExplorer");
// dojo.require("com.ibm.rdm.web.links.TraceabilityClient");
dojo.require("com.ibm.rdm.web.links.LinkCache");
dojo.require ("com.ibm.rdm.web.links.Tree");
dojo.require("dojox.collections.ArrayList");
dojo.require("dojox.collections.Queue");
// MBA
dojo.require("com.ibm.rdm.web.links.TreeLayout");



/* GLOBAL VARIABLES */
var visitedNodes     = new dojox.collections.ArrayList();                                     /* Visited nodes  */
var parent_queue     = new dojox.collections.Queue();
var children_queue   = new dojox.collections.Queue();
var column_to_draw   = new dojox.collections.Queue();
var previousParent   = null;                                        /* Var to track previous visited parent node */
var prevChild        = null;
var rootNode         = null;                                                              /*   Root node         */
var container = null;
var callback = null;
var callbackScope = null;

var PADDING_SIZE = 120;

com.ibm.rdm.web.links.GraphicalLinksExplorer.createGraph = function(_container, _resourceUri, rootNodeData, callbackFn, scope){
	  init();
      container = _container;
      callback = callbackFn;
      callbackScope = scope;
      processLinks(_resourceUri, rootNodeData);
}

//Ankur
function init() {
	visitedNodes.clear();
	parent_queue.clear();
	children_queue.clear();
	column_to_draw.clear();
	previousParent = null;
	rootNode = null;
	container = null;
    callback = null;
    callbackScope = null;
}

function processLinks(url, rootNodeData){
    var putChildrenDeferred = null;                                                     /*    Deferred object   */
    var initialData = {                                                                  /* Object data response */
        targetUri: rootNodeData.url,
        iconUri:   rootNodeData.iconUri,
        targetDisplayName: rootNodeData.label
    };
    var id = initialData.targetUri.substring(initialData.targetUri.lastIndexOf("/") + 1);
    rootNode = new com.ibm.rdm.web.links.Tree.TreeNode(id);
    rootNode.data = initialData;
    parent_queue.enqueue(rootNode);
    doProcessLinks();                                                /* Visit the children of the requested node */
}

function doProcessLinks(response){
    var parentToProcess = (parent_queue.count > 0) ? parent_queue.dequeue() : null;
    if (parentToProcess != null) {
        if (!visitedNodes.contains(parentToProcess.data.targetUri)) {
            putChildrenDeferred = putChildrenInQueue(parentToProcess);  /* Process children via deferred method  */
            putChildrenDeferred.addCallback(doProcessLinks);
            putChildrenDeferred.addErrback(function(e){
                alert("Error in putChildrenDeferred" + e);
                return e;
            });
        }
        else
            doProcessLinks();                     /* current node already visited. Do not request children again */
    }
    else {
        parent_queue.clear();                                               /* clear parent queue                */
        parent_queue = children_queue.clone();                              /* children queue becomes the parent */

        previousParent = null;
        children_queue.clear();                                             /* clear the children queue          */
        if (parent_queue.count > 0)
            doProcessLinks();
        else
            postTraversal();                                                /* no more parents to process. Done  */
        
    }
    return response;
}

function putChildrenInQueue(requestor_parent){
    // MBA
    // var deferred = com.ibm.rdm.web.links.TraceabilityClient.fetchOutgoingLinks(requestor_parent.data.targetUri);
    var deferred = com.ibm.rdm.web.links.LinkCache.fetchTracebilityLinks(requestor_parent.data.targetUri);
    var thisParentChildren = new dojox.collections.Queue();
    deferred.addCallback(function(item){
        /*   item contains an Array object of the type {targetUri:String, iconUri:String, linkLabel:String}      */

        for (var i = 0; i < item.length; i++) {
            var id = item[i].targetUri.substring(item[i].targetUri.lastIndexOf("/") + 1);
            var treeNode = new com.ibm.rdm.web.links.Tree.TreeNode(id);
            treeNode.data = item[i];
            treeNode.parent = requestor_parent;

            if (prevChild != null) {
                treeNode.leftNeighbor = prevChild;
                prevChild.rightNeighbor = treeNode;
            }
            prevChild = treeNode;
            children_queue.enqueue(treeNode);
            thisParentChildren.enqueue(treeNode); /* Add the created Tree node as a child of the requestor parent */
        }
        if (item.length == 0){        
           var previousChildren = (children_queue.count > 0) ? children_queue.toArray() : null;
           if(previousChildren != null){
             prevChild = previousChildren[previousChildren.length - 1]; 
           }
        }

        requestor_parent.children = thisParentChildren.toArray();

        if (previousParent != null && requestor_parent.children.length > 0  && previousParent.children.length > 0){
           requestor_parent.children[0].leftNeighbor = previousParent.children[previousParent.children.length -1];
           previousParent.children[previousParent.children.length - 1].rightNeighbor = requestor_parent.children[0];
        }
        previousParent = requestor_parent;
        thisParentChildren.clear();
        visitedNodes.add(requestor_parent.data.targetUri); /* Mark node as visited */
        return item;
    });
    deferred.addErrback(function(e){
      alert("An error occured: " + e);
      return e;
    });
    return deferred;
}
/* ----------------------------------------------------------------
 * The postTraversal() method is called at the very end of        /
 * doProcessLinks(). At this point a Tree structure for the       /
 * requested links has been completed.                            /
 *---------------------------------------------------------------*/
function postTraversal(){
      var Tree =  new com.ibm.rdm.web.links.Tree(rootNode);
      var WalkerLayout = new com.ibm.rdm.web.links.TreeLayout.WalkerLayout();
      Tree.setLayout(WalkerLayout);
      WalkerLayout._treePosition();
      Tree.render(container, WalkerLayout.XMax + PADDING_SIZE, WalkerLayout.YMax + PADDING_SIZE);
      callback.call(callbackScope, {w:WalkerLayout.XMax + PADDING_SIZE, h:WalkerLayout.YMax + PADDING_SIZE});
}



})();
