/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.links.LinkCache");
dojo.require("com.ibm.rdm.web.links.Link"); 
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.links", "LinksWidget");

/**********************
 * GLOBAL VARIABLES   *
 *********************/
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

var MAX_IE_URL_LIMIT = 2083;            //Do we need this anymore???
var queryutils       = com.ibm.rdm.web.util.queryutils;
var MAX_CHAR         = 30;              // Maximum amount of characters to display in view 

com.ibm.rdm.web.links.LinkCache = {
		
/**********************
 * PRIVATE VARIABLES  *
 *********************/
		
	// Deferred to be returned to caller
	_fetchOutoingLinksDeferred: null,
	_fetchIncomingLinksDeferred: null,
	_fetchCALMOutoingLinksDeferred: null,
	
	startup: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", 
	        									   "LinksWidget");
	},
	
    fetchTracebilityLinks: function(resourceURI) {
		return this.fetchOutgoingLinks(resourceURI, null, true);
	},
	
/**********************
 * PUBLIC METHODS     *
 *********************/
	
	fetchOutgoingLinks: function(resourceURI, ioArgs, hideXHRErrors, revision, doc, contentType) {
		
		// summary:     Invokes the private method _fetchOutgoingLinks.
    	//        
    	// returns:     Deferred Object
    	// 
		
		if (resourceURI.lastIndexOf("#") != -1) {
			resourceURI = resourceURI.substring(0, resourceURI.lastIndexOf("#"));
		}

		var linksDeferred = new dojo.Deferred();
		
		var linkObj = new com.ibm.rdm.web.links.Link({
        	_outgoing:true,
        	artifactUri: resourceURI
        });
		
		if (linkObj.isExternalLink()) {
			linksDeferred.callback([]);
			return linksDeferred; 
		}
	
		if (this._fetchOutoingLinksDeferred && (this._fetchOutoingLinksDeferred.fired == -1)) 
			return this._fetchOutoingLinksDeferred;
		
		this._fetchOutoingLinksDeferred = linksDeferred;
	    // Start the process of getting links! 
	    this._fetchOutgoingLinks(resourceURI, contentType, doc, revision);		    
	    return this._fetchOutoingLinksDeferred;
	},
	
	fetchIncomingLinks: function(resourceURI) {
		
		// summary:     This method retrives a list of links pointing to the current document
    	//        
    	// description:     
    	// returns:     
    	// 
		
		if (resourceURI.lastIndexOf("#") != -1) {
			resourceURI = resourceURI.substring(0, resourceURI.lastIndexOf("#"));
		}

		var linksDeferred = new dojo.Deferred();
		
		var linkObj = new com.ibm.rdm.web.links.Link({
        	_outgoing:false,
        	artifactUri: resourceURI
        });
		
		if (linkObj.isExternalLink()) {
			linksDeferred.callback([]);
			return linksDeferred; 
		}
	
		if (this._fetchIncomingLinksDeferred && (this._fetchIncomingLinksDeferred.fired == -1)) 
			return this._fetchIncomingLinksDeferred;
		
		this._fetchIncomingLinksDeferred = linksDeferred;
        // Start the process of getting Incoming links! 
    	this._fetchIncomingLinks(resourceURI);
	    return this._fetchIncomingLinksDeferred;
    },

    fetchOutgoingCALMLinks: function(resourceURI, contentType){
		
    	// summary:     This method returns a deferred with a list of link objects of 
    	//              type "ImplementedBy". 
    	//           
    	// returns:     Deferred Object 
    	
		if (resourceURI.lastIndexOf("#") != -1) {
			resourceURI = resourceURI.substring(0, resourceURI.lastIndexOf("#"));
		}

		var linksDeferred = new dojo.Deferred();
		
		var linkObj = new com.ibm.rdm.web.links.Link({
        	_outgoing:true,
        	artifactUri: resourceURI
        });
		
		if (linkObj.isExternalLink()) {
			linksDeferred.callback([]);
			return linksDeferred; 
		}
	
		if (this._fetchCALMOutoingLinksDeferred && (this._fetchCALMOutoingLinksDeferred.fired == -1)) 
			return this._fetchCALMOutoingLinksDeferred;
		
		this._fetchCALMOutoingLinksDeferred = linksDeferred;
        // Start the process of getting CALM links! 
    	this._fetchOutgoingCALMLinks(resourceURI, contentType);
	    return this._fetchCALMOutoingLinksDeferred;
	},
    
    
/**********************
 * PRIVATE METHODS    *
 *********************/
	
    _fetchOutgoingLinks: function(resourceURI, contentType, doc, etag) {

		// summary:     This method starts the process of getting outgoing links for the current document 
   	    //              in the ResourceViewer.      
        //
    	// description: This method checks the type of document to be processed. Glossaries are a special
    	//              case since they contain terms which also need to be searched for links. The method
    	//              _fetchOutgoingLinksFromDocument retrieves links from any other artifact type.
    	// returns:     Void   
    	//
    	
	    if(contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY)
			this._fetchOutgoingLinksFromGlossary(resourceURI, doc);
	    else
	    	this._fetchOutgoingLinksFromDocument(resourceURI, contentType, doc, etag);
    },
    
    _fetchOutgoingLinksFromDocument: function(resourceURI, contentType, doc, etag){
    	
    	// summary:     This method retrieves description about the artifacts the current document is linking to.
    	//        
    	// description: This method creates linkNode type objects for each link found in the document. Finally,
    	//              _getOutgoingLinksDescriptions is invoked to get resource descriptions for the artifacts
    	//              this document is linking to.
    	// returns:     Void.
    	// 
    	
    	//Check if this resource is a wrapper resource
    	if (com.ibm.rdm.web.common.MimeTypeRegistry.isWrapperResource(contentType))   	
    		this._handleGetLinksForWrapperResource(resourceURI, doc, contentType);
    	else{    
    		var linkNodes = com.ibm.rdm.web.links.linkutils.getLinkNodesFromDocument(doc, contentType); 
    		//No links were found. Invoke callback.
    		if (linkNodes.length == 0) {
    			this._invokeCallback([]);
    			return;
    		} 	
    		//Get the name of the document in the viewer
    		var title = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(doc,contentType);
    		//Create link nodes. 
    		var outgoingLinkObjects= this._createOutgoingLinkNodeObjects(linkNodes,resourceURI,null,title, false);
    		this._getOutgoingLinksDescriptions(outgoingLinkObjects,resourceURI);
    	}
    },
    _handleGetLinksForWrapperResource: function(resourceURI, doc, contentType){
    	
       var originalResourceURI = resourceURI; 
  	   var wrapperResourceURI = originalResourceURI.replace(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES,com.ibm.rdm.web.FRONTING_SERVICE_WRAPPER_RESOURCES);
    	var deferred= RDMClient.doGet({uri: wrapperResourceURI});   	
    	deferred.addCallback(this, function(response){   	  
  		   var linkNodes = com.ibm.rdm.web.links.linkutils.getLinkNodesFromDocument(doc, contentType); 		   
  		   if (linkNodes.length == 0) {
  			   this._invokeCallback([]);
  			   return;
  		   } 	
  		   //Get the name of the document in the viewer
  		   var title = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(doc,contentType);
  		   //Create link nodes. 
  		   var outgoingLinkObjects= this._createOutgoingLinkNodeObjects(linkNodes,resourceURI,null,title, true);
  		   this._getOutgoingLinksDescriptions(outgoingLinkObjects,resourceURI);
    	});
    	deferred.addErrback(function(response) {
    		this._invokeCallback([]);
    		com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
    				this._messages.errorFetchingLinksTryLater, 
    				this._messages.errorXQueryOutgoingLinks	+ "\n" + response
    		);
    		return response;
    	});  		
    },
	_fetchOutgoingLinksFromGlossary: function(/* Fully qualified URL */resourceURI, /* Artifact document */doc) {
     	
      	// summary:     Get all outgoing links for a glossary.
    	//        
    	// description: This method starts the process of getting outgoing links for the current artifact 
    	//              which is of type Glossary. There is a special case for handling Glossaries:
    	//              (1) Get any links found in the Glossary document, 
        //              (2) Run a query to get all the terms associated with the Glossary. 
    	//              (3) Parse each term found in (2) for links 
    	//              (4) Get resource descriptions for all those artifacts this document is linking to
    	// returns:     Void.
    	//  
        
    	// Query to retrieve links from a Glossary artifact
		var glossaryLinkQuery = "links:hasLinkType=<" + com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES + 
		    	                "/GlossaryTerms> and links:linkTarget=<" + resourceURI + ">";	
		var glossaryLinkProperties = "links:linkSource&namespaces=links=http://www.ibm.com/xmlns/rrm/1.0/"; 
		
		var termsInGlossary = [];
		var termResourceURIList = [];
		// Total outoing links to artifacts
		var totalOutgoingLinks = [];
		// Get link elements in the Glossary document
		var glossaryDoc = doc;
		//Get the name of the document in the viewer
		var glossaryTitle = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(glossaryDoc, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY);
		// Get the title of this Glossary, this will be used to indicate the source of the links.
		//var glossaryTitle = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(glossaryDoc, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY);
	    var glossaryLinkNodes = com.ibm.rdm.web.links.linkutils.getLinkNodesFromDocument(glossaryDoc, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY); 
	    var glossaryOutgoingLinkObjects = this._createOutgoingLinkNodeObjects(glossaryLinkNodes,resourceURI,null,glossaryTitle, false);
	    //Total up the number of outgoing links for this resource
	    totalOutgoingLinks = totalOutgoingLinks.concat(glossaryOutgoingLinkObjects);

	 	var extractTermsFromGlossary= function(termResourceURIList){

	 		// Get the term entries links from the result of the query. Then send get the document for every
	 		// term in the Glossary to be parsed for links 
	 		
	 		if (termResourceURIList.length == 0){
	 			//No terms found in this glossary document. Process any links and invoke the callback.
	 			this._getOutgoingLinksDescriptions(totalOutgoingLinks,resourceURI);
	 		}
	 		
	 		//Create an array of deferreds to get the document of each term
	 		var termDocDeferreds=[];
	 		for(var i=0; i<termResourceURIList.length; i++) {
	 			termDocDeferreds.push(RDMClient.doGet({uri: termResourceURIList[i]}));	    
	 		}
	 		//Add the deferreds for term documents to a DeferredList	 		
	 		var termDocDeferredList = new dojo.DeferredList(termDocDeferreds);

	 		termDocDeferredList.addCallback(this, function(response){	
 				// DeferredList returns and array of response
	 			for(var i=0; i < response.length; i++){
	 				var termDocument= response[i][1].doc;
	 				var termTitle = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(termDocument, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM);
	 			    // Get link elements in the current term document
	 				var termLinkNodes = com.ibm.rdm.web.links.linkutils.getLinkNodesFromDocument(termDocument,  com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM);  
	 				// Create link objects for each link in term document
 					var termOutgoingURIs = this._createOutgoingLinkNodeObjects(termLinkNodes,resourceURI, termResourceURIList[i],termTitle, false);					
 				    // Add results to the total of outoing links to artifacts
 					if(termOutgoingURIs != null)
 						totalOutgoingLinks = totalOutgoingLinks.concat(termOutgoingURIs);						
	 			}
	 			// We are done. Get extra resource details for those artifacts linked to and invoke callback for deferred.						
	 			this._getOutgoingLinksDescriptions(totalOutgoingLinks,resourceURI);
 			});
	 		
	 		termDocDeferredList.addErrback(function(response) {
 				this._invokeCallback([]);
 				com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
 						this._messages.errorFetchingLinksTryLater, 
 						this._messages.errorXQueryOutgoingLinks	+ "\n" + response
 				);
 				return response;
 			});  		
	 	};
		
		var deferred = ProjectCache.executeQuery({
			query: glossaryLinkQuery, 
			properties: glossaryLinkProperties,
			skipCache: false,
			parser: function(response) {
				var termLinks = [];
				var entries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "entry");
				for(var i=0; i<entries.length; i++){
					termLinks.push(com.ibm.rdm.web.util.domutils.getEntryBindingValueByName(entries[i], "linkSource"));
				}
				return termLinks;
			}
		});
    	deferred.addCallback(this, extractTermsFromGlossary);	
	},
	
	_createOutgoingLinkNodeObjects: function( /*DOM Object*/ linkNodes, 
			                                  /*String    */ defaultLinkParentURI,
			                                  /*String    */ preferredLinkParentURI,
			                                  /*String    */ sourceDisplayName,
			                                  /*boolean   */ isSourceWrapperResource){
				
		// summary:     This method creates link objects for a give array of URIs. The link objects are 
		//              saved in the global associative array _mapResourceURIToLinks[{key}].outgoingLinks. 
    	//        
    	// description: This method determines if a link points to an artifact based on the the URI. If the URI is 
		//              absolut, it will consider it to be a link to an artifact. The origin of the link can be the parent
		//              document of the link Node or as Glossary artifact. If the origin of the link is a Glossary, then 
    	// returns:     List of outgoing URIs to artifacts.
    	//  
			
		var linkNodeObjects =[];
    	// Define the parent URI for this link. If this is a link from a term, we adjust the parent URI to point to the term 
		// and not its Glossary host
    	var linkOriginURI = (!preferredLinkParentURI)?defaultLinkParentURI:preferredLinkParentURI;
    	// Create link objects for each URI in URIArray      	
    	for (var i = 0; i < linkNodes.length; i++) {
    		var linkObj = new com.ibm.rdm.web.links.Link({
    			_outgoing:true,
    			artifactUri:linkOriginURI //This is the URI of the parent artifact for this link.
    		});
    		linkObj.init(linkNodes[i]);
    		
    		// If the current link is external (i.e: https://www.ibm.com) and it is not a placeholder type link (i.e: link
    		// withouth a target URI. Then we need to fill some of its fields.
    		if (linkObj.isExternalLink && (linkObj.contentType != com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER)){
    			// Current link does not point to an artifact or placeholder. Set its display and target 	 			
    			linkObj.name = linkObj.uri;
    			if(!linkObj.displayName)
    				linkObj.displayName=linkObj.uri; 
    			linkObj.targetDisplayName =(linkObj.displayName.length > MAX_CHAR) ? dojo.string.substitute(this._messages.more,[linkObj.displayName.substring(0, MAX_CHAR)]): linkObj.displayName;
            }
    		    		
    		//If the source of this link is the current document, set the sourceName 
        	if(linkObj.sourceId == com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID || isSourceWrapperResource){
        		linkObj.sourceId = com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID;
        		linkObj.sourceDisplayName = sourceDisplayName;
        	}
        		
        	
    		linkNodeObjects.push(linkObj);
    	}
    	
    	
    	

    	return linkNodeObjects;
	},
	
	_getOutgoingLinksDescriptions: function( /* Link[]*/            linkObjs, 
			                                 /* resource URI key */ resourceURI){
		
		// summary:     This method gets a description for an artifact and appends this information to an existing
		//              link object. 
    	//        
    	// description: This method expects an array of link objects as an argument. The array is passed to 
		//              getBatchResourceDescriptions and the description results are added to each link object. 
    	// returns:     Modified list of outgoing link artifact.
    	//  
		
		// List of links to artifacts
		var listLinksToArtifacts = [];
		var listArtifactURIs = []
		var totalOutgoingLinks = [];

		//Filter links objects to get an array of internal links only. 
		for(var i=0; i < linkObjs.length; i++){
			if ( !linkObjs[i].isExternalLink && (linkObjs[i].contentType != com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER) ){
				listLinksToArtifacts.push(linkObjs[i]);
			    listArtifactURIs.push(linkObjs[i].uri);
			}else
				totalOutgoingLinks.push(linkObjs[i]);	
		}
				
		// If there are no links to artifacts that need to be processed, we're done. Invoke call back with whatever links
		// links to external sources and/or placeholder links we have.
		if (listLinksToArtifacts.length == 0) {
			this._invokeCallback(totalOutgoingLinks);
			return;
		} 	
		
		var getBatchResourceDescriptionsComplete = function(linksArray) {

			for(var i=0; i < listLinksToArtifacts.length; i++){
				var currentResourceURI = listLinksToArtifacts[i].uri;  

				if(linksArray[currentResourceURI]){
					var name = linksArray[currentResourceURI].name;
					var displayName = name;
					var targetDisplayName =(displayName.length > MAX_CHAR) ? dojo.string.substitute(this._messages.more,[displayName.substring(0, MAX_CHAR)]):displayName;
					var lastModifiedDate = linksArray[currentResourceURI].lastModifiedDate;
					var lastModifiedBy = linksArray[currentResourceURI].lastModifiedBy;
					var contentType = linksArray[currentResourceURI].contentType;
					var iconUri = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(contentType);
					var etag = linksArray[currentResourceURI].etag;				
					listLinksToArtifacts[i].resourceURI       = resourceURI;
					listLinksToArtifacts[i].name              = name;
					listLinksToArtifacts[i].displayName       = displayName;
					listLinksToArtifacts[i].targetDisplayName = targetDisplayName;
					listLinksToArtifacts[i].lastModifiedDate  = lastModifiedDate;
					listLinksToArtifacts[i].lastModifiedBy    = lastModifiedBy;
					listLinksToArtifacts[i].contentType       = contentType;
					listLinksToArtifacts[i].iconUri           = iconUri;
					listLinksToArtifacts[i].etag              = etag; 	
				}else{
					//No response from the server about this resource. Mark the link to this resource as invalid.
					listLinksToArtifacts[i].isLinkToValidArtifact = false;
				}

			}    				
			// We are done. Send back the total array of links to the deferred 
			totalOutgoingLinks = totalOutgoingLinks.concat(listLinksToArtifacts);	
			this._invokeCallback(totalOutgoingLinks);
		};
 
		/* Get resource description for each artifact pointed to in document */	
		var deferred = ProjectCache.getBatchResourceDescriptionsMultiFetch(listArtifactURIs);
		deferred.addCallback(this, getBatchResourceDescriptionsComplete);
		deferred.addErrback(function(response) {
			this._invokeCallback([]);
			com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
					this._messages.errorFetchingLinksTryLater, 
					this._messages.errorXQueryOutgoingLinks	+ "\n" + response
			);
			return response;
		});   	  	    	
	},
	
	_fetchIncomingLinks: function(resourceURI){
		// summary:     This method starts the process of getting incoming links for the current document 
   	    //              in the ResourceViewer.      
        //
    	// description: This method gets incoming links.
    	// returns:     Void   
    	//
		
		var incomingLinksQuery= "link=<"+ resourceURI + ">";
		var incomingLinksNamespaces = queryutils.H_QUALIFIER + "=" + queryutils.H_NAMESPACE + "#";
		var incomingLinksProperties ="link" + "&namespaces=" + incomingLinksNamespaces;
		var incomingResourceURIList=[];
		var linkNodeObjects =[];
		
		var getBatchResourceDescriptionsComplete = function(linksArray) {

			var linksResult =[];
			var resourceCounter=0;
			for(i in linksArray){
	
				var currentResourceURI = linkNodeObjects[resourceCounter].uri;  
				
				if(linksArray[currentResourceURI]){
					var name = linksArray[currentResourceURI].name;
					var displayName = name;
					var targetDisplayName =(displayName.length > MAX_CHAR) ? dojo.string.substitute(this._messages.more,[displayName.substring(0, MAX_CHAR)]):displayName;
					var lastModifiedDate = linksArray[currentResourceURI].lastModifiedDate;
					var lastModifiedBy = linksArray[currentResourceURI].lastModifiedBy;
					var contentType = linksArray[currentResourceURI].contentType;
					var iconUri = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(contentType);
					var etag = linksArray[currentResourceURI].etag;		
					
					linkNodeObjects[resourceCounter].resourceURI       = resourceURI;
					linkNodeObjects[resourceCounter].name              = name;
					linkNodeObjects[resourceCounter].displayName       = displayName;
					linkNodeObjects[resourceCounter].targetDisplayName = targetDisplayName;
					linkNodeObjects[resourceCounter].lastModifiedDate  = lastModifiedDate;
					linkNodeObjects[resourceCounter].lastModifiedBy    = lastModifiedBy;
					linkNodeObjects[resourceCounter].contentType       = contentType;
					linkNodeObjects[resourceCounter].iconUri           = iconUri;
					linkNodeObjects[resourceCounter].etag              = etag; 						
					linksResult.push(linkNodeObjects[resourceCounter]);
					
					
				}
				resourceCounter++;
			}
			// We are done. Send back the array of links to the deferred 	
			this._invokeIncomingLinkCallback(linksResult);
		};

		
		var incomingLinkQueryResponse= function(incomingResourceURIList){
	 		for(var i=0; i < incomingResourceURIList.length; i++){	 			
	 			var linkObj = new com.ibm.rdm.web.links.Link({
	    			_outgoing:false,
	    			artifactUri:resourceURI //This is the URI of the parent artifact for this link.
	    		});	 	
	 			linkObj.uri = incomingResourceURIList[i];
	 			linkNodeObjects.push(linkObj);
	 		}
	 		
	 		if(incomingResourceURIList.length == 0)   //No incoming links found send empty string
	 			return this._invokeIncomingLinkCallback([]);
	 		var deferred = ProjectCache.getBatchResourceDescriptionsMultiFetch(incomingResourceURIList);
			deferred.addCallback(this, getBatchResourceDescriptionsComplete);
			deferred.addErrback(function(response) {
				this._invokeIncomingLinkCallback([]);
				com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
						this._messages.errorFetchingLinksTryLater, 
						this._messages.errorXQueryOutgoingLinks	+ "\n" + response
				);
				return response;
			});   	  	    	
	 	
	 		
		}
		var deferred = ProjectCache.executeQuery({
			query: incomingLinksQuery, 
			properties: incomingLinksProperties,
			skipCache: false,
			parser: function(response) {
				var incomingResourceURIList = [];
				var entries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "entry");
		 		for(var i=0; i < entries.length; i++){	 			
		 			incomingResourceURIList.push(com.ibm.rdm.web.util.domutils.getEntryBindingValueByName(entries[i], "resource"));
		 		}
				return incomingResourceURIList;
			}
		});
    	deferred.addCallback(this, incomingLinkQueryResponse);	
		
	  
			
	},
	
	_fetchOutgoingCALMLinks: function(resourceURI, contentType){
		
		// summary:     This method starts the process of getting CALM links for the current document 
   	    //              in the ResourceViewer.      
        //
    	// description: This method gets CALM links by using the links service. It then gets details 
		//              about each link resource by requesting compact rendering. 
    	// returns:     Void   
    	//
		var calmLinkType = com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES + "/ImplementedBy"
		
		var validatedByLinkType = com.ibm.rdm.web.FRONTING_SERVICE_LINK_TYPES + "/ValidatedBy"
		
		onGetCALMLinksComplete = function(calmLinks){
						
			var linkNodeObjects =[];
			//Create link objects for ImplementedBy and ValidatedBy CALM links and return them.
			
			var implementedByLinks = calmLinks["ImplementedBy"];
			var validatedByLinks = calmLinks["ValidatedBy"];
			var totalImplementedByLinks =[];
			var totalValidatedByLinks=[];
			var modifyLabel = function(originalLabel){   
				if(!originalLabel)
					return "...";
				var hasIpInLabel = /^\[[0-9]|[A-Z|a-z]+:[0-9]+\]/; //RQM send validated by labels beginning with an IP address. Remove this from the label.
				originalLabel =  dojo.trim(originalLabel);			
				if (hasIpInLabel.test(originalLabel))
				    return originalLabel.substring(originalLabel.indexOf("]")+ 1); //Trim the IP from the label
			    else
			    	return originalLabel;
			};		
			if(implementedByLinks){
				for(var i=0; i < implementedByLinks.length; i++){
					var linkObj = new com.ibm.rdm.web.links.Link({
						_outgoing:true,
						artifactUri:resourceURI //This is the URI of the parent artifact for this link.
					});
					//Mark this link as a CALM link type
					linkObj.isCALMLink = true;
					linkObj.uri = implementedByLinks[i].linkTarget;
					linkObj.sourceId = linkObj.uri;
					linkObj.displayName = linkObj.uri;   
					var linkLabel =  implementedByLinks[i].label;
					linkObj.targetDisplayName = modifyLabel(linkLabel);
					linkObj.sourceDisplayName = linkObj.targetDisplayName;
					//Add link to list of Link Objects
					totalImplementedByLinks.push(linkObj);
				}
			}
			if(validatedByLinks){
				for(var i=0; i < validatedByLinks.length; i++){
					var linkObj = new com.ibm.rdm.web.links.Link({
						_outgoing:true,
						artifactUri:resourceURI //This is the URI of the parent artifact for this link.
					});
					//Mark this link as a CALM link type
					linkObj.isCALMLink = true;
					linkObj.uri = validatedByLinks[i].linkTarget;
					linkObj.sourceId = linkObj.uri;
					linkObj.displayName = linkObj.uri;  				
    				var linkLabel =  validatedByLinks[i].label;
					linkObj.targetDisplayName = modifyLabel(linkLabel);
					linkObj.sourceDisplayName = linkObj.targetDisplayName;
					//Add link to list of Link Objects
					totalValidatedByLinks.push(linkObj);
				}
			}
			linkNodeObjects["ImplementedBy"]=totalImplementedByLinks;
			linkNodeObjects["ValidatedBy"]=totalValidatedByLinks;
			//Return array of link Objects	
			this._invokeCallbackforCALM(linkNodeObjects);   	
		}
		
		var deferred= RDMClient.doGet({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_LINKS,  
			parser:	function(response) {
			
			var totalCalmLinks =[];
			var calmLinks =[];
			var validatedByLinks = [];
			var resourcesParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
			var parsedCalmLinks = resourcesParser.parse().RDF;

			for(var resource in parsedCalmLinks) { 
				if (resource == "xmlAttributes") {
					continue;
				}
				try {
					var resourceDescription = parsedCalmLinks[resource];
					var linkSource = resourcesParser.getProperty(resourceDescription, 'linkSource.xmlAttributes.resource');
					//Get link details for the requested artifact only
					if(linkSource == resourceURI){							
						var linkTarget = resourcesParser.getProperty(resourceDescription, 'linkTarget.xmlAttributes.resource');							
						var hasLinkType = resourcesParser.getProperty(resourceDescription, 'hasLinkType.xmlAttributes.resource');
						var etag = resourcesParser.getProperty(resourceDescription, 'etag.value');
						var label = resourcesParser.getProperty(resourceDescription, 'label.value');
												
						if(hasLinkType == calmLinkType){
							calmLinks.push({
								linkSource: linkSource,
								linkTarget: linkTarget,
								hasLinkType: hasLinkType,
								etag: etag,
								label: label
							});
							
							
						}	
						if(hasLinkType == validatedByLinkType){
							validatedByLinks.push({
								linkSource: linkSource,
								linkTarget: linkTarget,
								hasLinkType: hasLinkType,
								etag: etag,
								label: label
							});
						}
						
						totalCalmLinks["ImplementedBy"]=calmLinks;
						totalCalmLinks["ValidatedBy"]=validatedByLinks;
					}							
				}
				catch(e) {
					console.error(e);
				}
			}
			//return calmLinks;
			return totalCalmLinks
		}
		}
		);
		deferred.addCallback(this, onGetCALMLinksComplete);
		deferred.addErrback(this, function(response) {
			console.error("Error while performing a GET for the links service");
			this._invokeCallbackforCALM([]);
			com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(
					this._messages.errorFetchingLinksTryLater, 
					dojo.string.substitute(this._messages.errorGETOutgoingLinks, [resourceURI]) + "\n" + response
			);
			return response;}
		);
	},
		
    _invokeCallback: function(linksArray) {
		
		// summary:     Final method called in getting links process. It performs a callback on the
        //              deferred returned in earlier stages.  
		//
    	// returns:     Void
    	// 		
		this._fetchOutoingLinksDeferred.callback(linksArray);
    },
    
    _invokeIncomingLinkCallback:function(linksArray){
    	
    	// summary:     Final method called in getting links process. It performs a callback on the
        //              deferred returned in earlier stages.  
		//
    	// returns:     Void
    	// 
    	this._fetchIncomingLinksDeferred.callback(linksArray);
    },
    _invokeCallbackforCALM: function(linksArray) {
		
		// summary:     Final method called in getting CALM links. It performs a callback on the
        //              deferred returned in earlier stages. 
    	//
    	// returns:     Void
    	//    	
		this._fetchCALMOutoingLinksDeferred.callback(linksArray);
    }

};

})();
