/*
 * Copyright (c) 2007, 2009. IBM Corporation
 * All rights reserved.
 */
(function() { 
dojo.provide("com.ibm.rdm.web.links.LinkRowWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.links.linkutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("jazz.app.ResourceLink");

dojo.requireLocalization("com.ibm.rdm.web.links", "LinksWidget");


var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var eventTopics = com.ibm.rdm.web.commenting.EventTopics;


dojo.declare( "com.ibm.rdm.web.links.LinkRowWidget", [dijit._Widget, dijit._Templated], {

	templateString: "<tr dojoAttachPoint=\"_linkRow\"><td dojoAttachPoint=\"_img\"></td><td class=\"link-result\"><span class=\"linked-resource-result\" dojoAttachPoint=\"_linkInfo\"></span><span class=\"linked-source-result\" dojoAttachPoint=\"_sourceInfo\"></span></td></tr>",
	
	status: null,
	contentType: null,
	lastModifiedDate: null,
	lastModifiedBy: null,
	resourceViewer: null,
	viewType: null,
    	
	_linkObj: null,
	_messages: null,
	_artifactURI: null,
	hasrrmRelationship: false,
	
	SOURCE_NAME_LENGTH: 25,

	postCreate: function() {
	
	    
	this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksWidget");

     
	this.inherited("postCreate", arguments);
	this.status = com.ibm.rdm.web.links.LinkRowWidget.SHOWING;

	
	if(!this._linkObj.isCALMLink){
		this.contentType = this._linkObj.contentType;
		this.lastModifiedDate = this._linkObj.lastModifiedDate;
		this.lastModifiedBy = this._linkObj.lastModifiedBy;

		//Create the image type for the link
		var imageURI;
		var resourceLabel;
		imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(this.contentType);
		resourceLabel = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(this.contentType);
		if(imageURI) {
			var img = document.createElement("img");
			img.setAttribute("src", imageURI);
			// accessibility, provide alt and title text for image
			img.setAttribute("alt", resourceLabel);
			img.setAttribute("title", resourceLabel);
			img.setAttribute("tabIndex", "0");
			img.className = "content-type-entry-image";
		}
		
		if (this.hasrrmRelationship){
			
			var createFromLink = com.ibm.rdm.web.common.ResourceLinksHandler.getResourceLink(
					decodeURIComponent(this._linkObj.uri),
					this._linkObj.targetDisplayName,
					false
			);
			
			
			var spanCreatedLabel = document.createElement("span");
			spanCreatedLabel.style.marginRight = "2px";
			spanCreatedLabel.style.color = "#808080";
			spanCreatedLabel.appendChild(document.createTextNode(this._messages.createdFrom));
			var spanImg  = document.createElement("span");
			spanImg.style.marginRight = "2px";
			spanImg.appendChild(img);
			this._img.colSpan="3";
			this._img.appendChild(spanCreatedLabel);
			this._img.appendChild(spanImg);
			this._img.appendChild(createFromLink);
			return;
		}
		else
			this._img.appendChild(img);			
	}
	
	if (this.contentType != com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER) 
		var targetAnchor = ResourceLinksHandler.getResourceLink(decodeURIComponent(this._linkObj.uri), this._linkObj.targetDisplayName, true, true);
	if (!this._linkObj.userHasPermissionToRedirect) {
		this.connect(targetAnchor, "onclick", this._handleNoPermissionLinkClicked);
	} else if (this.contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_PLACEHOLDER) {
		//Do we want to show the Placeholder links?
		var sourceURL = this._linkObj.uri;
		var targetAnchor = document.createElement("a");
		targetAnchor.setAttribute("href", "#");
		targetAnchor.setAttribute("onclick", false);
		targetAnchor.appendChild(document.createTextNode(sourceURL));
		this._linkInfo.appendChild(targetAnchor);
		this.connect(targetAnchor, "onclick", this._handlePlaceHolderLinkClicked); 
		return;
	} else if (this.contentType == com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_EXTERNAL) {
		this.connect(targetAnchor, "onclick", this._handleExternalLinkClicked); 
	} else {
		this.connect(targetAnchor, "onclick", this._handleLinkClicked);			
	}

	if (this._linkObj.isOutgoingLink()) {
 
		var sourceName = this._linkObj.sourceDisplayName;
		if (sourceName.length > this.SOURCE_NAME_LENGTH) {
			sourceName = dojo.string.substitute(this._messages.more, [sourceName.substring(0, this.SOURCE_NAME_LENGTH)]);
		} 
		var label = dojo.string.substitute(this._messages.fromSource, [sourceName]);
		var sourceId = this._linkObj.sourceId;
		// assign something as the source id so that the link will be displayed
		// properly
		if (sourceId == null) { sourceId = "xyz"; };

		var sourceAnchor = ResourceLinksHandler.getResourceLink(sourceId, label, false, true);
		this.connect(sourceAnchor, "onclick", this._handleSourceClicked);

		if(this._linkObj.isCALMLink){ //Check if this link is of CALM type
			
			var CALMLinkAnchor = document.createElement("a");
			var CALMIconImage = document.createElement("span");
			CALMLinkAnchor.appendChild(document.createTextNode(this._linkObj.sourceDisplayName));

			var resourceLink = new jazz.app.ResourceLink({
			  uri: sourceId,
			  iconNode: CALMIconImage
			}, CALMLinkAnchor);
						
			this._linkInfo.appendChild(CALMIconImage);
			this._linkInfo.appendChild(CALMLinkAnchor);
		}
		else{
			this._linkInfo.appendChild(targetAnchor);
			if(sourceName != "<unknown>")
				this._sourceInfo.appendChild(sourceAnchor);
		}
	}
	else {//Link is of Incoming type
		var sourceURL = this._linkObj.uri;
		var sourceAnchor = ResourceLinksHandler.getResourceLink(sourceURL, this._linkObj.name, true);
		this.connect(sourceAnchor, "onclick", this._handleSourceClicked);
		this._linkInfo.appendChild(sourceAnchor);
	}
    
    },

    startup: function() {
    	this.inherited("startup", arguments);
    },
	
    destroy: function() {
    	this.inherited(arguments);
    },
    
    hide: function() {
    	this.status = com.ibm.rdm.web.links.LinkRowWidget.HIDDEN;
    	this._linkRow.style.display = "none";
    },

    unhide: function() {
    	this.status = com.ibm.rdm.web.links.LinkRowWidget.SHOWING;
    	this._linkRow.style.display = "";
    },
	
    /////////////////////////////////////////////////////////////////////////////// 
    // Method used to create a link to the artifacts that are linked
    _handleResourceContextClicked: function (event) {
    	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
    },
    
    _handleLinkClicked: function(event) {
		dojo.stopEvent(event);
		var clickedResourceURI = event.target.attributes.getNamedItem("href").value;
		var reviewId = (this.resourceViewer && this.resourceViewer.reviewId);
		var projectSnapshotURI = (this.resourceViewer && this.resourceViewer.projectSnapshotURI);
		viewerutils.openResourceUsingAppropriateViewer(
			clickedResourceURI, reviewId, projectSnapshotURI);
    },
     
    _handleSourceClicked: function(event, resourceURI) {
		dojo.stopEvent(event);
	    var source = event.currentTarget;
	    var locationId = source.attributes.getNamedItem("href").value;
	    
		if (locationId == com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID) {
			locationId = com.ibm.rdm.web.links.linkutils.ARTIFACT_LOCATION_ID;
		}
    	var topic = eventTopics.getTopic(eventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, this._artifactURI);
    	dojo.publish(topic, [locationId, this._artifactURI]);
    },

    /////////////////////////////////////////////////////////////////////////////// 
    // Method specific for a placeholder link that goes no where
    // For now it just displays an alert with a message    
    _handlePlaceHolderLinkClicked: function( event ) {
        event.preventDefault();
        event.stopPropagation();
        alert(this._messages.placeholderLinkFromRRCC);   
    },
    
    _handleNoPermissionLinkClicked: function( event ) {
        event.preventDefault();
        event.stopPropagation();
        alert(dojo.string.substitute(this._messages.artifactDoesNotExist, [this._linkObj.uri]));    
    },    

    /////////////////////////////////////////////////////////////////////////////// 
    // Method used to create a link to the resources outside application and to
    // the internet
    _handleExternalLinkClicked: function( event ) {
        event.preventDefault();
        event.stopPropagation();
        window.open( (dojo.isIE ? event.srcElement.href : event.target.href) );
    }
	
});

com.ibm.rdm.web.links.LinkRowWidget.HIDDEN = "hidden";
com.ibm.rdm.web.links.LinkRowWidget.SHOWING = "showing";

})();
