/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.links.linkutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.MimeTypes");

dojo.requireLocalization("com.ibm.rdm.web.links", "LinksWidget");

var domutils = com.ibm.rdm.web.util.domutils;

com.ibm.rdm.web.links.linkutils = {

    ///////////////////////////////////////////////////////////////////////////////////
    // Method that returns link nodes from the document given its content type
	getLinkNodesFromDocument: function( doc, contentType ) {
		var linkTagInfo = com.ibm.rdm.web.links.linkutils.linkTagInfo[ contentType ];
		var linkNodes = []; 
		if( linkTagInfo ) {
			for( var i=0; i < linkTagInfo.tag.length; i++ ) {
				dojo.forEach(
					domutils.getElementsByTagNameNS(
							doc, linkTagInfo.tag[i], 
							linkTagInfo.namespace[i], linkTagInfo.qualifier[i]
					), 
					function( link ) {
						if (linkTagInfo.tag[i] == "frame") {
							if (link.getAttribute("master")) { 
								linkNodes.push(link);
							}
						}else if(linkTagInfo.tag[i] == "Diagram"){
							if (link.getAttribute("master")) { 
								linkNodes.push(link);
							}
						}else if(linkTagInfo.tag[i] == com.ibm.rdm.web.util.queryutils.UIACTIVITY_QUALIFIER)	{
							var activityPath = link.getAttribute("path");
							if(activityPath){
								link.setAttribute("href", activityPath);	
								linkNodes.push(link);
							}							
						}
						else 
						{
							linkNodes.push(link);	
						}
					}
				);
			}
		}
		
		// See it there are any <rrm:hasLink> in the document. Convert these 
		// to <base:link> types for example: <base:link href="rrcX" title="Doc Title"/>
		
		var hasLinkNamespace =  com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":hasLink";
		var linkTargetNamespace =  com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":linkTarget";
		var rrmRelationship =  com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":relationship";
		var rdfsLabel = com.ibm.rdm.web.util.queryutils.RDFS_QUALIFIER + ":label";
		var rrmLink =  com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":link";
		var rrmTitle = com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":title";
		var rrmHref = com.ibm.rdm.web.util.queryutils.RRM_QUALIFIER + ":href";
		var rrmLinks = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(doc,hasLinkNamespace); 

		for (var i=0; i < rrmLinks.length; i++){
			var linkNode = rrmLinks[i];
			var linkTarget = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode,linkTargetNamespace);
			var resourceURI = com.ibm.rdm.web.util.domutils.getAttributeNS(linkTarget[0], "resource", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER);
			var tempLabel= com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode, rdfsLabel);	
			var relationshipNode = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode, rrmRelationship); 
			var targetLabel = (tempLabel && tempLabel.length > 0)?tempLabel[0].firstChild.nodeValue:null;			
			var tempTitle = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode,rrmTitle);			
			var tempHref  = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(linkNode,rrmHref);
			var linkTitle = (tempTitle && tempTitle.length > 0)?tempTitle[0].firstChild.nodeValue:null;	
			var linkHref = (tempHref && tempHref.length > 0)?tempHref[0].firstChild.nodeValue:null;	
			
			if(resourceURI){			
				if(targetLabel){
					var baseLinkNode = document.createElement("rrm:Link");
					baseLinkNode.setAttribute("href",resourceURI );
					baseLinkNode.setAttribute("title", targetLabel);
					if (relationshipNode.length > 0)
						baseLinkNode.setAttribute("rrm:relationship", com.ibm.rdm.web.util.queryutils.getTextNodeValue(relationshipNode[0].firstChild));
					linkNodes.push(baseLinkNode);	
				}else if(linkTarget){
					var baseLinkNode = document.createElement("rrm:Link");
					baseLinkNode.setAttribute("href",resourceURI );
					baseLinkNode.setAttribute("title", linkTitle);
					linkNodes.push(baseLinkNode);	
				}
			}else if(linkTitle && linkHref){
				var baseLinkNode = document.createElement("rrm:Link");
				baseLinkNode.setAttribute("href",linkHref );
				baseLinkNode.setAttribute("title", linkTitle);
				linkNodes.push(baseLinkNode);	
			}
			
		}	
		
		return linkNodes;
	},

    ///////////////////////////////////////////////////////////////////////////////////
    // Method that returns an object containing whether the url is from the same application
    // and what content type it is
	getURLInfo: function( url ) {
		return { sameLocation: this.sameLocation( url ), format: com.ibm.rdm.web.common.MimeTypes.LINK_TYPE_EXTERNAL };
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Check if the url points to an artifact from a different repository
	isFromAnotherRepository: function (url) {
	    var pathToResource = com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + "/";
		if (!this.sameLocation(url) && url.search(pathToResource) == -1) {
			return true;
		}
		return false;		
	},
	
    ///////////////////////////////////////////////////////////////////////////////////
	// Inspired from Code from dojox.off.file._sameLocation  dojo 1.0.2. 
	// Purpose: Determines whether the url is from your web application or is to the internet
	// Website: http://api.dojotoolkit.org/jsdoc/dojox/HEAD/dojox.off.files._sameLocation
	sameLocation: function(url) {		
		url = new dojo._Url(url);
		baseResourcesURI = new dojo._Url(com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES);
		var isWebAppURI;
		if(!url.scheme && !url.port && !url.host){ 
			return true;
		}
		if(!url.scheme && url.host && url.port
			&& baseResourcesURI.host == url.host
			&& baseResourcesURI.port == url.port) {
			return true;
		}
		if(!url.scheme && url.host && !url.port
			&& baseResourcesURI.host == url.host
			&& baseResourcesURI.port == 80) {
			return true;
		}
		isWebAppURI = baseResourcesURI.scheme == url.scheme
							&& baseResourcesURI.host == url.host
							&& (baseResourcesURI.port == url.port || !baseResourcesURI.port && !url.port);
		return isWebAppURI;
    }
    
};

com.ibm.rdm.web.links.linkutils.linkTagInfo = [];

/*dojo.forEach(com.ibm.rdm.web.common.MimeTypes.WRAPPERS, function(mime) {
	com.ibm.rdm.web.links.linkutils.linkTagInfo[mime.contentType] = 
		{ 	tag: ["link"], 
			namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE], 
			qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER] 
		};
});*/

com.ibm.rdm.web.links.linkutils.linkTagInfo[com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_UPLOADED] =
com.ibm.rdm.web.links.linkutils.linkTagInfo[com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY] =
	{ 	tag: ["link"], 
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER] 
	};	

com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT ] =
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT ] =
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_ACTOR ] =
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE ] =
	{ 	tag: ["a", "img", "story", "richtext", "diagram", "link"], 
		namespace: [com.ibm.rdm.web.util.queryutils.H_NAMESPACE, 
					com.ibm.rdm.web.util.queryutils.H_NAMESPACE, 
					com.ibm.rdm.web.util.queryutils.RPC_NAMESPACE, 
					com.ibm.rdm.web.util.queryutils.RPC_NAMESPACE,
					com.ibm.rdm.web.util.queryutils.RPC_NAMESPACE,
					com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.H_QUALIFIER, 
					com.ibm.rdm.web.util.queryutils.H_QUALIFIER, 
					com.ibm.rdm.web.util.queryutils.RPC_QUALIFIER, 
					com.ibm.rdm.web.util.queryutils.RPC_QUALIFIER,
					com.ibm.rdm.web.util.queryutils.RPC_QUALIFIER,
					com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER] 
	};

com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD ] =
{	tag: ["link", "frame", "UserImage"],
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE, "http://jazz.net/xmlns/alm/rm/Storyboard/v0.1", com.ibm.rdm.web.util.queryutils.USERINTERFACE_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER, "story", com.ibm.rdm.web.util.queryutils.USERINTERFACE_QUALIFIER]
	};
	

com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH ] =
{	tag: ["link", "Reuse", "UserImage", "Diagram"],
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE, com.ibm.rdm.web.util.queryutils.PART_NAMESPACE, com.ibm.rdm.web.util.queryutils.USERINTERFACE_NAMESPACE, com.ibm.rdm.web.util.queryutils.SKETCH_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER, com.ibm.rdm.web.util.queryutils.PART_QUALIFIER, com.ibm.rdm.web.util.queryutils.USERINTERFACE_QUALIFIER, com.ibm.rdm.web.util.queryutils.SKETCH_QUALIFIER]
};
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART ] = 
{	tag: ["link", "Reuse", "UserImage"],
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE, com.ibm.rdm.web.util.queryutils.PART_NAMESPACE, com.ibm.rdm.web.util.queryutils.USERINTERFACE_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER, com.ibm.rdm.web.util.queryutils.PART_QUALIFIER, com.ibm.rdm.web.util.queryutils.USERINTERFACE_QUALIFIER]
};
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS ] =
com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW ] =
{	tag: ["link", "UIActivity"],
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE, "http://jazz.net/xmlns/alm/rm/ScreenFlow/v0.1"], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER, "flow"]
};

com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM ] =
	{	tag: ["link", "synonym", "relatedTerm"],
		namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE,
		            com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE,
		            com.ibm.rdm.web.util.queryutils.TERM_NAMESPACE], 
		qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER,
		            com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER,
		            com.ibm.rdm.web.util.queryutils.TERM_QUALIFIER]
	};



com.ibm.rdm.web.links.linkutils.linkTagInfo[ com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM ] =
	{	tag:["actor","usecase","link"],
		namespace: [com.ibm.rdm.web.util.queryutils.USECASECONTEXT_NAMESPACE,
					com.ibm.rdm.web.util.queryutils.USECASECONTEXT_NAMESPACE, 
					com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE],
		qualifier: [com.ibm.rdm.web.util.queryutils.USECASECONTEXT_QUALIFIER,
					com.ibm.rdm.web.util.queryutils.USECASECONTEXT_QUALIFIER, 
					com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER]
	};

com.ibm.rdm.web.links.linkutils.linkTagInfo[com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION] =
{ 	tag: ["link"], 
	namespace: [com.ibm.rdm.web.util.queryutils.BASE_NAMESPACE], 
	qualifier: [com.ibm.rdm.web.util.queryutils.BASE_QUALIFIER] 
};


com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID = "#Artifact#";
com.ibm.rdm.web.links.linkutils.IMAGE_ID = "#Image#";
com.ibm.rdm.web.links.linkutils.EMBEDDED_DIAGRAM_SOURCE_ID = "#EmbeddedDiagram#";
com.ibm.rdm.web.links.linkutils.EMBEDDED_REUSE_SOURCE_ID = "#Reuse#";
com.ibm.rdm.web.links.linkutils.ARTIFACT_SOURCE_ID = "#Artifact#";
com.ibm.rdm.web.links.linkutils.EMBEDDED_STORYBOARD_SOURCE_ID = "#EmbeddedStoryBoard#";
com.ibm.rdm.web.links.linkutils.ARTIFACT_LOCATION_ID = "";



