/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.provide("com.ibm.rdm.web.pages.ResourcesPage");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.project.ProjectDashboard");
dojo.require("com.ibm.rdm.web.project.ProjectSection");
dojo.require("com.ibm.rdm.web.project.ProjectDetailsSection");
dojo.require("com.ibm.rdm.web.search.QuickSearch");
dojo.require("com.ibm.rdm.web.search.SearchQuery");
dojo.require("com.ibm.rdm.web.search.results.RecentCommentResults");
dojo.require("com.ibm.rdm.web.search.results.RecentResourceResults");
dojo.require("com.ibm.rdm.web.sidebar.SidebarLayout");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewer.ResourceViewer");
dojo.require("com.ibm.rdm.web.common.ErrorSectionWidget");
dojo.require("dijit.layout.StackContainer");
dojo.require("com.ibm.rdm.web.links.LinkCache");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.pages", "ResourcesPage");
dojo.requireLocalization("com.ibm.rdm.web.pages", "ChangesSection");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.pages.ActionViewCache");
dojo.require("com.ibm.rdm.web.review.ReviewsViewer");
dojo.require("com.ibm.rdm.web.review.ReviewCache");


dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");


var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var _ActionRegistry = null;
var _ErrorSectionWidget = null;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ActionViewCache = com.ibm.rdm.web.pages.ActionViewCache;
var domutils = com.ibm.rdm.web.util.domutils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

var ACTION_PREFIX = "com.ibm.rdm.web.pages.";
var ACTION_NAMES = [
    "showArtifact",
    "showArtifactHistory",
    "showArtifactInReviewContext",
    "showProjectDashboard",
    "showSearch",
    "showUserDashboard"
];

dojo.declare("com.ibm.rdm.web.pages.ResourcesPage", [dijit._Widget, dijit._Templated], {
    templateString: "<div dojoAttachPoint=\"_contentNode\"></div>",
    _messages: null,
    // Keeps the last main content that was displayed 
    _lastMainContent: null,
    // If this is true, _lastMainContent was not cached, and it must be destroyed
    _shouldLastMainContentBeDestroyed: true,
    _projectSection: null,
    _globalRefreshButton: null,
    _globalRefreshLink: null,
	_footer: null,
	_contentNode: null,
    domNode: document.createElement("DIV"),
    _footerHeight: 0,
    _userDashboardWidget: null,
    _recentCommentsSidebar: null,
    _recentArtifactsSidebar: null,
    _recentRequirementsSidebar: null,
    _reviewsSidebar: null,
		
	postCreate: function() {
		this.inherited(arguments);
	
        this._initializeCaching();
        this._setFavicon();
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ResourcesPage");

        //error section
        this._ErrorSectionWidget = new com.ibm.rdm.web.common.ErrorSectionWidget({_resourcesPageWidget:this});
        this.domNode.appendChild(this._ErrorSectionWidget.domNode);


        dojo.addClass(this._contentNode, "net-jazz-ajax-SideNavigationLayout");

        // TODO: We should use a controller here probably. Check what WI does
        _ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
        for (var i=0; i<ACTION_NAMES.length; i++) {
            _ActionRegistry.registerAction(ACTION_PREFIX + ACTION_NAMES[i], this, ACTION_NAMES[i]);
        }
        //net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode().appendChild(this.domNode);
        //this._globalRefreshButton = dijit.byId("globalRefreshButton");
        //this._globalRefreshLink = dojo.byId("globalRefreshLink");
        //this.connect(this._globalRefreshButton, "onClick", this.handleGlobalRefresh);
        //this.connect(this._globalRefreshLink, "onclick", this.handleGlobalRefresh);
		// Add a footer containing the IBM logo.  Please note that we tweak
		// marginBottom setting for JAF's root node as this allows us to
		// use the existing CSS settings to "float" the footer to the bottom
		// of the page, regardless of how tall or short the page is.  We also
		// make a similar tweak for the left nav layout for the same reason.

		//var rootNode = net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
		//dojo.body().appendChild(footer.domNode);
		//rootNode.style.marginBottom = "-" + footer.domNode.clientHeight + "px";
		//this.leftNavLayout.domNode.style.paddingBottom = "0px";
		//this.leftNavLayout.domNode.style.marginBottom = "-" + footer.domNode.clientHeight + "px";
        
        this._footerHeight = dojo.coords(jazz.app._JazzApplication.prototype.ui._footer.domNode).h;
	},

	handleRefresh: function() {
		com.ibm.rdm.web.common.UpdateContext.refreshHasOccurred();
		var widget = dijit.byNode(this._lastMainContent);
        if (widget.handleRefresh) {
            widget.handleRefresh();
		}
	},
    
    _setFavicon: function() {
    	var docHead = document.getElementsByTagName("head")[0];
    	var links = docHead.getElementsByTagName("link");
    	for (var i = 0; i < links.length; i++) {
    		var link = links[i];
    		if (link.rel=="shortcut icon") {
    			docHead.removeChild(link);
    			break;
    		}
    	}
    	var link = document.createElement("link");
    	link.rel = "shortcut icon";
    	link.href = "com.ibm.rdm.web/common/images/icons/favicon.ico";
    	docHead.appendChild(link);
    },

    _initializeCaching: function() {
    	com.ibm.rdm.web.common.UpdateContext.isChangeContextAllowed();
        com.ibm.rdm.web.links.LinkCache.startup();
        com.ibm.rdm.web.review.ReviewCache.startup();
    },

    showArtifact: function(args) {
    	dojo.removeClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
        var cachedActionView = ActionViewCache.getCachedActionView("showArtifact", args.artifactURI);
        var commentURI = args.commentURI;
        /*
        if (cachedActionView && args.refresh && args.refresh != "false") {
                cachedActionView = null; 
                this._removeCachedActionView("showArtifact", projectURI, cachedActionView);
                window.location.href = "#" + ACTION_PREFIX + "showArtifact"
                                        + "&artifactURI=" + args.artifactURI;
                return;
                
        }
        */
        if (cachedActionView) {
        	com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
            var shouldLastMainContentBeDestroyed = false;
            this._setMainContent(
				cachedActionView.widget.domNode, null, true,
				shouldLastMainContentBeDestroyed);
            cachedActionView.widget.updateBrowserTitle();
            if (commentURI != undefined) {
            	cachedActionView.widget.setCommentToSelect(commentURI);
            }
			if (cachedActionView.widget.viewHasBeenRestored) {
				cachedActionView.widget.viewHasBeenRestored();
			}
//            if (cachedActionView.widget._projectURI)
//            	this._projectSection.highlightProject(cachedActionView.widget._projectURI);
        } else {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
	        var resourceViewer = new com.ibm.rdm.web.viewer.ResourceViewer({
				resourcesPage: this,
	            resourceURI: args.artifactURI,
				footerHeight: this._footerHeight,
				viewType: com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifact
	        });
	        ActionViewCache.cacheActionView("showArtifact", args.artifactURI, resourceViewer);
	        var shouldLastMainContentBeDestroyed = false;
	        this._setMainContent(
				resourceViewer.domNode, null, true, shouldLastMainContentBeDestroyed);
	        resourceViewer.startup();
            if (commentURI != undefined) {
            	resourceViewer.setCommentToSelect(commentURI);
            }
//	        resourceViewer.getProjectURI().addCallback(
//				this._projectSection, this._projectSection.highlightProject);
		}
    },
    
    showArtifactHistory: function(args) {
    	dojo.removeClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
		var parameterHashCode = (args.artifactURI);
        var cachedActionView = ActionViewCache.getCachedActionView("showArtifactHistory", parameterHashCode);
        var revision = args.revision;
        var projectSnapshotURI = args["rrc:baseline"];

		// Note: The way this is implemented, we re-use an existing view ONLY
		// if we can find one that corresponds to the same resource at the
		// same "enveloping revision level" (i.e. where the headRevision query
		// param matches).  In this case we re-use this view and inform it
		// that it a context change is needed in order to select the passed
		// revision query param.
        if (cachedActionView) {
            var shouldLastMainContentBeDestroyed = false;
            this._setMainContent(
				cachedActionView.widget.domNode, null, true,
				shouldLastMainContentBeDestroyed);
            cachedActionView.widget.updateBrowserTitle();
            cachedActionView.widget.contentAreaViewer.showRevision(revision, projectSnapshotURI);
        } else {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
	        var resourceViewer = new com.ibm.rdm.web.viewer.ResourceViewer({
				resourcesPage: this,
	            resourceURI: args.artifactURI,
				footerHeight: this._footerHeight,
				viewType: com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory,
				revision: revision,
				projectSnapshotURI: projectSnapshotURI
	        });
	        ActionViewCache.cacheActionView("showArtifactHistory", parameterHashCode, resourceViewer);
	        var shouldLastMainContentBeDestroyed = false;
	        this._setMainContent(
				resourceViewer.domNode, null, true, shouldLastMainContentBeDestroyed);
	        resourceViewer.startup();
		}
    },

    showArtifactInReviewContext: function(args) {
    	dojo.removeClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
		var artifactURI = args.artifactURI;
        var revision = args.revision;
		var reviewId = args.reviewId;
        var commentURI = args.commentURI;
		var href =
			UriHelper.getResourceInReviewContextJafLink(
				artifactURI, revision, reviewId);
		var cacheKeyCode = href.substring(href.indexOf("&") + 1);
        var cachedActionView =
			ActionViewCache.getCachedActionView(
				"showArtifactInReviewContext", cacheKeyCode);
		var shouldLastMainContentBeDestroyed = false;
        if (cachedActionView) {
        	com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
            this._setMainContent(
				cachedActionView.widget.domNode, null, true,
				shouldLastMainContentBeDestroyed);
            cachedActionView.widget.updateBrowserTitle();
            if (commentURI) {
            	cachedActionView.widget.setCommentToSelect(commentURI);
            }
            cachedActionView.widget.viewHasBeenRestored();
        } else {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
	        var resourceViewer = new com.ibm.rdm.web.viewer.ResourceViewer({
				resourcesPage: this,
	            resourceURI: artifactURI,
				footerHeight: this._footerHeight,
				viewType: com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext,
				revision: revision,
				reviewId: reviewId
	        });
	        ActionViewCache.cacheActionView(
				"showArtifactInReviewContext", cacheKeyCode, resourceViewer);
	        this._setMainContent(
				resourceViewer.domNode, null, true, shouldLastMainContentBeDestroyed);
            if (commentURI) {
            	resourceViewer.setCommentToSelect(commentURI);
            }
	        resourceViewer.startup();
		}
    },

    showSearch: function(args) {
    	dojo.removeClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
    	net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(
    			this._messages["advancedSearch"]);
		com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
        var searchQuery = new com.ibm.rdm.web.search.SearchQuery({
			footerHeight: this._footerHeight
        });
        if (args.showAdvancedSearch) {
            searchQuery.handleOpenAdvanced(args.searchText);
        } else if (args.searchText) {
            searchQuery.handlePerformSearchRequest(args.searchText);
        }
        this._setMainContent(searchQuery.domNode);
        searchQuery.startup();
//        this._projectSection.removeHighlight();
    },

    showProjectDashboard: function(args) {
		dojo.removeClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
        var projectURI = encodeURI(args.projectURI);
//        this._projectSection.highlightProject(projectURI);
        var cachedActionView = ActionViewCache.getCachedActionView("showProjectDashboard", projectURI);
        /*
        if (cachedActionView && args.refresh && args.refresh!="false") {
                cachedActionView = null; 
                this._removeCachedActionView("showProjectDashboard", args.projectURI, cachedActionView);
                window.location.href = "#action=" + ACTION_PREFIX + "showProjectDashboard"
                                        + "&projectURI=" + args.projectURI;
                return;
        }
        */
        com.ibm.rdm.web.common.ProjectCache.getProject(projectURI).addCallback(this, function(project){
	    	var projectPageString = dojo.string.substitute(this._messages["projectDashboard"], [project.name]);
	        net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(projectPageString);
        });
        if (cachedActionView) {
            var shouldLastMainContentBeDestroyed = false;
            this._setMainContent(
				cachedActionView.widget.domNode, null, true, shouldLastMainContentBeDestroyed);
        } else {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
	        var projectDashboard = new com.ibm.rdm.web.project.ProjectDashboard({
	            projectURI: projectURI,
				footerHeight: this._footerHeight,
				resourcesPage: this
	        });
	        var sidebarLayout = new com.ibm.rdm.web.sidebar.SidebarLayout({
				bottomOffset: this._footerHeight,
	            contentNode: projectDashboard.domNode
			});
	        var projectDetails = new com.ibm.rdm.web.project.ProjectDetailsSection({
	            projectURI: projectURI,
				width: sidebarLayout.getSidebarSectionWidth()
	        });
	        var recentComments = new com.ibm.rdm.web.search.results.RecentCommentResults({
	            scopingProjectURIs: [projectURI],
	            viewType: com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.RECENT_COMMENTS,
	            displayContext: com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION,
	            resourcesPage: this
	        });
	        var recentArtifacts = new com.ibm.rdm.web.search.results.RecentResourceResults({
	            scopingProjectURIs: [projectURI],
	            viewType: com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_ARTIFACTS,
	            displayContext: com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION,
	            resourcesPage: this
	        });
	        var recentRequirements = new com.ibm.rdm.web.search.results.RecentResourceResults({
	            scopingProjectURIs: [projectURI],
	            viewType: com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS,
	            displayContext: com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION,
	            resourcesPage: this
	            
	        });
	        var reviewsViewer = new com.ibm.rdm.web.review.ReviewsViewer({
	            scopingProjectURIs: [projectURI],
	            resourcesPage: this
	            // May need to add scopingProjectURIs, viewType and displayContext, to make it work the same way for dashboard viewlet
	        });
			sidebarLayout.addSidebarSection(this._messages["projectDetails"], projectDetails.domNode, true);
	        this._reviewsSidebar = sidebarLayout.addSidebarSection(this._messages["reviewsViewer"], reviewsViewer.domNode, false);
	        this._recentCommentsSidebar = sidebarLayout.addSidebarSection(this._messages["recentComments"], recentComments.domNode, false);
	        this._recentArtifactsSidebar = sidebarLayout.addSidebarSection(this._messages["recentArtifacts"], recentArtifacts.domNode, false);
	        this._recentRequirementsSidebar = sidebarLayout.addSidebarSection(this._messages["recentRequirements"], recentRequirements.domNode, false);
	        ActionViewCache.cacheActionView("showProjectDashboard", projectURI, sidebarLayout);
	        var shouldLastMainContentBeDestroyed = false;
	        this._setMainContent(sidebarLayout.domNode, null, true, shouldLastMainContentBeDestroyed);
	        sidebarLayout.startup();
		}
    },
    
    showUserDashboard: function() {
    	dojo.addClass(com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication.config.buttons[0]._widget.domNode, "selected");//HACK: 19503: Web client: dashboard button in the web client banner is not 'on/teal blue' when the user is in the User Dashboard
    	if (!this._userDashboardWidget) {
			var dashboardServiceResponseHandler = new ServiceResponseHandler(this, "onPersonalDashboardsInfoRetrieved", "onDashboardError");
			DashboardClient.getLeftNavInfo(null, dashboardServiceResponseHandler);	 
    	}
    	else {
    		this.displayPersonalDashboard();
    	}
    },
    onPersonalDashboardsInfoRetrieved: function(response) {
    	var personalDashboards = response.personalDashboards;
		if (personalDashboards && personalDashboards.length > 0) {
			this.createPersonalDashboardWidget(personalDashboards[0].itemId, personalDashboards[0].title);
			this.displayPersonalDashboard();
		}
		else {
			var dashboardServiceResponseHandler = new ServiceResponseHandler(this, "onPersonalDashboardCreated", "onDashboardError");
			DashboardClient.createDashboard(
				null, 
				DashboardConstants.SCOPE_CONTRIBUTOR, 
				response.contributorItemId, 
				"com.ibm.rdm.web.dashboard.templates.personal", 
				true, 
				dashboardServiceResponseHandler);
		}
    },
    
    onPersonalDashboardCreated: function(response) {
		if (response.data && response.data.dashboard) {
			this.createPersonalDashboardWidget(response.data.dashboard.itemId, response.data.dashboard.title);
			this.displayPersonalDashboard();
		}
		else {
			if (response.status == "ERROR" && response.msg) {
				com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(this._messages.personalDashboardError, response.msg);
			}
			else {
				this.onDashboardError(response);
			}
		}
    },
    
    createPersonalDashboardWidget: function(dashboardId, title) {
		this._userDashboardWidget = new Dashboard({
			scope: DashboardConstants.SCOPE_CONTRIBUTOR,
			scopeItem: {id: dashboardId}, 
			projectArea: null
		});

		//HACK: hide delete and share dashboard buttons because there is not public api to do so
		this._userDashboardWidget._contentWidget.metaNav.connect(
			this._userDashboardWidget._contentWidget.metaNav,
			"init",
			dojo.hitch(this, function(){
				var deleteButton = dojo.query("img[src$='com.ibm.team.dashboard.web/graphics/metanav/icons/delete.gif']", this._userDashboardWidget.domNode)[0];
				var shareButton = dojo.query("img[src$='com.ibm.team.dashboard.web/graphics/metanav/icons/share-dash.gif']", this._userDashboardWidget.domNode)[0];
				if (deleteButton && shareButton) {
					clearTimeout(this._hackDashboarTimeout);
					this._hackDashboarTimeout = null;
					deleteButton.parentNode.style.display = "none";
					shareButton.parentNode.style.display = "none";
				}
				else {
					console.error("unable to hide dashboard buttons"); //non-nls, internal error
				}
             })
		);
		
    },             
		
    displayPersonalDashboard: function() {
    	var homePageString = dojo.string.substitute(this._messages["homePageForUser"], [com.ibm.rdm.web.AUTHENTICATED_USER.userId]);
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(homePageString );
		com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
		this._setMainContent(this._userDashboardWidget.domNode, null, false);
	},
	
	onDashboardError: function(errorResponse) {
		var errorString = (errorResponse && errorResponse.message)?errorResponse.message:this._messages.personalDashboardError;
		com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorMessage(this._messages.personalDashboardError, errorString);
	},

    updateResize: function() {
		// summary:
		//    This function should be called whenever a previously orphaned DOM
		//    node is re-parented back into the active document's DOM.  In
		//    practice we do this whenever we bring a previously cached page
		//    back into view.
        if (this._lastMainContent) {
	    	var contentNodeWidget = dijit.byNode(this._lastMainContent);
	        if (contentNodeWidget && contentNodeWidget.handleWindowResize) {
				contentNodeWidget.handleWindowResize();
	        }
        }
    },

    ///////////////////////////////////////////////////////////////////////////////////
    // Helpers

    _createBannerNode: function(title) {
        var bannerNode = document.createElement("div");
        bannerNode.className = "rdm-main-content-banner";
        var titleDiv = document.createElement("div");
        titleDiv.className = "title-div";
        bannerNode.appendChild(titleDiv);
        var titleSpan = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
            "", title);
        titleDiv.appendChild(titleSpan);
        return bannerNode;
    },

    _setMainContent: function(contentNode, bannerNode, doSetOverflowToHidden, shouldLastMainContentBeDestroyed) {
        // shouldLastMainContentBeDestroyed:
        //     If the action view (contentNode) is cached so that a refresh does not happen with back/forward buttons
        //     this should be set to false, so this._lastMainContent will not destroyed.
        // Note: bannerNode and doSetOverflowToHidden are optional.
        // As for doSetOverflowToHidden ... we're making use of this to avoid
        // the appearance of a vertical scrollbar when using the our sidebar is
        // in use (until we can track down what would normally be causing this
        // scrollbar to appear ...)
        
        if (this._lastMainContent && this._shouldLastMainContentBeDestroyed) {
            var widget = (
                this._lastMainContent.nodeType == 1
                && dijit.byNode(this._lastMainContent));
            if (widget) {
                widget.destroyRecursive();
            }
        }

        var overflowValue = (doSetOverflowToHidden ? "hidden" : "");
        if (dojo.isIE) {
            document.documentElement.style.overflow = overflowValue;
        } else {
            document.body.style.overflow = overflowValue;
        }
        if (this._lastMainContent && this._lastMainContent.parentNode) {
        	this._contentNode.removeChild(this._lastMainContent);
        }
        this._contentNode.appendChild(contentNode);
		// Note: We take care here to scroll the window back up to the top if
		// it's currently been scrolled down (so that the top area of the new
		// content doesn't "go missing").
		if (dojo.isIE) {
			if (window.screenTop > 0) {
				window.scrollTo(0, 0);
			}
		} else {
			if (window.scrollY > 0) {
				window.scrollTo(0, 0);
			}
		}

        this._lastMainContent = contentNode;
        this.updateResize();
        this._shouldLastMainContentBeDestroyed = shouldLastMainContentBeDestroyed;
    }   
});
var ResourcesPage = com.ibm.rdm.web.pages.ResourcesPage;


dojo.declare("com.ibm.rdm.web.pages.ChangesSection", [dijit._Widget, dijit._Templated], {
    templateString: "<div class=\"rdm-changes-section\"></div>",
    _messages: null,
    postCreate: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ChangesSection");
        this.domNode.innerHTML = (
            // Recent Artifacts
            "<div class=\"section-entry\">"
            + "<div class=\"icon\"><img src=\"com.ibm.rdm.web/common/images/icons/dtool16/recent-artifacts.gif\"/></div>"
            + "<a class=\"link-text\" href='#action=com.ibm.rdm.web.pages.showRecentArtifacts'>"
            + this._messages["artifacts"]
            + "</a>"
            + "</div>"
            // Recent Requirements
            + "<div class=\"section-entry\">"
            + "<div class=\"icon\"><img src=\"com.ibm.rdm.web/common/images/recent-requirements.gif\"/></div>"
            + "<a class=\"link-text\" href='#action=com.ibm.rdm.web.pages.showRecentRequirements'>"
            + this._messages["requirements"]
            + "</a>"
            + "</div>"
            // Recent Comments
            + "<div class=\"section-entry\">"
            + "<div class=\"icon\"><img src=\"com.ibm.rdm.web/common/images/recent-comments.gif\"/></div>"
            + "<a class=\"link-text\" href='#action=com.ibm.rdm.web.pages.showRecentComments'>"
            + this._messages["comments"]
            + "</a>"
            + "</div>"
        );
    }
    
});

dojo.declare("com.ibm.rdm.web.pages.UserDashboardSection", [dijit._Widget, dijit._Templated], {
    templateString: "<div class=\"rdm-changes-section\"></div>",
    _messages: null,
    postCreate: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ResourcesPage");
        this.domNode.innerHTML = (
            // Recent Artifacts
            "<div class=\"section-entry\">"
            + "<div class=\"icon\"><img src=\"com.ibm.rdm.web/user/images/icons/view/dashboard-icon.png\"/></div>"
            + "<a id=\"userDashboard\" class=\"user-dashboard\" href=\"#action=com.ibm.rdm.web.pages.showUserDashboard\">" 
            + this._messages["user"]
            + "</a>"
            + "</div>"
        );
    }
});

var ChangesSection = com.ibm.rdm.web.pages.ChangesSection;
com.ibm.rdm.web.pages.ResourcesPage.DELIMETER = " > ";
com.ibm.rdm.web.pages.ResourcesPage.PRODUCT_NAME = "IBM Rational Requirements Composer";

})();
