/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.parsers.xml2json");

dojo.declare(
    "com.ibm.rdm.web.parsers.xml2json",
    null,
    {
    	_xmlDoc: null,
    	_objectsMapNS: {},
    	
    	constructor: function(xml) {
   			this._xmlDoc = xml;
   			this._objectsMapNS = {};
    	},
    	parse: function(includeNamespaces) {
    		if (includeNamespaces)
    			return this._parseWithNamespaces(this._xmlDoc);
    		else
    			return this._parse(this._xmlDoc);
    	},
    	
    	getProperty: function(root, property /*string*/) {
    		var jsonPropertyValue = "";
    		try {
    			var splitArray = property.split(".");
    			var propertyValue = root;
    			for (var i=0; i<splitArray.length; i++) {
    				if (propertyValue == null || propertyValue == undefined) {
    					return "";
    				}
    				propertyValue = propertyValue[splitArray[i]];
    			}
    			jsonPropertyValue = propertyValue;
    		}
    		catch(e) {
    			//console.warn("xml2json: (property=" + property + ") " + e);
    		}
    		return jsonPropertyValue;
    	},
    	
    	getObjectsByNS: function(tag, namespace) {
			var map = this._objectsMapNS[tag + ":" + namespace];
			if (map) {
				return map;
			}
			else {
				return new Array();
			}
    	},
    	
    	//experimental, skips namespaces
		_parse: function(xmlDoc, parsedJSON) {
			//console.info("Experimental: XML2JSON()");
			if (!parsedJSON)
				parsedJSON = {};
			var childNodes = xmlDoc.childNodes;
			var nodeIndex = 1;
			for(var i=0; i<childNodes.length; i++) {
				var node = childNodes[i];
				var nodeName = dojo.isIE?(node.baseName):(node.localName);
				var nodeValue = dojo.isIE?(node.text):(node.textContent);
				if (nodeValue)
					nodeValue = dojo.trim(nodeValue);
				var nodeAttributes = node.attributes;
				var returnJSON = null;
				if (nodeName) {
					if (parsedJSON[nodeName]) {
						nodeName += nodeIndex;
						nodeIndex++;
					}
					parsedJSON[nodeName] = {};
					returnJSON = parsedJSON[nodeName];
					if (nodeValue && node.childNodes && node.childNodes.length == 1 && node.childNodes[0].nodeName == "#text") {
						parsedJSON[nodeName].value = nodeValue;
					}
					else if (node.childNodes && node.childNodes.length > 0){
						//returnJSON = parsedJSON[nodeName];
					}				
					if (nodeAttributes && nodeAttributes.length > 0) {
						parsedJSON[nodeName].xmlAttributes = {};
						for(var j=0; j<nodeAttributes.length; j++) {
							var attr = nodeAttributes[j];
							var attrName = dojo.isIE?(attr.baseName):(attr.localName);
							var attrValue = dojo.isIE?(attr.text):(attr.textContent);
							parsedJSON[nodeName].xmlAttributes[attrName] = attrValue;
						}
	
					}
				}
				this._parse(node, returnJSON);
			}
			return parsedJSON;
	    },
	    
	    //experimental, skips namespaces
		_parseWithNamespaces: function(xmlDoc, parsedJSON) {
			//console.info("Experimental: XML2JSON()");
			if (!parsedJSON)
				parsedJSON = {};
			var childNodes = xmlDoc.childNodes;
			var nodeIndex = 1;
			for(var i=0; i<childNodes.length; i++) {
				var node = childNodes[i];
				var noNamespaceName = dojo.isIE?(node.baseName):(node.localName);
				
				var nodeValue = dojo.isIE?(node.text):(node.textContent);
				if (nodeValue)
					nodeValue = dojo.trim(nodeValue);
				var nodeAttributes = node.attributes;
				var returnJSON = null;
				if (noNamespaceName) {
					var nodeName = noNamespaceName + ((node.namespaceURI)?":" + node.namespaceURI:"");
					var _nodeName = nodeName;
					if (parsedJSON[nodeName]) {
						nodeName += nodeIndex;
						nodeIndex++;
					}
					parsedJSON[nodeName] = {};
					returnJSON = parsedJSON[nodeName];
					if (nodeValue && node.childNodes && node.childNodes.length == 1 && node.childNodes[0].nodeName == "#text") {
						parsedJSON[nodeName].value = nodeValue;
					}
					else if (node.childNodes && node.childNodes.length > 0){
						//returnJSON = parsedJSON[nodeName];
					}				
					if (nodeAttributes && nodeAttributes.length > 0) {
						parsedJSON[nodeName].xmlAttributes = {};
						for(var j=0; j<nodeAttributes.length; j++) {
							var attr = nodeAttributes[j];
							var attrName = dojo.isIE?(attr.baseName):(attr.localName);
							var attrValue = dojo.isIE?(attr.text):(attr.textContent);
							parsedJSON[nodeName].xmlAttributes[attrName] = attrValue;
						}
	
					}
					if (!this._objectsMapNS[_nodeName]) {
						this._objectsMapNS[_nodeName] = new Array();
					}
					this._objectsMapNS[_nodeName].push(parsedJSON[nodeName]);
				}
				this._parseWithNamespaces(node, returnJSON);
			}
			return parsedJSON;
	    },
	    
	    _parseWithNamespaces_EXPERIMENTAL: function(xmlDoc, parsedJSON) {
			//console.info("Experimental: XML2JSON()");
			if (!parsedJSON)
				parsedJSON = {};
			var childNodes = xmlDoc.childNodes;
			var nodeIndex = 1;
			//debugger;
			for(var i=0; i<childNodes.length; i++) {
				var node = childNodes[i];
				var _nodeName = dojo.isIE?(node.baseName):(node.localName);
				var nodeName = (dojo.isIE?(node.tagName):(node.nodeName)) + ((node.namespaceURI)?"_" + node.namespaceURI:"");
				var nodeValue = dojo.isIE?(node.text):(node.textContent);
				if (nodeValue)
					nodeValue = dojo.trim(nodeValue);
				var nodeAttributes = node.attributes;
				var returnJSON = null;
				if (_nodeName) {
					if (parsedJSON[nodeName]) {
						parsedJSON[nodeName].push({});
						returnJSON = parsedJSON[nodeName][parsedJSON[nodeName].length-1]
					}
					else {
						parsedJSON[nodeName] = new Array({});
						returnJSON = parsedJSON[nodeName][0];
					}
					//returnJSON = parsedJSON[nodeName];
					if (nodeValue && node.childNodes && node.childNodes.length == 1 && node.childNodes[0].nodeName == "#text") {
						returnJSON.value = nodeValue;
					}
					else if (node.childNodes && node.childNodes.length > 0){
						//returnJSON = parsedJSON[nodeName];
					}				
					if (nodeAttributes && nodeAttributes.length > 0) {
						returnJSON['xmlAttributes'] = {};
						for(var j=0; j<nodeAttributes.length; j++) {
							var attr = nodeAttributes[j];
							var attrName = dojo.isIE?(attr.baseName):(attr.localName);
							var attrValue = dojo.isIE?(attr.text):(attr.textContent);
							returnJSON.xmlAttributes[attrName] = attrValue;
						}
	
					}
				}
				this._parseWithNamespaces(node, returnJSON);
			}
			return parsedJSON;
	    }
    }
);

})();
