/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.ProjectDetailsSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.requireLocalization("com.ibm.rdm.web.project", "ProjectDetailsSection");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

var VERTICAL_SCROLLBAR_WIDTH = 18;
var DEFAULT_USER_CELL_WIDTH = 80;
var PHOTO_WIDTH = 40;

dojo.declare(
	"com.ibm.rdm.web.project.ProjectDetailsSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    This class is responsible for the Project Details section that appears
	//    in the sidebar for the Project Dashboard.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/ProjectDetailsSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	projectURI: null,
	width: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	createdOn: null,
	projectMembers: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	// _currentUserPropertyInfos: an array of property lists
	//    Each element of this array is a property list of the form:
	//    {
	//        id: <userId>  // E.g. "Bob" or "bob@company.com"
	//        title: <userName> // E.g. "Bob" or "Bob Gazonkas"
	//        photoURI: <photoURI>
	//        userRole: <userRole>  // Note: The stored userRole is already translated.
	//    }
	_currentUserPropertyInfos: null,
	// _currentColumnCount: an int
	//    The current number of columns that are being used to display the users in
	//    a table fashion.
	_currentColumnCount: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentPane: null,
	_projectNameSpan: null,
	_createdOnSpan: null,
	_projectMembersSpan: null,
	_snapshotsSectionCollapsed: true,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	startup: function() {
		this._currentColumnCount = null;
		
		ProjectCache.getProject(this.projectURI).addCallback(this,
				function(project){
					this._projectNameSpan.appendChild(document.createTextNode(project.name));
					return project;
				}
		);
		
		this._updateCreatedOn();
		this._updateProjectSnapshots();
		// TODO The spec has a "Linked to: " field as well, which we need to add
		// (but I don't see it in the native client).
		this._updateProjectMembers();
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "ProjectDetailsSection");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", 
				"common");
		dojo.forEach(["createdOn", "projectMembers", "latestSnapshot", "more"], function(prop){
			this[prop] = this._messages[prop];
		}, this);
	},
	
	destroy: function() {
		this.inherited("destroy", arguments);
	},

	handleRefresh: function() {
		this._updateProjectMembers();
		this._updateProjectSnapshots();
	},

    containerDimensionsHaveChanged: function() {
		var coords = dojo.coords(this.domNode.parentNode);
        var height = coords.h;
        var width = coords.w;
        var contentPaneTop = (
            this._contentPane.domNode.offsetTop
            - this._contentPane.domNode.parentNode.offsetTop);
        var scrollHeightToUse = (height - contentPaneTop);
		this._relayoutTableIfNeeded();
        this._contentPane.resize({ h: scrollHeightToUse, w: width });
    },

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_updateCreatedOn: function() {
		var self = this;
		var onGetComplete = function(response) {
			var createdDate = self._getCreatedDate(response.doc);
			var createdNode = document.createTextNode(self._composeShortDate(createdDate));
			self._createdOnSpan.appendChild(createdNode);
		};
		ErrorHandler.warn("TODO: no project information support");
		//var deferred =
		//	RDMClient.doGet(this.projectURI);
		//deferred.addCallback(onGetComplete);
	},

	_updateProjectMembers: function() {
		var self = this;
		var onProjectMembersComplete = function(userPropertyInfos) {
			self._storeUserPropertyInfos(userPropertyInfos);
			self._updateRoleInUserPropertyInfos(userPropertyInfos);
			self._relayoutTableIfNeeded(true);
	    	return userPropertyInfos;
		}
		var deferred = com.ibm.rdm.web.common.ProjectCache.getProjectUsersWithRoles(this.projectURI);
		deferred.addCallback(this, onProjectMembersComplete);
	},
	
	_updateProjectSnapshots: function() {
		this._latestSnapshotNode.innerHTML = "";
		this._latestSnapshotNode.appendChild(document.createTextNode(this._commonMessages.loading));
		this._allSnapshotsNode.innerHTML = "";
		this._snapshotsSectionCollapsed = (this._snapshotsSectionCollapsed)?true:false;
		dojo.disconnect(this._snapshotLinkHandle);
		var snapshotsDeferred = ProjectCache.getProjectSnapshots(this.projectURI);
		snapshotsDeferred.addCallback(this, function(snapshots){
			this._latestSnapshotNode.innerHTML = "";
			function snapshotDateComparator(snapshot1, snapshot2) {
	           var a = snapshot1.snapshotDate;
	           var b = snapshot2.snapshotDate;
	           return (a<b)?1:((a>b)?-1:0);
	        }
	        var projectSnapshots = [];
	        for (var snapshotURI in snapshots) {
	        	projectSnapshots.push(snapshots[snapshotURI]);
	        }
	        projectSnapshots.sort(snapshotDateComparator);
	        if (projectSnapshots.length > 0) {
	        	var snapshotDiv = document.createElement("DIV");
				snapshotDiv.appendChild(document.createTextNode(projectSnapshots[0].title));
				this._latestSnapshotNode.appendChild(snapshotDiv);
	        }
	        if (projectSnapshots.length > 1) {
				for (var i=1; i<projectSnapshots.length; i++) {
					var snapshotDiv = document.createElement("DIV");
					snapshotDiv.appendChild(document.createTextNode(projectSnapshots[i].title));
					this._allSnapshotsNode.appendChild(snapshotDiv);
				}
				dojo.addClass(this._snapshotsLinkNode, "visible");
				dojo.removeClass(this._snapshotsLinkNode, "hidden");
				
		        this._snapshotLinkHandle = dojo.connect(this._snapshotsLinkNode, "onclick", this, function(event) {
		        	if (!this._snapshotsSectionCollapsed) {
		        		dojo.addClass(this._allSnapshotsNode, "hidden");
		        		dojo.removeClass(this._allSnapshotsNode, "visible");
		        		this._snapshotsSectionCollapsed = true;
		        		this._snapshotsLinkNode.innerHTML = "";
		        		this._snapshotsLinkNode.appendChild(document.createTextNode(this._messages.more));
		        	}
		        	else {
		        		dojo.addClass(this._allSnapshotsNode, "visible");
		        		dojo.removeClass(this._allSnapshotsNode, "hidden");
		        		this._snapshotsSectionCollapsed = false;
		        		this._snapshotsLinkNode.innerHTML = "";
		        		this._snapshotsLinkNode.appendChild(document.createTextNode(this._messages.less));
		        	}
		        });
	        }
	        
		});
		snapshotsDeferred.addErrback(this, function(snapshots){
			this._latestSnapshotNode.innerHTML = "";
		});
	},

	_storeUserPropertyInfos: function(userPropertyInfos) {
		// summary:
		//    For very first call:
		//    - Sets the '_currentUserPropertyInfos' field to an initial list
		//      of user information.  This information includes the 'id' and
		//      'photoURI' for every user in the current project.  However, it
		//      does _not_ include the appropriate 'userRole' for each of these
		//      users (it will be set to null).
		//    For subsequent calls:
		//    - Subsequent calls to this same function will fill reset the
		//      '_currentUserPropertyInfos' field with appropriate values for
		//      the 'id' and 'photoURI' for every user in the current project.
		//      In addition, it will also set the 'userRole' to the last known
		//      'userRole' value for each user.  This is done with the
		//      expectation that a call will subsequently be made to
		//      '_updateRoleInUserPropertyInfos' soon after the call to this
		//      function (in order to patch the role up if it has changed).
		//      We do this to avoid needless flashing in the UI.
		// userPropertyInfos:
		//    The callback parameter passed by the ProjectCache.getProjectUsers
		//    function.
		var oldUserPropertyInfos = this._currentUserPropertyInfos;
		var getLastUserRole = function(user) {
			return user.userRoles[0].label;
		};
		this._currentUserPropertyInfos = [];
		for (var userUri in userPropertyInfos) {
			var userRoleToUse = getLastUserRole(userPropertyInfos[userUri]);
			var userPropertyInfo = {
				userId: userPropertyInfos[userUri].userId,
				photoUri:  userPropertyInfos[userUri].photoUri,
				name: userPropertyInfos[userUri].name,
				// Note: We're setting this to be the same as the last known
				// user role value for this user (as soon, a call will be made
				// to '_updateRoleInUserPropertyInfos' to fix this up if needed).
				userRole: userRoleToUse
			};
			this._currentUserPropertyInfos.push(userPropertyInfo);
			this._currentUserPropertyInfos[userPropertyInfo.id] = userPropertyInfo;
		}
	},

	_updateRoleInUserPropertyInfos: function(userRoleInfos) {
		// summary:
		//    Contribute an appropriate 'userRole' property to each user property
		//    list found in the '_currentUserPropertyInfos' field.  The
		//    "appropriate" value for each user is found in the passed
		//    'userRoleInfos'.
		// userRoleInfos:
		//    The callback parameter passed by the ProjectCache.getProjectUserRoles
		//    function.
		var self = this;
		var getMostPrivilegedRole = function(userRoles) {
			var isAdministrator = false;
			var isWriter = false;
			var isCommenter = false;
			for (var i = 0; i < userRoles.length; i++) {
				switch (userRoles[i]) {
					case "Administrator":
						isAdministrator = true;
						break;
					case "Writer":
						isWriter = true;
						break;
					case "Commenter":
						isCommenter = true;
						break;
				}
			}
			if (isAdministrator) {
				return self._messages["administrator"];
			}
			else if (isWriter) {
				return self._messages["writer"];
			}
			else if (isCommenter) {
				return self._messages["commenter"];
			}
		}
		for (var userURI in userRoleInfos) {
			var userRoleInfo = userRoleInfos[userURI];
			var userName = userRoleInfo.userId;
			var userRoles = userRoleInfo.userRoles;
			var mostPrivilegedRole = this._currentUserPropertyInfos[0].userRole; //getMostPrivilegedRole(userRoles);
			this._currentUserPropertyInfos[0].userRole = mostPrivilegedRole;
		}
	},

	_addProjectMembersTable: function(userPropertyInfos, numColumns) {
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		var self = this;
		var addPhotoRow = function(startIndex) {
			var tr = document.createElement("tr");
			for (var i = startIndex, column = 0;
				 column < numColumns && i < userPropertyInfos.length;
				 i++, column++
				) {
				userProperties = userPropertyInfos[i];
				self._addProjectMemberPhotoTD(userProperties, tr);
			}
			tbody.appendChild(tr);
		}
		var addDetailsRow = function(startIndex) {
			var tr = document.createElement("tr");
			for (var i = startIndex, column = 0;
				 column < numColumns && i < userPropertyInfos.length;
				 i++, column++
				) {
				userProperties = userPropertyInfos[i];
				self._addProjectMemberDetailsTD(userProperties, tr);
			}
			tbody.appendChild(tr);
		};
		var rowUserPropsStartIndex = 0;
		for (var i = 0; i < userPropertyInfos.length; i++) {
			if ((i % (numColumns)) === 0) {
				addPhotoRow(i);
				addDetailsRow(i);
			}
		}
		
		this._projectMembersDiv.appendChild(table);
	},

	_removeProjectMembersTable: function() {
		this._projectMembersDiv.innerHTML = "";
	},

	_calculateAppropriateColumnCount: function(clientWidth) {
		clientWidth -= VERTICAL_SCROLLBAR_WIDTH;
		var singleColumnWidth = (this._calculateAverageColunmnWidth() || DEFAULT_USER_CELL_WIDTH);
		return Math.floor(clientWidth / singleColumnWidth);
	},

	_calculateAverageColunmnWidth: function() {
		var averageColumnWidth = null;
		if (this._projectMembersDiv.firstChild) {
			var allColumnWidths = 0;
			var tbody = this._projectMembersDiv.firstChild.firstChild;
			var tr = tbody.firstChild;
			if (tr) {
				var numColumns = tr.childNodes.length;
				for (var td = tbody.firstChild.firstChild;
					 td != null;
					 td = td.nextSibling
					 ) {
					if (td.clientWidth <= PHOTO_WIDTH) {
						// It must be the case that the images haven't been
						// obtained yet, which would throw off the calculation
						// for average width.
						// In this case we'll return null -- meaning we can't
						// determine the average width.
						return null;
					}
					allColumnWidths += td.clientWidth;
				}
				averageColumnWidth = Math.floor(allColumnWidths / numColumns);
			}
		}
		return averageColumnWidth;
	},

	_relayoutTableIfNeeded: function(/* Optional */ doForceRelayout) {
		if (this._currentUserPropertyInfos && this.domNode.parentNode) {
			var clientWidth = this.domNode.parentNode.clientWidth;
			if (clientWidth) {
				var numColumnsToUse = this._calculateAppropriateColumnCount(clientWidth);
				if (doForceRelayout || (numColumnsToUse != this._currentColumnCount)) {
					this._removeProjectMembersTable();
					this._addProjectMembersTable(this._currentUserPropertyInfos, numColumnsToUse);
				}
				this._currentColumnCount = numColumnsToUse;
			}
		}
	},

	_addProjectMemberPhotoTD: function(userPropertyInfo, tr) {
		var img = document.createElement("img");
		var td = document.createElement("td");
		var photoUri = userPropertyInfo.photoUri;
		img.className = "user-image";
		td.className = "user-photo-row-table-data";
		img.setAttribute("src", photoUri);
		td.appendChild(img);
		tr.appendChild(td);
	},

	_addProjectMemberDetailsTD: function(userPropertyInfo, tr) {
		var usernameNode = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
			"user-name",
			userPropertyInfo.name);
		var projectMemberHover = new com.ibm.rdm.web.common.UserHover({projectURI : this.projectURI}, null);
        projectMemberHover.createHoverPopup(usernameNode, userPropertyInfo);
		var td = document.createElement("td");
		td.className = "user-details-row-table-data";
		td.appendChild(usernameNode);
		/* Removing role information and displaying the info in a hover. Leaving the code here just in case...
		if (userPropertyInfo.userRole) {
			td.appendChild(document.createElement("br"));
			var roleNode = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
				"user-role",
				userPropertyInfo.userRole);
			td.appendChild(roleNode);
		}
		*/
		tr.appendChild(td);
	},

	_getCreatedDate: function(doc) {
		// summary:
		//    Parse out the line that contains something like the following:
		//        <updated>2008-09-10T17:35:24-04:00</updated>
		//    and return this element's value as a Date instance.
        var nodes = doc.getElementsByTagName("updated");
        var createdString = nodes[0].firstChild.nodeValue;
        var createdDate = dojo.date.stamp.fromISOString(createdString);
        return createdDate;
	},

    _composeShortDate: function(date) {
        return dojo.date.locale.format(date, { formatLength: "short" });
    }
});



})();
