/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.ResultSetView");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.search.results.RecentCommentResults");
dojo.require("dijit.layout.StackContainer");
dojo.require("com.ibm.rdm.web.common.CollapseableTitleSection");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.server.client.RecentFeedsClient");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.requireLocalization("com.ibm.rdm.web.project", "ResultSetView");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RecentFeedsClient = com.ibm.rdm.web.server.client.RecentFeedsClient;
var COMMENT_HIGHLIGHT_EXPIRATION_HRS = 48; //Do not hightlight comments older than limit set here (in hours)

dojo.declare(
	    "com.ibm.rdm.web.project.ResultSetView",
	    [dijit._Widget, dijit._Templated],
{
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/ResultSetView.html"),
    widgetsInTemplate: true,
    
    projectURI: null,
    
    
    ///////////////////////////////////////////////////////////////////////////////////
	// Private Fields
    _messages: null,
    _viewType: null,
    _sortOrder: null,
    _groupOrder: null,
    _stackContainer: null,
    _contentPane: null,
    _contentPaneSize: null,
    _listContentPane: null,
    _groupContentPane: null,
    _arrayListOfResources: null,
    _mapContentTypesToResourceInfo: null,
    _mapUserToResourceInfo: null,
    _arrayListOfRecentArtifacts: null,
    _resultTable: null,
    _collapseableHeaders: null,
    _isRecentArtifactsHighlighted: null,
    _isRecentCommentedHighlighted: null,
    _currentGroupingList: null,
	_mapResourceURIToClickHandler: null,
	_lastSelected: null,
	_resultSetTable: null,
	_usersMap: null,
	_userRolesMap: null,
	// _isMessageDisplayed: Boolean
	//    Set to true only when the this._messages.loadingArtifactsMessage or
	//    this._messages.filterNeededMessage is being displayed.
	_isMessageDisplayed: null,
    
    postCreate: function() {
	    this._collapseableHeaders = [];
	    this._arrayListOfResources = [];
        this._listContentPane = new dijit.layout.ContentPane({}, this._listedResourceEntriesDiv);
        this._groupContentPane = new dijit.layout.ContentPane({}, this._groupedResourceEntriesDiv);
        this._stackContainer = new dijit.layout.StackContainer({}, this._stackContainerNode);
        this._contentPane = this._listContentPane; //TODO Change to what user selected last
        this._mapResourceURIToClickHandler = new Object;
		this._isMessageDisplayed = false;
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "ResultSetView");
        
        com.ibm.rdm.web.project.ResultSetView.headerTitles.NAME = this._messages.name;
        com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE = this._messages.type;
        com.ibm.rdm.web.project.ResultSetView.headerTitles.USER = this._messages.lastModifiedBy;
        com.ibm.rdm.web.project.ResultSetView.headerTitles.DATE = this._messages.lastModifiedDate;
        com.ibm.rdm.web.project.ResultSetView.headerTitles.MANAGEDREQUIREMENT = this._messages.managedRequirement;
        
        this._populateUserRoleInfo();
    },
    
    startup: function() {
        this._listContentPane.startup();
        this._groupContentPane.startup();
    	this._stackContainer.startup(); 
    },
    
    destroy: function() {
        this._listContentPane.destroyRecursive();
        this._groupContentPane.destroyRecursive();
    	this._stackContainer.destroyRecursive(); 
    	dojo.forEach(this._collapseableHeaders, function(widget){
    		widget.destroyRecursive();
    	});
    },
    
    showArtifacts: function() {
		this._isMessageDisplayed = false;
    	//TODO Change to view according to user preferences
    	this._sort();
    },

	showLoadingArtifactsMessage: function() {
		this._showMessage(this._messages.loadingArtifactsMessage);
	},

	showFilterNeededMessage: function() {
		this._showMessage(this._messages.filterNeededMessage);
	},

	showNoResultsMessage: function(isMoreThanOneFilterSpecified) {
		if (isMoreThanOneFilterSpecified) {
			this._showMessage(this._messages.noMatchesForFilters);
		} else {
			this._showMessage(this._messages.noMatchesForFilter);
		}
	},

    resize: function(args) {
        // summary:
        //    This function is here to receive resize calls that are made when
        //    this widget is used as the sole content of a ContentPane.  Please
        //    see ConentPane's 'resize' function for more information.
/*    	this.newArgs = args;
	    if(!this.resizeInProgress) {	
	    	var self = this;
	    	var resizeFn = function() {
	    	}
	        setTimeout(resizeFn, 500);
	        this.resizeInProgress = true;*/
    	if(args && args.w >= 0 && args.h >= 0) {
    		var width = args.w;
    		var height = args.h;
    		//this._stackContainer.style.height = height + "px";
    		//this._stackContainer.style.width = width + "px";
    		if (dojo.isIE) {
    			// TODO Is there a better approach for this?
    			// Without the following code, the horz scrollbar will always
    			// appear when the vert scrollbar is present, even in those
    			// [most] cases where it shouldn't ...
    			if (this._resultTable) {
    				this._resultTable.style.width =
    					Math.max(0, (args.w));
    			}
    		};
    		//height = height;
    		var contentPaneSize = {
    				h: height,
    				w: width
    		};
    		this._contentPane.resize(contentPaneSize);
    		this._contentPaneSize = args;
    	}
    },
    
    
    ///////////////////////////////////////////////////////////////////////////////////
	// Helper Methods
    highlightRecentArtifacts: function() {

    	if (this._isRecentArtifactsHighlighted) {
    		dojo.forEach(this._arrayListOfRecentArtifacts, function(row) { 
    			dojo.removeClass(row, "recently-modified-artifact-highlight");
    		});
    		this._isRecentArtifactsHighlighted = false;
    	} else {
    		dojo.forEach(this._arrayListOfRecentArtifacts, function(row) { 
    			dojo.addClass(row, "recently-modified-artifact-highlight");
    		});
    		this._isRecentArtifactsHighlighted = true;
    	}   	
    },
    
    highlightRecentCommentedArtifacts: function(){    	
    	var self = this;
    	var recentlyCommentedDeferred = self._getRecentlyCommentedArtifacts();
    	recentlyCommentedDeferred.addCallback(function(commentedItems){
    		if (self._isRecentCommentedHighlighted) {       		
    			dojo.forEach(commentedItems, function(item){
    				var trElem = document.getElementById(item);
    				if(trElem != null){
    					dojo.removeClass(trElem, "recently-commented-artifact-highlight");
    				}	
    			});
    			self._isRecentCommentedHighlighted = false;
    		}else {       		
    			dojo.forEach(commentedItems, function(item){
    				var trElem = document.getElementById(item);
    				if(trElem != null){
    					dojo.addClass(trElem, "recently-commented-artifact-highlight");
    				}     		
    			});
    			self._isRecentCommentedHighlighted = true;
    		}
    	});   	  	
    },
    
    _getRecentlyCommentedArtifacts: function(){ 
		//summary:
		//method returns a deferred with the recently commented artifacts for the current project   	
    	var self = this;
    	var now= new Date();
    	var deferred =  new dojo.Deferred();  
    	var recentCommentsDeferred = RecentFeedsClient.getRecentComments(self.projectURI, com.ibm.rdm.web.search.results.RecentCommentResults.dateComparator); 
    	var commentedArtifacts= new Object();   
    	recentCommentsDeferred.addCallback(function(comments){
    		var results=[];
    		dojo.forEach(comments, function(comment){   			
    			//Get recent comments which have not exceeded the allowed time			
    			var hoursPassed = dojo.date.difference(comment.updated, now, "hour");
    	        if(hoursPassed < COMMENT_HIGHLIGHT_EXPIRATION_HRS){
    	        	var idSuffix = "#" + self.getGroupOrder();
        			commentedArtifacts[comment.resourceUri + idSuffix]=true;
    	        }
    		});
    		for(prop in commentedArtifacts){results.push(prop)}; 
    		deferred.callback(results);
    	   return comments;
    	});   	
    	return deferred;
    },
    
    _sort: function() {
		if( this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE ) {
			this.groupByType();
		} else if( this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER ) {
			this.groupByUser();
		} else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.DATE) {
			this.groupByDate();
		} else {
			this.noGrouping();
		}
    },

    
    sortByName: function(){
    	if( this._sortOrder != com.ibm.rdm.web.project.ResultSetView.sortBy.NAME ) {
    		this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.NAME;
    		
    		this._sort();
    		this.projectToolbar._checkMenuItem();
    	}
    },
    
    sortByType: function() {
    	if( this._sortOrder != com.ibm.rdm.web.project.ResultSetView.sortBy.TYPE ) {
    		this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.TYPE;
    		
    		this._sort();
    		this.projectToolbar._checkMenuItem();
    	}
    },
    
    sortByUser: function() {
    	if( this._sortOrder != com.ibm.rdm.web.project.ResultSetView.sortBy.USER ) {
    		this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.USER;
    		
    		this._sort();
    		this.projectToolbar._checkMenuItem();
    	}
    },
    
    sortByDate: function() {
    	if( this._sortOrder != com.ibm.rdm.web.project.ResultSetView.sortBy.DATE ) {
    		this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.DATE;
    		
    		this._sort();
    		this.projectToolbar._checkMenuItem();
    	}
    },
    
    noGrouping: function() {
    	this._groupOrder = com.ibm.rdm.web.project.ResultSetView.groupBy.NONE;
    	/*
    	if (this._sortByTypeNode) {
    		this._sortByTypeNode.attr("disabled", false);
    	}
    	this._sortByUserNode.attr("disabled", false);
    	*/
   		this._showListedResources(this._arrayListOfResources);
    },
    
    groupByUser: function() {
    	this._groupOrder = com.ibm.rdm.web.project.ResultSetView.groupBy.USER;
    	this._mapUserToResourceInfo = new Object;
    	var userGrouping = [];
    	var self = this;
    	dojo.forEach( this._arrayListOfResources, function( resourceInfo ) {
    		var lastModifiedBy = com.ibm.rdm.web.util.queryutils.getLastPathSegment(resourceInfo.lastModifiedBy);
    		lastModifiedBy = decodeURIComponent(lastModifiedBy);
    		if(!self._mapUserToResourceInfo[lastModifiedBy]) {
    			self._mapUserToResourceInfo[lastModifiedBy] = new Array;
    			userGrouping.push(lastModifiedBy);
    		}
    		self._mapUserToResourceInfo[lastModifiedBy].push(resourceInfo);
    	});
    	userGrouping.sort(function(name1, name2){
    		name1 = name1.toLowerCase();
    		name2 = name2.toLowerCase();
    		if (name1 < name2) {
    			return -1;
    		} else if (name1 > name2) {
    			return 1;
    		} else {
    			return 0;
    		}
    	});
    	/*
    	if (this._sortByTypeNode) {
    		this._sortByTypeNode.attr("disabled", false);
    	}
    	this._sortByUserNode.attr("disabled", true);
    	*/
		if(this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.USER) {
			this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.NAME;
		}
    	this._currentGroupingList = userGrouping;
      	var onComplete = function(userMap) {
    		this._showGroupedResources(this._mapUserToResourceInfo, userMap);    	
     	};
        ProjectCache.getAllUsers().addCallback(this, onComplete);
    },
    
    groupByDate: function() {
    	this._groupOrder = com.ibm.rdm.web.project.ResultSetView.groupBy.DATE;
    	this._mapUserToResourceInfo = new Object;
    	var self = this;
    	dojo.forEach(this._arrayListOfResources, function(resourceInfo) {
    		var lastModifiedDate = resourceInfo.lastModifiedDate;
        	var dateRange = DateTimeHelper.getDateRange(lastModifiedDate);
    		if (!self._mapUserToResourceInfo[dateRange]) {
    			self._mapUserToResourceInfo[dateRange] = new Array;
    		}
    		self._mapUserToResourceInfo[dateRange].push(resourceInfo);	
    	});
    	/*
    	if (this._sortByTypeNode) {
    		this._sortByTypeNode.attr("disabled", false);
    	}
    	this._sortByUserNode.attr("disabled", true);
    	*/
    	this._currentGroupingList = [DateTimeHelper.DATE_RANGE_TYPE.Today,
    	                             DateTimeHelper.DATE_RANGE_TYPE.Yesterday,
    	                             DateTimeHelper.DATE_RANGE_TYPE.PastWeek,
    	                             DateTimeHelper.DATE_RANGE_TYPE.PastMonth,
    	                             DateTimeHelper.DATE_RANGE_TYPE.Earlier];
    	this._showGroupedResources(this._mapUserToResourceInfo);    	
    },
  
	getTimeRangeLabelToUse: function(range) {
		var rangeString = null;
		switch (range) {
			case DateTimeHelper.DATE_RANGE_TYPE.Today: 
				rangeString = this._messages.today;
				break;
            case DateTimeHelper.DATE_RANGE_TYPE.Yesterday:
				rangeString = this._messages.yesterday;
				break;
            case DateTimeHelper.DATE_RANGE_TYPE.PastWeek:
				rangeString = this._messages.pastWeek;
				break;
            case DateTimeHelper.DATE_RANGE_TYPE.PastMonth:
				rangeString = this._messages.pastMonth;
				break;
            case DateTimeHelper.DATE_RANGE_TYPE.Earlier:
 				rangeString = this._messages.earlier;
				break;
            default:
				rangeString = "";
        }
        return rangeString;
	},
    
    groupByType: function() {
    	this._groupOrder = com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE;
    	var self = this;
    	var typeGrouping = [];
    	this._mapContentTypesToResourceInfo = new Object;
    	dojo.forEach( this._arrayListOfResources, function( resourceInfo ) {
    		if(!self._mapContentTypesToResourceInfo[resourceInfo.contentType]) {
    			typeGrouping.push(resourceInfo.contentType);
    			self._mapContentTypesToResourceInfo[resourceInfo.contentType] = new Array;
    		}
    		self._mapContentTypesToResourceInfo[resourceInfo.contentType].push( resourceInfo );	
    	});
    	typeGrouping.sort(function(type1, type2){
    		name1 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(type1).toLowerCase();
    		name2 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(type2).toLowerCase();
    		if (name1 < name2) {
    			return -1;
    		} else if (name1 > name2) {
    			return 1;
    		} else {
    			return 0;
    		}
    	});
    	/*
    	this._sortByUserNode.attr("disabled", false);
    	this._sortByTypeNode.attr("disabled", true);
    	*/
		if(this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.TYPE) {
			this._sortOrder = com.ibm.rdm.web.project.ResultSetView.sortBy.NAME;
		}
		this._currentGroupingList = typeGrouping;
    	this._showGroupedResources(this._mapContentTypesToResourceInfo);
    },
    
    appendResults: function(results) {
    	this._arrayListOfResources = this._arrayListOfResources.concat(results);
    	//this._appendResultsToTable(results);
    },
    
    setResults: function(results) {
    	this._arrayListOfResources = results;
    },
    
    getGroupOrder: function() {
    	return this._groupOrder;
    },
    
    getSortOrder: function() {
    	return this._sortOrder;
    },
    
    getToggleButtonState: function() {
    	return this._isRecentArtifactsHighlighted;
    },

    _constructTableHeader: function() {
    	var table = document.createElement("table");
    	table.className = "dashboard-result-set";
    	table.setAttribute("cellSpacing", "0px");
    	var tbody = document.createElement("tbody");
    	table.appendChild( tbody );
    	var numHeaderColumns = 0;
    	var headerTitles = [com.ibm.rdm.web.project.ResultSetView.headerTitles.NAME,
    	                    com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE,
    	                    com.ibm.rdm.web.project.ResultSetView.headerTitles.USER,
    	                    com.ibm.rdm.web.project.ResultSetView.headerTitles.DATE,
    	                    com.ibm.rdm.web.project.ResultSetView.headerTitles.MANAGEDREQUIREMENT];
    	var headerRow = document.createElement( "tr" );
    	headerRow.className = "table-header";
    	tbody.appendChild( headerRow );
    	for (var i = 0; i < headerTitles.length; i++) {
    		var headerNode = null;
    		if (this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS) {
    			if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
	            	if (headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.USER &&
	            		headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE){
	            		headerNode = document.createElement( "th" );
	            	}
	            } else {
	            	if (headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE) {
	            		headerNode = document.createElement( "th" );
	            	}
	            }
    		}
			else if (this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS ||
					this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTIONS || 
					this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.COLLECTION) {
	            if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE) {
	            	if (headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE) {
	            		headerNode = document.createElement( "th" );
	            	}
	            } else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
	            	if(headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.USER){
	            		headerNode = document.createElement( "th" );
	            	}
	            } else {
	            	if (headerTitles[i] != com.ibm.rdm.web.project.ResultSetView.headerTitles.MANAGEDREQUIREMENT){
	            		headerNode = document.createElement( "th" );
	            	}
	            }
    		}
            if (headerNode) {
            	numHeaderColumns++;
        		headerNode.appendChild( document.createTextNode( headerTitles[i] ) );
        		headerNode.className = "header-cell";
        			if (headerTitles[i] == com.ibm.rdm.web.project.ResultSetView.headerTitles.NAME) {
        					this.connect(headerNode, "onclick", this.sortByName);
        			}
        			if (headerTitles[i] == com.ibm.rdm.web.project.ResultSetView.headerTitles.TYPE) {
    						this.connect(headerNode, "onclick", this.sortByType);
        			}
        			if (headerTitles[i] == com.ibm.rdm.web.project.ResultSetView.headerTitles.USER) {
							this.connect(headerNode, "onclick", this.sortByUser);
        			}
        			if (headerTitles[i] == com.ibm.rdm.web.project.ResultSetView.headerTitles.DATE) {
							this.connect(headerNode, "onclick", this.sortByDate);
        			}		
        		headerRow.appendChild( headerNode );    
            }
    	}
    	var nodes = headerRow.childNodes;
    	for (var i = 0; i < nodes.length; i++) {
    		switch (numHeaderColumns) {
    		case 4:
    			dojo.addClass(nodes[i],"four-column");
    			break;
    		case 3:
    			dojo.addClass(nodes[i],"three-column");
    			break;
    		case 2:
    			dojo.addClass(nodes[i],"two-column");
    			break;
    		}
    		
    	}
    	this._arrayListOfRecentArtifacts = [];
    	this._resultTable = table;
    	//this._checkMenuItem();
    	return table;
    },
    
    _sortArray: function(resourceInfos) {
        if(this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.NAME){
        	resourceInfos.sort(compareResourceName);
        } else if( this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.TYPE){
        	resourceInfos.sort(compareResourceType);
        } else if( this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.USER){
        	resourceInfos.sort(compareResourceModifiedBy);
        } else if( this._sortOrder == com.ibm.rdm.web.project.ResultSetView.sortBy.DATE){
        	resourceInfos.sort(compareResourceModifiedDate);
        } else if( !this._sortOrder ) {
        	resourceInfos.sort(compareResourceModifiedDate);
        }
    },
    
    updateResult: function(resourceInfo) {
		var resourceURI = resourceInfo.resourceURI;
		var tdElements = dojo.byId(resourceInfo.resourceURI + "#" + this._groupOrder).getElementsByTagName("td");
		var linkTD = tdElements[0];
		com.ibm.rdm.web.util.domutils.removeChildNodes(linkTD);
		this.disconnect(this._mapResourceURIToClickHandler[resourceURI]);
		this._mapResourceURIToClickHandler[resourceURI] = null;
		linkTD.appendChild(this._createResourceLink(resourceInfo));
		var lastModifiedByTD = null;
		if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.NONE
			|| this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.DATE) {
			lastModifiedByTD = tdElements[2];
		} else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE) {
			lastModifiedByTD = tdElements[1];
		}
		if (lastModifiedByTD) {
			com.ibm.rdm.web.util.domutils.removeChildNodes(lastModifiedByTD);
			lastModifiedByTD.appendChild(this._createUserNode(resourceInfo.lastModifiedBy));
		}
    },
    
    _getTimeWindowLabel: function(dateRange) {
        var label = null;
        if (dateRange == com.ibm.rdm.web.commenting.UPDATED_TODAY) {
        	label = this._messages.today;
		} else if (dateRange == com.ibm.rdm.web.commenting.UPDATED_YESTERDAY) {
			label = this._messages.yesterday;
		} else if (dateRange == com.ibm.rdm.web.commenting.UPDATED_PAST_WEEK) {
			label = this._messages.pastWeek;
		} else if (dateRange == com.ibm.rdm.web.commenting.UPDATED_PAST_MONTH) {
			label = this._messages.pastMonth;
		} else if (dateRange == com.ibm.rdm.web.commenting.UPDATED_EARLIER) {
			label = this._messages.earlier;
		}
        return label;
    },
    
    _constructTableRow: function(resourceInfo) {
    	var self = this;
        var tr = document.createElement("tr");
        tr.id = resourceInfo.resourceURI + "#" + this._groupOrder;
        // Add the resource image
        td = document.createElement("td");
        if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.NONE) {
        	td.className = "result-set-column-cell";
        } else { 
        	// indent the first column
        	td.className = "result-set-grouped-column-one-cell";
        }
		td.appendChild(this._createResourceLink(resourceInfo));
		dojo.addClass(td, "inactive-text");
        tr.appendChild(td);
        if (this._groupOrder != com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE &&
            !(this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS)) {
        	// Add the resource type
        	td = document.createElement("td");
        	td.appendChild(document.createTextNode( 
        			com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(resourceInfo.contentType)));
        	td.className = "result-set-column-cell";
        	dojo.addClass(td, "inactive-text");
        	tr.appendChild(td);
		}
             
        if (this._groupOrder != com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
            // Add the "Last Modified By"
            td = document.createElement("td");
			dojo.addClass(td, "result-set-column-cell");
			this._updateProjectMemberFullName(resourceInfo, td);
			this._createUserHover(resourceInfo, td);
			dojo.addClass(td, "inactive-text");
            tr.appendChild(td);
        }
        // Add the "Last Modified Date"
        td = document.createElement("td");
        td.className = "result-set-column-cell";
        try {
	        var date = dojo.date.locale.format(
	            resourceInfo.lastModifiedDate, {formatLength:"medium"});
	        td.appendChild(document.createTextNode(date));
        } catch (e) {
			var message = dojo.string.substitute(this._messages["errorParseDate"], [e]);
			console.error(message);
        }
        dojo.addClass(td, "inactive-text");
        tr.appendChild(td);
        
        // Add the "Managed Requirement if needed"
        var reqProTag = resourceInfo.reqProTag;
        var linkToReqPro = resourceInfo.linkToReqPro;
        td = document.createElement("td");
        td.className = "result-set-column-cell";
        if ((this._viewType === com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS ||
         this._groupOrder === com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE) && reqProTag && linkToReqPro){
        	var nodeManagedReq = com.ibm.rdm.web.util.domutils.createAnchor(reqProTag, decodeURIComponent(linkToReqPro));
            this.connect(nodeManagedReq, "onclick", this._handleManagedReqClicked);
        	td.appendChild(nodeManagedReq);
        }
        dojo.addClass(td, "inactive-text");
        tr.appendChild(td);
        
        // Add row event handling
        this.connect(tr, "onclick", function() {
        	if (self._lastSelected) {
        		if (self._lastSelected == tr) {
        			dojo.removeClass(self._lastSelected, "row-selected");
        			self._lastSelected = null;
        		} else {
        			dojo.removeClass(self._lastSelected, "row-selected");
        			dojo.addClass(tr, "row-selected");
        			self._lastSelected = tr;
        		}
        	} else {
        		dojo.addClass(tr, "row-selected");
        		self._lastSelected = tr;
        	}
        });
   
        try {
	        var dateRange = DateTimeHelper.getDateRange(resourceInfo.lastModifiedDate);
	        if (dateRange == com.ibm.rdm.web.common.DateTimeHelper.DATE_RANGE_TYPE.Today ||
	        		dateRange == com.ibm.rdm.web.common.DateTimeHelper.DATE_RANGE_TYPE.Yesterday) {
	        	this._arrayListOfRecentArtifacts.push(tr);
	        }
        } catch (e) {
        	console.error(e);
        }
        return tr;
    },
    
    _updateProjectMemberFullName: function(resourceInfo, td) {
		var link = resourceInfo.resourceURI;
		var onProjectMemberComplete = function(userNames) {
			this._usersMap = userNames;
			var fullUserName = userNames[resourceInfo.lastModifiedBy].name;
			var emailAddress = userNames[resourceInfo.lastModifiedBy].email;
			var nodeMailTo = com.ibm.rdm.web.util.domutils.createMailTo(emailAddress,
					resourceInfo.displayName, 
					fullUserName,
					encodeURIComponent(link));
			dojo.addClass(nodeMailTo, "active-text");
			td.appendChild(nodeMailTo);
			return userNames;
		};
		if (this._usersMap) {
			dojo.hitch(this, onProjectMemberComplete,this._usersMap)();
		}
		else {
			ProjectCache.getAllUsers().addCallback(this, onProjectMemberComplete);
		}
	},
	
	_showMessage: function(messageToDisplay) {
		this._isMessageDisplayed = true;
	    this._arrayListOfResources = [];
		if (!this.resultTable) {
			var table = this._constructTableHeader();
			this._preDisplayActivity(table);
		}
		if (this._resultTable) {
			var tbody = this._resultTable.firstChild;
			// Remove any result set rows that are currently displayed.
			var resultRow = tbody.firstChild.nextSibling;
			while (resultRow) {
				var nextRow = resultRow.nextSibling;
				tbody.removeChild(resultRow);
				resultRow = nextRow;
			}
			// Display a single row containing the text of the passed
			// messageToDisplay.
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode(messageToDisplay));
			tr.appendChild(td);
			tbody.appendChild(tr);
			td.setAttribute("colSpan", tbody.firstChild.childNodes.length);
			td.className = "user-message-table-data";
		}
	},
    
    _showGroupedResources: function(mapResourceInfoArrays, userMap) {
		if (this._isMessageDisplayed) {
			return;
		}
   		function getUserInfo(userId, userInfos) {
    		for (var userURL in userInfos) {
        		if (userInfos[userURL].userId === userId) {
        			return userInfos[userURL];
        		}
        	}
        	return null;
        };
    	this._contentPane = this._groupContentPane;
    	var table = this._constructTableHeader();
    	var tbody = table.firstChild;
    	var headerCount = 0;
        for (var i = 0; i < this._currentGroupingList.length; i++) {
        	var prop = this._currentGroupingList[i];
            var resourceInfos = mapResourceInfoArrays[prop];
            if (!resourceInfos) {
            	continue;
            }
            this._sortArray(resourceInfos);
            var tr = document.createElement("tr");
            var th = document.createElement("th");
            var node;
            if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE) {
	            var contentTypeDisplayName = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(prop);
	            node = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
	                "content-type-title", contentTypeDisplayName);
	            th.setAttribute("colSpan", "4");
            } else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
        		var userInfo = getUserInfo(prop, userMap);
        		var author = (userInfo ? userInfo.name : prop);
	            node = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
		                "", author);
	            th.setAttribute("colSpan", "4");
            } else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.DATE) {
            	// get message for property
            	var rangeString = this.getTimeRangeLabelToUse(prop);
            	node = com.ibm.rdm.web.util.domutils.createSpannedTextNode(
		                "", rangeString);
            	th.setAttribute("colSpan", "4");
            }
            var collapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection(
            		{
            			sectionTitle: node.firstChild.nodeValue
            		});
            this._collapseableHeaders.push(collapseableHeader);
            th.appendChild(collapseableHeader.domNode);
            th.className = "content-type-table-header";
            tr.appendChild(th);
            tbody.appendChild(tr);
            var rows = [];
            var contentType;
            if (resourceInfos[0]) {
            	contentType = resourceInfos[0].contentType;
            }
            for (var j = 0; j < resourceInfos.length; j++) {
                var resourceInfo = resourceInfos[j];
				var tr = this._constructTableRow(resourceInfo);
                tbody.appendChild(tr);
                rows.push(tr);
                headerCount++;
            }
            if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE ) {
            	collapseableHeader.setHeaderImage(com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceInfo.contentType));
            } else if (this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.USER) {
            	collapseableHeader.setHeaderImage("com.ibm.rdm.web/common/images/user.gif");
            }
            collapseableHeader.setContent(rows);
        }
        this.updateNumArtifacts(headerCount);
        this._preDisplayActivity(table);
/*    	this._contentPane.setContent(table);
    	this._stackContainer.selectChild(this._contentPane);
        // TODO Is there a better approach for this?
        // For some reason, IE doesn't know to show horz/vert scrollbars when
        // you ask for a dashboard that contains enough artifacts to require
        // scrollbars.  The following workaround "jiggles" the content pane's
        // dimensions so that scrollbars appear when necessary.
        if (dojo.isIE) {
            var size = this._contentPaneSize;
            size.h -= 1;
            this._contentPane.resize(size);
            size.h += 1;
            this._contentPane.resize(size);
        } else {
        	this.resize(this._contentPaneSize);
//        }
*/
	},
    
	_appendResultsToTable: function(resourceInfos) {
		var tbody = this._resultSetTable.firstChild;
        for (var i = 0; i < resourceInfos.length; i++) {
        	var resourceInfo = resourceInfos[i];
        	var tr = this._constructTableRow(resourceInfo);
        	tbody.appendChild(tr);
        }
    	if (this._isRecentArtifactsHighlighted) {
    		this._isRecentArtifactsHighlighted = false;
    		this.highlightRecentArtifacts();
    		this._isRecentCommentedHighlighted =false;
    		this.highlightRecentCommentedArtifacts();  
    	}   	
	},

    _showListedResources: function(resourceInfos) {
		if (this._isMessageDisplayed) {
			return;
		}
    	this._contentPane = this._listContentPane;
    	var table = this._constructTableHeader();
    	var tbody = table.firstChild;
    	var date = new Date();
    	this._sortArray(resourceInfos);
    	ErrorHandler.debug(dojo.date.difference(date, null, "millisecond") + "ms to sort Array");
    	date = new Date();
        for (var i = 0; i < resourceInfos.length; i++) {
        	var resourceInfo = resourceInfos[i];
			var tr = this._constructTableRow(resourceInfo);
        	tbody.appendChild(tr);   
        }
        
        this.updateNumArtifacts(resourceInfos.length);
        
        ErrorHandler.debug(dojo.date.difference(date, null, "millisecond") + "ms to create Table");
        this._preDisplayActivity(table);
/*    	this._contentPane.setContent(table);
    	//TODO this._stackContainer._containerContentBox
        this._stackContainer.selectChild(this._contentPane);
        // TODO Is there a better approach for this?
        // For some reason, IE doesn't know to show horz/vert scrollbars when
        // you ask for a dashboard that contains enough artifacts to require
        // scrollbars.  The following workaround "jiggles" the content pane's
        // dimensions so that scrollbars appear when necessary.
        if (dojo.isIE) {
            var size = this._contentPaneSize;
            size.h -= 1;
            this._contentPane.resize(size);
            size.h += 1;
            this._contentPane.resize(size);
        } else {
        	this.resize(this._contentPaneSize);
//        }
*/    },

	updateNumArtifacts: function(num) {
		// Update the number of artifacts in the project header
		if (num == 1) {
			this.numArtifacts.childNodes[0].nodeValue = this._messages.showingOne;
		} else {
			this.numArtifacts.childNodes[0].nodeValue = dojo.string.substitute(this._messages.showingN, [num]);
		}
	},
    
    _preDisplayActivity: function(table) {
    	this._contentPane.attr('content', table);
    	this._resultSetTable = table;
    	this._stackContainer.selectChild(this._contentPane);
    	this.resize(this._contentPaneSize);
    	if (this._isRecentArtifactsHighlighted) {
    		this._isRecentArtifactsHighlighted = false;
    		this.highlightRecentArtifacts();
    	}
    	if (this._isRecentCommentedHighlighted){
    		this._isRecentCommentedHighlighted =false;
    		this.highlightRecentCommentedArtifacts();  
    	}  
    },

    _createResourceLink: function(resourceInfo) {
		var contentType = (
			 this._groupOrder == com.ibm.rdm.web.project.ResultSetView.groupBy.TYPE ?
			 null : resourceInfo.contentType
		);
        // Add the resource image.
        var div = document.createElement("div");
        if (contentType) {
	        var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(contentType);
	        if (imageURI) {
	            var img = document.createElement("img");
	            img.setAttribute("src", imageURI);
	            img.className = "content-type-entry-image";
	            div.appendChild(img);
	        }
        }
          
        // Add the resource link.
		var resourceURI = (
			resourceInfo.resourceURI
			+ (
				(resourceInfo.inSnapshot && resourceInfo.isCollectionArtifactAssociatedWithSnapshot) ?
				"?revision=" + resourceInfo.etag + "&rrc:baseline=" + encodeURIComponent(resourceInfo.projectSnapshotURI) :
				""
			  )
		);
		var label = (
			resourceInfo.displayName
			+ (
				(!resourceInfo.inSnapshot && resourceInfo.isCollectionArtifactAssociatedWithSnapshot) ?
				this._messages.artifactNotInSnapshot :
				""
			  )
		);
        var link = ResourceLinksHandler.getResourceLink(resourceURI, label, true);
		if (dojo.isIE) {
			// This is a workaround for Jazz Foundation defect 97086.
			link.style.position = "relative";
		}
        div.appendChild(link);
        return div;
    },

	_createUserNode: function(userName) {
		return document.createTextNode(userName);
	},
    
    _handleResourceContextClicked: function (event) {
    	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
    },
    
    _handleManagedReqClicked: function (event) {
    	 event.preventDefault();
         event.stopPropagation();
         window.open( (dojo.isIE ? event.srcElement.href : event.target.href) );
    },
        
	_createUserHover: function(resourceInfo, node){
		var userRoles = this._userRolesMap;
		for (var userUri in userRoles){	
			if(resourceInfo.lastModifiedBy == userRoles[userUri].url){
				var userInfo = userRoles[userUri];
				var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : this.projectURI}, null);
				userHover.createHoverPopup(node, userInfo);					
			}
		}
    },
    
    _populateUserRoleInfo: function(){
    	var onProjectMembersComplete = function(userPropertyInfos) {
			this._userRolesMap = userPropertyInfos;
		}
    	
    	var deferred = ProjectCache.getProjectUsersWithRoles(this.projectURI);
		deferred.addCallback(this, onProjectMembersComplete);
    }
    	
});

compareResourceName = function(resourceInfo1, resourceInfo2) {
	var compareTypes = function(resourceInfo1, resourceInfo2) {
	    var resourceDate1 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(resourceInfo1.contentType);
	    var resourceDate2 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(resourceInfo2.contentType);
	    return ((resourceDate1 == resourceDate2) ?
	    	0 :
	        ((resourceDate1 < resourceDate2) ? -1 : 1));
	};
    var resourceName1 = resourceInfo1.displayName;
    if (resourceName1) { resourceName1 = resourceName1.toLowerCase(); }
    var resourceName2 = resourceInfo2.displayName;
    if (resourceName2) { resourceName2 = resourceName2.toLowerCase(); }
    return ((resourceName1 == resourceName2) ?
    	compareTypes(resourceInfo1, resourceInfo2) :
        ((resourceName1 < resourceName2) ? -1 : 1));
};

compareResourceModifiedDate = function(resourceInfo1, resourceInfo2) {
    var resourceDate1 = resourceInfo1.lastModifiedDate;
    var resourceDate2 = resourceInfo2.lastModifiedDate;
    return ((resourceDate1 == resourceDate2) ?
    	compareResourceName(resourceInfo1, resourceInfo2) :
        ((resourceDate1 > resourceDate2) ? -1 : 1));
};

compareResourceModifiedBy = function(resourceInfo1, resourceInfo2) {
    var resourceDate1 = resourceInfo1.lastModifiedBy;
    var resourceDate2 = resourceInfo2.lastModifiedBy;
    return ((resourceDate1 == resourceDate2) ?
    	compareResourceName(resourceInfo1, resourceInfo2) :
        ((resourceDate1 < resourceDate2) ? -1 : 1));
};

compareResourceType = function(resourceInfo1, resourceInfo2) {
    var resourceDate1 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(resourceInfo1.contentType);
    var resourceDate2 = com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(resourceInfo2.contentType);
    return ((resourceDate1 == resourceDate2) ?
    	compareResourceName(resourceInfo1, resourceInfo2) :
        ((resourceDate1 < resourceDate2) ? -1 : 1));
};

com.ibm.rdm.web.project.ResultSetView.groupBy = {
		NONE: 4,
		TYPE: 5,
		USER: 6,
		DATE: 7
};

com.ibm.rdm.web.project.ResultSetView.sortBy = {
		NAME: 0,
		TYPE: 1,
		USER: 2,
		DATE: 3
};

// These get populated in postCreate()
com.ibm.rdm.web.project.ResultSetView.headerTitles = {
}

})();
