/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.project.UploadFileWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dojo.io.iframe");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.project.projectutils");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
// dojo.requireLocalization("com.ibm.rdm.web.project", "UploadFileWidget");
var UriHelper = com.ibm.rdm.web.common.UriHelper;


dojo.declare(
    "com.ibm.rdm.web.project.UploadFileWidget",
    [dijit._Widget, dijit._Templated],
{
    // summary:
    // Provides a dialog to upload artifacts to server. 
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/UploadFileWidget.html"),
    currentProject: this.projectURI,
    currentResultView: null,
    widgetsInTemplate: true,
    _dialogWidget: null,
    
    postCreate: function() {
    	
    	console.log("In UploadFileWidget postcreate");   	
        this.inherited("postCreate", arguments); 
        this.initialize();       
    },
    
    initialize: function(){
    	this.createMimeTypesMenu(); 
    	this._reset();
    }
    ,
    _onFileNameChange: function(evt){
    	
    	var selectedFile = this._uploadFilename.value;
    	
    	//IE passes the file path along with the file name. FF only passes the file name.
    	//We need to make sure we only capture the filename
    	
    	var lastForwardSlashIndex = selectedFile.lastIndexOf("/"); //Unix path separator
    	var lastBackSlashIndex = selectedFile.lastIndexOf("\\"); //Windows path separator
    	
    	if(lastForwardSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastForwardSlashIndex + 1);
    	}else if(lastBackSlashIndex != -1){
    		selectedFile = selectedFile.substring(lastBackSlashIndex + 1);
    	}   	   	
    	this._artifactName.value = selectedFile;
    }
    ,
    
    _onFinishButtonClick: function(evt){    	    	
        
       if(this._uploadFilename.value== ""){
    	   return;
       }
       this.showMessage("Uploading file...");
       
       var uri = UriHelper.WEB_APP_JRSPROXY +  "/upload";
       var dialogWidget = this._dialogWidget;
       //Submit the form using an IFRAME
    	dojo.io.iframe.send({
    		form: "uploadForm",
    		handleAs: "html",
    		url: uri,
    		load: function(response, ioArgs){
    		   console.log("File uploaded sucessfully!");
    		   dialogWidget.hide();
    	       return response;
    	    },
    	    error: function(response, ioArgs){
    	      console.log("error");
    	      console.log(response, ioArgs);
    	      return response;
    	    }    	
    	});
    	
    	//Refresh the page here with the same call to "globalRefreshButton"
    	
    	
    },   
    _onCancelButtonClick: function(evt){
    	this._cancel();
    },
    
    _reset: function(){
    	this._uploadFilename.value=""; 
    	this._artifactName.value = "";
    	if (this.currentProject){
    		var project = com.ibm.rdm.web.project.projectutils.getProjectName(this.currentProject);
    		this._projectSelection.value = project;
    		this._projectName.value = project;

    	}
    		
        //Set folder location. 
    	this._folderSelection.value = "MyFolder";
    	var url="https://localhost:9443/jazz/resources/rdm/projectresources/9/rrc_tag_49772a57-4d33-408e-bdb1-a1c48b38dfe3"; 
    	this._folderName.value =url;
    	this.clearMessage();
    	
    },
    
    _cancel: function(){
    	this._reset();
    	if(this._dialogWidget != null){
    		this._dialogWidget.hide();
    	}
    },
    
    show: function(){
    	this._reset();
    	this._dialogWidget.show();
    },
    showMessage: function(msg) {
    	this._statusImageNode.style.display = "";
        this._statusMessageNode.appendChild(document.createTextNode(msg));
    },
    clearMessage: function() {
    	this._statusImageNode.style.display = "none";
    	if (this._statusMessageNode.firstChild != null)
    		this._statusMessageNode.removeChild(this._statusMessageNode.firstChild);
    },
    
    createMimeTypesMenu: function(){
    
      //Create selection drop down for available mime types   		
		var selectNode = document.createElement("select");
		selectNode.setAttribute('name', 'mimeTypes');
					
    	dojo.forEach(com.ibm.rdm.web.common.MimeTypes.ARTIFACTS_ORDERED, function(resourceType) {
    		//var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceType.contentType);  		
    		var optionNode = document.createElement("option");
    		optionNode.setAttribute('value', resourceType.contentType);
          	var textNode = document.createTextNode(resourceType.name);
          	optionNode.appendChild(textNode);
          	selectNode.appendChild(optionNode);  
    	});
    	
    	dojo.forEach(com.ibm.rdm.web.common.MimeTypes.ABBREVIATED_WRAPPERS, function(resourceType) {
    		//var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceType.contentType);
    		var optionNode = document.createElement("option");
    		optionNode.setAttribute('value', resourceType.contentType);
          	var textNode = document.createTextNode(resourceType.name);
          	optionNode.appendChild(textNode);
          	selectNode.appendChild(optionNode);  
    	});
      	
    	this._mimeTypes.appendChild(selectNode);
 
    }

});


