/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.ArtifactInReviewViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.commenting.ResourceViewerWidget");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ArtifactBanner");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.reviewChangeNotifier");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewUpdateMediator");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ArtifactBanner = com.ibm.rdm.web.review.ArtifactBanner;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var Headers = com.ibm.rdm.web.server.Headers;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var reviewChangeNotifier = com.ibm.rdm.web.review.reviewChangeNotifier;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewUpdateMediator = com.ibm.rdm.web.review.reviewUpdateMediator;


dojo.declare(
	"com.ibm.rdm.web.review.ArtifactInReviewViewer",
	[dijit._Widget, dijit._Templated],
{
    // summary:
	//    Provides a viewer for display of an artifact in the context of a
	//    Review.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ArtifactInReviewViewer.html"),
	widgetsInTemplate: false,

    ///////////////////////////////////////////////////////////////////////////
    // Attach points

	_artifactBannerDiv: null,
	_resourceViewerWidgetDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

 	resourceViewer: null,
 	resourceURI: null,
	revision: null,
	reviewId: null,
	resourceViewerWidget: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_reviewParser: null,
	_numReviewLoadsPerformed: null,
	_artifactBanner: null,
	_reviewPostIsInProgress: null,
	_onReviewChanged: null,
	_onReviewSaveIsInProgress: null,
	_onReviewSaveHasCompleted: null,
	_onReviewSaveHasFailed: null,
	_lastWidth: null,
	_lastHeight: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.review", "reviewAndApprovalCommon");
		this._numReviewLoadsPerformed = 0;
		this._reviewPostIsInProgress = false;
		this._lastWidth = null;
		this._lastHeight = null;
		// Add the banner at the top that contains vote casting buttons.
		this._artifactBanner = new ArtifactBanner(
			{
				resourceViewer: this.resourceViewer,
				artifactInReviewViewer: this,
				resourceURI: this.resourceURI,
				disabled: true
			},
			this._artifactBannerDiv
		);
		// Add the content area which appears below the banner and holds
		// the artifact that is under review.
		this._resourceViewerWidgetDiv.appendChild(this.resourceViewerWidget.domNode);
	},

	startup: function() {
		this.inherited("startup", arguments);
		this._updateUsingNewestReviewContent();
		this.resourceViewerWidget.startup();
	},

	destroy: function() {
		this.resourceViewerWidget.destroyRecursive();
		this._removeReviewListeners();
		this.inherited("destroy", arguments);
	},
	
	loadResource: function(contentType, revision) {
		// TODO Need to add code here to ask ReviewCache for latest version
		// (due to the fact that something like the user's role in the review
		// may have changed).
	},
	
	setDimensions: function(width, height) {
		this._lastWidth = width;
		this._lastHeight = height;
		var contentAreaHeight = height - this._artifactBanner.domNode.clientHeight;
		this.resourceViewerWidget.setDimensions(width, contentAreaHeight);
	},

	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.

		// FIXME Implement
	},

	showAsBusy: function(isBusy) {
		if (isBusy) {
			this.domNode.style.cursor = "progress";
		} else {
			this.domNode.style.cursor = "";
		}
	},

	isBusy: function() {
		return (this.domNode.style.cursor === "progress");
	},

	viewHasBeenRestored: function() {
		// summary:
		//    This viewer was just brought back from the "DOM cache" and it's
		//    been asked to redisplay itself.
		// description:
		//    If the participant clicks through a second time on the same
		//    artifact within a Review, this function is called to account for
		//    any Review changes that may have occurred since the last time it
		//    was displayed.  For example, perhaps the last time it displayed
		//    using an banner containing enabled vote casting buttons (as the
		//    Review's state was Started).  However, if the user clicked the
		//    back button (to get back to the Review) and then clicked it the
		//    same artifact link again, it _may_ be the case that the Review
		//    is now in the Paused state, in which case the vote casting
		//    buttons should now display as disabled.
		var reviewParser = this._reviewParser;
		var isInformal = (!reviewParser.isFormal());
		if (isInformal) {
			// Update the artifact under review as well as the review related
			// content (including the vote last cast on this artifact).
			this._updateUsingNewestArtifactContentAndNewestReviewContent();
		} else {
			// Update the review related content (including the vote last
			// cast on this artifact).  Also, if appropriate, update the
			// artifact content.
			var doUpdateContentIfStartTimestampHasChanged = (
				this._reviewParser.isFormal()
				&& this._reviewParser.getReviewState() != reviewTypes.REVIEW_STATE_TYPES.Draft
			);
			this._updateUsingNewestReviewContent(doUpdateContentIfStartTimestampHasChanged);
		}
	},

	viewHasBeenRefreshed: function() {
		// summary:
		//    The artifact content has just been refreshed.  This function is
		//    called to make sure that the associated review content is ALSO
		//    appropriately refreshed when this occurs.
		this._updateUsingNewestReviewContent();
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_updateUsingNewestArtifactContentAndNewestReviewContent: function() {
		this.resourceViewer.resourcesPage.handleRefresh();
	},

	_updateUsingNewestReviewContent: function(/*Boolean?*/doUpdateContentIfStartTimestampHasChanged) {
		// summary:
		//    Get the newest review content and then make appropriate review
		//    related updates.
		// description:
		//    Get the newest review content, and make appropriate updates to the
		//    review banner and all other review related widgets that are
		//    listening via the reviewUpdateMediator.  In addition, if this is
		//    a formal review in which the review timestamp has changed, we also
		//    take care to update the artifact content area.
		var self = this;
		var originalStartTimestamp = (
			this._reviewParser != null ?
			this._reviewParser.getStartTimestamp() :
			null
		);
		var onReviewLoaded = function(response) {
			var reviewParser = response.doc;
			var artifactInfo = reviewParser.getArtifactInfo(self.resourceURI);
			self._reviewParser = reviewParser;
			self._artifactBanner.attr("reviewParser", reviewParser);
			if (!self._artifactBanner.isStarted()) {
				self._artifactBanner.startup();
				// Note: The artifact banner doesn't populate its contents until
				// startup has been called.  However, in populating the contents
				// the banner's height grows to accommodate the vote casting
				// buttons that it has decided to show (based on the user's role
				// within the review).  Therefore, due to the fact that the
				// banner's height grows, we need to make sure to appropriately
				// redraw the content area so that the status bar doesn't get
				// "pushed down" so much that it "goes missing" (and when this
				// happens a gap would also appear beneath the "Links" section
				// in the sidebar).
				if (self._lastWidth && self._lastHeight) {
					self.setDimensions(self._lastWidth, self._lastHeight);
				}
			}
			self._addReviewListeners();
			// Transition the artifact from "Not started" to "In progress" if it
			// makes sense to do so.
			var originalAritifactStatusType = artifactInfo.myArtifactStatusType;
			reviewUpdateMediator.artifactUnderReviewHasBeenOpened(
				reviewParser, artifactInfo);
			if (originalAritifactStatusType !== artifactInfo.myArtifactStatusType
				&& !reviewParser.getParticipantResultUri()
				) {
				self._reviewPostIsInProgress = true;
			}
			if (originalStartTimestamp
				&& originalStartTimestamp != self._reviewParser.getStartTimestamp()
				) {
				// The review timestamp has shifted within a formal review.
				// This means the artifact content might be different now,
				// so we force a refresh to occur.
				self._updateUsingNewestArtifactContentAndNewestReviewContent();
			}
			// Update the enablement state of the vote casting buttons if this
			// is the first time the review has been loaded for this artifact.
			self._numReviewLoadsPerformed++;
			if (self._numReviewLoadsPerformed === 1
				&& reviewutils.canVote(reviewParser)
				&& originalAritifactStatusType !== reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
				) {
				// The vote casting buttons were originally created using the
				// "disabled" state ... change things so that they're enabled.
				self._artifactBanner.attr("disabled", false);
			}
			return response;
		};
		var onReviewError = function(response) {
			self._showAppropriateLoadError();
			return response;
		};
		var deferred = ReviewCache.getReview(this.reviewId);
		deferred.addCallback(onReviewLoaded);
		deferred.addErrback(onReviewError);
	},

	_showAppropriateLoadError: function() {
		// summary:
		//    Display an appropriate error as a result of not being able to
		//    load the associated review.
		var resourceURI = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + "/" + this.reviewId;
		ErrorHandler.warn(
			"Unable to load the associated review '" + resourceURI + "'"
		);
		var self = this;
		var performHeadReqestAndShowAppropriateError = function(projectURI) {
			// Check if this abnormal response was returned due to the fact
			// that the underlying Review has been deleted.
			var headers = [];
			headers[Headers.OWNING_CONTEXT] = projectURI;
			var headDeferred = RDMClient.doHead({
				uri: resourceURI,
				xhrArgs: { headers: headers },
				skipCache: true,
				skipErrors: true
			});
			headDeferred.addCallback(this, function(response){
				self._showAssociatedReviewCouldNotBeLoadedError(resourceURI);
				return response;
			});
			headDeferred.addErrback(this, function(response){
				var status = response.ioArgs.xhr.status;
				if (status === 410) { // Note: 410 is "Gone"
					ErrorHandler.warn(
						"The associated review, '"
						+ resourceURI
						+ "', has been deleted"
					);
  					self._showAssociatedReviewHasBeenDeletedError(resourceURI);
				} else {
					self._showAssociatedReviewCouldNotBeLoadedError(resourceURI);
				}
				return response;
			});
		}
		var onDescriptionComplete = function(indexedProperties) {
			projectURI = indexedProperties.projectURI;
			performHeadReqestAndShowAppropriateError(projectURI);
			return indexedProperties;
		};
		var onDescriptionError = function() {
			self._showAssociatedReviewCouldNotBeLoadedError(resourceURI);
		}
		var deferred = ProjectCache.getResourceDescriptionMultiFetch(resourceURI);
		deferred.addCallback(onDescriptionComplete);
	},

	_showAssociatedReviewHasBeenDeletedError: function(resourceURI) {
		var msg = this._messages["associatedReviewHasBeenDeleted"];
		var details = dojo.string.substitute(
			this._messages["associatedReviewHasBeenDeletedDetails"],
			[
				resourceURI
			]
		);
		ErrorHandler.getErrorSectionWidget().showErrorMessage(
		   msg, details);
	},

	_showAssociatedReviewCouldNotBeLoadedError: function(resourceURI) {
		var msg = this._messages["associatedReviewCouldNotBeLoaded"];
		var details = dojo.string.substitute(
			this._messages["associatedReviewCouldNotBeLoadedDetails"],
			[
				resourceURI
			]
		);
		ErrorHandler.getErrorSectionWidget().showErrorMessage(
		   msg, details);
	},

	_addReviewListeners: function() {
		// summary:
		//    Add listeners so that we know when a new version of the associated
		//    review is available, or when save related events occur.
		// description:
		//    Register for callbacks so that we're informed of:
		//    - Updated review content
		//    - Save events that occur when the user attepts to cast a vote
		if (! this._onReviewChanged) {
			// When the user attempts to cast a vote within this view the
			// callbacks will be invoked in the following order:
			// (1) onReviewSaveIsInProgress
			// (2) onReviewSaveHasCompleted
			// (3) onReviewChanged
			// Please note that when 'onReviewSaveHasCompleted' is called, the
			// passed 'reviewParser' (which is the same instance as
			// this._reviewParser) will ALREADY contain the appropriate revision
			// information (accessible via 'getParticipantResultRevision').
			// This is by design, and it allows us to enable the artifact
			// banner WITHOUT having to wait for 'onReviewChanged' to be called.
			// To be more specific, it enables the user to cast a vote one way
			// and then very quickly vote another way.  The alternative approach
			// would be to defer re-enabling the artifact banner (and the vote
			// casting buttons within it) until we hear that a newer version of
			// the review has been loaded via 'onReviewChanged', but this could
			// potentially take several seconds and it's a nicer user experience
			// to have this banner disabled for a minimal amount of time.  More
			// specifically, it enables us to avoid 409 conflict errors when
			// saving, as the user's participant results revision is already
			// appropriately "baked into" this._reviewParser (and this revision
			// etag is always passed whenever performing PUTs for the
			// participant results).
			var self = this;
			var onReviewChanged = function(reviewParser) {
				self._reviewParser = reviewParser;
				self._artifactBanner.attr("reviewParser", reviewParser);
			};
			var onReviewSaveIsInProgress = function(reviewParser) {
				self._artifactBanner.attr("disabled", true);
			};
			var onReviewSaveHasCompleted = function(reviewParser) {
				self._artifactBanner.attr("disabled", false);
			};
			var onReviewSaveHasFailed = function(reviewParser) {
				// Note: The error widget has already reported the problem before
				// this callback is invoked
				self._artifactBanner.attr("disabled", false);
			};
			reviewChangeNotifier.addReviewListener(
				this._reviewParser.getSelfUri(),
				onReviewChanged,
				onReviewSaveIsInProgress,
				onReviewSaveHasCompleted,
				onReviewSaveHasFailed);
			this._onReviewChanged = onReviewChanged;
			this._onReviewSaveIsInProgress = onReviewSaveIsInProgress;
			this._onReviewSaveHasCompleted = onReviewSaveHasCompleted;
			this._onReviewSaveHasFailed = onReviewSaveHasFailed;
		}
	},

	_removeReviewListeners: function() {
		if (this._onReviewChanged) {
			reviewChangeNotifier.removeReviewListener(
				this._reviewParser.getSelfUri(),
				this._onReviewChanged,
				this._onReviewSaveIsInProgress,
				this._onReviewSaveHasCompleted,
				this._onReviewSaveHasFailed);
		}
	}
});


})();
