/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.MyReview");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.review.ArtifactTableDescriptionSection");
dojo.require("com.ibm.rdm.web.review.ReviewArtifactsVotingSection");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.VoteCastingButtons");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewUpdateMediator");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ArtifactTableDescriptionSection = com.ibm.rdm.web.review.ArtifactTableDescriptionSection;
var ReviewArtifactsVotingSection = com.ibm.rdm.web.review.ReviewArtifactsVotingSection;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var VoteCastingButtons = com.ibm.rdm.web.review.VoteCastingButtons;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewUpdateMediator = com.ibm.rdm.web.review.reviewUpdateMediator;
var reviewutils = com.ibm.rdm.web.review.reviewutils;

var BORDER_AFFORDANCE = 51;


dojo.declare(
	"com.ibm.rdm.web.review.MyReview",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    An editor which enables a participant to cast his/her votes
	//    within a particular Review.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/MyReview.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_dueDateSpan: null,
	_reviewerInstructionsLabelSpan: null,
	_reviewerInstructionsTextArea: null,
	_artifactTableDescriptionSectionDiv: null,
	_groupSection: null,
	_reviewArtifactsVotingSectionDiv: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceViewer: null,
	reviewParser: null,
	reviewContainer: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_artifactTableDescriptionSection: null,
	_reviewArtifactsVotingSection: null,
	_lastWidth: null,
	_timerHandle: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		var self = this;
		this._artifactTableDescriptionSection = new ArtifactTableDescriptionSection(
			{ doUseAbbreviatedDisplay: true },
			this._artifactTableDescriptionSectionDiv);
		this._reviewArtifactsVotingSection = new ReviewArtifactsVotingSection(
			{
				resourceViewer: this.resourceViewer,
				reviewContainer: this.reviewContainer,
				groupSection: this._groupSection
			},
			this._reviewArtifactsVotingSectionDiv);
		this._populateContents();
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	setWidth: function(width) {
		this._lastWidth = width;
		this._resizeTextareaControl(width);
		this._reviewArtifactsVotingSection.setWidth(width);
	},

	paneHasBeenSelected: function() {
		var self = this;
		if (this._timerHandle) {
			window.clearTimeout(this._timerHandle);
		}
    	this._timerHandle = window.setTimeout(
        	function () {
				// Note by SPC: This is kind of a hack, but without doing this
				// twice the instructions textarea won't properly account for
				// the addition or removal of the pane's vertical scrollbar
				// (i.e. when it's added or removed, the textarea's width would
				// be improperly set to too little or too much by an amount
				// equals to the scrollbar's width [around 18 pixels]).
				self.setWidth(self._lastWidth);
				window.setTimeout(
					function () {
						self.setWidth(self._lastWidth);
					},
					250 //interval between dispatches
				);
    		},
    		250 //interval between dispatches
    	);
	},

	startup: function() {
		this.inherited(arguments);
	},

	markMyReviewStatusAsDone: function(artifactStatusType) {
		if (! this.reviewContainer.isBusy()) {
			var self = this;
			var onSaveComplete = function() {
				self.reviewContainer.showAsBusy(false);
				// Note: We're already hooked up to the reviewChangeNotifier
				// so we there's no need to separately update the UI elements
				// here ...
			};
			var onSaveCancelled = function() {
				self.reviewContainer.showAsBusy(false);
				// Note: We're already hooked up to the reviewChangeNotifier
				// so we there's no need to separately update the UI elements
				// here ...
			};
			var onSaveError = function() {
				self.reviewContainer.showAsBusy(false);
			};
			this.reviewContainer.showAsBusy(true);
			reviewUpdateMediator.markMyReviewStatusAsDone(
				this.reviewParser, onSaveComplete, onSaveCancelled, onSaveError);
		}
	},

	reviewSaveIsInProgress: function(reviewParser) {
		// summary:
		//    The review is in the process of being saved.
		// description:
		//    The user has perfomed a UI gesture which as resulted in a save
		//    request going off to the server.  Disable all UI controls whose
		//    action would result in a new save being performed.
		this._reviewArtifactsVotingSection.reviewSaveIsInProgress(reviewParser);
	},

	reviewSaveIsNoLongerInProgress: function(reviewParser) {
		// summary:
		//    The review is no longer in the process of being saved (either the
		//    save succeeded or returned an error code).
		// description:
		//    The outstanding save request that was issued due to the user
		//    performing a UI gesture has completed.  Re-enable the UI controls
		//    whose action would result in a new save being performed.
		this._reviewArtifactsVotingSection.reviewSaveIsNoLongerInProgress(reviewParser);
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		this.reviewParser = reviewParser;
		if (this._reviewArtifactsVotingSection) {
			this._reviewArtifactsVotingSection.attr(
				"reviewParser", reviewParser);
		}
		this._updateDueDateAndInstructions();
	},

	_resizeTextareaControl: function(accordionPaneWidth) {
		var labelSpanWidth = dojo.coords(this._reviewerInstructionsLabelSpan).w;
		if (labelSpanWidth > 0) {
			var groupSectionWidth = dojo.coords(this._groupSection).w;
			var borderAffordance = accordionPaneWidth - groupSectionWidth;
			var textareaWidthToUse = groupSectionWidth - labelSpanWidth - BORDER_AFFORDANCE;
			this._reviewerInstructionsTextArea.style.width = textareaWidthToUse + "px";
		}
	},

	_populateContents: function() {
		this._updateDueDateAndInstructions();
		this._artifactTableDescriptionSection.attr(
			"reviewParser", this.reviewParser);
		this._artifactTableDescriptionSection.startup();
		this._reviewArtifactsVotingSection.attr(
			"reviewParser", this.reviewParser);
		this._reviewArtifactsVotingSection.startup();
	},

	_updateDueDateAndInstructions: function() {
		var dueDate = this.reviewParser.getDueDate();
		var dueDateAsString = dojo.date.locale.format(
			dueDate,
			{ selector: "date", formatLength: "full" }
		);
		this._dueDateSpan.innerHTML = dueDateAsString;
		var instructions = this.reviewParser.getInstructions();
		if (dojo.isIE) {
			// Note: On IE attempting to use innerHTML instead will result
			// in removal of any newlines that are found in the text.
			this._reviewerInstructionsTextArea.innerText = instructions;
		} else {
			this._reviewerInstructionsTextArea.innerHTML = instructions;
		}
	}
});


})();
