/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewAccordionPane");

dojo.require("dijit._Widget");
dojo.require("dijit.layout.AccordionContainer");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.ReviewProgressBar");
dojo.require("com.ibm.rdm.web.review.VoteCastingButtons");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var ReviewProgressBar = com.ibm.rdm.web.review.ReviewProgressBar;
var VoteCastingButtons = com.ibm.rdm.web.review.VoteCastingButtons;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;


dojo.declare(
	"com.ibm.rdm.web.review._ReviewAccordionPane",
	dijit.layout.AccordionPane,
{
	// summary:
	//    Serves as the superclass for all review accordion panes.

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	reviewParser: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_twistieImage: null,

	_YOUR_REVIEW_IMAGE_SRC: "com.ibm.rdm.web/review/images/icons/obj16/my-review.gif",
	_REVIEW_STATE_IMAGE_SRC: "com.ibm.rdm.web/review/images/icons/obj16/Review-overall.gif",

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function(){
		this.inherited(arguments);
		var twistieImage = this._createTwistie();
		var div = document.createElement("div");
		div.className = "titleSectionDiv";
		this.titleTextNode.parentNode.appendChild(div);
		div.appendChild(twistieImage);
		div.appendChild(this._createTitleSectionContent());
		this._twistieImage = twistieImage;
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	reviewSaveIsInProgress: function(reviewParser) {
		// summary:
		//    The review is in the process of being saved.
		// description:
		//    The user has perfomed a UI gesture which has resulted in a save
		//    request going off to the server.  If appropriate, subclasses
		//    should override this function so that they may react accordingly
		//    by updating their UI controls (e.g. disable all write based
		//    UI controls).
	},

	reviewSaveIsNoLongerInProgress: function(reviewParser) {
		// summary:
		//    The review is no longer in the process of being saved (either the
		//    save succeeded or returned an error code).
		// description:
		//    The outstanding save request that was issued due to the user
		//    performing a UI gesture has completed.  If appropriate, subclasses
		//    should override this function so that they may react accordingly
		//    by updating their UI controls (e.g. re-enable their write based
		//    UI controls).
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: It's expected that subclasses will overload this function.
		//
		this.reviewParser = reviewParser;
	},

	_createTitleSectionContent: function() {
		// summary:
		//    Create the content that is to be shown in the title section.
		//    Subclasses MUST override this function.
		// returns: DomNode
		//    The DOM node to add to the section title area.
 		throw new Error("Unimplemented API: com.ibm.rdm.web.review._ReviewAccordionPane._createTitleSectionContent");
	},

	_createTwistie: function() {
		var img = document.createElement("img");
		img.className = "expand-collapse-image";
 		this._setExpandedStateIcon(img, this.selected);
		return img;
	},

	_setExpandedStateIcon: function(img, doShowAsExpanded) {
		img.setAttribute(
			"src",
			(doShowAsExpanded ?
				"com.ibm.rdm.web/sidebar/images/collapse.gif" :
				"com.ibm.rdm.web/sidebar/images/expand.gif")
		);
		if (doShowAsExpanded) {
			img.setAttribute("title", this._messages["collapseTooltip"]);
			img.setAttribute("alt", this._messages["collapseAlt"]);
		} else {
			img.setAttribute("title", this._messages["expandTooltip"]);
			img.setAttribute("alt", this._messages["expandAlt"]);
		}
	},

	_setSelectedState: function(/*Boolean*/ isSelected){
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._twistieImage) {
			this._setExpandedStateIcon(this._twistieImage, this.selected);
		}
	},

	_onTitleClick: function() {
		// summary:
		//    This is an override of a superclass function.
		var accordionContainer = this.getParent();
		var paneToSelect =
			(accordionContainer._adjacent(true) || accordionContainer._adjacent(false));
		if (paneToSelect) {
			accordionContainer.selectChild(paneToSelect);
		}
	},

	_appendTableDataElement: function(tr, /*DomNode?*/tdContent) {
		var td = document.createElement("td");
		tr.appendChild(td);
		if (tdContent) {
			td.appendChild(tdContent);
		}
		return td;
	},

	_createIconCell: function(imageSrc, /*String?*/ cssClassName) {
		var img = document.createElement("img");
		img.src = imageSrc;
		if (cssClassName) {
			img.className = cssClassName;
		}
		return img;
	}
});


dojo.declare(
	"com.ibm.rdm.web.review.MyReviewAccordionPane",
	com.ibm.rdm.web.review._ReviewAccordionPane,
{
	// summary:
	//    Shows title pane content for the "Participant's Review" accordion
	//    pane.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_myOverallReviewProgressBar: null,
	_currentStatusTD: null,
	_myRoleTD: null,
	_voteCastingButtons: null,
	_isReviewSaveInProgress: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function(){
		this.inherited(arguments);
		this._isReviewSaveInProgress = false;
	},

	onSelected: function(){
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		this.myReview.paneHasBeenSelected();
	},

	reviewSaveIsInProgress: function(reviewParser) {
		// summary:
		//    This is an override of a superclass function.
		this._isReviewSaveInProgress = true;
		this._updateDoneButtonEnablementState();
	},

	reviewSaveIsNoLongerInProgress: function(reviewParser) {
		// summary:
		//    This is an override of a superclass function.
		this._isReviewSaveInProgress = false;
		this._updateDoneButtonEnablementState();
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._started) {
			// Update the "<review-status>" found in the
			// "Participant's Review": <review-status>" UI element.
			this._updateMyReviewStatus(this._currentStatusTD);
			// Update the "<role>" found in the "Your Role: <role>"
			// UI element.
			this._updateRole(this._myRoleTD);
			// Update the Done button enablement state.
			this._updateDoneButtonEnablementState();
		}
	},

	_updateDoneButtonEnablementState: function() {
		this._voteCastingButtons.attr(
			"reviewParser", this.reviewParser);
		this._voteCastingButtons.attr(
			"disabled",
			(this._isReviewSaveInProgress
				|| (! reviewutils.canVote(this.reviewParser)))
		);
		this._voteCastingButtons.updateVotingButtons();
	},

	_createUserIconCell: function() {
		var img = document.createElement("img");
		var onUserInfoComplete = function(mapUserURIToUserInfo) {
			var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
			var userInfo = mapUserURIToUserInfo[myUserURI];
			var photoURI = (
				userInfo ?
				userInfo.photoUri :
				"com.ibm.team.repository.web/ui/internal/admin/users/images/no-photo.png"
			);
			img.className = "userImage";
			img.src = photoURI;
			return mapUserURIToUserInfo;
		};
		ProjectCache.getAllUsers().addCallback(this, onUserInfoComplete);
		return img;
	},

	_createTitleSectionContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var table = document.createElement("table");
		table.className = "titleSectionTable";
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		table.appendChild(tbody);
		tbody.appendChild(tr);
		// Append the icon td element.
		this._appendTableDataElement(tr, this._createUserIconCell());
		// Append the "Participant's Review": <review-status>" td element 
		var td = this._appendTableDataElement(
			tr, this._createMyReviewStatusCell());
		// Append a divider.
		this._appendTableDataElement(
			tr,
			this._createIconCell("com.ibm.rdm.web/common/images/review-banner-divider.gif"));
		// Append the "Your Role: <role>" td element.
		var td = this._appendTableDataElement(tr);
		this._updateRole(td);
		this._myRoleTD = td;
		// Append a divider.
		// Note: css rules will make sure this only shows when section is expanded.
		var doneCellDividerImage = this._createIconCell(
			"com.ibm.rdm.web/common/images/review-banner-divider.gif",
			"doneDividerImage");
		this._appendTableDataElement(tr, doneCellDividerImage);
		// Append a "Done" button.
		// Note: css rules will make sure this only shows when section is expanded.
		var self = this;
		var onVoteCastHandler = function(artifactStatusType, doPromptForComments) {
			self.myReview.markMyReviewStatusAsDone();
		};
		var voteCastingButtons = new VoteCastingButtons(
			{
				viewType: VoteCastingButtons.VIEW_TYPE.Review,
				onVoteCastHandler: onVoteCastHandler
			}
		);
		voteCastingButtons.attr("reviewParser", this.reviewParser);
		voteCastingButtons.updateVotingButtons();
		voteCastingButtons.startup();
		voteCastingButtons.attr(
			"disabled", (! reviewutils.canVote(this.reviewParser)));
		this._appendTableDataElement(tr, voteCastingButtons.domNode);
		this._voteCastingButtons = voteCastingButtons;
		// Finally, return this newly created table.
		return table;
	},

	_createMyReviewStatusCell: function() {
		// summary:
		//    Create a table that will represent the
		//        "Participant's Review: <progressBar>"
		//    or
		//        "Participant's Review: <icon-status> <string-status>"
		//    content within the pane's title area.
		// description:
		//    This is implemented as a table due to the fact that dojo ProgressBar's
		//    are represented as a div with embedded divs and it's probematic to try
		//    to make them inline instead of block (at least on IE).
		//    As a side-effect this will set the following field:
		//        this._currentStatusTD
		var table = document.createElement("table");
		table.className = "titleSectionTable";
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		table.appendChild(tbody);
		tbody.appendChild(tr);
		// Add a "Participant's Review" field label.
		var fieldLabelSpan =
			domutils.createSpannedTextNode(
				"titleFieldLabelSpan",
				this._messages["yourReviewPaneTitle"]);
		this._appendTableDataElement(tr, fieldLabelSpan);
		// Add an appropriate "Participant's Review" field value.
		var td = this._appendTableDataElement(tr);
		td.className = "titleStatusTableData";
		this._updateMyReviewStatus(td);
		this._currentStatusTD = td;
		// Return the table cell element to use.
		return table;
	},

	_updateMyReviewStatus: function(parentNode) {
		// summary:
		//    Update the "<progress-bar>" or "<icon-status> <string-status>"
		//    that appears just to the right of "Participant's Review" in
		//    the pane's title area.
		var self = this;
		var fillWithMyOverallReviewProgressBar = function(parentNode, participantStatusInfo) {
			var percentageVotesCast = participantStatusInfo.percentageVotesCast;
			var reviewProgressBar = new ReviewProgressBar({
				progress: percentageVotesCast,
				viewType: ReviewProgressBar.VIEW_TYPE.Completed
			});
			self._myOverallReviewProgressBar = reviewProgressBar;
			parentNode.appendChild(reviewProgressBar.domNode);
		};
		var fillWithMyOverallReviewIconAndLabel = function(parentNode, myReviewStatusType) {
			var statusDisplayName =
				reviewTypes.getArtifactStatusTypeDisplayName(myReviewStatusType);
			var img = reviewutils.createArtifactStatusTypeImage(myReviewStatusType);
			img.className = "userStatusImage";
			var statusDisplayNameSpan =
				domutils.createSpannedTextNode("titleFieldValueSpan", statusDisplayName);
			parentNode.appendChild(img);
			parentNode.appendChild(statusDisplayNameSpan);
		};
		var getMyReviewCurrentStatusType = function(userURI, participantResultInfos, participantStatusInfo) {
			var reviewArtifactStatusType = participantResultInfos[userURI].artifactStatusType;
			if (! reviewArtifactStatusType) {
				if (participantStatusInfo.percentageVotesCast === 0
					&& participantStatusInfo.numInProgress === 0
					) {
					reviewArtifactStatusType = reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
				}
			}
			return reviewArtifactStatusType;
		};
		var userURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		var participantResultInfos = this.reviewParser.getParticipantResultInfos();
		var participantInfos = this.reviewParser.getParticipantInfos();
		var participantStatusInfo = participantInfos[userURI].participantStatusInfo;
		var myReviewStatusType =
			getMyReviewCurrentStatusType(userURI, participantResultInfos, participantStatusInfo);
		// Clear out the current overall review status info (if needed).
		if (this._myOverallReviewProgressBar) {
			this._myOverallReviewProgressBar.destroyRecursive();
			this._myOverallReviewProgressBar = null;
		}
		// Clear out the last displayed content (if needed).
		if (parentNode.firstChild) {
			domutils.removeChildNodes(parentNode);
		}
		// Add up to date overall review status info.
		if (myReviewStatusType) {
			fillWithMyOverallReviewIconAndLabel(
				parentNode, myReviewStatusType);
		} else {
			fillWithMyOverallReviewProgressBar(
				parentNode, participantStatusInfo);
		}
	},

	_updateRole: function(parentNode) {
		// summary:
		//    Update the "<role>" that appears in the "Your Role: <role>"
		//    section of the pane's title area.
		var role = reviewTypes.getParticipantRoleTypeDisplayName(
			this.reviewParser.getRole());
		var createTextNode = function(label, cssClassName) {
			var textNode = document.createTextNode(label);
			if (cssClassName) {
				textNode.className = cssClassName;
			}
			return textNode;
		};
		// Clear out the last displayed content (if needed).
		if (parentNode.firstChild) {
			domutils.removeChildNodes(parentNode);
		}
		parentNode.appendChild(
			domutils.createSpannedTextNode(
				"titleFieldLabelSpan", this._messages["yourRole"]));
		parentNode.appendChild(
			domutils.createSpannedTextNode("titleFieldValueSpan", role));
	}
});


dojo.declare(
	"com.ibm.rdm.web.review.OverallReviewAccordionPane",
	com.ibm.rdm.web.review._ReviewAccordionPane,
{
	// summary:
	//    Shows title pane content for the "Overall Review" accordion pane.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_reviewStateCellsTR: null,
	_reviewStateTableDataCells: null,
	_reviewOverallCompletionProgressBar: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function(){
		this._reviewStateCellsTR = null;
		this._reviewStateTableDataCells = null;
		this._reviewOverallCompletionProgressBar = null;
		this.inherited(arguments);
	},

	onSelected: function(){
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		this.overallReview.paneHasBeenSelected();
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    This is an override of a superclass function.
		this.inherited(arguments);
		if (this._started) {
			// Update the "<icon-state> <string-state> <progress-bar>" UI element.
			this._createOrUpdateReviewStateCells();
		}
	},

	_createTitleSectionContent: function() {
		// summary:
		//    This is an override of a superclass function.
		var table = document.createElement("table");
		table.className = "titleSectionTable";
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		table.appendChild(tbody);
		tbody.appendChild(tr);
		// Append the icon td element.
		var img = this._createIconCell(
			this._REVIEW_STATE_IMAGE_SRC, "overallReviewImage");
		this._appendTableDataElement(tr, img);
		// Append the "Overall Review: <review-state>" td element 
		var td = this._appendTableDataElement(
			tr, this._createReviewStateCells());
		// Finally, return this newly created table.
		return table;
	},

	_createReviewStateCells: function() {
		// summary:
		//    Create a table that will represent the review state information
		//    in either the format [when review is "Started"]:
		//      Overall Review: Draft -> Started -> Reviewed -> Finalized
		//    or the format [when review is "Paused"]:
		//      Overall Review: Draft -> Paused -> Reviewed -> Finalized
		// description:
		//    This is implemented as a table due to the fact that dojo ProgressBar's
		//    are represented as a div with embedded divs and it's probematic to try
		//    to make them inline instead of block (at least on IE).
		//    As a side-effect this will set the following fields:
		//        this._reviewStateCellsTR
		//        this._reviewStateTableDataCells
		//    and may set the following field:
		//        this._reviewOverallCompletionProgressBar
		var table = document.createElement("table");
		table.className = "titleSectionTable";
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		table.appendChild(tbody);
		tbody.appendChild(tr);
		// Add a "Overall Review" field label.
		var fieldLabelSpan =
			domutils.createSpannedTextNode(
				"titleFieldLabelSpan",
				this._messages["overallReviewPaneTitle"]);
		this._appendTableDataElement(tr, fieldLabelSpan);
		// Add the state transitions after the "Overall Review" field value.
		this._reviewStateCellsTR = tr;
		this._createOrUpdateReviewStateCells();
		// Return the table cell element to use.
		return table;
	},

	_createOrUpdateReviewStateCells: function() {
		// summary:
		//    Add the review state cells to the title area.
		// description:
		//    The review state cells will be composed of either:
		//      Draft -> Started -> Reviewed -> Finalized
		//    or:
		//      Draft -> Paused -> Reviewed -> Finalized
		//    and only one of these states will displayed as the active state.
		//    As a side-effect this will set the following field:
		//        this._reviewStateTableDataCells
		//    and may set the following field:
		//        this._reviewOverallCompletionProgressBar
		var tr = this._reviewStateCellsTR;
		// Clear out the current overall review status info (if needed).
		if (this._reviewOverallCompletionProgressBar) {
			this._reviewOverallCompletionProgressBar.destroyRecursive();
			this._reviewOverallCompletionProgressBar = null;
		}
		var reviewStateTypes = [
			reviewTypes.REVIEW_STATE_TYPES.Draft,
			(
				this.reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Paused ?
				reviewTypes.REVIEW_STATE_TYPES.Paused :
				reviewTypes.REVIEW_STATE_TYPES.Started
			),
			reviewTypes.REVIEW_STATE_TYPES.Reviewed,
			reviewTypes.REVIEW_STATE_TYPES.Finalized
		];
		var doCreateNewTableDataCells = (null === this._reviewStateTableDataCells);
		if (doCreateNewTableDataCells) {
			this._reviewStateTableDataCells = [];
		}
		var reviewStateTableDataCells = [];
		for (var i = 0; i < reviewStateTypes.length; i++) {
			var reviewStateType = reviewStateTypes[i];
			var isLastState = (i === reviewStateTypes.length-1);
			if (doCreateNewTableDataCells) {
				// Create a TD to hold the review state icon.
				var td = this._appendTableDataElement(tr);
				this._addReviewStateContent(td, reviewStateType);
				// Save the review state TD away so that we can get to it
				// easily when we need to refresh.
				this._reviewStateTableDataCells.push(td);
				// If needed, create a TD to hold the review state transition
				// arrow icon.
				if (!isLastState) {
					td = this._appendTableDataElement(tr);
					this._addTransitionArrowContent(td);
				}
			} else {
				var td = this._reviewStateTableDataCells[i];
				// Clear out the last displayed content.
				domutils.removeChildNodes(td);
				// Update the TD content.
				this._addReviewStateContent(td, reviewStateType);
			}
		}
	},

	_addTransitionArrowContent: function(parentNode) {
		var img = document.createElement("img");
		img.src = "com.ibm.rdm.web/review/images/icons/obj16/state-arrows.gif";
		img.className = "titleFieldValueImage";
		parentNode.appendChild(img);
	},

	_addReviewStateContent: function(parentNode, reviewStateType) {
		// summary:
		//    Update the "<icon-state> <string-state>" or
		//    "<icon-started-state> In progress <progress-bar>" that appears
		//    in one of the cells to the right of "Overall Review" in the pane's
		//    title area.
		//    As a side-effect this function may set the following field:
		//        this._reviewOverallCompletionProgressBar
		var self = this;
		var appendReviewStateIconAndLabel = function(parentNode, reviewStateType) {
			// Note: UX wants "Started" to display as "In progress" in the context of this
			// pane's title ...
			var statusDisplayName = (
				reviewStateType === reviewTypes.REVIEW_STATE_TYPES.Started ?
				self._messages["inProgress"] :
				statusDisplayName = reviewTypes.getReviewStateDisplayName(reviewStateType)
			);
			var img = reviewutils.createReviewStateImage(reviewStateType);
			img.className = "titleFieldValueImage";
			var statusDisplayNameSpan =
				domutils.createSpannedTextNode("titleFieldValueSpan", statusDisplayName);
			parentNode.appendChild(img);
			parentNode.appendChild(statusDisplayNameSpan);
		};
		var appendReviewOverallCompletionProgressBar = function(parentNode, percentageCompleted) {
			var reviewProgressBar = new ReviewProgressBar({
				progress: percentageCompleted,
				viewType: ReviewProgressBar.VIEW_TYPE.Completed
			});
			self._reviewOverallCompletionProgressBar = reviewProgressBar;
			parentNode.appendChild(reviewProgressBar.domNode);
		};
		// Add an icon and label to show the current state.  If the current
		// state is Started, we also add an overall progress bar.
		var currentReviewStateType = this.reviewParser.getReviewState();
		if (reviewStateType === reviewTypes.REVIEW_STATE_TYPES.Started
			&& currentReviewStateType === reviewTypes.REVIEW_STATE_TYPES.Started
			) {
			var table = document.createElement("table");
			var tbody = document.createElement("tbody");
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			table.appendChild(tbody);
			tbody.appendChild(tr);
			tr.appendChild(td);
			appendReviewStateIconAndLabel(td, reviewStateType);
			td = document.createElement("td");
			td.className = "titleStatusTableData";
			tr.appendChild(td);
			var percentCompleted = this.reviewParser.getPercentReviewCompleted();
			appendReviewOverallCompletionProgressBar(td, percentCompleted);
			parentNode.appendChild(table);
		} else {
			appendReviewStateIconAndLabel(parentNode, reviewStateType);
		}
		if (currentReviewStateType === reviewStateType) {
			dojo.removeClass(parentNode, "titleStatusTableData");
			dojo.addClass(parentNode, "activeTitleStatusTableData");
		} else {
			dojo.removeClass(parentNode, "activeTitleStatusTableData");
			dojo.addClass(parentNode, "titleStatusTableData");
		}
	}
});


})();
