/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewArtifactsCommentWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.CollapseableTitleSection");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");
dojo.require("dijit.Toolbar");
dojo.require("com.ibm.rdm.web.commenting.CommentingClient");
dojo.require("com.ibm.rdm.web.search.results.CommentEntryTable");
dojo.requireLocalization("com.ibm.rdm.web.review", "ReviewArtifactsCommentWidget");
dojo.require("dojox.collections.Dictionary");


dojo.declare("com.ibm.rdm.web.review.ReviewArtifactsCommentWidget", 
	[dijit._Widget, dijit._Templated], 
	
{
    // summary:
	//    Provides a viewer for displaying the comments associated to the artifacts that are
	//    in the review.
    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ReviewArtifactsCommentWidget.html"),

	widgetsInTemplate: true,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
    viewer: null,
	resourceUri: null,
    hasStarted: false,
	
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    
    // widget elements
	_commentsListNode: null,
	// tBody for top-level table, the rows in this table hold comment entries (CommentEntryTables)
	// in No Grouping mode or groups (CollapseableTitleSections) in group mode
	_commentsTableTbody: null,
	_contentPane: null,
	_sortByDateMenuItem: null, 
	_sortByParticipantMenuItem: null, 
	_sortByArtifactMenuItem: null, 
	_noGroupingMenuItem: null, 
	_groupByParticipantMenuItem: null, 
	_groupByArtifactMenuItem: null, 

	_messages: null,
	_sidebarSection: null,
	_comments: null,	
	_reviewParser: null,	
	
	_sortBy: null,
	_groupBy: null,
	_currentSortSelection: null,
	_currentGroupSelection: null,
	
	_projectURI: null,
	
	// dictionary of comments grouped by participant and artifact
	_commentsByParticipant: null,
	_commentsByArtifact: null,
	
	// dictionary of CommentEntryTables keyed by comment.linkSelfUri, these
	// tables can be reused as sort/group modes are changed
	_commentEntries: null,
	// dictionary of CollapseableTitleSections keyed by group id 
	_commentGroups: null,
	
	// tBodies for group tables, the rows in a group table hold comment entries (CommentEntryTables)
	_commentGroupTables: null,
	// non-participants can comment on review artifacts, use this key for grouping non-participant
	// comments when in Group By Participant mode
	_OTHERS_KEY: "others",
	
	// project users information
	_userInfo: null,
	 
	_deferredArray: null,
	
	_commentsCreated: false,
	
    postCreate: function() {
    	this.inherited("postCreate", arguments);
        this._deferredArray = [];
    },

    startup: function() {
    	this.hasStarted = true;
        this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this._handleWindowResize, force: true});
        this._sortBy = com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_DATE;
        this._groupBy = com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_NOGROUPING;
        this._currentSortSelection = this._sortByDateMenuItem;
        this._currentGroupSelection = this._noGroupingMenuItem;
 		this._createCommentsTable(this._commentsListNode);
 		this._commentEntries = new dojox.collections.Dictionary();
 		this._commentGroups = new dojox.collections.Dictionary();
 		this._commentGroupTables = new dojox.collections.Dictionary();
 		this._comments = new Array();
 		
        this.loadComments();
        this._resizeContentPaneIfNeeded();
    },
    
    postMixInProperties: function() {
        this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "ReviewArtifactsCommentWidget");
	},
	
    destroy: function() {
        jazz.util.ResizeNotifier.unregister(this._resizeEvent);
        this._destroyWidgets();
        if (this._deferredArray) {
            for (var i = 0; i < this._deferredArray.length; i++) {
                var deferred = this._deferredArray[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        }
		this.inherited(arguments);
    },
    
	setSidebarSection: function(sidebar) {
    	this._sidebarSection = sidebar;
    },
    
    loadComments: function() {
    	var self = this;
    	this._showMessage(this._messages.loadingComments);
    	if (this._commentsCreated) {
    		// clear out existing comments
			for(var i = this._commentsTableTbody.rows.length; i > 0;i--) {
				var cell = this._commentsTableTbody.rows[i-1].cells[0];
				com.ibm.rdm.web.util.domutils.removeChildNodes(cell);
				this._commentsTableTbody.deleteRow(i -1);
			}
		}
		// get associated review
		var onReviewLoaded = function(response) {
			self._reviewParser = response.doc;
            self._showComments();
            self._commentsCreated = true;
			return response;
		};
		var deferred = com.ibm.rdm.web.review.ReviewCache.getReview(this.viewer.reviewId);
		deferred.addCallback(onReviewLoaded);

    },
    
    setProjectURI: function(projURI) {
    	this._projectURI = projURI;
    },
    
    refreshComments: function() {
    	if (this._commentsCreated) {
    		// TODO (LPK) - save expanded state of groups and reinstate 
    		this._showComments();
    	}
    },
    
    _showComments: function() {
    	var self = this;
        var deferred = com.ibm.rdm.web.commenting.CommentingClient.getReviewComments(this._reviewParser.getReviewId());
        this._deferredArray.push(deferred);
		var onCommentsRetrieved = function(response) {
        	self._resetState(true);
			self._comments = new Array();
			// comments could reference artifacts that are no longer in the review, remove these comments
			var artifactInfos = self._reviewParser.getArtifactInfos();
			var resourceURIs = new dojox.collections.Dictionary();
			for (var i = 0; i < artifactInfos.length; i++) {
				var artifactInfo = artifactInfos[i];
				resourceURIs.add(artifactInfo.resourceURI, null);
			}
			for (var i = 0; i < response.length; i++) {
				var comment = response[i];
				comment.projectURI = self._projectURI;
				if (resourceURIs.containsKey(comment.resourceUri)) {
					self._comments.push(comment);
				}
			}
			self._createCommentEntries();
			return response;
		};
		deferred.addCallback(onCommentsRetrieved);
        deferred.addErrback(this, function(error) {
        	this._clearMessage();
        	// TODO (LPK) reset state as appropriate
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7213, [], error
    		);
        });
        return deferred;
    },
    
    _createCommentEntries: function(sortOnly) {
		this._sortComments();
 		this._groupComments();
		
		// remove rows in the existing tables
		var groupTables = this._commentGroupTables.getValueList();
		for(var i = groupTables.length; i > 0;i--) {
			var groupTable = groupTables[i-1];
			for (var j = groupTable.rows.length; j > 0; j--) {
				var cell = groupTable.rows[j-1].cells[0];
				com.ibm.rdm.web.util.domutils.removeChildNodes(cell);
				groupTable.deleteRow(j-1);
			}
		}
		
		// no need to recreate the groups in sortOnly mode
		if (!sortOnly || this._groupBy == com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_NOGROUPING) {
			this._commentGroupTables = new dojox.collections.Dictionary();
			this._commentGroups = new dojox.collections.Dictionary();
			for(var i = this._commentsTableTbody.rows.length; i > 0;i--) {
				var cell = this._commentsTableTbody.rows[i-1].cells[0];
				com.ibm.rdm.web.util.domutils.removeChildNodes(cell);
				this._commentsTableTbody.deleteRow(i -1);
			}
		}

		if (this._groupBy == com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_PARTICIPANT) {
			if (sortOnly) {
				var groupIds = this._commentGroups.getKeyList();
				for (var i=0; i<groupIds.length; i++) {
					var groupId = groupIds[i];
					var comments = this._commentsByParticipant.item(groupId);
					var groupTBody = this._commentGroupTables.item(groupId);
					for (var j=0; j<comments.length; j++) {
            			this._createCommentEntry(comments[j], groupTBody);
					}
				}
			} else {
				// create participant group panes, add comments
				var self = this;
				// gather participant URIs, grab names, sort
        		var sortedUsers = new Array();
        		function userComparator(userURI1, userURI2) {
        			var userInfo1 = self._userInfo[userURI1];
        			var userInfo2 = self._userInfo[userURI2];
        			if (!userInfo1) return -1;
        			if (!userInfo2) return 1;
					var a = userInfo1.name.toLowerCase();
					var b = userInfo2.name.toLowerCase();
           			return (a>b)?1:((a<b)?-1:0);
       			}
				var participantInfos = this._reviewParser.getParticipantInfos();
				for (var i=0; i < participantInfos.length; i++) {
					var participantInfo = participantInfos[i];
					sortedUsers.push(participantInfo.userURI);
				}
				sortedUsers.sort(userComparator);
				for (var i=0; i < sortedUsers.length; i++) {
					var userURI = sortedUsers[i];
					var comments = this._commentsByParticipant.item(userURI);
        			var userInfo = this._userInfo[userURI];
        			var userName = null;
        			if (userInfo) {
        				userName = userInfo.name;
        			} else {
        				userName = userURI;
        			}
					var title = dojo.string.substitute(this._messages.groupN, [userName, comments.length]);
					var group = this._createCommentGroup(userURI, title, this._commentsTableTbody);
					group.collapse();
					var groupTBody = this._createGroupCommentsTable(group);
					this._commentGroupTables.add(userURI, groupTBody);
					for (var j=0; j<comments.length; j++) {
            			this._createCommentEntry(comments[j], groupTBody);
					}
				};
				var comments = this._commentsByParticipant.item(this._OTHERS_KEY);
				var title = dojo.string.substitute(this._messages.groupN, [this._messages.nonParticipants, comments.length]);
				var group = this._createCommentGroup(this._OTHERS_KEY, title, this._commentsTableTbody);
				group.collapse();
				var groupTBody = this._createGroupCommentsTable(group);
				this._commentGroupTables.add(this._OTHERS_KEY, groupTBody);
				for (var j=0; j<comments.length; j++) {
            		this._createCommentEntry(comments[j], groupTBody);
				}
			}
		} else if (this._groupBy == com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_ARTIFACT) {
			if (sortOnly) {
				var groupIds = this._commentGroups.getKeyList();
				for (var i=0; i<groupIds.length; i++) {
					var groupId = groupIds[i];
					var comments = this._commentsByArtifact.item(groupId);
					var groupTBody = this._commentGroupTables.item(groupId);
					for (var j=0; j<comments.length; j++) {
            			this._createCommentEntry(comments[j], groupTBody);
					}
				}
			} else {
				// create artifact group panes, add comments, artifactInfos should represent the
				// appropriate order in which to display the artifacts
				var artifactInfos = this._reviewParser.getArtifactInfos();
				for (var i=0; i < artifactInfos.length; i++) {
					var artifactInfo = artifactInfos[i];
					var comments = this._commentsByArtifact.item(artifactInfo.resourceURI);
					var title = dojo.string.substitute(this._messages.groupN, [artifactInfo.artifactName, comments.length]);
					var group = this._createCommentGroup(artifactInfo.resourceURI, title, this._commentsTableTbody);
					group.collapse();
					var groupTBody = this._createGroupCommentsTable(group);
					this._commentGroupTables.add(artifactInfo.resourceURI, groupTBody);
					for (var j=0; j<comments.length; j++) {
            			this._createCommentEntry(comments[j], groupTBody);
					}
				};
			}
		} else {
        	for (var i = 0; i < this._comments.length; i++) {
            	this._createCommentEntry(this._comments[i], this._commentsTableTbody);
        	}
		}
		var message = dojo.string.substitute(this._messages.artifactCommentsN, [this._comments.length]);
 		this._showMessage(message);
    },
    
    _groupComments: function() {
		switch (this._groupBy) {
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_PARTICIPANT:
    			this._commentsByParticipant = new dojox.collections.Dictionary();
				var participantInfos = this._reviewParser.getParticipantInfos();
				for (var i = 0; i < participantInfos.length; i++) {
					var participantInfo = participantInfos[i];
					this._commentsByParticipant.add(participantInfo.userURI, new Array());
				}
				this._commentsByParticipant.add(this._OTHERS_KEY, new Array());
    			for (var i=0; i<this._comments.length; i++) {
    				var comment = this._comments[i];
    				var commentAuthor = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS + "/" + encodeURIComponent(comment.author); 
    				var commentGroup = this._commentsByParticipant.item(commentAuthor);
    				if (commentGroup) {
    					commentGroup.push(comment);
    				} else {
    					var commentGroup = this._commentsByParticipant.item(this._OTHERS_KEY);
    					commentGroup.push(comment);
    				}
    			}
				break;
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_ARTIFACT:
    			this._commentsByArtifact = new dojox.collections.Dictionary();
				var artifactInfos = this._reviewParser.getArtifactInfos();
				for (var i = 0; i < artifactInfos.length; i++) {
					var artifactInfo = artifactInfos[i];
					this._commentsByArtifact.add(artifactInfo.resourceURI, new Array());
				}
    			for (var i=0; i<this._comments.length; i++) {
    				var comment = this._comments[i];
    				var commentGroup = this._commentsByArtifact.item(comment.resourceUri);
    				commentGroup.push(comment);
    			}
				break;
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_NOGROUPING:
				break;
		};
    },
    
    _sortComments: function() {
    	var self = this;
    	
        function dateComparator(comment1, comment2) {
           var a = new Date(comment1.updated);
           var b = new Date(comment2.updated);
           return (a<b)?1:((a>b)?-1:0);
        }
        
        function artifactComparator(comment1, comment2) {
			var resDesc1 = self._reviewParser.getArtifactInfo(comment1.resourceUri);
			var resDesc2 = self._reviewParser.getArtifactInfo(comment2.resourceUri);
			var a = resDesc1.artifactName;
			var b = resDesc2.artifactName;
           	return (a>b)?1:((a<b)?-1:0);
       	}
       	
        function userComparator(comment1, comment2) {
       	    var a = comment1.author.toLowerCase();
            var b = comment2.author.toLowerCase();
           	return (a>b)?1:((a<b)?-1:0);
       	}
       	
		switch (this._sortBy) {
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_DATE:
				// sort by artifact within date
				this._comments.sort(artifactComparator);
				this._comments.sort(dateComparator);
				break;
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_ARTIFACT:
				// sort by date within artifact
				this._comments.sort(dateComparator);
				this._comments.sort(artifactComparator);
				break;
			case com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_PARTICIPANT:
				// sort by date within users
				this._comments.sort(dateComparator);
				this._comments.sort(userComparator);
				break;
		};
    },
    
    _createGroupCommentsTable: function(groupCollapseableSection) {
		// summary:
		// 	Create a table for the group.  The rows in the table will hold comment
		//	entries.
		// returns: tBody for the table, used for deleting rows in the table
    	var commentsTable = document.createElement("table");
 		if (!dojo.isIE) {
        	commentsTable.setAttribute("cellSpacing", "0");
  		}
		commentsTable.className = "results-table";
        var tBody = document.createElement("tbody");
        commentsTable.appendChild(tBody);
    	groupCollapseableSection.setContent(commentsTable);
        return tBody;
    },
    
    _createCommentsTable: function() {
		// summary:
		// 	Create a table for the section.  The rows in the table will hold comment
		//	entries in No Grouping mode or groups in Group By mode
		// returns: tBody for the table, used for deleting rows in the table
    	var commentsTable = document.createElement("table");
 		if (!dojo.isIE) {
        	commentsTable.setAttribute("cellSpacing", "0");
  		}
		commentsTable.className = "results-table";
    	this._commentsListNode.appendChild(commentsTable);
        this._commentsTableTbody = document.createElement("tbody");
        commentsTable.appendChild(this._commentsTableTbody);
    },
    
    _createCommentGroup: function(groupId, name, tbody) {
		// summary:
		// 	Create a CollapseableTitleSection for grouping comments
		// returns: CollapseableTitleSection
        var tr = document.createElement("tr");
		tbody.appendChild(tr);
		var group = this._commentGroups.item[groupId];
		if (!group) {
			group = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			group.setTitle(name);
			this._commentGroups.add(groupId, group);
		}
     	var tableColumn = com.ibm.rdm.web.search.domutils.createTD(group.domNode);
     	tr.appendChild(tableColumn);
     	return group;
    },
    
    _createCommentEntryTable: function(comment, resDesc) {
		var resDesc = this._reviewParser.getArtifactInfo(comment.resourceUri);
		var entryTable = null;
		if (resDesc) {
			entryTable = new com.ibm.rdm.web.search.results.CommentEntryTable(
				{
					comment:comment, 
					artifactName: resDesc.artifactName,
  					artifactContentType: resDesc.contentType,
  					artifactReviewRevision: resDesc.revision
      			}
			, null);
		} else {
     		entryTable = new com.ibm.rdm.web.search.results.CommentEntryTable(
            	{
            		comment:comment, 
              		artifactName: "",
              		artifactContentType: null
            	}
          	, null);
  		}
      	return entryTable;
    },
    
    _createCommentEntry: function(comment, tbody) {
        var tr = document.createElement("tr");
		tbody.appendChild(tr);
		var commentEntry = this._commentEntries.item(comment.linkSelfUri);
		var entryTable = null;
		if (commentEntry) {
			entryTable = commentEntry;
		} else {
			var resDesc = this._reviewParser.getArtifactInfo(comment.resourceUri);
			entryTable = this._createCommentEntryTable(comment, resDesc);
        	this._commentEntries.add(comment.linkSelfUri, entryTable);    
      		entryTable.addToolTip();
      		entryTable.startup();
		}
             
     	var tableColumn = com.ibm.rdm.web.search.domutils.createTD(entryTable.domNode);
     	tr.appendChild(tableColumn);
		this.connect(tableColumn, "onmouseover", function(event) {
			this._onCommentMouseOver(event, tableColumn, entryTable);
		});
   		this.connect(tableColumn, "onmouseout", function(event) {
			this._onCommentMouseOut(event, tableColumn, entryTable);
		});
    	return tr;
	},
        
    _onCommentMouseOver: function(event, tableColumn, entry) {
        if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        dojo.addClass(tableColumn, "highlight");
        dojo.removeClass(entry._replyToAnchor, "hideElement");
        dojo.addClass(entry._replyToAnchor, "showElement"); 
        if (!entry.comment.isElementDeleted) {
    		dojo.addClass(entry._replyToAnchor, "enabled");
    		dojo.removeClass(entry._replyToAnchor, "disabled");
        } else {
    		dojo.removeClass(entry._replyToAnchor, "enabled");
    		dojo.addClass(entry._replyToAnchor, "disabled");
        }  
    },

	_onCommentMouseOut: function(event, tableColumn, entry) {
        if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        dojo.removeClass(tableColumn, "highlight");
        dojo.removeClass(entry._replyToAnchor, "showElement");
        dojo.addClass(entry._replyToAnchor, "hideElement");   
    },
        
    _showMessage: function(message) {
    	this._clearMessage();
        this._sidebarSection.setSectionTitleInfo(message);
    },
    
    _clearMessage: function() {
    	this._sidebarSection.clearSectionTitleInfo();
    },
        
    _resizeContentPaneIfNeeded: function() {
    	var self = this;
    	var domBasedResizeContentPane = function() {
        	if (!self.domNode.parentNode) { return; }
       		var height = self.domNode.parentNode.clientHeight;
     		var width = self.domNode.parentNode.clientWidth;
       		var contentPaneTop = (self._contentPane.domNode.offsetTop - self._contentPane.domNode.parentNode.offsetTop);
      		var scrollHeightToUse = (height - contentPaneTop);
       		self._contentPane.resize({ h: scrollHeightToUse, w: width });
     	};
     	if (this._contentPane) {
        	domBasedResizeContentPane();
    	}
	},

    _handleWindowResize: function() {
        this._resizeContentPaneIfNeeded();
    },
        
    containerDimensionsHaveChanged: function() {
        this._resizeContentPaneIfNeeded();
    },

    _resetState: function(destroyWidgets) {
 		this._commentsByArtifact = null;
 		this._commentsByParticipant = null;
    	if (destroyWidgets) {
        	this._destroyWidgets();
    	} else {
    	}
    },
    
    _destroyWidgets: function() {
    	if (this._commentEntries) {
        	var entries = this._commentEntries.getValueList();
    		for (var i=0; i<entries; i++) {
    			var entry = entries[i];
    			entry.destroyRecursive();
    		}
    	}
    	if (this._commentGroups) {
        	var groups = this._commentGroups.getValueList();
    		for (var i=0; i<groups; i++) {
    			var group = groups[i];
    			group.destroyRecursive();
    		}
    	}
    },
    
    _updateSortingGrouping: function(sortBy, groupBy, menuItem) {
    	this._showMessage(this._messages.sortingComments);
		this._resetState(false);
		var previousSelection;
				
		if (sortBy) {
			this._sortBy = sortBy;
			previousSelection = this._currentSortSelection;
			this._currentSortSelection = menuItem;
		}
		
		if (groupBy) {
			this._groupBy = groupBy;
			previousSelection = this._currentGroupSelection;
			this._currentGroupSelection = menuItem;
			
			if (groupBy == com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_PARTICIPANT) {
				this._sortByParticipantMenuItem.attr("disabled", true);
				if (this._currentSortSelection == this._sortByParticipantMenuItem) {
					dojo.removeClass(this._sortByParticipantMenuItem.iconNode, "commentingToolbarDropdownBullet");
					dojo.addClass(this._sortByDateMenuItem.iconNode, "commentingToolbarDropdownBullet");
					this._currentSortSelection = this._sortByDateMenuItem;
				}
				this._sortByArtifactMenuItem.attr("disabled", false);
			} else if (groupBy == com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_ARTIFACT) {
				this._sortByParticipantMenuItem.attr("disabled", false);
				this._sortByArtifactMenuItem.attr("disabled", true);
				if (this._currentSortSelection == this._sortByArtifactMenuItem) {
					dojo.removeClass(this._sortByArtifactMenuItem.iconNode, "commentingToolbarDropdownBullet");
					dojo.addClass(this._sortByDateMenuItem.iconNode, "commentingToolbarDropdownBullet");
					this._currentSortSelection = this._sortByDateMenuItem;
				}
			} else {
				this._sortByParticipantMenuItem.attr("disabled", false);
				this._sortByArtifactMenuItem.attr("disabled", false);
			} 
		}
	    dojo.removeClass(previousSelection.iconNode, "commentingToolbarDropdownBullet");
		dojo.addClass(menuItem.iconNode, "commentingToolbarDropdownBullet");
	},
	
	_onFilterClick: function(event) {
	},
	
	_onSortByDateMenuItemClick: function(event) {
		this._updateSortingGrouping(com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_DATE,
			null,
			this._sortByDateMenuItem);
		this._createCommentEntries(true);
	},
	
	_onSortByParticipantMenuItemClick: function(event) {
		var self = this;
		var onUsersLoaded = function(response) {
         	self._userInfo = response;
			self._updateSortingGrouping(com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_PARTICIPANT,
				null,
				self._sortByParticipantMenuItem);
			self._createCommentEntries(true);
		};
		var onUsersLoadedError = function(error) {
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7215, [], error
    		);
		};
		var deferred = com.ibm.rdm.web.common.ProjectCache.getProjectUsers(this._projectURI);
		this._deferredArray.push(deferred);
		deferred.addCallback(onUsersLoaded);	
		deferred.addErrback(onUsersLoadedError);	
	},
		
	_onSortByArtifactMenuItemClick: function(event) {
		this._updateSortingGrouping(com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.SORTBY_ARTIFACT,
			null,
			this._sortByArtifactMenuItem);
		this._createCommentEntries(true);
	},
		
	_onNoGroupingMenuItemClick: function(event) {
		this._updateSortingGrouping(null,
			com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_NOGROUPING, 
			this._noGroupingMenuItem);
		this._createCommentEntries();
	},
		
	_onGroupByParticipantMenuItemClick: function(event) {
		var self = this;
		var onUsersLoaded = function(response) {
         	self._userInfo = response;
			self._updateSortingGrouping(null,
				com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_PARTICIPANT, 
				self._groupByParticipantMenuItem);
			self._createCommentEntries();
		};
		var onUsersLoadedError = function(error) {
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7215, [], error
    		);
		};
		var deferred = com.ibm.rdm.web.common.ProjectCache.getProjectUsers(this._projectURI);
		this._deferredArray.push(deferred);
		deferred.addCallback(onUsersLoaded);	
		deferred.addErrback(onUsersLoadedError);	
	},
		
	_onGroupByArtifactMenuItemClick: function(event) {
		this._updateSortingGrouping(null, 
			com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering.GROUPBY_ARTIFACT, 
			this._groupByArtifactMenuItem);
		this._createCommentEntries();
	}
	
});

})();


com.ibm.rdm.web.review.ReviewArtifactsCommentWidget.Ordering = {
	SORTBY_PARTICIPANT: "SORTBY_PARTICIPANT",
	SORTBY_ARTIFACT: "SORTBY_ARTIFACT",
	SORTBY_DATE: "SORTBY_DATE",
	GROUPBY_NOGROUPING: "GROUPBY_NOGROUPING",
	GROUPBY_ARTIFACT: "GROUPBY_ARTIFACT",
	GROUPBY_PARTICIPANT: "GROUPBY_PARTICIPANT"
};
