/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewArtifactsViewerSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ReviewHoverPopup");
dojo.require("com.ibm.rdm.web.review.ReviewProgressBar");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ReviewHoverPopup = com.ibm.rdm.web.review.ReviewHoverPopup;
var ReviewProgressBar = com.ibm.rdm.web.review.ReviewProgressBar;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;


dojo.declare(
	"com.ibm.rdm.web.review.ReviewArtifactsViewerSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a widget used to display a list of artifacts under review
	//    within a particular Review artifact (but does NOT support casting
	//    votes for these artifacts).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ReviewArtifactsViewerSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_reviewArtifactsTableBody: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	reviewParser: null,
	onArtifactRowHighlighted: null,
	onArtifactRowUnhighlighted: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_mapResourceURIToTableRow: null,
	// _connectHandles: Array
	//    Contains all dojo.connect handles that we need to dojo.disconnect
	//    whenever application refresh is requested.
    _connectHandles: null,
	// _refreshWidgets: Array
	//    Contains all widgets that should be explicitly destroyed every time
	//    application refresh is requested.
	_refreshWidgets: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._mapResourceURIToTableRow = new Object();
		this._connectHandles = [];
		this._refreshWidgets = [];
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	startup: function() {
		this._updateArtifactsTable();
	},

	destroy: function() {
		this._removeConnectHandles();
		this.inherited("destroy", arguments);
	},
	
	highlightArtifactIndex: function(artifactIndex) {
		var tr = this._reviewArtifactsTableBody.childNodes[artifactIndex];
		dojo.addClass(tr, "highlightRow");
	},

	unhighlightArtifactIndex: function(artifactIndex) {
		var tr = this._reviewArtifactsTableBody.childNodes[artifactIndex];
		dojo.removeClass(tr, "highlightRow");
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		this.reviewParser = reviewParser;
		this._updateArtifactsTable();
	},

	_updateArtifactsTable: function() {
		// If table rows are currently present, remove them.
		if (this._reviewArtifactsTableBody.firstChild) {
			// Clear out the event handlers used for such things as highlighting
			// rows and clicking on rows.
			this._removeConnectHandles();
			// Clear out the widgets.  For every row, we've got one widget; the
			// ResourceLink found in the second TD element.
			this._destroyAllRefreshWidgets();
			// Remove the rows.
			domutils.removeChildNodes(this._reviewArtifactsTableBody);
		}
		var artifactInfos = this.reviewParser.getArtifactInfos();
		// Add new table rows.
		for (var i = 0; i < artifactInfos.length; i++) {
			var artifactInfo = artifactInfos[i];
			this._addArtifactTableRow(artifactInfo, this.reviewParser.getReviewId());
		}
		// If this is an informal review, we need to update the Artifact name
		// and Version column cells with appropriate information about the
		// current HEAD version (for every artifact in the review).
		if (! this.reviewParser.isFormal()) {
			this._updateAppropriateTableColumnsWithLiveContent();
		}
	},

	_removeConnectHandles: function() {
		if (this._connectHandles) {
			for (var i = 0; i < this._connectHandles.length; i++) {
				dojo.disconnect(this._connectHandles[i]);
			}
		}
		this._connectHandles = [];
	},

	_addRefreshWidget: function(widget) {
		this._refreshWidgets.push(widget);
	},

	_destroyAllRefreshWidgets: function() {
		dojo.forEach(
			this._refreshWidgets,
			function(widget) {
				widget.destroyRecursive();
			}
		);
		this._refreshWidgets = [];
	},

	_updateAppropriateTableColumnsWithLiveContent: function() {
		// summary:
		//    Called for "informal" reviews to populate the Version column and
		//    and Artifact (i.e. the "artifact name") column with information
		//    about the current HEAD versions of all review artifacts.
		var deferred = this.reviewParser.getLiveArtifactsVersionAndNameInfo();
		deferred.addCallback(this, function(resourceDescriptionsObject){
			for (resourceURI in resourceDescriptionsObject) {
				var resourceDescriptionInfo = resourceDescriptionsObject[resourceURI];
				var resourceURI = resourceDescriptionInfo.resourceURI;
				var lastModifiedDate = new Date(resourceDescriptionInfo.lastModifiedDate);
				var displayName = resourceDescriptionInfo.displayName;
				var tr = this._mapResourceURIToTableRow[resourceURI];
				var nameTD = tr.childNodes[0];
				var nameAnchor = nameTD.childNodes[1];
				var versionTD = tr.childNodes[1];
				var versionSpan = versionTD.firstChild;
				nameAnchor.firstChild.nodeValue = displayName;
				versionSpan.className = "";
				versionSpan.firstChild.nodeValue =
					reviewutils.composeLastModifiedDateDisplayLabel(lastModifiedDate);
			}
			return resourceDescriptionsObject;
		});
	},

	// TODO Refactor this ... it's too big ...
	_addArtifactTableRow: function(artifactInfo, reviewId) {
		var tr = document.createElement("tr");
		this._mapResourceURIToTableRow[artifactInfo.resourceURI] = tr;
		this._reviewArtifactsTableBody.appendChild(tr);
		var nameTD = document.createElement("td");
		var versionTD = document.createElement("td");
		var statusTD = document.createElement("td");
		var img = document.createElement("img");
		img.src = MimeTypeRegistry.getImageURI(artifactInfo.contentType);
		img.className = "artifactImage";
		var resourceURI = artifactInfo.resourceURI;
		var revision = artifactInfo.revision;
		var reviewParser = this.reviewParser;
		var doUseArtifactInReviewViewerWhenClickOccurs = (
			revision		// Check if it's a formal review in non-Draft state
			|| ((!reviewParser.isFormal()) // ... or an informal review in a non-Draft state
				&& reviewParser.getReviewState() !== reviewTypes.REVIEW_STATE_TYPES.Draft)
		);
		var anchor =
			ResourceLinksHandler.getResourceLink(
				resourceURI,
				artifactInfo.artifactName,
				true, true);
		if (dojo.isIE) {
			// This is a workaround for Jazz Foundation defect 97086.
			anchor.style.position = "relative";
		}
		this._addRefreshWidget(dijit.byNode(anchor));
		this._connectHandles.push(dojo.connect(anchor, "onclick", function(event) {
			if (doUseArtifactInReviewViewerWhenClickOccurs) {
				dojo.stopEvent(event);
				var href =
					UriHelper.getResourceInReviewContextJafLink(
						resourceURI, revision, reviewId);
				ResourceLinksHandler.showArtifactInReview(href);
			} else {
				ResourceLinksHandler.handleAnchorClicked(event);
			}
		}));
		nameTD.appendChild(img);
		nameTD.appendChild(anchor);
		this._createVersionCellContent(versionTD, artifactInfo);
		var overallArtifactStatusInfo = artifactInfo.overallArtifactStatusInfo;
		var percentageVotesCast = overallArtifactStatusInfo.percentageVotesCast;
		if (percentageVotesCast < 100) {
			var reviewProgressBar = new ReviewProgressBar({
				progress: percentageVotesCast,
				viewType: ReviewProgressBar.VIEW_TYPE.Reviewed
			});
			statusTD.appendChild(reviewProgressBar.domNode);
		} else {
			var img = null;
			var statusLabel = null;
			if (overallArtifactStatusInfo.numDisapproved > 0) {
				img = reviewutils.createArtifactStatusTypeImage(
					reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved);
				img.className = "statusColumnImage";
				statusLabel = reviewTypes.getArtifactStatusTypeDisplayName(
					reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved);
			} else {
				img = reviewutils.createArtifactStatusTypeImage(
					reviewTypes.ARTIFACT_STATUS_TYPES.Approved);
				img.className = "statusColumnImage";
				statusLabel = reviewTypes.getArtifactStatusTypeDisplayName(
					reviewTypes.ARTIFACT_STATUS_TYPES.Approved);
			}
			var statusDisplayNameSpan =
				domutils.createSpannedTextNode("statusColumnTextSpan", statusLabel);
			statusTD.appendChild(img);
			statusTD.appendChild(statusDisplayNameSpan);
		}
		tr.appendChild(nameTD);
		tr.appendChild(versionTD);
		tr.appendChild(statusTD);
		this._addRowHighlightSupport(tr);
		var reviewHoverPopup = new ReviewHoverPopup({
			viewType: ReviewHoverPopup.VIEW_TYPE.SINGLE_ARTIFACT_SUMMARY_STATUS,
			artifactUnderReviewURI: artifactInfo.resourceURI,
			artifactUnderReviewId: artifactInfo.id
        });
		reviewHoverPopup.createHoverPopup(statusTD, this.reviewParser);
	},

	_createVersionCellContent: function(versionTD, artifactInfo) {
		var versionLabelToUse = artifactInfo.versionLabel;
		if (versionLabelToUse.length === 0) {
			versionLabelToUse = (
				this.reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Draft ?
				this._messages["notYetSpecified"] :
				"..."
			);
			versionTD.innerHTML = (
				"<span class=\"versionUnsetSpan\">" + versionLabelToUse + "</span>"
			);
		} else {
			versionTD.innerHTML = (
				"<span>" + versionLabelToUse + "</span>"
			);
		}
	},

	_addRowHighlightSupport: function(tr) {
		var self = this;
		this._connectHandles.push(dojo.connect(tr, "onmouseover", function(event) {
			dojo.stopEvent(event);
			dojo.addClass(tr, "highlightRow");
			if (self.onArtifactRowHighlighted) {
				self.onArtifactRowHighlighted(tr.rowIndex-1);
			}
		}));
		this._connectHandles.push(dojo.connect(tr, "onmouseout", function(event) {
			dojo.stopEvent(event);
			dojo.removeClass(tr, "highlightRow");
			if (self.onArtifactRowUnhighlighted) {
				self.onArtifactRowUnhighlighted(tr.rowIndex-1);
			}
		}));
	}
});


})();
