/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewArtifactsVotingSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.VoteCastingButtons");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewUpdateMediator");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var VoteCastingButtons = com.ibm.rdm.web.review.VoteCastingButtons;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;

var BORDER_AFFORDANCE = 44;


dojo.declare(
	"com.ibm.rdm.web.review.ReviewArtifactsVotingSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a widget used to display a list of artifacts under review
	//    and enables the user to cast votes for these artifacts (approve,
	//    disapprove or abstain).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ReviewArtifactsVotingSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_voteCastingButtonsDiv: null,
	_headerCheckBox: null,
	_statusHeaderSpan: null,
	_reviewArtifactsTable: null,
	_reviewArtifactsTableBody: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceViewer: null,
	reviewParser: null,
	reviewContainer: null,
	groupSection: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_artifactInfos: null,
	_voteCastingButtons: null,
	_numCheckBoxesSelected: null,
	_isReviewReadOnly: null,
	_mapResourceURIToTableRow: null,
	_isReviewSaveInProgress: null,
	// _connectHandles: Array
	//    Contains all dojo.connect handles that we need to dojo.disconnect
	//    whenever application refresh is requested.
	_connectHandles: null,
	// _refreshWidgets: Array
	//    Contains all widgets that should be explicitly destroyed every time
	//    application refresh is requested.
	_refreshWidgets: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		var self = this;
		// Hook up the header cell checkbox.
		this.connect(this._headerCheckBox, "onClick", this._onHeaderCheckBoxClicked);
		// Hook up the vote casting buttons.
		var onVoteCastHandler = function(artifactStatusType, doPromptForComments) {
			self._castArtifactVotes(artifactStatusType, doPromptForComments);
		};
		this._isReviewSaveInProgress = false;
		this._voteCastingButtons = new VoteCastingButtons(
			{
				viewType: VoteCastingButtons.VIEW_TYPE.ArtifactsTable,
				onVoteCastHandler: onVoteCastHandler,
				disabled: true
			}
		);
		this._voteCastingButtonsDiv.appendChild(this._voteCastingButtons.domNode);
		this._connectHandles = [];
		this._numCheckBoxesSelected = 0;
		this._mapResourceURIToTableRow = new Object();
		this._refreshWidgets = [];
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	startup: function() {
		this._isReviewReadOnly = (! reviewutils.canVote(this.reviewParser));
		this._voteCastingButtons.attr("reviewParser", this.reviewParser);
		this._voteCastingButtons.startup();
		this._populateContents();
		this.inherited(arguments);
	},

	destroy: function() {
		this._removeConnectHandles();
		this.inherited("destroy", arguments);
	},

	setWidth: function(width) {
		this._layout();
	},

	reviewSaveIsInProgress: function(reviewParser) {
		// summary:
		//    The review is in the process of being saved.
		// description:
		//    The user has perfomed a UI gesture which has resulted in a save
		//    request going off to the server.  Disable all UI controls whose
		//    action would result in a new save being performed.
		this._isReviewSaveInProgress = true;
		this._updateVoteCastingButtonsEnablementState();
	},

	reviewSaveIsNoLongerInProgress: function(reviewParser) {
		// summary:
		//    The review is no longer in the process of being saved (either the
		//    save succeeded or returned an error code).
		// description:
		//    The outstanding save request that was issued due to the user
		//    performing a UI gesture has completed.  Re-enable the UI controls
		//    whose action would result in a new save being performed.
		this._isReviewSaveInProgress = false;
		this._updateVoteCastingButtonsEnablementState();
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		var oldRole = (this.reviewParser && this.reviewParser.getRole());
		var newRole = reviewParser.getRole();
		var isRoleDifferent = (oldRole != newRole);
		this.reviewParser = reviewParser;
		this._isReviewReadOnly = (! reviewutils.canVote(reviewParser));
		this._voteCastingButtons.attr("reviewParser", reviewParser);
		if (newRole && isRoleDifferent) {
			// The role is different and this means we need to recreate the
			// vote casting buttons.  The reason we recreate these vote casting
			// buttons is due to the fact that the votes you can cast may
			// be role specific (e.g. an Approver's buttons are different
			// than the buttons used by a Reviewer or Optional Reviewer).
			this._voteCastingButtons.updateVotingButtons();
			// Note: The width of the vote casting buttons _may_ now be
			// different as the buttons may have changed.  To accommodate
			// this new width, we re-layout our contents ...
			this._layout();
		}
		if (this._started) {
			this._updateArtifactsTable();
		}
		// It _may_ be the case that the vote casting buttons to the right of
		// the table are currently enabled (as a row may have been selected
		// before the user clicked on the Done button).  Update these buttons
		// now so that they're appropriately disabled, if needed.
		this._updateVoteCastingButtonsEnablementState();
	},

	_showAsBusy: function(isBusy) {
		this.reviewContainer.showAsBusy(isBusy);
		this._updateVoteCastingButtonsEnablementState();
	},

	_layout: function() {
		var voteCastingButtonsWidth = dojo.coords(this._voteCastingButtonsDiv).w;
		if (voteCastingButtonsWidth > 0) {
			var groupSectionWidth = dojo.coords(this.groupSection).w;
			var tableWidthToUse = groupSectionWidth - voteCastingButtonsWidth - BORDER_AFFORDANCE;
			this._reviewArtifactsTable.style.width = tableWidthToUse + "px";
		}
	},

	_removeConnectHandles: function() {
		if (this._connectHandles) {
			for (var i = 0; i < this._connectHandles.length; i++) {
				dojo.disconnect(this._connectHandles[i]);
			}
		}
		this._connectHandles = [];
	},

	_castArtifactVotes: function(artifactStatusType, doPromptForComments) {
		// summary:
		//    Called when the user clicks on one of the vote casting buttons
		//    found to the right of the table.
		// description:
		//    Find all rows that are currently selected and cast a vote for
		//    the artifacts that are found in these selected rows.
		var artifactInfos = this._artifactInfos;
		var childNodes = this._reviewArtifactsTableBody.childNodes;
		var selectedArtifactInfos = [];
		for (var i = 0; i < childNodes.length; i++) {
			tableRow = childNodes[i];
			var checkBox = dijit.byNode(tableRow.firstChild.firstChild);
			if (checkBox.checked) {
				selectedArtifactInfos.push(artifactInfos[i]);
				artifactInfos[i].myArtifactStatusType = artifactStatusType;
			}
		}
		var self = this;
		var onSaveComplete = function() {
			self._showAsBusy(false);
			// Note: We're already hooked up to the reviewChangeNotifier
			// so we there's no need to separately update the UI table row
			// elements here ... but we DO need to update the header cell
			// checkbox and disable the vote casting buttons (until the
			// user next selects a check box).
			self._headerCheckBox.attr("checked", false);
		};
		var onSaveCancelled = function() {
			self._showAsBusy(false);
		};
		var onSaveError = function(response) {
			self._showAsBusy(false);
		};
		this._showAsBusy(true);
		com.ibm.rdm.web.review.reviewUpdateMediator.castArtifactVotes(
			this.reviewParser,
			selectedArtifactInfos,
			doPromptForComments,
			onSaveComplete,
			onSaveCancelled,
			onSaveError
		);
	},

	_populateContents: function() {
		this._voteCastingButtons.updateVotingButtons();
		this._updateArtifactsTable();
		this._updateVoteCastingButtonsEnablementState();
	},

	_addRefreshWidget: function(widget) {
		this._refreshWidgets.push(widget);
	},

	_destroyAllRefreshWidgets: function() {
		dojo.forEach(
			this._refreshWidgets,
			function(widget) {
				widget.destroyRecursive();
			}
		);
		this._refreshWidgets = [];
	},

	_updateArtifactsTable: function() {
		this._headerCheckBox.attr("disabled", this._isReviewReadOnly);
		var artifactInfos = this.reviewParser.getArtifactInfos();
		var reviewId = this.reviewParser.getReviewId();
		var self = this;
		var clearOutLastTable = function() {
			self._numCheckBoxesSelected = 0;
			// Clear out the event handlers used for such things as highlighting
			// rows and clicking on rows.
			self._removeConnectHandles();
			// Clear out the widgets.  For every row, this includes the CheckBox
			// found in the first TD as well as the ResourceLink found in the
			// next TD.
			self._destroyAllRefreshWidgets();
			// Remove the rows.
			domutils.removeChildNodes(self._reviewArtifactsTableBody);
		}
		// If table rows are currently present, remove them.
		if (this._reviewArtifactsTableBody.firstChild) {
			clearOutLastTable();
		}
		// Add new table rows.
		for (var i = 0; i < artifactInfos.length; i++) {
			var artifactInfo = artifactInfos[i];
			this._addArtifactTableRow(artifactInfo, reviewId);
		}
		// If this is an informal review, we need to update the Artifact name
		// and Version column cells with appropriate information about the
		// current HEAD version (for every artifact in the review).
		if (! this.reviewParser.isFormal()) {
			this._updateAppropriateTableColumnsWithLiveContent();
		}
		this._artifactInfos = artifactInfos;
	},

	_updateAppropriateTableColumnsWithLiveContent: function() {
		// summary:
		//    Called for "informal" reviews to populate the Version column and
		//    and Artifact (i.e. the "artifact name") column with information
		//    about the current HEAD versions of all review artifacts.
		var deferred = this.reviewParser.getLiveArtifactsVersionAndNameInfo();
		deferred.addCallback(this, function(resourceDescriptionsObject){
			for (resourceURI in resourceDescriptionsObject) {
				var resourceDescriptionInfo = resourceDescriptionsObject[resourceURI];
				var resourceURI = resourceDescriptionInfo.resourceURI;
				var lastModifiedDate = new Date(resourceDescriptionInfo.lastModifiedDate);
				var displayName = resourceDescriptionInfo.displayName;
				var tr = this._mapResourceURIToTableRow[resourceURI];
				var nameTD = tr.childNodes[1];
				var nameAnchor = nameTD.childNodes[1];
				var versionTD = tr.childNodes[2];
				var versionSpan = versionTD.firstChild;
				nameAnchor.firstChild.nodeValue = displayName;
				versionSpan.className = "";
				versionSpan.firstChild.nodeValue =
					reviewutils.composeLastModifiedDateDisplayLabel(lastModifiedDate);
			}
			return resourceDescriptionsObject;
		});
	},

	_addArtifactTableRow: function(artifactInfo, reviewId) {
		var tr = document.createElement("tr");
		this._mapResourceURIToTableRow[artifactInfo.resourceURI] = tr;
		this._reviewArtifactsTableBody.appendChild(tr);
		// Add TD elements to this TR.
		var appendTableDataElement = function(tr) {
			var td = document.createElement("td");
			tr.appendChild(td);
			return td;
		}
		var checkBoxTD = appendTableDataElement(tr);
		checkBoxTD.className = "checkboxTableData";
		var nameTD = appendTableDataElement(tr);
		var versionTD = appendTableDataElement(tr);
		var statusTD = appendTableDataElement(tr);
		this._createCheckBoxCellContent(checkBoxTD);
		this._createNameCellContent(nameTD, artifactInfo, reviewId);
		this._createVersionCellContent(versionTD, artifactInfo);
		this._createStatusCellContent(statusTD, artifactInfo);
		// Add appropriate event handling to this TR.
		var self = this;
		this._connectHandles.push(dojo.connect(tr, "onmouseover", function(event) {
			dojo.stopEvent(event);
			if (! self.reviewContainer.isBusy()) {
				dojo.addClass(tr, "highlightRow");
			}
		}));
		this._connectHandles.push(dojo.connect(tr, "onmouseout", function(event) {
			dojo.stopEvent(event);
			if (! self.reviewContainer.isBusy()) {
				dojo.removeClass(tr, "highlightRow");
			}
		}));
		var self = this;
		this._connectHandles.push(dojo.connect(tr, "onclick", function(event) {
			dojo.stopEvent(event);
			if ((! self.reviewContainer.isBusy()) && (! self._isReviewReadOnly)) {
				self._toggleRowSection(tr);
			}
		}));
	},

	_toggleRowSection: function(tr) {
		var checkBox = dijit.byNode(tr.firstChild.firstChild);
		checkBox.attr("checked", (! checkBox.checked));
		this._handleCheckBoxHasBeenToggled(checkBox);
	},

	_createCheckBoxCellContent: function(checkBoxTD) {
		var checkBox = new dijit.form.CheckBox({
			onClick: dojo.hitch(this, this._onArtifactCheckBoxClicked),
			disabled: this._isReviewReadOnly
		});
		checkBoxTD.appendChild(checkBox.domNode);
		this._addRefreshWidget(checkBox);
	},

	_createNameCellContent: function(nameTD, artifactInfo, reviewId) {
		var img = document.createElement("img");
		img.src = MimeTypeRegistry.getImageURI(artifactInfo.contentType);
		img.className = "artifactImage";
		var self = this;
		var anchor =
			ResourceLinksHandler.getResourceLink(
				artifactInfo.resourceURI,
				artifactInfo.artifactName,
				true, true);
		if (dojo.isIE) {
			// This is a workaround for Jazz Foundation defect 97086.
			anchor.style.position = "relative";
		}
		this._addRefreshWidget(dijit.byNode(anchor));
		this.connect(anchor, "onclick", function(event) {
			dojo.stopEvent(event);
			var href = UriHelper.getResourceInReviewContextJafLink(
				artifactInfo.resourceURI,
				artifactInfo.revision,
				reviewId
			);
			ResourceLinksHandler.showArtifactInReview(href);
		});
		nameTD.appendChild(img);
		nameTD.appendChild(anchor);
	},

	_createVersionCellContent: function(versionTD, artifactInfo) {
		var versionLabelToUse = artifactInfo.versionLabel;
		if (versionLabelToUse.length === 0) {
			versionLabelToUse = (
				this.reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Draft ?
				this._messages["notYetSpecified"] :
				"..."
			);
			versionTD.innerHTML = (
				"<span class=\"versionUnsetSpan\">" + versionLabelToUse + "</span>"
			);
		} else {
			versionTD.innerHTML = (
				"<span>" + versionLabelToUse + "</span>"
			);
		}
	},

	_createStatusCellContent: function(statusTD, artifactInfo) {
		var artifactStatusType = artifactInfo.myArtifactStatusType;
		var statusDisplayName =
			reviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType);
		var img = reviewutils.createArtifactStatusTypeImage(artifactStatusType);
		img.className = "statusColumnImage";
		var textSpanCssClassName = (
			artifactStatusType === reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted ?
			"statusColumnLightTextSpan" :
			"statusColumnTextSpan"
		);
		var statusDisplayNameSpan =
			domutils.createSpannedTextNode(textSpanCssClassName, statusDisplayName);
		statusTD.appendChild(img);
		statusTD.appendChild(statusDisplayNameSpan);
	},

	_onHeaderCheckBoxClicked: function(event) {
		var checked = this._headerCheckBox.checked;
		dojo.forEach(
			this._reviewArtifactsTableBody.childNodes,
			function(tr) {
				var checkBox = dijit.byNode(tr.firstChild.firstChild);
				checkBox.attr("checked", checked);
			}
		);
		this._numCheckBoxesSelected = (
			checked ?
			this._reviewArtifactsTableBody.childNodes.length :
			0
		);
		this._updateVoteCastingButtonsEnablementState();
	},

	_onArtifactCheckBoxClicked: function(event) {
		dojo.stopEvent(event);
		var checkBox = dijit.byId(event.target);
		this._handleCheckBoxHasBeenToggled(checkBox);
	},

	_updateVoteCastingButtonsEnablementState: function() {
		this._voteCastingButtons.attr(
			"disabled",
			(this._isReviewReadOnly
			    || this._isReviewSaveInProgress
				|| this.reviewContainer.isBusy()
				|| this._numCheckBoxesSelected === 0)
		);
	},

	_handleCheckBoxHasBeenToggled: function(checkBox) {
		this._numCheckBoxesSelected += (checkBox.checked ? 1 : -1);
		this._updateVoteCastingButtonsEnablementState();
	}
});


})();
