/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewContainer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.AccordionContainer");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rdm.web.review.MyReview");
dojo.require("com.ibm.rdm.web.review.OverallReview");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("com.ibm.rdm.web.review.ReviewAccordionPane");
dojo.require("com.ibm.rdm.web.review.reviewChangeNotifier");
dojo.require("com.ibm.rdm.web.review.reviewTypes");


var MyReview = com.ibm.rdm.web.review.MyReview;
var MyReviewAccordionPane = com.ibm.rdm.web.review.MyReviewAccordionPane;
var OverallReview = com.ibm.rdm.web.review.OverallReview;
var OverallReviewAccordionPane = com.ibm.rdm.web.review.OverallReviewAccordionPane;
var ReviewHisoryAccordionPane = com.ibm.rdm.web.review.ReviewHistoryAccordionPane;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var ReviewStateAccordionPane = com.ibm.rdm.web.review.ReviewStateAccordionPane;
var reviewChangeNotifier = com.ibm.rdm.web.review.reviewChangeNotifier;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


dojo.declare(
	"com.ibm.rdm.web.review.ReviewContainer",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    This widget presents content that is either based on MyReview or
	//    OverallReview.

 	templateString: "<div class=\"rdmReviewContainer\"><div dojoAttachPoint=\"_contentDiv\"></div></div>",

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceViewer: null,
	reviewParser: null,
	width: null,
	height: null,
	displayType: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_accordionContainer: null,
	_reviewPages: null,
	_reviewAccordionPanes: null,
	_onReviewChanged: null,
	_onReviewSaveIsInProgress: null,
	_onReviewSaveHasCompleted: null,
	_onReviewSaveHasFailed: null,
	_lastWidth: null,
	_isBusy: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._isBusy = false;
		this._reviewAccordionPanes = [];
		this._reviewPages = [];
		this._populateContents();
		this._contentDiv.appendChild(this._accordionContainer.domNode);
		this.setDimensions(this.width, this.height);
	},

	startup: function() {
		this.inherited(arguments);
		this._accordionContainer.startup();
		this._addReviewListeners();
		this._setWidthOfAccordionPanes(this._lastWidth);
	},

	destroy: function() {
		this._removeReviewListeners();
		this.inherited("destroy", arguments);
	},

	setDimensions: function(width, height) {
		if (width && height) {
			this.domNode.style.width = width + "px";
			this.domNode.style.height = height + "px";
			if (this._accordionContainer) {
				this._accordionContainer.resize({
					w: width,
					h: height
				});
			}
			this._lastWidth = width;
			this._setWidthOfAccordionPanes(width);
		}
	},

	showAsBusy: function(isBusy) {
		if (isBusy) {
			dojo.query("div", this.domNode).style("cursor", "progress");
		} else {
			dojo.query("div", this.domNode).style("cursor", "");
		}
		this._isBusy = isBusy;
	},

	isBusy: function() {
		return this._isBusy;
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setWidthOfAccordionPanes: function(width) {
		if (width > 0) {
			dojo.forEach(
				this._reviewPages,
				function(reviewPage) {
					if (reviewPage.setWidth) {
						reviewPage.setWidth(width);
					}
				}
			);
		}
	},

	_addReviewListeners: function() {
		var self = this;
		var onReviewChanged = function(reviewParser) {
			// Save away this new reviewParser.
			self.reviewParser = reviewParser;
			// When our display of a Review moves from state Draft to Started,
			// we need to add the "Participant's Review" accordion pane.
			var doAddMyReviewPane = false;
			if (reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Started
				&& self._reviewAccordionPanes[0] instanceof com.ibm.rdm.web.review.OverallReviewAccordionPane
				&& reviewParser.getRole() // Returns non-null only if current user is a participant
				) {
				doAddMyReviewPane = true;
			}
			// When our display of a Review moves from state Started, Paused, or
			// Reviewed to Finalized, we need to remove the "Participant's
			// Review" accordion pane.  We also need to remove the
			// "Participant's Review" pane if we notice that the pane is
			// currently present, but the user's role changes such that
			// he/she is no longer a participant in the review.
			else if (
				self._reviewAccordionPanes[0] instanceof com.ibm.rdm.web.review.MyReviewAccordionPane
				&& (reviewParser.getReviewState() === reviewTypes.REVIEW_STATE_TYPES.Finalized
					|| reviewParser.getRole() == null)
				) {
				var myReviewPane = self._reviewAccordionPanes[0];
				self._accordionContainer.removeChild(myReviewPane);
				self._removeMyReviewContentAndAccordionPane();
			}
			// Notify all panes that there's a new reviewParser.
			dojo.forEach(
				self._reviewAccordionPanes,
				function(reviewAccordionPane) {
					reviewAccordionPane.attr("reviewParser", reviewParser);
				}
			);
			// Notify all pages appearing in these panes that there's a new
			// reviewParser.
			dojo.forEach(
				self._reviewPages,
				function(reviewPage) {
					reviewPage.attr("reviewParser", reviewParser);
				}
			);
			// Now add the MyReview pane, if needed.
			if (doAddMyReviewPane) {
				var myReviewPane = self._createMyReviewContentAndAccordionPane();
				self._accordionContainer.addChild(myReviewPane, 0);
			}
		};
		var onReviewSaveIsInProgress = function(reviewParser) {
			var myReviewPane = self._getMyReviewAccordionPane();
			if (myReviewPane) {
				myReviewPane.reviewSaveIsInProgress(reviewParser);
				var myReview = self._getMyReview();
				if (myReview) {
					myReview.reviewSaveIsInProgress(reviewParser);
				}
			}
		};
		var onReviewSaveHasCompletedOrFailed = function(reviewParser) {
			var myReviewPane = self._getMyReviewAccordionPane();
			if (myReviewPane) {
				myReviewPane.reviewSaveIsNoLongerInProgress(reviewParser);
				var myReview = self._getMyReview();
				if (myReview) {
					myReview.reviewSaveIsNoLongerInProgress(reviewParser);
				}
			}
		}
		var onReviewSaveHasCompleted = function(reviewParser) {
			onReviewSaveHasCompletedOrFailed(reviewParser);
		};
		var onReviewSaveHasFailed = function(reviewParser) {
			// Note: The error widget has already reported the problem before
			// this callback is invoked.
			onReviewSaveHasCompletedOrFailed(reviewParser);
		};
		reviewChangeNotifier.addReviewListener(
			this.reviewParser.getSelfUri(),
			onReviewChanged,
			onReviewSaveIsInProgress,
			onReviewSaveHasCompleted,
			onReviewSaveHasFailed);
		this._onReviewChanged = onReviewChanged;
		this._onReviewSaveIsInProgress = onReviewSaveIsInProgress;
		this._onReviewSaveHasCompleted = onReviewSaveHasCompleted;
		this._onReviewSaveHasFailed = onReviewSaveHasFailed;
	},

	_removeReviewListeners: function() {
		if (this._onReviewChanged) {
			reviewChangeNotifier.removeReviewListener(
				this.reviewParser.getSelfUri(),
				this._onReviewChanged,
				this._onReviewSaveIsInProgress,
				this._onReviewSaveHasCompleted,
				this._onReviewSaveHasFailed);
		}
	},

	_getMyReviewAccordionPane: function() {
		return (
			this._reviewAccordionPanes[0] instanceof com.ibm.rdm.web.review.MyReviewAccordionPane ?
			this._reviewAccordionPanes[0] :
			null
		);
	},

	_getMyReview: function() {
		return (
			this._reviewPages[0] instanceof com.ibm.rdm.web.review.MyReview ?
			this._reviewPages[0] :
			null
		);
	},

	_populateContents: function() {
		var review = null;
		var reviewState = this.reviewParser.getReviewState();
		var doShowMyReview = (
			(
				reviewState === reviewTypes.REVIEW_STATE_TYPES.Started
				|| reviewState === reviewTypes.REVIEW_STATE_TYPES.Paused
				|| reviewState === reviewTypes.REVIEW_STATE_TYPES.Reviewed
			)
			&& this.reviewParser.getRole() // Returns non-null only if current user is a participant
		);
		var accordionContainer = new dijit.layout.AccordionContainer({}, null);
		if (doShowMyReview) {
			var myReviewPane = this._createMyReviewContentAndAccordionPane();
			var overallReviewPane = this._createOverallReviewContentAndAccordionPane();
			accordionContainer.addChild(myReviewPane);
			accordionContainer.addChild(overallReviewPane);
		} else {
			var overallReviewPane = this._createOverallReviewContentAndAccordionPane();
			accordionContainer.addChild(overallReviewPane);
		}
		// TODO Implement a ReviewHistoryAccordionPane and make use of it here.
		this._accordionContainer = accordionContainer;
	},

	_createMyReviewContentAndAccordionPane: function() {
		var myReview = new MyReview({
			resourceViewer: this.resourceViewer,
			reviewParser: this.reviewParser,
			reviewContainer: this
		});
		var myReviewPane = new MyReviewAccordionPane({
			title: "one",
			reviewParser: this.reviewParser,
			myReview: myReview
		});
		myReviewPane.attr("content", myReview.domNode);
		// Note: As a side-effect, we add new entries TO THE START OF the
		// '_reviewPages' and '_reviewAccordionPanes' fields.
		this._reviewPages.unshift(myReview);
		this._reviewAccordionPanes.unshift(myReviewPane);
		return myReviewPane;
	},

	_removeMyReviewContentAndAccordionPane: function(myReviewPane) {
		var accordionPane = this._getMyReviewAccordionPane();
		var myReview = this._getMyReview();
		if (accordionPane) {
			this._reviewAccordionPanes.shift();
			accordionPane.destroyRecursive();
		}
		if (myReview) {
			this._reviewPages.shift();
		}
	},

	_createOverallReviewContentAndAccordionPane: function() {
		var overallReview = new OverallReview({
			resourceViewer: this.resourceViewer,
			reviewParser: this.reviewParser,
			reviewContainer: this
		});
		var overallReviewPane = new OverallReviewAccordionPane({
			title: "two",
			reviewParser: this.reviewParser,
			overallReview: overallReview
		});
		overallReviewPane.attr("content", overallReview.domNode);
		// Note: As a side-effect, we add new entries TO THE END OF the
		// '_reviewPages' and '_reviewAccordionPanes' fields.
		this._reviewPages.push(overallReview);
		this._reviewAccordionPanes.push(overallReviewPane);
		return overallReviewPane;
	}
});


})();
