/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
dojo.provide("com.ibm.rdm.web.review.ReviewHoverPopup");

dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");

dojo.require("jazz.ui.HoverPopup");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var HoverPopup = jazz.ui.HoverPopup;
var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var reviewutils = com.ibm.rdm.web.review.reviewutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;


dojo.declare("com.ibm.rdm.web.review.ReviewHoverPopup", [dijit._Widget, dijit._Templated], {
	// summary:
	//    This class is used to enable hover popups in various areas related to
	//    reviews.

	templateString: "<div class=\"rdmReviewHoverPopup\"><div dojoAttachPoint=\"_contentDiv\"></div></div>",

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	// viewType: String
	//    This value MUST set to one of the values found in
	//        com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.
	viewType: null,

	// userURI: String
	//    This field MUST be passed when the viewType is set to
	//        com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.SINGLE_PARTICIPANT_SUMMARY_STATUS.
	//    A userURI as specified in the content of a ReviewParser object.
	userURI: null,

	// artifactUnderReviewURI: String
	//    This field MUST be passed when the viewType is set to
	//        com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.SINGLE_ARTIFACT_SUMMARY_STATUS.
	//    This field should hold a value that originated from a the following
	//    field within a particular ReviewParser:
	//        artifactInfo.resourceURI
	artifactUnderReviewURI: null,
	// artifactUnderReviewId: String
	//    This field MUST be passed when the viewType is set to
	//        com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.SINGLE_ARTIFACT_SUMMARY_STATUS.
	//    This field should hold a value that originated from a the following
	//    field within a particular ReviewParser:
	//        artifactInfo.id
	artifactUnderReviewId: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_hoverPopup: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},
	
	createHoverPopup: function(node, reviewParser) {
		var self = this;
		this._hoverPopup = new HoverPopup({
			around: [node],
			createContents: function() {
				var node = null;
				switch(self.viewType) {
 					case com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.SINGLE_PARTICIPANT_SUMMARY_STATUS:
						node = self._createSingleParticipantSummaryStatus(reviewParser);
						break;
					case com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.SINGLE_ARTIFACT_SUMMARY_STATUS:
						node = self._createSingleArtifactSummaryStatus(reviewParser);
						break;
					// TODO Implement this one ...
// 					case com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE.OVERALL_REVIEW_SUMMARY_STATUS:
				}
				return node;
			}
		}, null);  
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_createSingleParticipantSummaryStatus: function(reviewParser) {
		var userURI = this.userURI;
		var participantInfos = reviewParser.getParticipantInfos();
		var userParticipantInfo = participantInfos[userURI];
		var percentageVotesCast = "0";
		if (userParticipantInfo.participantStatusInfo) {
			percentageVotesCast = userParticipantInfo.participantStatusInfo.percentageVotesCast;
		}
		var table = document.createElement("table");
		var td = null;
		table.className = "singleParticipantHoverPopupTable";
		// Add the header row.
		var thead = document.createElement("thead");
		table.appendChild(thead);
		var tr = document.createElement("tr");
		thead.appendChild(tr);
		var userSpan = document.createElement("span");
		userSpan.className = "userName";
		td = this._appendNewTH(tr, userSpan);
		td.colSpan = "2";
		reviewutils.fillUserSpanAndAddUserImage(userURI, userSpan);
		var percentageReviewedLabel =
			dojo.string.substitute(this._messages["percentReviewed"], [percentageVotesCast + "%"]);
		td = this._appendNewTH(tr, document.createTextNode(percentageReviewedLabel));
		td.colSpan = "2";
		td.className = "percentReviewed";
		// Add the table body rows.
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		var artifactInfos = reviewParser.getArtifactInfos();
		var artifactResultInfos = reviewParser.getArtifactResultInfos();
		var singleParticipantResultInfos = artifactResultInfos[userURI];
		var index = 0;
		var mapArtifactIdToTargetedUserArtifactStatus = new Object();
		dojo.forEach(
			artifactResultInfos[userURI],
			function(targetedUserResultInfo) {
				mapArtifactIdToTargetedUserArtifactStatus[targetedUserResultInfo.artifact] =
					targetedUserResultInfo.artifactStatusType;
			}
		);
		dojo.forEach(
			artifactInfos,
			function(artifactInfo) {
				var displayIndex = ++index;
				var artifactStatusType = (
					mapArtifactIdToTargetedUserArtifactStatus[artifactInfo.id]
					|| reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
				);
				var tr = document.createElement("tr");
				tbody.appendChild(tr);
				var td = null;
				// Add the index cell.
				this._appendNewTD(
					tr,
					document.createTextNode(index + "."),
					"index");
				// Add the artifact name cell.
				this._appendNewTD(
					tr,
					document.createTextNode(artifactInfo.artifactName),
					"artifactName");
				// Add the status icon cell.
				this._appendNewTD(
					tr,
					reviewutils.createArtifactStatusTypeImage(artifactStatusType),
					"statusIcon");
				// Add the status text cell.
				this._appendNewTD(
					tr,
					document.createTextNode(
						reviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType)),
					"statusLabel");
			},
			this
		);
		return table;
	},

	_createSingleArtifactSummaryStatus: function(reviewParser) {
		var artifactId = this.artifactUnderReviewId;
		var participantInfos = reviewParser.getParticipantInfos();
		var artifactInfos = reviewParser.getArtifactInfos();
		var artifactResultInfos = reviewParser.getArtifactResultInfos();
		var artifactInfo = artifactInfos[artifactId];
		var table = document.createElement("table");
		var td = null;
		table.className = "singleArtifactHoverPopupTable";
		// Add the header row.
		var thead = document.createElement("thead");
		table.appendChild(thead);
		var tr = document.createElement("tr");
		thead.appendChild(tr);
		var img = document.createElement("img");
		img.src = MimeTypeRegistry.getImageURI(artifactInfo.contentType);
		img.className = "artifactImage";
		var td = this._appendNewTH(tr, img);
		var artifactNameSpan = document.createElement("span");
		artifactNameSpan.innerHTML = artifactInfo.artifactName;
		artifactNameSpan.className = "artifactName";
		td.appendChild(artifactNameSpan);
		td.colSpan = "2";
		// TODO Fill in that overallStatusTD with appropriate status info ...
		// (but we need UX guidance on when we can say something like Disapproved
		// [for example, can this only be shown when at least one Approver or
		// Required Reviewer casts a vote of Disapproved, or should we wait until
		// this participant has ALSO specified that they are Done?]).
		// For now, we'll just not show a rollup status in the header area ...
		var overallStatusTD = this._appendNewTH(tr, document.createTextNode(" "));
		td.colSpan = "2";
		// Add the table body rows.
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		dojo.forEach(
			participantInfos,
			function(participantInfo) {
				var userURI = participantInfo.userURI;
				var singleParticipantResultInfos = artifactResultInfos[userURI];
				var artifactStatusType =
					this._getVoteCastByParticipant(
						userURI, singleParticipantResultInfos);
				var tr = document.createElement("tr");
				tbody.appendChild(tr);
				var td = null;
				// Add a cell containing the user image and user name.
				var userSpan = document.createElement("span");
				userSpan.className = "userName"
				this._appendNewTD(tr, userSpan);
				reviewutils.fillUserSpanAndAddUserImage(userURI, userSpan);
				// Add the status icon cell.
				this._appendNewTD(
					tr,
					reviewutils.createArtifactStatusTypeImage(artifactStatusType),
					"statusIcon");
				// Add the status text cell.
				this._appendNewTD(
					tr,
					document.createTextNode(
						reviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType)),
					"statusLabel");
			},
			this
		);
		return table;
	},

	_getVoteCastByParticipant: function(userURI, singleParticipantResultInfos) {
		var artifactStatusType = reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted;
		for (var i = 0; i < singleParticipantResultInfos.length; i++) {
			var resultInfo = singleParticipantResultInfos[i];
			if (resultInfo.artifact === this.artifactUnderReviewId) {
				artifactStatusType = resultInfo.artifactStatusType;
			}
		}
		return artifactStatusType;
	},

	_appendNewTH: function(tr, childNode, /*String?*/cssClassName) {
		var td = document.createElement("th");
		tr.appendChild(td);
		td.appendChild(childNode);
		if (cssClassName) {
			td.className = cssClassName;
		}
		return td;
	},

	_appendNewTD: function(tr, childNode, /*String?*/cssClassName) {
		var td = document.createElement("td");
		tr.appendChild(td);
		td.appendChild(childNode);
		if (cssClassName) {
			td.className = cssClassName;
		}
		return td;
	}
});


com.ibm.rdm.web.review.ReviewHoverPopup.VIEW_TYPE = {
	SINGLE_PARTICIPANT_SUMMARY_STATUS: "Single participant summary status",
	SINGLE_ARTIFACT_SUMMARY_STATUS: "Single artifact summary status",
	OVERALL_REVIEW_SUMMARY_STATUS: "Overall review summary status"
}
