/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewParser");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.review.ParticipantResultsParser");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");

var ParticipantResultParser = com.ibm.rdm.web.review.ParticipantResultsParser;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;


// Here are examples of Review artifacts that this ReviewParser class might be
// asked to parse ...

// Representation used on 8/30/09 for an informal review ...
//
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <rrmReview:reviewDueDate>2009-09-06T10:56:11.604Z</rrmReview:reviewDueDate>
//     <rrmReview:owner>https://127.0.0.1:9443/jazz/users/TestJazzAdmin1</rrmReview:owner>
//     <dc:title>Informal</dc:title>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="https://127.0.0.1:9443/jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//     <rrmReview:reviewTimestamp>2009-08-30T10:56:33.226Z</rrmReview:reviewTimestamp>
//     <rrmReview:formal>false</rrmReview:formal>
//     <dc:description></dc:description>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="https://127.0.0.1:9443/rdm/resources/rrc123"/>
//         <rrmReview:artifactId>https://127.0.0.1:9443/rdm/resources/rrc123</rrmReview:artifactId>
//         <dc:contributor rdf:resource="https://127.0.0.1:9443/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc1</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 8/31/09 for a formal review based on a collection ...
//
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:formal>true</rrmReview:formal>
//     <dc:description></dc:description>
//     <rrmReview:owner>https://127.0.0.1:9443/jazz/users/TestJazzAdmin1</rrmReview:owner>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>1</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="https://127.0.0.1:9443/rdm/resources/rrc124?revision=_f5TcoZVTEd6FzvjE3bX-NQ"/>
//         <rrmReview:artifactId>https://127.0.0.1:9443/rdm/resources/rrc124?revision=_f5TcoZVTEd6FzvjE3bX-NQ</rrmReview:artifactId>
//         <dcterms:modified>2009-08-30T10:54:32.298Z</dcterms:modified>
//         <dc:contributor rdf:resource="https://127.0.0.1:9443/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc2</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <dc:title>FRC1</dc:title>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="https://127.0.0.1:9443/rdm/resources/rrc123?revision=_eBe_YZVTEd6FzvjE3bX-NQ"/>
//         <rrmReview:artifactId>https://127.0.0.1:9443/rdm/resources/rrc123?revision=_eBe_YZVTEd6FzvjE3bX-NQ</rrmReview:artifactId>
//         <dcterms:modified>2009-08-30T10:54:19.734Z</dcterms:modified>
//         <dc:contributor rdf:resource="https://127.0.0.1:9443/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc1</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:reviewTimestamp>2009-08-31T11:46:47.704Z</rrmReview:reviewTimestamp>
//     <rrmReview:hasCollection rdf:parseType="Resource">
//       <rrmReview:collectionName>Col1</rrmReview:collectionName>
//       <rrmReview:collection rdf:resource="https://127.0.0.1:9443/rdm/resources/rrc126"/>
//     </rrmReview:hasCollection>
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <rrmReview:reviewDueDate>2009-09-07T11:46:34.652Z</rrmReview:reviewDueDate>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="https://127.0.0.1:9443/jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 9/04/09 for a formal review based on a collection associated with a snapshot ...
//
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <dc:title>FCS</dc:title>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//     <rrmReview:hasCollection rdf:parseType="Resource">
//       <rrmReview:collectionSnapshot>2009-09-04T14:11:14.000Z</rrmReview:collectionSnapshot>
//       <rrmReview:collectionName>SnapCol1</rrmReview:collectionName>
//       <rrmReview:collection rdf:resource="https://localhost:9443/rdm/resources/rrc184"/>
//     </rrmReview:hasCollection>
//     <dc:description></dc:description>
//     <rrmReview:reviewDueDate>2009-09-11T14:39:43.168Z</rrmReview:reviewDueDate>
//     <rrmReview:formal>true</rrmReview:formal>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="https://localhost:9443/rdm/resources/rrc177?revision=_fW0JMZj5Ed62MOWY8zDLqA"/>
//         <rrmReview:artifactId>https://localhost:9443/rdm/resources/rrc177?revision=_fW0JMZj5Ed62MOWY8zDLqA</rrmReview:artifactId>
//         <dcterms:modified>2009-09-04T02:20:18.627Z</dcterms:modified>
//         <dc:contributor rdf:resource="https://localhost:9443/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc1</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:reviewTimestamp>2009-09-04T14:39:49.948Z</rrmReview:reviewTimestamp>
//     <rrmReview:owner>https://localhost:9443/jazz/users/TestJazzAdmin1</rrmReview:owner>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 9/19/09 for a formal review ...
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:reviewDueDate>2009-09-26T11:53:35.645Z</rrmReview:reviewDueDate>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Approver</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="/jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//     <dc:description></dc:description>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="../resources/rrc98?revision=_7lIp0aUSEd6-38oE2LC_yQ"/>
//         <rrmReview:artifactId>https://localhost:9443/rdm/resources/rrc98?revision=_7lIp0aUSEd6-38oE2LC_yQ</rrmReview:artifactId>
//         <dcterms:modified>2009-09-19T11:52:39.405Z</dcterms:modified>
//         <dc:contributor rdf:resource="/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc1</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:formal>true</rrmReview:formal>
//     <dc:title>F1</dc:title>
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <rrmReview:reviewTimestamp>2009-09-19T11:54:34.329Z</rrmReview:reviewTimestamp>
//     <rrmReview:owner>https://localhost:9443/jazz/users/TestJazzAdmin1</rrmReview:owner>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 9/23/09 for a formal review ...
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="../../jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="../resources/rrc89"/>
//         <rrmReview:artifactId>../resources/rrc89</rrmReview:artifactId>
//         <dc:contributor rdf:resource="../../jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dc:title>Doc1</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:formal>true</rrmReview:formal>
//     <rrmReview:reviewStatus>Draft</rrmReview:reviewStatus>
//     <rrmReview:reviewDueDate>2009-09-30T20:58:01.727Z</rrmReview:reviewDueDate>
//     <dc:description></dc:description>
//     <rrmReview:owner>../../jazz/users/TestJazzAdmin1</rrmReview:owner>
//     <dc:title>AAA</dc:title>
//   </rrmReview:Review>
// </rdf:RDF>
 
// Representation used on 10/01/09 for a formal review (note that absolute paths ARE used in this case) ...
// <rdf:RDF
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="../resources/rrc113?revision=_rY5lM6w4Ed6BHIsY82esNg"/>
//         <rrmReview:artifactId>../resources/rrc113?revision=_rY5lM6w4Ed6BHIsY82esNg</rrmReview:artifactId>
//         <dcterms:modified>2009-09-28T14:10:29.763Z</dcterms:modified>
//         <dc:contributor rdf:resource="https://VM2003Oracle10.x.x.x.x:9443/jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.sid.sketch+xml</dc:format>
//         <dc:title>Gather Personal Info</dc:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:owner>https://VM2003Oracle10.x.x.x.x:9443/jazz/users/Oscar</rrmReview:owner>
//     <dc:title>Rev5</dc:title>
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <dc:description></dc:description>
//     <rrmReview:reviewDueDate>2009-10-05T18:29:37.984Z</rrmReview:reviewDueDate>
//     <rrmReview:formal>true</rrmReview:formal>
//     <rrmReview:reviewTimestamp>2009-09-28T20:49:57.078Z</rrmReview:reviewTimestamp>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="https://VM2003Oracle10.x.x.x.x:9443/jazz/users/TestJazzAdmin1"/>
//     </rrmReview:hasParticipant>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//       <dc:contributor rdf:resource="https://VM2003Oracle10.x.x.x.x:9443/jazz/users/Oscar"/>
//     </rrmReview:hasParticipant>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 10/16/09 for a formal review:
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="../resources/rrc37"/>
//         <rrmReview:artifactId>../resources/rrc37</rrmReview:artifactId>
//         <dc:contributor rdf:resource="../../jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.requirement+xml</dc:format>
//         <dcterms:title>Browser Compliance</dcterms:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <dcterms:description></dcterms:description>
//     <rrmReview:hasParticipant rdf:parseType="Resource">
//       <rrmReview:participantInfo rdf:parseType="Resource">
//         <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//         <dc:contributor rdf:resource="../../jazz/users/sam"/>
//       </rrmReview:participantInfo>
//     </rrmReview:hasParticipant>
//     <rrmReview:reviewDueDate>2009-10-23T14:58:14.429Z</rrmReview:reviewDueDate>
//     <dcterms:title>Eric1</dcterms:title>
//     <rrmReview:reviewStatus>Draft</rrmReview:reviewStatus>
//     <rrmReview:reviewInstructions>whatever</rrmReview:reviewInstructions>
//     <rrmReview:owner>../../jazz/users/TestJazzAdmin1</rrmReview:owner>
//     <rrmReview:formal>true</rrmReview:formal>
//   </rrmReview:Review>
// </rdf:RDF>

// Representation used on 10/19/09 for a formal review:
// <rdf:RDF
//     xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/"
//     xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"
//     xmlns:dcterms="http://purl.org/dc/terms/"
//     xmlns:dc="http://purl.org/dc/elements/1.1/"
//     xmlns:rrmReview="http://www.ibm.com/xmlns/rrm/reviews/1.0/"
//     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
//   <rrmReview:Review rdf:about="">
//     <dcterms:title>F_R1</dcterms:title>
//     <rrmReview:reviewTimestamp>2009-10-19T17:05:01.511Z</rrmReview:reviewTimestamp>
//     <rrmReview:hasArtifactReview>
//       <rrmReview:ReviewArtifact>
//         <rrmReview:artifactIndex>0</rrmReview:artifactIndex>
//         <rrmReview:artifact rdf:resource="../resources/rrc94?revision=_b9F7wrzREd6pGtSxIiQHCw"/>
//         <rrmReview:artifactId>../resources/rrc94?revision=_b9F7wrzREd6pGtSxIiQHCw</rrmReview:artifactId>
//         <dcterms:modified>2009-10-19T17:04:17.788Z</dcterms:modified>
//         <dc:contributor rdf:resource="../../jazz/users/TestJazzAdmin1"/>
//         <dc:format>application/x-com.ibm.rdm.richtext+xml</dc:format>
//         <dcterms:title>Doc1</dcterms:title>
//       </rrmReview:ReviewArtifact>
//     </rrmReview:hasArtifactReview>
//     <rrmReview:reviewStatus>Started</rrmReview:reviewStatus>
//     <rrmReview:hasParticipant>
//       <rrmReview:ParticipantInfo>
//         <rrmReview:reviewRole>Required Reviewer</rrmReview:reviewRole>
//         <dc:contributor rdf:resource="../../jazz/users/joe"/>
//       </rrmReview:ParticipantInfo>
//     </rrmReview:hasParticipant>
//     <rrmReview:reviewDueDate>2009-10-26T17:01:55.681Z</rrmReview:reviewDueDate>
//     <rrmReview:owner>../../jazz/users/TestJazzAdmin1</rrmReview:owner>
//     <rrmReview:formal>true</rrmReview:formal>
//     <dcterms:description></dcterms:description>
//   </rrmReview:Review>
// </rdf:RDF>
 
dojo.declare(
	"com.ibm.rdm.web.review.ReviewParser",
	null,
{
	// summary:
	//    Parses the content found in Review artifacts.
	// description:
	//    Please note that although the constructor initializes most fields
	//    with appropriate values, two subsequent calls MUST be made to the
	//    following functions in order for this class to be properly
	//    initialized:
	//        setProjectUri
	//        updateParticipantResults

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_resourceURI: null,
	_projectURI: null,
	_revision: null,
	_lastModifiedDate: null,
	_role: null,
	// _participantResultURI: String
	//    See the getParticipantResultURI function for details about this field.
	_participantResultURI: null,
	// _participantResultRevision: String
	//    See the getParticipantResultRevision function for details about this field.
	_participantResultRevision: null,
	_reviewName: null,
	//_creatorUserId: null,
	_creatorUserURI: null,
	_dueDate: null,
	// _isFormal: Boolean
	//    See the isFormal function for details about this field.
	_isFormal: null,
	// _collectionName: String
	//    See the getCollectionName function for details about this field.
	_collectionName: null,
	// _collectionName: String
	//    See the getCollectionName function for details about this field.
	_collectionSnapshotDate: null,
	// _collectionSnapshotDate: String
	//    See the getCollectionSnapshotDate function for details about this field.
	_collectionSnapshotDate: null,
	// _startTimestamp: string
	//    See the getStartTimestamp function for details about this field.
	_startTimestamp: null,
	// _startTimestampDate: Date
	//    See the getStartTimestampDate function for details about this field.
	_startTimestampDate: null,
	// _reviewState: String
	//    See the getReviewState function for details about this field.
	_reviewState: null,
	// _instructions: String
	//    See the getInstructions function for details about this field.
	_instructions: null,
	// _artifactInfos: Array (of property lists)
	//    See the getArtifactInfos function for details about this field.
	_artifactInfos: null,
	// _participantInfos: Array (of property lists)
	//    See the getParticipantInfos function for details about this field.
	_participantInfos: null,
	// _participantResultInfos: Array (of property lists)
	//    See the getParticipantResultInfos function for details about this
	//    field.
	_participantResultInfos: null,
	// _artifactResultInfos: Array (of property lists)
	//    See the getArtifactResultInfos function for details about this field.
	_artifactResultInfos: null,
	// _percentReviewCompleted: integer
	//    See the getPercentReviewCompleted function for details about this
	//    field.
	_percentReviewCompleted: null,
	// _resourceDescriptionsDeferred: Deferred
	//    See the getLiveArtifactsVersionAndNameInfo function for details about
	//    this field.
	_resourceDescriptionsDeferred: null,
	// _mapUserUriToOriginalUserPath: Object
	//    The user paths found within the review may be either relative or
	//    absolute.  This hash map is used to map each participant's fully
	//    qualified user URI to the original user path stored in the review.
	_mapUserUriToOriginalUserPath: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(keywordArgs) {
		// summary:
		//    Parses the content found within a Review artifact.
		// keywordArgs.response:
		//    The response returned by a doGet for the review artifact.
		var response = keywordArgs.response;
		var lastModifiedDateString = response.ioArgs.xhr.getResponseHeader("Last-Modified");
		// FIXME Add code here to get the ParticipantResults (once Eric starts generating them).
		var resourceURI = XhrUtils.getResponseURI(response);
		var index = resourceURI.indexOf("?");
		this._mapUserUriToOriginalUserPath = new Object();
		this._resourceURI = (index === -1 ? resourceURI : resourceURI.substring(0, index));
		this._revision = XhrUtils.getResponseEtag(response);
		this._lastModifiedDate = new Date(lastModifiedDateString);
		this._artifactResultInfos = [];
		this._participantResultInfos = [];
		this._parseContent(response.doc);
		this._initResultRelatedFields(this._participantInfos);
	},

	getSelfUri: function() {
		// summary:
		//    Get the resource URI for this review (e.g.
		//    "https://localhost:9443/rdm/resources/_h8ZFADsaEd6IT-BLBFfrNQ").
		return this._resourceURI;
	},

	getProjectUri: function() {
		return this._projectURI;
	},

	getReviewId: function() {
		// summary:
		//    Get the shortname resource id for this review (e.g.
		//    "_h8ZFADsaEd6IT-BLBFfrNQ").
		return queryutils.getLastPathSegment(this._resourceURI);
	},

	getRevision: function() {
		// summary:
		//    Get the revision of this review (e.g. "_Zzb6sTs3Ed6IT-BLBFfrNQ").
		return this._revision;
	},

	getLastModifiedDate: function() {
		// summary:
		//    Get the date at which this review was last modified.
		// returns: Date object
		return this._lastModifiedDate;
	},

	getReviewName: function() {
		// summary:
		//    Get the review name for this review (e.g. "My Test Review").
		return this._reviewName;
	},

	isFormal: function() {
		// summary:
		//    Returns a boolean representing whether the review is a formal
		//    review.
		// description:
		//    A "formal review" contains artifacts that are pinned to a
		//    particiular version, and when these artifacts are opened, the
		//    header area cues the user into the fact the review artifact
		//    under display is pinned to a particular version.
		//    An "informal review" contains artifacts that are "live"
		//    (i.e. always from HEAD), and when these artifacts are opened,
		//    the header area cues the user into the fact that the review
		//    artifact under display is the "live" version of the artifact.
		return this._isFormal;
	},

	getCollectionName: function() {
		// summary:
		//    Gets the name of the collection that this review is based on
		//    or 'null' if it's NOT based on a collection.
		return this._collectionName;
	},

	getCollectionSnapshotDate: function() {
		// summary:
		//    Gets the date of the snapshot associated with a collection that
		//    this review is based on, or 'null' (if it's NOT based on a
		//    collection associated with a snapshot).
		return this._collectionSnapshotDate;
	},

	getCollectionUri: function() {
		// summary:
		//    Gets the URI for the collection that this review is based on
		//    or 'null' if it's NOT based on a collection.
		return this._collectionURI;
	},

	getReviewState: function() {
		// summary:
		//    Get the review's state (a value found in reviewTypes.REVIEW_STATE_TYPES).
		return this._reviewState;
	},

	//getCreatorUserId: function() {
	//	return this._creatorUserId;
	//},
	
	getCreatorUserURI: function() {
		return this._creatorUserURI;
	},

	getRole: function() {
		// summary:
		//    Gets the participant role type for the current user.
		// returns:
		//    If the current user is not a participant, null will be returned.
		//    Otherwise (if current user is a participant) the returned value
		//    will be one of the values found in the enum represented by
		//    com.ibm.rdm.web.review.reviewTypes.PARTICIPANT_ROLE_TYPES.
		return this._role;
	},

	setParticipantResultUri: function(participantResultURI) {
		// summary:
		//    Sets the ParticipantResult resource URI associated with the
		//    current user for this review.
		// description:
		//    Normally the ParticipantResult resource URI is set via a call
		//    to 'updateParticipantResults' (in this class) when all
		//    participant results are read into a ReviewParser instance.
		//    However, when the user casts his/her very first vote, the POST
		//    that goes off will return a resource URI and this
		//    'setParticipantResultUri' function will be called to store this
		//    new URI away for subsequent use in the next vote cast.
		this._participantResultURI = participantResultURI;
	},

	getParticipantResultUri: function() {
		// summary:
		//    Gets the ParticipantResult associated with the current user
		//    for this review.
		// description:
		//    If the current user is the review creator or has ever cast any
		//    votes in the context of this review, a ParticipantResult
		//    artifact will be associated with this user.  This function
		//    returns the URI for this ParticipantResult (if it exists).
		// returns:
		//    A ParticipantResult URI, if one exists; otherwise null.
		return this._participantResultURI;
	},

	setParticipantResultRevision: function(participantResultRevision) {
		// summary:
		//    Sets the revision for the ParticipantResult associated with
		//    the current user for this review.
		// description:
		//    We store this value away so that we can prevent possible save
		//    collisions when the user attempts to vote.  After the first
		//    vote cast for a review, all subsequent vote casts will ask the
		//    current reviewParser for this field (via the
		//    'getParticipantResultRevision' function) so that it can be
		//    passed as the "If-Match" header value when performing the PUT
		//    of the user's participant results.
		this._participantResultRevision = participantResultRevision;
	},

	getParticipantResultRevision: function() {
		// summary:
		//    Gets the revision for the ParticipantResult associated with
		//    the current user for this review.
		return this._participantResultRevision;
	},

	getDueDate: function() {
		// summary:
		//    Gets the due date for this review.
		// returns: Date object
		return this._dueDate;
	},

	getStartTimestamp: function() {
		// summary:
		//    Gets the start timestamp for this review.
		// description:
		//    This timestamp represents the time at which the review was
		//    started, or if ever paused and restarted, the date at which
		//    it was last restarted.
		// returns: string
		return this._startTimestamp;
	},

	getStartTimestampDate: function() {
		// summary:
		//    Gets the start timestamp for this review.
		// description:
		//    This timestamp represents the time at which the review was
		//    started, or if ever paused and restarted, the date at which
		//    it was last restarted.
		// returns: Date object
		return this._startTimestampDate;
	},

	getInstructions: function() {
		// summary:
		//    Get the review instructions.
		// returns: String
		return this._instructions;
	},

	getArtifactInfos: function() {
		// summary:
		//    Gets information about the artifacts involved in the review.
		// returns:
		//    An array of property lists, each property list having the
		//    following form:
		//       {
		//           artifactName: <string such as "Document1">
		//           resourceURI: <string such as "_lckswDjhEd67TdjgK4pk9g">
		//           revision: <string such as "_CfoHgf3iEd22i4tXbUWVZg">
		//           contentType: <string such as "application/x-com.ibm.rdm.richtext+xml">
		//           lastModifiedDate: <string such as "2009-03-16T09:03:49">
		//           id: <string such as "Artifact__h8M3wDsaEd6tvb-bjZtYIw">
		//           versionLabel: <string>
		//           myArtifactStatusType: <a type from reviewTypes.ARTIFACT_STATUS_TYPES>
		//           overallArtifactStatusInfo: <a property list [see overallArtifactStatusInfo below]>
		//           persistedURI: <string such as "_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ">
		//       }
		//    where overallArtifactStatusInfo is of the following form:
		//       {
		//           percentageVotesCast: <an integer from 0 to 100>
		//           numApproved: <an integer>
		//           numDisapproved: <an integer>
		//           numAbstained: <an integer>
		//           numReviewed: <an integer>
		//       }
		//    This array also serves as a hashmap, where the key is the id (e.g.
		//    "Artifact__eZTagBl1Ed6Y_KkX2z2gTQ") and the value is the appropriate
		//    artifactInfo entry.
		return this._artifactInfos;
	},

	getArtifactInfo: function(resourceURI) {
		// summary:
		//    Gets information about a particular resource involved in the review.
		// returns:
		//    A property list having the following form:
		//       {
		//           artifactName: <string such as "Document1">
		//           resourceURI: <string such as "_lckswDjhEd67TdjgK4pk9g">
		//           revision: <string such as "_CfoHgf3iEd22i4tXbUWVZg">
		//           contentType: <string such as "application/x-com.ibm.rdm.richtext+xml">
		//           lastModifiedDate: <string such as "2009-03-16T09:03:49">
		//           id: <string such as "Artifact__h8M3wDsaEd6tvb-bjZtYIw">
		//           versionLabel: <string>
		//           myArtifactStatusType: <a type from reviewTypes.ARTIFACT_STATUS_TYPES>
		//           overallArtifactStatusInfo: <a property list [see overallArtifactStatusInfo below]>
		//           persistedURI: <string such as "_lckswDjhEd67TdjgK4pk9g?revision=_zObo4To7Ed62cpPlu7nHxQ">
		//       }
		//    where overallArtifactStatusInfo is of the following form:
		//       {
		//           percentageVotesCast: <an integer from 0 to 100>
		//           numApproved: <an integer>
		//           numDisapproved: <an integer>
		//           numAbstained: <an integer>
		//           numReviewed: <an integer>
		//       }
		// resourceURI: string
		//    A resource URI such as "https://localhost:9443/rdm/resources/_lckswDjhEd67TdjgK4pk9g"
		var artifactInfo = null;
		// TODO Consider converting this over to use a hashmap.
		for (var i = 0; i < this._artifactInfos.length; i++) {
			if (this._artifactInfos[i].resourceURI === resourceURI) {
				artifactInfo = this._artifactInfos[i];
				break;
			}
		}
		return artifactInfo;
	},

	getParticipantInfos: function() {
		// summary:
		//    Gets information about the participants involved in the review.
		// returns: Array (holding property lists, but also serves has a hashtable)
		//    An array of property lists, each property list having the
		//    following form:
		//       {
		//           resourceURI: "https://localhost:9443/rdm/reviews/results/_VQ5n8WgLEd6NlcEl9xCf0g"
		//           userURI: <userURI>
		//           participantRoleType: <a type from reviewTypes.PARTICIPANT_ROLE_TYPES>
		//           participantStatusInfo: <a property list [see below]>
		//       }
		//    where participantStatusInfo is of the form:
		//       {
		//           percentageVotesCast: <an integer from 0 to 100>
		//           numApproved: <an integer>
		//           numDisapproved: <an integer>
		//           numAbstained: <an integer>
		//           numReviewed: <an integer>
		//           numInProgress: <an integer>
		//       }
		//    In addition this returned array also serves as a hashmap,
		//    keyed by userURI, in which the key value represents the
		//    participantInfo property associated with that user.
		return this._participantInfos;
	},

	getParticipantResultInfos: function() {
		// summary:
		//    Gets the overall review votes cast by each participant.
		// returns: Array (holding property lists, but also serves has a hashtable)
		//    An array of property lists, each property list having the
		//    following form:
		//       {
		//           userURI: <userURI>
		//           artifactStatusType: <a type from reviewTypes.ARTIFACT_STATUS_TYPES or null>
		//       }
		//    In addition this returned array also serves as a hashmap,
		//    where the key is the userURI, and the value is the
		//    participantResultInfo property list pertaining to that user.
		//    For example to get at the overall review vote cast by userURI
		//    "https://localhost:9443/jazz/users/Joe", you could use something
		//    like the following:
		//    |    var participantResultInfos = reviewParser.getParticipantResultInfos();
		//    |    var joeReviewVote = participantResultInfos["https://localhost:9443/jazz/users/Joe"].artifactStatusType
		//    Please note that the artifactStatusType MAY be null.  Only if a
		//    user has cast the "overall review" vote will his/her property
		//    list's artifactStatusType property hold a non-null value
		//    (representing an enum value such as
		//    reviewTypes.ARTIFACT_STATUS_TYPES.Approved).
		return this._participantResultInfos;
	},

	getArtifactResultInfos: function() {
		// summary:
		//    Gets all artifact results cast by all participants involved in the review.
		// returns: Array (holding property lists, but also serves has a hashtable)
		//    An array of property lists, each property list having the
		//    following form:
		//       {
		//           userURI: <userURI>
		//           artifact: <something like "Artifact__eYce4Bl1Ed6Y_KkX2z2gTQ">
		//           artifactStatusType: <a type from reviewTypes.ARTIFACT_STATUS_TYPES>
		//       }
		//    In addition this returned array also serves as a hashmap,
		//    keyed by userURI, in which the key values represent the array of
		//    property lists pertaining to that user.  For example to get at
		//    the artifact voting information for userURI:
		//        https://localhost:9443/jazz/users/Joe
		//    you could use something like the following:
		//    |    var artifactResultInfos = reviewParser.getArtifactResultInfos();
		//    |    var joeArtifactResultInfos = artifactResultInfos["https://localhost:9443/jazz/users/Joe"];
		//    and the resulting 'joeArtifactResultInfos' array would ONLY contain
		//    elements in which the userURI is "https://localhost:9443/jazz/users/Joe".
		return this._artifactResultInfos;
	},

	getPercentReviewCompleted: function() {
		// summary:
		//    Gets an overall completion percentage for the review.
		// returns:
		//    An integer representing the percentage completion for the
		//    review (a value between 0 and 100).
		return this._percentReviewCompleted;
	},

	getLiveArtifactsVersionAndNameInfo: function() {
		// summary:
		//    Returns a deferred to which the caller may use addCallback to
		//    obtain "describe" information about all artifacts involved
		//    in the review.
		if (! this._resourceDescriptionsDeferred) {
			var resourceURIs = [];
			for (var i = 0; i < this._artifactInfos.length; i++) {
				resourceURIs.push(this._artifactInfos[i].resourceURI);
			}
			var deferred =
				ProjectCache.getBatchResourceDescriptionsMultiFetch(resourceURIs);
			deferred.addCallback(this, function(resourceDescriptionsObject){
				return resourceDescriptionsObject;
			});
			this._resourceDescriptionsDeferred = deferred;
		}
		return this._resourceDescriptionsDeferred;
	},

	getUserPathToUseWhenSaving: function(userURI) {
		// summary:
		//    Return the user path that should be used when saving a vote cast
		//    by a particular participant.
		return this._mapUserUriToOriginalUserPath[userURI];
	},

	setProjectUri: function(projectURI) {
		this._projectURI = projectURI;
	},

	updateParticipantResults: function(participantResultsParser) {
		// summary:
		//    Update appropriate fields found within this class to account for
		//    all of the individual ParticipantResult vote casting artifacts
		//    represented by the passed 'participantResultsParser'.
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		this._initResultRelatedFields(this._participantInfos);
		this._resetResultRelatedFields(participantResultsParser);
		this._fillParticipantStatusProperties(
			this._participantInfos, this._artifactInfos, this._artifactResultInfos);
		this._fillPercentReviewCompleted(
			this._participantInfos, this._artifactInfos, this._artifactResultInfos);
		var myParticipantInfo = this._participantInfos[myUserURI];
		this._participantResultURI = (
			myParticipantInfo ?
			myParticipantInfo.resourceURI :
			null
		);
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_initResultRelatedFields: function(participantInfos) {
		// summary:
		//    Initialize some result related fields.
		// description:
		//    This method initializes the following fields:
		//        this._artifactResultInfos
		//        this._participantResultInfos
		//    Eventually _resetResultRelatedFields will be called to set them
		//    to appropriate values pertaining to the votes from the review's
		//    participants.
		//    This is a two stage process due to the fact that two GETs are
		//    needed to be made to the server: one for the Review and one for
		//    the feed representing the participant results associated with the
		//    review.
		var participantResultInfos = [];
		var artifactResultInfos = [];
		// Initialize the following fields:
		//    this._participantResultInfos
		//    this._artifactResultInfos
		dojo.forEach(participantInfos, function(participantInfo) {
			artifactResultInfos[participantInfo.userURI] = [];
			var participantResultInfo = {
				userURI: participantInfo.userURI,
				artifactStatusType: null
			};
			participantResultInfos.push(participantResultInfo);
			participantResultInfos[participantInfo.userURI] = participantResultInfo;
		});
		this._artifactResultInfos = artifactResultInfos;
		this._participantResultInfos = participantResultInfos;
		// Initialize the following fields within all of this._artifactInfos:
		//    myArtifactStatusType
		//    overallArtifactStatusInfo
		// to appropriate values.
		dojo.forEach(
			this._artifactInfos,
			function(artifactInfo) {
				var overallArtifactStatusInfo =
					this._composeOverallArtifactStatusInfo(
						[], participantInfos);
				artifactInfo.myArtifactStatusType = null;
				artifactInfo.overallArtifactStatusInfo = overallArtifactStatusInfo;
			},
			this
		);
	},

	_resetResultRelatedFields: function(participantResultsParser) {
		// summary:
		//    Incorporate newly retrieved participant results into the
		//    appropriate fields found within this class.
		// description:
		//    This method adds participant results to the following fields:
		//        this._artifactResultInfos
		//        this._participantResultInfos
		//        this._artifactInfos
		//    and adds the resourceURI (for ParticipantResult artifact) to
		//    each property value found in:
		//        this._participantInfos
		var participantResults = participantResultsParser.getParticipantResults();
		// Fill the following fields:
		//    this._participantInfos
		//    this._participantResultInfos
		//    this._artifactResultInfos
 		dojo.forEach(
			participantResults,
			function(participantResultsElement) {
				var resourceURI = participantResultsElement.resourceURI;
				var userURI = participantResultsElement.userURI;
				var isDone = participantResultsElement.isDone;
				var artifactResults = participantResultsElement.artifactResults;
				var participantInfo = this._participantInfos[userURI];
				if (participantInfo) {
					// Tweak this._participantInfos.
					participantInfo.resourceURI = resourceURI;
					// Tweak this._participantResultInfos.
	 				if (isDone) {
	 					this._participantResultInfos[userURI].artifactStatusType =
	 						reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed;
	 				}
					// Tweak this._artifactResultInfos.
					dojo.forEach(
						artifactResults,
						function(artifactResult) {
							var resultInfo = {
								userURI: userURI,
								artifact: artifactResult.artifactId,
								artifactStatusType: artifactResult.participantResultStatus
							};
							this._artifactResultInfos.push(resultInfo);
							var userArtifactResultInfos = this._artifactResultInfos[userURI];
							userArtifactResultInfos.push(resultInfo);
						},
						this
					);
				}
			},
			this
		);
		var mapArtifactIdToMyArtifactStatus = new Object;
		dojo.forEach(
			this._artifactResultInfos[com.ibm.rdm.web.AUTHENTICATED_USER.uri],
			function(myResultInfo) {
				mapArtifactIdToMyArtifactStatus[myResultInfo.artifact] =
					myResultInfo.artifactStatusType;
			},
			this
		);
		var mapArtifactIdToArtifactResultInfos = new Object;
		dojo.forEach(
			this._artifactResultInfos,
			function(resultInfo) {
				var artifactResultInfos = mapArtifactIdToArtifactResultInfos[resultInfo.artifact];
				if (! artifactResultInfos) {
					artifactResultInfos = [];
					mapArtifactIdToArtifactResultInfos[resultInfo.artifact] = artifactResultInfos;
				}
 				artifactResultInfos.push(resultInfo);
			},
			this
		);
		// Set the following fields within all of this._artifactInfos:
		//    myArtifactStatusType
		//    overallArtifactStatusInfo
		// to appropriate values.
		dojo.forEach(
			this._artifactInfos,
			function(artifactInfo) {
				var id = artifactInfo.id;
				var myArtifactStatusType = (
					mapArtifactIdToMyArtifactStatus[id]
					|| reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
				);
				var singleArtifactResultInfos = mapArtifactIdToArtifactResultInfos[id];
				var overallArtifactStatusInfo =
					this._composeOverallArtifactStatusInfo(
						singleArtifactResultInfos, this._participantInfos);
				artifactInfo.myArtifactStatusType = myArtifactStatusType;
				artifactInfo.overallArtifactStatusInfo = overallArtifactStatusInfo;
			},
			this
		);
	},

	_parseContent: function(doc) {
		var getRole = function(userURI, participantInfos) {
			for (var i = 0; i < participantInfos.length; i++) {
				if (participantInfos[i].userURI === userURI) {
					return participantInfos[i].participantRoleType;
				}
			}
		};
		var myUserURI = com.ibm.rdm.web.AUTHENTICATED_USER.uri;
		this._parseCreator(doc);
		this._parseReviewName(doc);
		this._parseIsFormal(doc);
		this._parseCollectionNameUriAndSnapshotDate(doc);
		this._parseDueDate(doc);
		this._parseStartTimestamp(doc);
		this._parseReviewState(doc);
		this._parseInstructions(doc);
		this._parseParticipants(doc);
		this._parseArtifacts(doc, this._participantInfos);
		this._role = getRole(myUserURI, this._participantInfos);
	},

	_parseCreator: function(doc) {
		// summary:
		//    Fill this._creatorUserId with content parsed from the passed doc.

		// TODO Resolve why IE doesn't like the following, as this would be faster ...
		//this._creatorUserId = doc.firstChild.getAttribute("creator");

// 		var nodes = domutils.getElementsByTagNameNS(
// 			doc, "Review",
// 			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
// 		);
// 		this._creatorUserId = nodes[0].getAttribute("creator");
		// FIXME ... The spec and Eric currently aren't specifying creator
		this._creatorUserURI = null;
		//this._creatorUserId = null;
	},

	_parseReviewName: function(doc) {
		// summary:
		//    Fill this._reviewName with content parsed from the passed doc.
		this._reviewName =
			queryutils.getArtifactNameFromArtifactContent(
				doc, com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW);
	},

	_parseIsFormal: function(doc) {
		// summary:
		//    Fill this._isFormal with content parsed from the passed doc.
		var formalTextNodeValue = this._getTextNodeValue(
			doc, "formal",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		// Note: for backward compatability we'll check for not equals false
		// instead of equals true ...
		this._isFormal = (formalTextNodeValue !== "false");
	},

	_parseCollectionNameUriAndSnapshotDate: function(doc) {
		// summary:
		//    Fill this._collectionName and this._collectionURI with content
		//    parsed from the passed doc.
		var self = this;
		var getCollectionNameFromHasCollectionNode = function(hasCollectionNode) {
			return self._getTextNodeValue(
				hasCollectionNode, "collectionName",
				queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
			);
		};
		var getCollectionSnapshotDateFromHasCollectionNode = function(hasCollectionNode) {
			var dateAsString = self._getTextNodeValue(
				hasCollectionNode, "collectionSnapshot",
				queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
			);
			return (
				dateAsString ?
				dojo.date.stamp.fromISOString(dateAsString) :
				null
			);
		};
		var getCollectionUriFromHasCollectionNode = function(hasCollectionNode) {
			var nodes = domutils.getElementsByTagNameNS(
				hasCollectionNode, "collection",
				queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
			);
			var relativeResource = domutils.getAttributeNS(
				nodes[0], "resource",
				queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER
			);
			return queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(
				self._resourceURI, relativeResource);
		};
		var nodes = domutils.getElementsByTagNameNS(
			doc, "hasCollection",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		if (nodes.length === 1) {
			var hasCollectionNode = nodes[0];
			this._collectionName =
				getCollectionNameFromHasCollectionNode(hasCollectionNode);
			this._collectionURI =
				getCollectionUriFromHasCollectionNode(hasCollectionNode);
			this._collectionSnapshotDate =
				getCollectionSnapshotDateFromHasCollectionNode(hasCollectionNode);
		} else {
			this._collectionName = null;
			this._collectionURI = null;
			this._collectionSnapshotDate = null
		}
	},

	_parseDueDate: function(doc) {
		// summary:
		//    Fill this._dueDate with content parsed from the passed doc.
		var dueDateAsString = this._getTextNodeValue(
			doc, "reviewDueDate",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		this._dueDate = (
			dueDateAsString ?
			dojo.date.stamp.fromISOString(dueDateAsString) :
			null
		);
	},

	_parseReviewState: function(doc) {
		// summary:
		//    Fill this._reviewState with content parsed from the passed doc.
		var getReviewStateType = function(reviewStateAttribute) {
			switch (reviewStateAttribute) {
				case "Draft":
					return reviewTypes.REVIEW_STATE_TYPES.Draft;
				case "Started":
					return reviewTypes.REVIEW_STATE_TYPES.Started;
				case "Paused":
					return reviewTypes.REVIEW_STATE_TYPES.Paused;
				case "Finalized":
					return reviewTypes.REVIEW_STATE_TYPES.Finalized;
				case "Complete":
					return reviewTypes.REVIEW_STATE_TYPES.Reviewed;
			};
		};
		var reviewStateAttribute = this._getTextNodeValue(
			doc, "reviewStatus",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		this._reviewState = (
			reviewStateAttribute ?
			getReviewStateType(reviewStateAttribute) :
			reviewTypes.REVIEW_STATE_TYPES.Draft
		);
	},

	_parseStartTimestamp: function(doc) {
		// summary:
		//    Fill this._startTimestamp with content parsed from the passed doc.
		var startTimestampString = this._getTextNodeValue(
			doc, "reviewTimestamp",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		this._startTimestamp = startTimestampString;
		this._startTimestampDate = dojo.date.stamp.fromISOString(startTimestampString);
	},

	_parseInstructions: function(doc) {
		// summary:
		//    Fill this._instructions with content parsed from the passed doc.
		this._instructions = this._getTextNodeValue(
			doc, "reviewInstructions",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
	},

	_parseParticipants: function(doc) {
		// summary:
		//    Fill this._participantInfos with content parsed from the
		//    passed doc (but don't fill in the properties for
		//    participantStatusInfo) [that's done by a separate call to
		//    _fillParticipantStatusProperties].
		var participantInfos = [];
		var self = this;
		var getParticipantRoleType = function(documentRoleAttribute) {
			switch (documentRoleAttribute) {
				case "Approver":
					return reviewTypes.PARTICIPANT_ROLE_TYPES.Approver;
				case "Required Reviewer":
					return reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer;
				case "Optional Reviewer":
					return reviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer;
			};
		};
		var getRoleFromHasParticipantNode = function(hasParticipantNode) {
			return self._getTextNodeValue(
				hasParticipantNode, "reviewRole",
				queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
			);
		};
		var nodes = domutils.getElementsByTagNameNS(
			doc, "hasParticipant",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		dojo.forEach(
			nodes,
			function(node) {
				var userURI = this._parseUserURIFromHasParticipantNodeAndStoreUserMapping(node);
				var role = getRoleFromHasParticipantNode(node);
				if (userURI) {
					var participantInfo = {
						userURI: userURI,
						participantRoleType: getParticipantRoleType(role),
						participantStatusType: reviewTypes.PARTICIPANT_STATUS_TYPES.NotStarted
					};
					participantInfos.push(participantInfo);
					participantInfos[userURI] = participantInfo;
				}
			},
			this
		);
		this._participantInfos = participantInfos;
	},

	_parseUserURIFromHasParticipantNodeAndStoreUserMapping: function(hasParticipantNode) {
		// summary:
		//    Add a new entry to this._mapUserUriToOriginalUserPath and return
		//    the user path found in 'hasParticipantNode', but fully qualified.
		var nodes = domutils.getElementsByTagNameNS(
			hasParticipantNode, "contributor",
			queryutils.DC_NAMESPACE, queryutils.DC_QUALIFIER
		);
		var userPath = domutils.getAttributeNS(
			nodes[0], "resource",
			queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER
		);
		var fullyQualifiedUserURI = null;
		if (userPath.substring(0, 3) === "../") {
			fullyQualifiedUserURI =
				queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(
					this._resourceURI, userPath);
		}
		else if (userPath.indexOf("/jazz") === 0) {
			// FIXME Remove this workaround for the fact that review participants
			// are stored like this:
			//    <dc:contributor rdf:resource="/jazz/users/TestJazzAdmin1"/>
			// instead of this:
			//    <dc:contributor rdf:resource="../../jazz/users/TestJazzAdmin1"/>
			// as soon as server code automatically transitions the review's state from
			// "Started" to "Complete".
			userPath = "../.." + userPath;
			fullyQualifiedUserURI =
				queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(
					this._resourceURI, userPath);
		}
		else {
			fullyQualifiedUserURI = userPath;
		}
		// Add this new entry to this._mapUserUriToOriginalUserPath.
		this._mapUserUriToOriginalUserPath[fullyQualifiedUserURI] = userPath;
		return fullyQualifiedUserURI;
	},

	_parseArtifacts: function(doc, participantInfos) {
		// summary:
		//    Fill this._artifactInfos with content parsed from the passed doc.
		// description:
		//    This function will ONLY add review related fields to the newly
		//    constructed this._artifactInfos.  Participant result related field
		//    will NOT be added here.  Instead, a subsequent call to:
		//        _resetResultRelatedFields
		//    will patch up these artifactInfos to have appropriate values for
		//    the following fields:
		//        myArtifactStatusType
		//        overallArtifactStatusInfo
		//    [This is due to the fact that obtaining the participant results
		//     requires a separate server call from the one used to get the
		//     review itself.]
		var artifactInfos = [];
		var nodes = domutils.getElementsByTagNameNS(
			doc, "ReviewArtifact",
			queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER
		);
		var self = this;
		var getPersistedURIFromReviewArtifactNode = function(node) {
			var nodes = domutils.getElementsByTagNameNS(
				node, "artifact", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER);
			var relativeResource = domutils.getAttributeNS(
				nodes[0], "resource",
				queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER
			);
			return queryutils.getFullyQualifiedUriFromAbsoluteResourceAndRelativeResource(
				self._resourceURI, relativeResource);
		};
		dojo.forEach(
			nodes,
			function(node) {
				var name = this._getTextNodeValue(
					node, "title", queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER);
				var contentType = this._getTextNodeValue(
					node, "format", queryutils.DC_NAMESPACE, queryutils.DC_QUALIFIER);
				var persistedURI = getPersistedURIFromReviewArtifactNode(node);
				var id = this._getTextNodeValue(
					node, "artifactId", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER);
				var lastModifiedDate = this._getTextNodeValue(
					node, "modified", queryutils.DCTERMS_NAMESPACE, queryutils.DCTERMS_QUALIFIER);
				var artifactIndex = this._getTextNodeValue(
					node, "artifactIndex", queryutils.REVIEW_NAMESPACE, queryutils.REVIEW_QUALIFIER);
				// TODO There's also a "contributor":
				//     <dc:contributor rdf:resource="https://localhost:9443/rdm/users/TestJazzAdmin1"/>
				// should we be parsing this as well?
				var questionMarkIndex = persistedURI.indexOf("?");
				var resourceShortName = (
					questionMarkIndex === -1 ?
					persistedURI :
					persistedURI.substring(0, questionMarkIndex)
				);
				var resourceURI = resourceShortName;
				var revision = null;
				if (questionMarkIndex !== -1) {
					var equalSignIndex = persistedURI.indexOf("=", questionMarkIndex+1);
					revision = persistedURI.substring(equalSignIndex+1);
				}
				var overallArtifactStatusInfo =
					this._composeOverallArtifactStatusInfo(
						[], participantInfos);
				artifactInfos[artifactIndex] = {
					artifactName: name,
					resourceURI: resourceURI,
					revision: revision,
					contentType: contentType,
					lastModifiedDate: lastModifiedDate,
					id: id,
					versionLabel: this._getLastModifiedDateLabel(lastModifiedDate),
					myArtifactStatusType: null,
					overallArtifactStatusInfo: overallArtifactStatusInfo,
					persistedURI: persistedURI
				};
			},
			this
		);
		// Make artifactInfos a hashmap keyed by id and value'd by artifactInfo.
		dojo.forEach(artifactInfos, function(artifactInfo) {
			artifactInfos[artifactInfo.id] = artifactInfo;
		});
		this._artifactInfos = artifactInfos;
	},

	_getTextNodeValue: function(node, unqualifiedTagName, namespace, qualifier) {
		var nodes = domutils.getElementsByTagNameNS(node, unqualifiedTagName, namespace, qualifier);
		return (
			nodes.length > 0 && nodes[0].firstChild ?
			nodes[0].firstChild.nodeValue :
			""
		);
	},

	_getLastModifiedDateLabel: function(lastModifiedString) {
		var lastModifiedDateLabel = "";
		if (lastModifiedString) {
			var date = dojo.date.stamp.fromISOString(lastModifiedString);
			lastModifiedDateLabel = reviewutils.composeLastModifiedDateDisplayLabel(date);
		}
		return lastModifiedDateLabel;
	},

	_fillParticipantStatusProperties: function(participantInfos, artifactInfos, artifactResultInfos) {
		// summary:
		//    Run through participantInfos and fill in the
		//    participantStatusInfo field.
		dojo.forEach(
			participantInfos,
			function(participantInfo) {
				var singleParticipantArtifactResultInfos =
					artifactResultInfos[participantInfo.userURI];
				participantInfo.participantStatusInfo =
					this._composeParticipantStatusInfo(
						singleParticipantArtifactResultInfos,
						artifactInfos.length);
			},
			this
		);
	},

	_fillPercentReviewCompleted: function(participantInfos, artifactInfos, artifactResultInfos) {
		// summary:
		//    Fill in this._percentReviewCompleted.
		var numApproversAndReviewers = 0; // Note: This does NOT include "Optional Reviewers"
		var numParticipants = participantInfos.length;
		var numArtifacts = artifactInfos.length;
		var numVotesCastByApproversAndReviewers = 0;
		dojo.forEach(
			participantInfos,
			function(participantInfo) {
				var userURI = participantInfo.userURI;
				var role = participantInfo.participantRoleType;
				if (role === reviewTypes.PARTICIPANT_ROLE_TYPES.Approver
					|| role === reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer
					) {
					numApproversAndReviewers++;
					var userArtifactResultInfos = artifactResultInfos[userURI];
					dojo.forEach(
						userArtifactResultInfos,
						function(userArtifactResultInfo) {
							var statusType = userArtifactResultInfo.artifactStatusType;
							if (statusType === reviewTypes.ARTIFACT_STATUS_TYPES.Approved
								|| statusType === reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved
								|| statusType === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained
								|| statusType === reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed
								) {
								numVotesCastByApproversAndReviewers++;
							}
						}
					);
				}
			}
		);
		this._percentReviewCompleted = (
			(numApproversAndReviewers === 0
				|| numVotesCastByApproversAndReviewers === 0
				|| numArtifacts === 0) ?
			0 :
			Math.round(
				100 *
				(numVotesCastByApproversAndReviewers
					/ (numApproversAndReviewers * numArtifacts)))
		);
	},

	_composeOverallArtifactStatusInfo: function(singleArtifactResultInfos, participantInfos) {
		var isUserRoleApproverOrReviewer = function(artifactResultInfo) {
			var userURI = artifactResultInfo.userURI;
			var role = participantInfos[userURI].participantRoleType;
			return (
				role === reviewTypes.PARTICIPANT_ROLE_TYPES.Approver
				|| role === reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer
			);
		};
		var numApproved = 0;
		var numDisapproved = 0;
		var numAbstained = 0;
		var numReviewed = 0;
		var numVotesCastByApproversAndReviewers = 0;
		dojo.forEach(
			singleArtifactResultInfos,
			function(artifactResultInfo) {
				var voteWasCast = false;
				switch (artifactResultInfo.artifactStatusType) {
					case reviewTypes.ARTIFACT_STATUS_TYPES.Approved:
						numApproved++;
						voteWasCast = true;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
						numDisapproved++;
						voteWasCast = true;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Abstained:
						numAbstained++;
						voteWasCast = true;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
						numReviewed++;
						voteWasCast = true;
						break;
				}
				if (voteWasCast && isUserRoleApproverOrReviewer(artifactResultInfo)) {
					numVotesCastByApproversAndReviewers++;
				}
			}
		);
		var numApproversAndReviewers = 0;
		dojo.forEach(
			participantInfos,
			function(participantInfo) {
				var role = participantInfo.participantRoleType;
				if (role === reviewTypes.PARTICIPANT_ROLE_TYPES.Approver
					|| role === reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer
					) {
					numApproversAndReviewers++;
				}
			}
		);
		var percentageVotesCast = (
			(numVotesCastByApproversAndReviewers === 0
				|| numApproversAndReviewers === 0) ?
			0 :
			Math.round(
				100 *
				(numVotesCastByApproversAndReviewers / numApproversAndReviewers)
			)
		);
		return {
			percentageVotesCast: percentageVotesCast,
			numApproved: numApproved,
			numDisapproved: numDisapproved,
			numAbstained: numAbstained,
			numReviewed: numReviewed
		};
	},

	_composeParticipantStatusInfo: function(singleParticipantArtifactResultInfos, numReviewArtifacts) {
		var numApproved = 0;
		var numDisapproved = 0;
		var numAbstained = 0;
		var numReviewed = 0;
		var numInProgress = 0;
		dojo.forEach(
			singleParticipantArtifactResultInfos,
			function(artifactResultInfo) {
				switch (artifactResultInfo.artifactStatusType) {
					case reviewTypes.ARTIFACT_STATUS_TYPES.Approved:
						numApproved++;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
						numDisapproved++;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Abstained:
						numAbstained++;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
						numReviewed++;
						break;
					case reviewTypes.ARTIFACT_STATUS_TYPES.InProgress:
						numInProgress++;
						break;
				}
			}
		);
		var numVotesCast = numApproved + numDisapproved + numAbstained + numReviewed;
		var percentageVotesCast = (
			numVotesCast === 0 || singleParticipantArtifactResultInfos.length === 0 ?
			0 :
			Math.round(100 * (numVotesCast / numReviewArtifacts))
		);
		return {
			percentageVotesCast: percentageVotesCast,
			numApproved: numApproved,
			numDisapproved: numDisapproved,
			numAbstained: numAbstained,
			numReviewed: numReviewed,
			numInProgress: numInProgress
		};			
	}
});


})();
