/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewParticipantsViewerSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.review.ReviewHoverPopup");
dojo.require("com.ibm.rdm.web.review.ReviewProgressBar");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ReviewHoverPopup = com.ibm.rdm.web.review.ReviewHoverPopup;
var ReviewProgressBar = com.ibm.rdm.web.review.ReviewProgressBar;
var UserHover = com.ibm.rdm.web.common.UserHover;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;

var MAX_ARTIFACTS_BEFORE_PROGRESS_BAR_IS_NEEDED = 10;


dojo.declare(
	"com.ibm.rdm.web.review.ReviewParticipantsViewerSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a widget used to display information about the participants
	//    involved in a review (including their name, role and current status).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ReviewParticipantsViewerSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_reviewParticipantsTableBody: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	reviewParser: null,
	onParticipantRowArtifactResultCellHighlighted: null,
	onParticipantRowArtifactResultCellUnhighlighted: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_connectHandles: null,
	_userHovers: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._userHovers = [];
		this._connectHandles = [];
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	startup: function() {
		this._populateContents();
        this.inherited(arguments);
	},

	destroy: function() {
		this._removeConnectHandles();
		this.inherited("destroy", arguments);
	},

	highlightArtifactIndex: function(artifactIndex) {
		var artifactInfos = this.reviewParser.getArtifactInfos();
		if (artifactInfos.length <= MAX_ARTIFACTS_BEFORE_PROGRESS_BAR_IS_NEEDED) {
			this._setHighlightForArtifactInUserStatusArtifactIconTable(
				artifactIndex,
				true			// doHighlight
			);
		}
	},

	unhighlightArtifactIndex: function(artifactIndex) {
		var artifactInfos = this.reviewParser.getArtifactInfos();
		if (artifactInfos.length <= MAX_ARTIFACTS_BEFORE_PROGRESS_BAR_IS_NEEDED) {
			this._setHighlightForArtifactInUserStatusArtifactIconTable(
				artifactIndex,
				false			// doHighlight
			);
		}
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		this.reviewParser = reviewParser;
		if (this._started) {
			// If table rows are currently present, do proper clean up.
			if (this._reviewParticipantsTableBody.firstChild) {
				this._removeUserHovers();
				domutils.removeChildNodes(this._reviewParticipantsTableBody);
				this._removeConnectHandles();
			}
			// Add new table rows with up to date content.
			this._populateContents();
		}
	},

	_removeConnectHandles: function() {
		if (this._connectHandles) {
			for (var i = 0; i < this._connectHandles.length; i++) {
				dojo.disconnect(this._connectHandles[i]);
			}
		}
		this._connectHandles = [];
	},

	_populateContents: function() {
		var participantInfos = this.reviewParser.getParticipantInfos();
		var participantResultInfos = this.reviewParser.getParticipantResultInfos();
		for (var i = 0; i < participantInfos.length; i++) {
			var participantInfo = participantInfos[i];
			var userURI = participantInfo.userURI;
			this._addParticipantTableRow(userURI, participantInfo);
		}
	},

	_addParticipantTableRow: function(userURI, participantInfo) {
		var participantRoleType = participantInfo.participantRoleType;
		var roleDisplayName =
			reviewTypes.getParticipantRoleTypeDisplayName(participantRoleType);
		var tr = document.createElement("tr");
		this._reviewParticipantsTableBody.appendChild(tr);
		var userTD = document.createElement("td");
		var roleTD = document.createElement("td");
		var statusTD = document.createElement("td");
		userTD.innerHTML =
			"<td><span class=\"userSpan\">" + userURI + "</span></td>";
		var roleImage = reviewutils.createParticipantRoleImage(participantRoleType);
		roleImage.className = "participantRoleImage";
		roleTD.appendChild(roleImage);
		var roleSpan = document.createElement("span");
		roleSpan.className = roleSpan;
		roleSpan.appendChild(document.createTextNode(roleDisplayName));
		roleTD.appendChild(roleSpan);
		tr.appendChild(userTD);
		tr.appendChild(roleTD);
		tr.appendChild(statusTD);
		this._fillStatusTableData(statusTD, userURI, participantInfo);
		var userSpan = userTD.firstChild;
		var self = this;
		var onUserInfoComplete = function(userInfo, userSpan) {
			self._addUserHover(userSpan, userInfo);
			self._moveTableRowToSortedRowPositionUsingUserName(tr, userInfo.name);
		};
		reviewutils.fillUserSpanAndAddUserImage(userURI, userSpan, onUserInfoComplete);
		// Add highlighting to the row when moused over.
		var self = this;
		this._connectHandles.push(dojo.connect(tr, "onmouseover", function(event) {
			dojo.stopEvent(event);
			dojo.addClass(tr, "highlightRow");
		}));
		this._connectHandles.push(dojo.connect(tr, "onmouseout", function(event) {
			dojo.stopEvent(event);
			dojo.removeClass(tr, "highlightRow");
		}));
	},

	_addUserHover: function(userSpan, userInfo) {
		var userHover = new UserHover(
			{ projectURI : this.reviewParser.getProjectUri() },
			null
		);
		userHover.createHoverPopup(userSpan, userInfo);
		this._userHovers.push(userHover);
	},

	_removeUserHovers: function() {
		dojo.forEach(
			this._userHovers,
			function(userHover) {
				userHover.destroy();
			}
		);
		this._userHovers = [];
	},

	_moveTableRowToSortedRowPositionUsingUserName: function(tr, userName) {
		var tbody = this._reviewParticipantsTableBody;
		var lowerCaseUserName = userName.toLowerCase();
		var foundNodeToInsertBefore = false;
		var compareTR = tbody.firstChild;
		var getRowUserName = function(tr) {
			var userNameTD = tr.firstChild;
			var userNameSpan = null;
			// Note: We can't just use:
			//     var userNameSpan = userNameTD.lastChild;
			// as this works for FF but doesn't for IE (as it returns DispHTMLUnknownElement).
			// Instead we hunt down the SPAN element when using IE ...
			if (dojo.isIE) {
				for (var child = userNameTD.firstChild; child != null; child = child.nextSibling) {
					if (child.nodeName.toLowerCase() === "span") {
						userNameSpan = child;
						break;
					}
				}
			} else {
				userNameSpan = userNameTD.lastChild;
			}
			var textNode = userNameSpan.firstChild;
			return (dojo.isIE ? textNode.nodeValue : textNode.textContent);
		};
		for (; compareTR != null; compareTR = compareTR.nextSibling) {
			if (tr != compareTR) {
				var compareUserName = getRowUserName(compareTR);
				if (lowerCaseUserName < compareUserName.toLowerCase()) {
					foundNodeToInsertBefore = true;
					dojo.place(tr, compareTR, "before");
					break;
				}
			}
		}
		if (!foundNodeToInsertBefore && compareTR != null && tr != compareTR) {
			dojo.place(tr, compareTR, "after");
		}
	},

	_fillStatusTableData: function(statusTD, userURI, participantInfo) {
		var artifactInfos = this.reviewParser.getArtifactInfos();
		if (artifactInfos.length > MAX_ARTIFACTS_BEFORE_PROGRESS_BAR_IS_NEEDED) {
			var progressBar =
				this._createUserStatusProgressBar(userURI, participantInfo);
			statusTD.appendChild(progressBar.domNode);
			// TODO Add text for number disapproved after the progress bar?
		} else {
			var artifactResultsIconTable =
				this._createUserStatusArtifactIconTable(userURI, participantInfo);
			statusTD.appendChild(artifactResultsIconTable);
		}
		var reviewHoverPopup = new ReviewHoverPopup({
			viewType: ReviewHoverPopup.VIEW_TYPE.SINGLE_PARTICIPANT_SUMMARY_STATUS,
			userURI: userURI
        });
		reviewHoverPopup.createHoverPopup(statusTD, this.reviewParser);
	},

	_createUserStatusProgressBar: function(userURI, participantInfo) {
		var participantStatusInfo = participantInfo.participantStatusInfo;
		// Note: If the review is in the the Draft state, the
		// participantStatusInfo would be set to 'undefined'.  In this
	    // case we'd use 0 for the percentageVotesCast.
		var percentageVotesCast = (
			participantStatusInfo ?
			participantStatusInfo.percentageVotesCast :
			0
		);
		return new ReviewProgressBar({
			progress: percentageVotesCast,
			viewType: ReviewProgressBar.VIEW_TYPE.Completed
		});
	},

	_createUserStatusArtifactIconTable: function(userURI, participantInfo) {
		var artifactInfos = this.reviewParser.getArtifactInfos();
		var artifactResultInfos = this.reviewParser.getArtifactResultInfos();
		var table = document.createElement("table");
		table.className = "artifactResultsIconTable";
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		var mapArtifactIdToTargetedUserArtifactStatus = new Object();
		dojo.forEach(
			artifactResultInfos[userURI],
			function(targetedUserResultInfo) {
				mapArtifactIdToTargetedUserArtifactStatus[targetedUserResultInfo.artifact] =
					targetedUserResultInfo.artifactStatusType;
			}
		);
		// Create all the TR's.
		var self = this;
		dojo.forEach(
			artifactInfos,
			function(artifactInfo) {
				var td = document.createElement("td");
				var artifactStatusType = (
					mapArtifactIdToTargetedUserArtifactStatus[artifactInfo.id]
					|| reviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
				);
				var statusDisplayName =
					reviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType);
				var img = reviewutils.createArtifactStatusTypeImage(artifactStatusType);
				img.title = statusDisplayName;
				td.appendChild(img);
				self._addTableDataHighlightSupport(td);
				tr.appendChild(td);
			}
		);
		table.appendChild(tbody);
		tbody.appendChild(tr);
		return table;
	},

	_setHighlightForArtifactInUserStatusArtifactIconTable: function(artifactIndex, doHighlight) {
		dojo.forEach(
			this._reviewParticipantsTableBody.childNodes,
			function(tr) {
				var artifactResultsIconTable = tr.lastChild.firstChild;
				var targetTD = artifactResultsIconTable.tBodies[0].firstChild.childNodes[artifactIndex];
				if (doHighlight) {
					dojo.addClass(targetTD, "highlight");
				} else {
					dojo.removeClass(targetTD, "highlight");
				}
			}
		)
	},

	_addTableDataHighlightSupport: function(td) {
		var self = this;
		this._connectHandles.push(dojo.connect(td, "onmouseover", function(event) {
			dojo.stopEvent(event);
			dojo.addClass(td, "highlight");
			if (self.onParticipantRowArtifactResultCellHighlighted) {
				self.onParticipantRowArtifactResultCellHighlighted(td.cellIndex);
			}
		}));
		this._connectHandles.push(dojo.connect(td, "onmouseout", function(event) {
			dojo.stopEvent(event);
			dojo.removeClass(td, "highlight");
			if (self.onParticipantRowArtifactResultCellUnhighlighted) {
				self.onParticipantRowArtifactResultCellUnhighlighted(td.cellIndex);
			}
		}));
	}
});


})();
