/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewProgressBar");

dojo.require("dijit.ProgressBar");
dojo.require("dijit._Widget");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var ProgressBar = dijit.ProgressBar;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;

var MIN_PROGRESS_BAR_WIDTH_IN_PIXELS = 110;
var PADDING_FOR_SIDES_OF_PROGRESS_LABEL_IN_PIXELS = 8;


dojo.declare(
	"com.ibm.rdm.web.review.ReviewProgressBar",
	dijit.ProgressBar,
{
	// summary:
	//    Provides a uniform look for progress bars that are used to display
	//    review related percentage complete or percentage reviewed
	//    information.

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	viewType: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	buildRendering: function(){
		this.inherited("buildRendering", arguments);
		dojo.addClass(this.domNode, "rdmReviewProgressBar");
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
		this.inherited("postMixInProperties", arguments);
	},

	report: function(/*float*/percent){
		var percent =
			dojo.number.format(
				percent,
				{ type: "percent", places: this.places, locale: this.lang }
			);
		var keyToUse = null;
		switch (this.viewType) {
			case com.ibm.rdm.web.review.ReviewProgressBar.VIEW_TYPE.Completed:
				keyToUse = this._messages["percentCompleted"];
				break;
			case com.ibm.rdm.web.review.ReviewProgressBar.VIEW_TYPE.Reviewed:
				keyToUse = this._messages["percentReviewed"];
				break;
		}
		var labelToUse = dojo.string.substitute(keyToUse, [percent]);
		this._setWidthToAppropriateValue(labelToUse);
		return labelToUse;
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setWidthToAppropriateValue: function(label) {
		var offsetWidth =
			viewerutils.calculateTextWidth(label, "Arial,sans-serif", "8pt", "bold");
		var widthToUse = Math.max(
			offsetWidth + PADDING_FOR_SIDES_OF_PROGRESS_LABEL_IN_PIXELS,
			MIN_PROGRESS_BAR_WIDTH_IN_PIXELS
		);
		this.domNode.style.width = widthToUse + "px";
	}
});


})();

com.ibm.rdm.web.review.ReviewProgressBar.VIEW_TYPE = {
	Completed: "Completed",
	Reviewed: "Reviewed"
};
