/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.ReviewsViewer");

dojo.require("com.ibm.rdm.web.review.ReviewEntry");
dojo.require("dijit._Templated");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.review.ReviewParser");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.Toolbar");
dojo.require("dijit.Menu");
dojo.require("com.ibm.rdm.web.common.CollapseableTitleSection");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.review.DashboardReviewParser");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

dojo.requireLocalization("com.ibm.rdm.web.review", "ReviewsViewer");

// Shortcut variables
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var ReviewParser = com.ibm.rdm.web.review.ReviewParser;
var ReviewEntry = com.ibm.rdm.web.review.ReviewEntry;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var DashboardReviewParser = com.ibm.rdm.web.review.DashboardReviewParser;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

var PAGE_SIZE = 7;

dojo.declare(
        "com.ibm.rdm.web.review.ReviewsViewer",
        [dijit._Widget, dijit._Templated],
{
    // summary:
    //    Displays a list of reviews and their details, and navigates to the review editor
    //    when a specific review is clicked.

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "review/templates/ReviewsViewer.html"),
    widgetsInTemplate: true,
    
    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
    scopingProjectURIs: null,
    projectURI: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    
    ///////////////////////////////////////////////////////////////////////////////////
    _reviewEntryWidgets: null,
    // Attach points
    _reviewTitleNode: null,
    _authorNode: null,
    _timestampNode: null,
    _sortReviewsBy: 0,
    _groupReviewsBy: 0,
    _filterDisplayReviews: 0,
    _messages: null,
    _collapseableHeaders: null,
    _hideFinalizedReviews: null,
    _reviewsAreGrouped: false,
    _currentPage: null,
    _totalPages: null,
    _mouseOverHandle: null,
    _mouseOutHandle: null,
    _paginationSet: false,
    
	postCreate: function() {
		this._firstLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/beginning.gif");		
		this._lastLinkImage.src = dojo.moduleUrl("com.ibm.rdm.web", "common/images/icons/etool16/end.gif");		
	},
	
    startup: function() {
        this._collapseableHeaders = [];
        this._reviewEntryWidgets = [];
        this.inherited("startup", arguments);       
        //Default sort by Name, Group by None
        this._sortReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_NAME;
        this._groupReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_NONE;
        this._filterDisplayReviews = com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS;
        dojo.addClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewToolbarDropdownBullet");
        //Accessibility
        this._reviewToolbarOptionsDrowpdown.setAttribute("alt", this._messages["sort"]);
	    this._reviewToolbarOptionsDrowpdown.setAttribute("title", this._messages["sort"]);
	    //Group by None icon set
	    dojo.addClass(this._reviewGroupByReviewNoneMenuItem.iconNode, "reviewToolbarDropdownBullet");
	    this._hideFinalizedReviews = false;
	    
	    if(this.myReviewsViewlet==true){
			// Note by SPC on 11/17/2009 ... I'm commenting the following lines out
			// so that the drop down is NOT removed.  The reason for this is that
			// I'm attempting to fix Defect 20692 and the fix involves making sure
			// that startup is called regardless of whether it's called in the
			// context of the sidebar or the user dashboard.  However, I found that
			// in the context of the user dashboard, startup was NOT being called.
			// Now that I'm making sure that startup IS called when used in the user
			// dashboard, we need to make sure we STILL see the drop down button
			// (as this drop down IS present in the bits baked into release 2.0.0.0).
// 	    	if(this._reviewToolbarOptionsDrowpdown.domNode.parentNode){
// 	    		this._reviewToolbarOptionsDrowpdown.domNode.parentNode.removeChild(this._reviewToolbarOptionsDrowpdown.domNode);
// 	    	}
	    	this._loadUserDashBoardReviews();
    	}
    	else{
    		this._loadProjectDashBoardReviews();
    	}

		this._showAllReviewsButton.attr('checked', true);
    },
    
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.review", "ReviewsViewer");
		dojo.forEach(
			[
				"sortByReviewName",
				"sortByMyRole",
				"sortByReviewStatus",
				"sortByDueDate",
				"sortByCreator",
				"sortByState",
				"sortByDateLastModified",
				"sort",
				"groupByNone",
				"groupByDate",
				"groupByAuthor",
				"groupByState",
				"groupByReviewStatus",
				"groupByModifiedDate",
				"groupByMyRole",
				"modifiedOn",
				"status",
				"today",
				"yesterday",
				"pastWeek",
				"pastMonth",
				"earlier",
				"none",
				"draft",
				"inProgress",
				"complete",
				"finalized",
				"hideFinalizedReviews",
				"sortingAndGroupingOptions",
				"reviewer",
				"optionalReviewer",
				"approver",
				"owner",
				"nonParticipant",
				"paused",
				"showMyReviews",
				"showActiveReviews",
				"showAllReviews",
				"previous",
				"next",
				"pageNOfM"
			],
			function(prop) { this[prop] = this._messages[prop]; },
			this
		);
	},
    
    destroy: function() {
    	if(this._collapseableHeaders){
    		for (var i = 0; i < this._collapseableHeaders.length; i++) {
    			this._collapseableHeaders[i].destroyRecursive();
    			this._collapseableHeaders[i] = null;
    		}
    	}
    	this._collapseableHeaders = [];
    	dojo.forEach(this._reviewEntryWidgets, 
                function(rew) {
					if (rew && rew.destroy) {
						rew.destroy();
					}
                });    	
    	this._reviewEntryWidgets = [];
    },
    
    handleRefresh: function() {
    	this._resetState();
	    if(this.myReviewsViewlet==true){
	    	this._loadUserDashBoardReviews();
    	}
    	else{
    		this._loadProjectDashBoardReviews();
    	}
    },
    
	_loadUserDashBoardReviews: function() {
    	var deferred = ProjectCache.getUserMemberProjects();
        deferred.addCallback(this, this._processUserDashBoardReviews); 
        //this._processUserDashBoardReviews();
    },
    
	_processUserDashBoardReviews: function(projects) {
		this._setLoadingReviewHeaderMessage();
    	//for (projectURL in projects) {
    		//this.projectURI = projectURL; 
            var postData = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL;
            var deferred = RDMClient.doGet({uri:postData}); 
            deferred.addCallback(this, this._handleJFSQuerySuccessResponse, projects);
            deferred.addErrback(this, function(error) {
            	com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            		7217, [], error
    			);
        	});
    	//} 
    	ErrorHandler.warn("TODO _processUserDashBoardReviews needs to be revisited.");
    },
    
	_loadProjectDashBoardReviews: function() {
    	var projectURI = this.scopingProjectURIs;
    	this.projectURI = projectURI;
    	var postData = com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL + encodeURI("?resourceContext=" + projectURI);
    	this._setLoadingReviewHeaderMessage();
    	
    	var deferred = RDMClient.doGet({uri:postData});    	
    	deferred.addCallback(this, this._handleJFSQuerySuccessResponse, null);
    	deferred.addErrback(this, function(error) {
    		domutils.createDefaultCursor();
            com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().showErrorNumber(
            	7217, [], error
    		);
        });	 	
    },
    
    _handleJFSQuerySuccessResponse: function(projects, response) {
    	var reviewDescriptions = [];
    	var roleDescriptions = [];
    	// clear the waiting cursor which is set in SideBarLayout._onSectionHeaderClick
    	if(!this.myReviewsViewlet){
    		domutils.createDefaultCursor();
    	}

    	if(response.doc){
    		var self = this;
    		responseDoc = response.doc;
    		var revCount = 0;
    		var descriptionNodes = domutils.getElementsByTagNameNS(
						response.doc, "Description",
						queryutils.RDF_NAMESPACE, 
						queryutils.RDF_QUALIFIER
				);			
    		for(var i=0; i<descriptionNodes.length; i++) {		
    			//Workaround for extra description nodes.
    			//This may need to be removed
    			var aboutAttribute = domutils.getAttributeNS(
    				descriptionNodes[i], "about", 
    				queryutils.RDF_NAMESPACE, 
    				queryutils.RDF_QUALIFIER)
    				
				var nodeIdAttribute = domutils.getAttributeNS(
    				descriptionNodes[i], "nodeID", 
    				queryutils.RDF_NAMESPACE, 
    				queryutils.RDF_QUALIFIER)
    				
    			var roleAttribute = this._getTextNodeValue(
    				descriptionNodes[i], "reviewRole", 
    				queryutils.REVIEW_NAMESPACE, 
    				queryutils.RRM_QUALIFIER)

    			if(aboutAttribute){	
					var descriptionNode = descriptionNodes[i];
					var resourceURI = queryutils.getResourceURIfromDescription(descriptionNode);
					var projectURI = queryutils.getProject(descriptionNode);
					var title = queryutils.getRawTitle(descriptionNode);
					var lastModifiedDate = queryutils.getModifiedDate(descriptionNode);
					
					//if review is in the project that user is not a member of, skip it
					if (projects && !projects[projectURI]) {
						continue;
					}
					
					var participantNodes = domutils.getElementsByTagNameNS(descriptionNodes[i], "hasParticipant",
																		queryutils.REVIEW_NAMESPACE,
																		queryutils.RRM_QUALIFIER);	
					var participantNodeIdsMap = null;
					for (var j=0; j<participantNodes.length; j++) {
						var participantNodeId = domutils.getAttributeNS(
	    					participantNodes[j], "nodeID", 
	    					queryutils.RDF_NAMESPACE, 
	    					queryutils.RDF_QUALIFIER);
	    				if(!participantNodeIdsMap){
	    					participantNodeIdsMap = {};
	    				}
	    				participantNodeIdsMap[participantNodeId] = participantNodes[j];
					}
					var reviewInfo = {
						descriptionNode: descriptionNode,
						resourceURI: resourceURI,
						projectURI: projectURI,
						title: title,
						lastModifiedDate: lastModifiedDate,
						participantNodeIdsMap: participantNodeIdsMap
					};
					reviewDescriptions.push(reviewInfo);
					
                	revCount++;
    			}
    			else if (nodeIdAttribute && roleAttribute){
    				var contributorNode = domutils.getElementsByTagNameNS(descriptionNodes[i], "contributor",
    																queryutils.DC_NAMESPACE, queryutils.DC_QUALIFIER);
        			var resourceNode = contributorNode[0].attributes.getNamedItem("rdf:resource");
        			var contributorUri = resourceNode.firstChild.nodeValue;
    				var roleInfo = {
						nodeId: nodeIdAttribute,
						reviewRole: roleAttribute,
						contributorUri: contributorUri
					};
					roleDescriptions.push(roleInfo);
    			}
    		}
    		this._createReviewEntries(projects, reviewDescriptions, roleDescriptions);
    		this._sortReviews(this._reviewEntryWidgets);		
    		}
    		if(this.myReviewsViewlet==true){
    			this._updateReviewUserDashHeader(this._reviewEntryWidgets.length);
    			this._updatePagination(this._reviewEntryWidgets.length);
    		}
    		else{
    			this._setHeaderReviewCount(revCount);
    		}
    },
    
    _createReviewEntries: function(projects, reviewDescriptions, roleDescriptions){
    	for(var i=0; i<reviewDescriptions.length; i++) {
    		var reviewRole = "";
    		var currentUser = com.ibm.rdm.web.AUTHENTICATED_USER;
    		var participantNodeIdsMap = reviewDescriptions[i].participantNodeIdsMap;

			for(var j=0; j<roleDescriptions.length; j++) {
				if (participantNodeIdsMap) {
					if(participantNodeIdsMap[roleDescriptions[j].nodeId]
					&& currentUser.uri === roleDescriptions[j].contributorUri ){
						reviewRole = roleDescriptions[j].reviewRole;
						break;
					}
				}
			}

    		var revParser = new DashboardReviewParser({
                    	"review": reviewDescriptions[i].descriptionNode
            	});
            revParser.setRole(reviewRole);
            var reviewEntry = new ReviewEntry({
            	"reviewParser": revParser,
            	"reviewResourceUri": reviewDescriptions[i].resourceURI,
            	"reviewModifiedDate": reviewDescriptions[i].lastModifiedDate,
            	"projectURI" : reviewDescriptions[i].projectURI
            	});
            	this._reviewsListNode.appendChild(reviewEntry.domNode);
                this._reviewEntryWidgets.push(reviewEntry);
    	}    	
    },
    
    _updatePagination: function(revCount){
		this._currentPage = 1;
        this._totalPages = Math.ceil(revCount/PAGE_SIZE);
        this._pageText.innerHTML = 
            dojo.string.substitute(this._messages.pageNOfM, [this._currentPage,this._totalPages]);
        if (this._totalPages == 0) {
        	this._footer.style.visibility = "hidden";
        } else {
        	this._footer.style.visibility = "";
        	if (!this._mouseOverHandle) {
        		this._mouseOverHandle = this.connect(this.domNode, "onmouseover", this._showNavOptions);
        	}
        	if (!this._mouseOutHandle) {
        		this._mouseOutHandle = this.connect(this.domNode, "onmouseout", this._hideNavOptions);
        	}
        }
    },
    
	_updateReviewUserDashHeader: function(revCount) {	
    	this.site.setTitle(dojo.string.substitute(this._messages.showingReviews, [revCount]));
    	this.site.setLoading(false);  	
    },
      
    _resetState: function() {
    	dojo.forEach(this._reviewEntryWidgets, 
                     function(rew) {
    					if (rew && rew.destroy) {
    						rew.destroy();
    					}
                     });    	
    	this._reviewEntryWidgets = [];
    },
 
	_sortReviews: function(reviewArray) {
    	switch(this._sortReviewsBy) {
			case com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_DATELASTMODIFIED:
				reviewArray.sort(compareDateModified);
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_DUEDATE:
				reviewArray.sort(compareDueDate);
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_NAME:
				reviewArray.sort(compareName);			
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_STATUS:
				reviewArray.sort(compareState);			
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_ROLE:
				reviewArray.sort(compareRole);			
				break;
    	}
    	if(this._reviewsAreGrouped == true){
    		this._groupReviews(reviewArray);
    	}
    	else{
    		this._createSortedReviewNodes(reviewArray);
    	}
    },
    
    _groupReviews: function(reviewArray) {
    	this._removeReviewNodes();
    	this.destroyCollapseableHeaders();
    	
    	if (this._filterDisplayReviews == com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS){
    		this._setHeaderReviewCount(reviewArray.length);
    	}
    	
    	if(this._groupReviewsBy == com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_STATUS) {
    		
			var draftCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(draftCollapseableHeader);
			this._reviewsListNode.appendChild(draftCollapseableHeader.domNode);
			
			var inProgressCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(inProgressCollapseableHeader);
			this._reviewsListNode.appendChild(inProgressCollapseableHeader.domNode);
			
			var pausedCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(pausedCollapseableHeader);
			this._reviewsListNode.appendChild(pausedCollapseableHeader.domNode);
			
			var completeCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(completeCollapseableHeader);
			this._reviewsListNode.appendChild(completeCollapseableHeader.domNode);
			
			var finalizedCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(finalizedCollapseableHeader);
			this._reviewsListNode.appendChild(finalizedCollapseableHeader.domNode);
			
			var statusDraftCount = 0;
			var statusInProgressCount = 0;
			var statusPausedCount = 0;
			var statusCompleteCount = 0;
			var statusFinalizedCount = 0;
			var numIncluded = 0;
			
    		for(z=0; z<reviewArray.length; z++){
    			if(this._checkFilterInclusions(reviewArray[z])){
    			switch(reviewArray[z].reviewParser.getReviewState()) {
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_DRAFT:
    					draftCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusDraftCount++;
    					break;
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_IN_PROGRESS:
    					inProgressCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusInProgressCount++;
    					break;
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_STARTED:
    					inProgressCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusInProgressCount++;
    					break;
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_PAUSED:		
    					pausedCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusPausedCount++;
    					break;
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_COMPLETE:
    					completeCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusCompleteCount++;
    					break;
    				case com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED:
    					finalizedCollapseableHeader.setContent(reviewArray[z].domNode);
    					statusFinalizedCount++;
    					break;
    			}
    			numIncluded++;
    			}
    		}
    		if (this._filterDisplayReviews !== com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS){
				this._setHeaderFilteredReviewCount(reviewArray.length, numIncluded);
    		}  						  						
    		// Set the title after we have a count
    		draftCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["draft"],
				[ this._createReviewCount(statusDraftCount) ]));
    		inProgressCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["inProgress"],
				[ this._createReviewCount(statusInProgressCount) ]));
    		pausedCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["paused"],
				[ this._createReviewCount(statusPausedCount) ]));
    		completeCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["complete"],
				[ this._createReviewCount(statusCompleteCount) ]));
    		finalizedCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["finalized"],
				[ this._createReviewCount(statusFinalizedCount) ]));

    		this._collapseGroupedItems();
    	}
    	
    	
    	if(this._groupReviewsBy == com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_ROLE) {
    		
			var reviewerCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(reviewerCollapseableHeader);
			this._reviewsListNode.appendChild(reviewerCollapseableHeader.domNode);
			
			var optionalReviewerCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(optionalReviewerCollapseableHeader);
			this._reviewsListNode.appendChild(optionalReviewerCollapseableHeader.domNode);
			
			var approverCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(approverCollapseableHeader);
			this._reviewsListNode.appendChild(approverCollapseableHeader.domNode);
			
			var ownerCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(ownerCollapseableHeader);
			this._reviewsListNode.appendChild(ownerCollapseableHeader.domNode);
			
			var noneCollapseableHeader = new com.ibm.rdm.web.common.CollapseableTitleSection({});
			this._collapseableHeaders.push(noneCollapseableHeader);
			this._reviewsListNode.appendChild(noneCollapseableHeader.domNode);
						
			var roleReviewerCount = 0;
			var roleOptionalReviewerCount = 0;
			var roleApproverCount = 0;
			var roleOwnerCount = 0;
			var noneCount = 0;
			var numIncluded = 0;
			
    		for(z=0; z<reviewArray.length; z++){
    			if(this._checkFilterInclusions(reviewArray[z])){
    			switch(reviewArray[z].reviewParser.getRole()) {
    				case reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer:
    					reviewerCollapseableHeader.setContent(reviewArray[z].domNode);
    					roleReviewerCount++;
    					break;
    				case reviewTypes.PARTICIPANT_ROLE_TYPES.Approver:
    					approverCollapseableHeader.setContent(reviewArray[z].domNode);
    					roleApproverCount++;
    					break;
    				case reviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer:
    					optionalReviewerCollapseableHeader.setContent(reviewArray[z].domNode);
    					roleOptionalReviewerCount++;
    					break;
    				default:
    					if(reviewArray[z].reviewParser.getCreatorUserURI()==
    			        	com.ibm.rdm.web.AUTHENTICATED_USER.uri){
    						ownerCollapseableHeader.setContent(reviewArray[z].domNode);
    						roleOwnerCount++;
    					}
    					else{
    						// Non participant
    						noneCollapseableHeader.setContent(reviewArray[z].domNode);
    						noneCount++;
    					}
    				}
    				numIncluded++;			 				
    			}	
    		}
    		
			if (this._filterDisplayReviews !== com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS){
				this._setHeaderFilteredReviewCount(reviewArray.length, numIncluded);
    		}
    		// Set the title after we have a count
    		reviewerCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["reviewer"],
				[ this._createReviewCount(roleReviewerCount) ]));
    		approverCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["approver"],
				[ this._createReviewCount(roleApproverCount) ]));
    		optionalReviewerCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["optionalReviewer"],
				[ this._createReviewCount(roleOptionalReviewerCount) ]));
    		ownerCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["owner"],
				[ this._createReviewCount(roleOwnerCount) ]));
    		noneCollapseableHeader.setTitle(dojo.string.substitute(
				this._messages["nonParticipant"],
				[ this._createReviewCount(noneCount) ]));
    		
    		this._collapseGroupedItems();
    	}
    },
    
    _createReviewCount: function(reviewCount){  	
    	if(reviewCount <= 0){
    		return this._messages["none"];
    	}
    	else{
    		return reviewCount;
    	} 	
    },
    
	_handleReviewClicked: function(event) {
		dojo.stopEvent(event);
		ResourceLinksHandler.handleAnchorClicked(event);
	},
	
    _expandGroupedItems: function(){
    	if(this._collapseableHeaders){
    		dojo.forEach(this._collapseableHeaders, function(collapseableHeader) {
    			if(collapseableHeader._contentDiv.childNodes.length > 0){
    				collapseableHeader.expand();
    			}
    		}); 
    	}
    },
    
    _collapseGroupedItems: function(){
    	if(this._collapseableHeaders){
    		dojo.forEach(this._collapseableHeaders, function(collapseableHeader) {
    			collapseableHeader.collapse();
    		}); 
    	}
    },
    
    _createSortedReviewNodes: function(reviews) {
    	this._removeReviewNodes();
    	var revCount = 0;
		switch(this._filterDisplayReviews) {
			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS:
				if(this.myReviewsViewlet==true && reviews.length > 0){
					if(!this._paginationSet){
						this._updatePagination(reviews.length)
		    		}
					this._paginationSet = true;
					for (var i = (this._currentPage-1)*PAGE_SIZE; i < reviews.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); i++){
						this._reviewsListNode.appendChild(reviews[i].domNode);
						revCount++;
					}
				}
				else{
		    		for (var i=0; i<reviews.length; i++){
		    			this._reviewsListNode.appendChild(reviews[i].domNode);
		    			revCount++;
		    		}
				}
				if(reviews){
					this._setHeaderReviewCount(reviews.length);
		    	}
		    	else {
		    		this._setHeaderReviewCount(0);
		    	}
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_MYREVIEWS:
				if(this.myReviewsViewlet==true && reviews.length > 0){
					var totalReviews = [];
					var j = 0;
					for (var i=0; i<reviews.length; i++){
		    			var reviewRole = reviews[i].reviewParser.getRole();
		    			// If reviewRole == null the user is not a participant
		    			if(reviewRole || reviews[i].reviewParser.getCreatorUserURI()==
			        		com.ibm.rdm.web.AUTHENTICATED_USER.uri){
		    				totalReviews[j] = reviews[i];
		    				j++;
		    			}
		    		}
		    		if(!this._paginationSet){
						this._updatePagination(totalReviews.length)
		    		}
					this._paginationSet = true;
					
					for (var i = (this._currentPage-1)*PAGE_SIZE; i < totalReviews.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); i++){
		    				this._reviewsListNode.appendChild(totalReviews[i].domNode);
		    				revCount++;
					}
				}
				else{
					// User is a participant or the creator
		    		for (var i=0; i<reviews.length; i++){
		    			var reviewRole = reviews[i].reviewParser.getRole();
		    			// If reviewRole == null the user is not a participant
		    			if(reviewRole || reviews[i].reviewParser.getCreatorUserURI()==
			        		com.ibm.rdm.web.AUTHENTICATED_USER.uri){
		    				this._reviewsListNode.appendChild(reviews[i].domNode);
		    				revCount++;
		    			}
		    		}
				}
				if(this.myReviewsViewlet==true){
		    		this._setHeaderFilteredReviewCount(reviews.length, totalReviews.length);
				}
				else{
					this._setHeaderFilteredReviewCount(reviews.length, revCount);
				}
				break;
			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ACTIVEREVIEWS:
				if(this.myReviewsViewlet==true && reviews.length > 0){
					var totalReviews = [];
					var j = 0;
		    		for (var i=0; i<reviews.length; i++){
		    			var reviewState = reviews[i].reviewParser.getReviewState();
		    			if(reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_DRAFT &&
		    					reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED){
		    				totalReviews[j] = reviews[i];
		    				j++;
		    			}
		    		}
		    		
		    		if(!this._paginationSet){
						this._updatePagination(totalReviews.length)
		    		}
					this._paginationSet = true;
		    		
		    		for (var i = (this._currentPage-1)*PAGE_SIZE; i < totalReviews.length && i < ((this._currentPage-1)*PAGE_SIZE+PAGE_SIZE); i++){
		    			this._reviewsListNode.appendChild(totalReviews[i].domNode);
		    			revCount++;
					}
				}
				else{
					for (var i=0; i<reviews.length; i++){
		    			var reviewState = reviews[i].reviewParser.getReviewState();
		    			if(reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_DRAFT &&
		    					reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED){
		    				this._reviewsListNode.appendChild(reviews[i].domNode);
		    				revCount++;
		    			}
		    		}		
				}
				
		    	this._setHeaderFilteredReviewCount(reviews.length,revCount);
				break;
		} 	
    },
    
    _checkFilterInclusions: function(review) {
    	if(review){
    		switch(this._filterDisplayReviews) {
    			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS:
    				return true;
    				break;
    			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_MYREVIEWS:
    				// User is a participant or the creator
    				var reviewRole = review.reviewParser.getRole();
    					// If reviewRole == null the user is not a participant
    					if(reviewRole || review.reviewParser.getCreatorUserURI()==
    			        	com.ibm.rdm.web.AUTHENTICATED_USER.uri){
    						return true;
    						}
    				break;
    			case com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ACTIVEREVIEWS:
    				var reviewState = review.reviewParser.getReviewState();
    				if(reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_DRAFT &&
    							reviewState != com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED){
    						return true;
    				}
    				break;
    			default:
    				return false;
    		}	
    	}
    },
    
    _removeReviewNodes: function() {
    	domutils.removeChildNodes(this._reviewsListNode);
    },
    
    _onSortByStatusItemClick: function() {
    	this._sortReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_STATUS;
    	this._sortReviews(this._reviewEntryWidgets);
    	this._removeReviewToolbarDropdownSortImageClass();
    	dojo.addClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIconWithDot");
    	dojo.addClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewSortByRoleIcon");
    	dojo.addClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewSortByReviewNameIcon");
    },
    
    _onSortByRoleItemClick: function() {
    	this._sortReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_ROLE;
    	this._sortReviews(this._reviewEntryWidgets);
    	this._removeReviewToolbarDropdownSortImageClass();
    	dojo.addClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIcon");
    	dojo.addClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewSortByRoleIconWithDot");
    	dojo.addClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewSortByReviewNameIcon");
    },
       
    _onSortByNameItemClick: function() {
    	this._sortReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Ordering.SORTBY_NAME;
    	this._sortReviews(this._reviewEntryWidgets);
    	this._removeReviewToolbarDropdownSortImageClass();
    	dojo.addClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIcon");
    	dojo.addClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewSortByRoleIcon");
    	dojo.addClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewSortByReviewNameIconWithDot");
    },
    
    _onGroupByNoneItemClick: function() {
    	this._groupReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_NONE;
    	this._reviewsAreGrouped = false;
    	//TODO GET RID OF THIS
    	this._removeReviewNodes();
    	///this.handleRefresh();
    	this._removeReviewToolbarDropdownGroupImageClass();
    	dojo.addClass(this._reviewGroupByReviewNoneMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.addClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIcon");
    	dojo.addClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewSortByRoleIcon");
    	this._sortReviews(this._reviewEntryWidgets);
        },
    
    _onGroupByStatusItemClick: function() {
    	this._groupReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_STATUS;
    	this._reviewsAreGrouped = true;
    	this._removeReviewToolbarDropdownGroupImageClass();
    	dojo.addClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIconWithDot");
    	dojo.addClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewSortByRoleIcon");
    	this._groupReviews(this._reviewEntryWidgets);
    },
    
    _onGroupByRoleItemClick: function() {
    	this._groupReviewsBy = com.ibm.rdm.web.review.ReviewsViewer.Grouping.GROUPBY_ROLE;
    	this._reviewsAreGrouped = true;
    	this._removeReviewToolbarDropdownGroupImageClass();
    	dojo.addClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIcon");
    	dojo.addClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewSortByRoleIconWithDot");
    	this._groupReviews(this._reviewEntryWidgets);
    },
    
    _showMyReviewsButtonClicked: function () {
    	this._filterDisplayReviews = com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_MYREVIEWS;
    	this._showActiveReviewsButton.reset();
    	this._showAllReviewsButton.reset();
       	if(this._reviewEntryWidgets){
       		this._paginationSet = false;
       		this._sortReviews(this._reviewEntryWidgets);
       	}
    },
    
    _showActiveReviewsButtonClicked: function () {
    	this._filterDisplayReviews = com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ACTIVEREVIEWS;
    	this._showMyReviewsButton.reset();
    	this._showAllReviewsButton.reset();
    	if(this._reviewEntryWidgets){
    		this._paginationSet = false;
    		this._sortReviews(this._reviewEntryWidgets);
       	}
    },
    
    _showAllReviewsButtonClicked: function () {
    	this._filterDisplayReviews = com.ibm.rdm.web.review.ReviewsViewer.Display.DISPLAY_ALLREVIEWS;
    	this._showMyReviewsButton.reset();
    	this._showActiveReviewsButton.reset();
    	if(this._reviewEntryWidgets){
    		this._paginationSet = false;
    		this._sortReviews(this._reviewEntryWidgets);
       	}
    },
    
    _removeReviewToolbarDropdownSortBulletClass: function() {
    	dojo.removeClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.removeClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.removeClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewToolbarDropdownBullet");  	
    },
    
    _removeReviewToolbarDropdownGroupBulletClass: function() {
    	dojo.removeClass(this._reviewGroupByReviewNoneMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.removeClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.removeClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewToolbarDropdownBullet");  	
    },
    
    _removeReviewToolbarDropdownGroupImageClass: function() {
    	dojo.removeClass(this._reviewGroupByReviewNoneMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.removeClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIconWithDot");
    	dojo.removeClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewSortByRoleIconWithDot");  	
    },
    
    _removeReviewToolbarDropdownSortImageClass: function() {
    	dojo.removeClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIconWithDot");
    	dojo.removeClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewSortByRoleIconWithDot");
    	dojo.removeClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewSortByReviewNameIconWithDot");  	
    },
    
    _addReviewToolbarDropdownGroupImageClass: function() {
    	dojo.addClass(this._reviewGroupByReviewNoneMenuItem.iconNode, "reviewToolbarDropdownBullet");
    	dojo.addClass(this._reviewGroupByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIconWithDot");
    	dojo.addClass(this._reviewGroupByRoleMenuItem.iconNode, "reviewSortByRoleIconWithDot");  	
    },
    
    _addReviewToolbarDropdownSortImageClass: function() {
    	dojo.addClass(this._reviewSortByReviewStatusMenuItem.iconNode, "reviewSortByReviewStatusIcon");
    	dojo.addClass(this._reviewSortByMyRoleMenuItem.iconNode, "reviewSortByRoleIcon");
    	dojo.addClass(this._reviewSortByReviewNameMenuItem.iconNode, "reviewSortByReviewNameIcon");  	
    },
    
    _onHideAllFinalizedReviews: function() {
    	this._hideFinalizedReviews = this._finalizedCheckBox.checked;
    	if(this._hideFinalizedReviews == true){
    		this._setHideFinalizedReviews();
    	}
    	else{
    		this._showFinalizedReviews();
    	}
    },
    
    _setHideFinalizedReviews: function() {    	
       	if(this._reviewEntryWidgets){
    		dojo.forEach(this._reviewEntryWidgets, function(reviewEntryWidget) {
    			if(reviewEntryWidget._reviewStateNode.innerHTML ==
    				com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED){
    				reviewEntryWidget.domNode.style.display="none";
    			}
    		}); 
    	}
    },
    
    _showFinalizedReviews: function() {  	
       	if(this._reviewEntryWidgets){
    		dojo.forEach(this._reviewEntryWidgets, function(reviewEntryWidget) {
    			if(reviewEntryWidget._reviewStateNode.innerHTML ==
    				com.ibm.rdm.web.review.ReviewsViewer.Status.STATUS_FINALIZED){
    				reviewEntryWidget.domNode.style.display="";
    			}
    		}); 
    	}
    },
    
    destroyCollapseableHeaders: function() {
    	if(this._collapseableHeaders){
    		for (var i = 0; i < this._collapseableHeaders.length; i++) {
    			this._collapseableHeaders[i].destroyRecursive();
    			this._collapseableHeaders[i] = null;
    		}
    	}
    	this._collapseableHeaders = [];
    },
    
    containerDimensionsHaveChanged: function() {
        var height = this.domNode.parentNode.clientHeight;
        var width = this.domNode.parentNode.clientWidth;
        var contentPaneTop = (
            this._contentPane.domNode.offsetTop
            - this._contentPane.domNode.parentNode.offsetTop);
        var scrollHeightToUse = (height - contentPaneTop);
        this._contentPane.resize({ h: scrollHeightToUse, w: width });
    },
    
	setDimensions: function(width, height) {
        this.domNode.style.height = height + "px";
        this.domNode.style.width = width + "px";
	},
    
    _getDateRange: function(lastModifiedDate) {
        var now = new Date();
        var todayDate = new Date(now.getFullYear(), now.getMonth(), now.getDate());
    	
		if (lastModifiedDate.getTime() > todayDate.getTime()) {
			return com.ibm.rdm.web.review.ReviewsViewer.Modified.MODIFIED_TODAY;
        }
        else if (lastModifiedDate.getTime() > (todayDate.getTime() - 1000*60*60*24)) {
        	return com.ibm.rdm.web.review.ReviewsViewer.Modified.MODIFIED_YESTERDAY;
        }
        else if (Math.abs(dojo.date.difference(todayDate, lastModifiedDate, "day")) < 7) {
        	return com.ibm.rdm.web.review.ReviewsViewer.Modified.MODIFIED_PAST_WEEK;
        }
        else if ((Math.abs(dojo.date.difference(todayDate, lastModifiedDate, "day")) < 30)) {
        	return com.ibm.rdm.web.review.ReviewsViewer.Modified.MODIFIED_PAST_MONTH;
        }
        else
        	return com.ibm.rdm.web.review.ReviewsViewer.Modified.MODIFIED_EARLIER;
    },

	_setHeaderReviewCount: function(totalNum) {
		if(this.myReviewsViewlet==true){
			this.site.setTitle(dojo.string.substitute(this._messages.showingReviews, [totalNum]));
    		this.site.setLoading(false);  	
		}
		else{
			var text = dojo.string.substitute(this._messages.showingReviews, [totalNum]);
	    	this.resourcesPage._reviewsSidebar.setSectionTitleInfo(text);
			
		}
	},
	
	_setHeaderFilteredReviewCount: function(totalNum, resultNum) {
		if(this.myReviewsViewlet==true){
		    this.site.setTitle(dojo.string.substitute(this._messages.showingReviewsNofM, [resultNum, totalNum]));
    		this.site.setLoading(false);  	
		}
		else{
			var text = dojo.string.substitute(this._messages.showingReviewsNofM, [resultNum, totalNum]);
			this.resourcesPage._reviewsSidebar.setSectionTitleInfo(text);
			
		}
	},
	
	_setLoadingReviewHeaderMessage: function() {
		if(!this.myReviewsViewlet==true){
			domutils.createHourGlassCursor();
	    	this.resourcesPage._reviewsSidebar.setSectionTitleInfo(this._messages.loadingReviews);
		}
	},
	
	_handleFirstPage: function() {
		this._currentPage = 1;
        dojo.addClass(this._first, "invisible");
        dojo.addClass(this._prev, "invisible");
        dojo.removeClass(this._last, "invisible");
        dojo.removeClass(this._next, "invisible");
        this._clearLastAndShowCurrentPageResults();
	},
	
	_handlePreviousPage: function(){
		if (this._currentPage > 1) {
        	this._currentPage--;
        	if (this._currentPage == 1) {
        		dojo.addClass(this._first, "invisible");
                dojo.addClass(this._prev, "invisible");
        	}
        	dojo.removeClass(this._next, "invisible");
            dojo.removeClass(this._last, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        }
	},
	
	_handleNextPage: function(){
		if (this._currentPage < this._totalPages) {
        	this._currentPage++;
        	if (this._currentPage == this._totalPages) {
        		dojo.addClass(this._next, "invisible");
                dojo.addClass(this._last, "invisible");		
        	}
        	dojo.removeClass(this._first, "invisible");
            dojo.removeClass(this._prev, "invisible");
        	this._clearLastAndShowCurrentPageResults();
        }	
	},
	
	_handleLastPage: function(){
		this._currentPage = this._totalPages;
    	dojo.addClass(this._next, "invisible");
        dojo.addClass(this._last, "invisible");
        dojo.removeClass(this._first, "invisible");
        dojo.removeClass(this._prev, "invisible");
        this._clearLastAndShowCurrentPageResults();
		
	},
	
	_showNavOptions: function(event) {
		if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        if (this._totalPages > 1) {
        	if (this._currentPage != this._totalPages) {
        		dojo.removeClass(this._next, "invisible");
        		dojo.removeClass(this._last, "invisible");
        	}
        	if (this._currentPage != 1) {
        		dojo.removeClass(this._first, "invisible");
                dojo.removeClass(this._prev, "invisible");
        	}
        }
	},
        
	_hideNavOptions: function(event) {
		if (event) {
        	event.preventDefault();
    		event.stopPropagation();
        }
        dojo.addClass(this._first, "invisible");
        dojo.addClass(this._last, "invisible");
        dojo.addClass(this._prev, "invisible");
        dojo.addClass(this._next, "invisible");        	
	},
	
	_clearLastAndShowCurrentPageResults: function() {
		this._pageText.innerHTML = dojo.string.substitute(this._messages.pageNOfM, [this._currentPage, this._totalPages]);
        this._sortReviews(this._reviewEntryWidgets);
	},

	_getTextNodeValue: function(node, unqualifiedTagName, namespace, qualifier) {
		var nodes = domutils.getElementsByTagNameNS(node, unqualifiedTagName, namespace, qualifier);
		return (
			nodes.length > 0 && nodes[0].firstChild ?
			nodes[0].firstChild.nodeValue :
			""
		);
	}
	    
    
}); // end of dojo.declare


compareDueDate = function(reviewArray1, reviewArray2) {
	  var dueDate1 = reviewArray1.reviewParser._dueDate;
	    var dueDate2 = reviewArray2.reviewParser._dueDate;
	    return ((dueDate1 == dueDate2) ?
	        	compareName(reviewArray1, reviewArray2) :
	            ((dueDate1 < dueDate2) ? -1 : 1));
};

compareDateModified = function(reviewArray1, reviewArray2) {
	  var DateModified1 = reviewArray1.reviewModifiedDate;
	    var DateModified2 = reviewArray2.reviewModifiedDate;
	    return ((DateModified1 == DateModified2) ?
	        	compareName(reviewArray1, reviewArray2) :
	            ((DateModified1 > DateModified2) ? -1 : 1));
};

compareName = function(reviewArray1, reviewArray2) {
	  var name1 = reviewArray1.reviewParser._reviewName.toLowerCase();
	    var name2 = reviewArray2.reviewParser._reviewName.toLowerCase();
	    return ((name1 < name2) ? -1 : 1);
};

//compareCreator = function(reviewArray1, reviewArray2) {
//	  var creator1 = reviewArray1.reviewParser._creatorUserId;
//	    var creator2 = reviewArray2.reviewParser._creatorUserId;
//	    return ((creator1 == creator2) ?
//	        	compareName(reviewArray1, reviewArray2) :
//	            ((creator1 < creator2) ? -1 : 1));
//};

compareState = function(reviewArray1, reviewArray2) {
	  var state1 = reviewArray1.reviewParser._reviewState;
	    var state2 = reviewArray2.reviewParser._reviewState;	    
	    return ((state1 == state2) ?
	        	compareName(reviewArray1, reviewArray2) :
	            ((state1 < state2) ? -1 : 1));
};

compareRole = function(reviewArray1, reviewArray2) {
	  var role1 = reviewArray1.reviewParser.getRole();
	    var role2 = reviewArray2.reviewParser.getRole();	    
	    return ((role1 == role2) ?
	        	compareName(reviewArray1, reviewArray2) :
	            ((role1 < role2) ? -1 : 1));
};


com.ibm.rdm.web.review.ReviewsViewer.Ordering = {
		
		SORTBY_CREATOR: "SORTBY_CREATOR",
		SORTBY_DUEDATE: "SORTBY_DUEDATE",
		SORTBY_REVIEWNAME: "SORTBY_REVIEWNAME",
		SORTBY_STATUS: "SORTBY_STATUS",
		SORTBY_DATELASTMODIFIED: "SORTBY_DATELASTMODIFIED",
		SORTBY_ROLE: "SORTBY_ROLE"
	};

com.ibm.rdm.web.review.ReviewsViewer.Grouping = {

		GROUPBY_AUTHOR: "GROUPBY_AUTHOR",
		GROUPBY_MODIFIEDDATE: "GROUPBY_MODIFIEDDATE",
		GROUPBY_STATE: "GROUPBY_STATE",
		GROUPBY_NONE: "GROUPBY_NONE",
		GROUPBY_STATUS: "GROUPBY_STATUS",
		GROUPBY_ROLE: "GROUPBY_ROLE"
	};

com.ibm.rdm.web.review.ReviewsViewer.Modified = {

		MODIFIED_TODAY: "MODIFIED_TODAY",
		MODIFIED_YESTERDAY: "MODIFIED_YESTERDAY",
		MODIFIED_PAST_WEEK: "MODIFIED_PAST_WEEK",
		MODIFIED_PAST_MONTH: "MODIFIED_PAST_MONTH",
		MODIFIED_EARLIER: "MODIFIED_EARLIER"
	};

com.ibm.rdm.web.review.ReviewsViewer.Status = {

		STATUS_DRAFT: "Draft",
		STATUS_IN_PROGRESS: "In Progress",
		STATUS_PAUSED: "Paused",
		STATUS_COMPLETE: "Complete",
		STATUS_FINALIZED: "Finalized",
		STATUS_STARTED: "Started"
	};

com.ibm.rdm.web.review.ReviewsViewer.Display = {

		DISPLAY_ALLREVIEWS: "DISPLAY_ALLREVIEWS",
		DISPLAY_MYREVIEWS: "DISPLAY_MYREVIEWS",
		DISPLAY_ACTIVEREVIEWS: "DISPLAY_ACTIVEREVIEWS"
	};
})();


