/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.review.VoteCastingButtons");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.review.VoteButton");
dojo.require("com.ibm.rdm.web.review.reviewTypes");
dojo.require("com.ibm.rdm.web.review.reviewutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");


var VoteButton = com.ibm.rdm.web.review.VoteButton;
var domutils = com.ibm.rdm.web.util.domutils;
var reviewTypes = com.ibm.rdm.web.review.reviewTypes;
var reviewutils = com.ibm.rdm.web.review.reviewutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;


dojo.declare("com.ibm.rdm.web.review.VoteCastingButtons",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Contributes the vote casting buttons that appear when looking at
	//    a Review or an artifact under Review.

	templateString: "<span class=\"rdmVoteCastingButtons\" dojoAttachPoint=\"_voteCastingButtonsSpan\"></span>",

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	reviewParser: null,
	// onVoteCastHandler:
	//    When the user clicks on one of the voting buttons, this handler will
	//    be invoked.  It will be passed the following two parameters:
	//      artifactStatusType
	//        [ A value from reviewTypes.ARTIFACT_STATUS_TYPES ]
	//      doPromptForComments
	//        [ A boolean ]
	onVoteCastHandler: null,
	disabled: null,
	// viewType:
	//    The value of this field must be one of the values found in the
	//	  com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE property list.
	viewType: null,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_voteCastingButtonsSpan: null,
	_participantRoleType: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_approveButton: null,
	_disapproveButton: null,
	_abstainButton: null,
	_reviewedButton: null,
	_doneButton: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	constructor: function() {
		this.disabled = false;
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.review", "reviewAndApprovalCommon");
	},

	// TODO Fold this functionality into _setReviewParserAttr?
	updateVotingButtons: function(/*Optional*/artifactInfo) {
		if (this.reviewParser) {
			var isDifferentRoleType = false;
			// Note: Only AFTER the first time this function is called will
			// this._participantRoleType hold a non-null value.
			if (this._participantRoleType
				&& this.reviewParser.getRole() !== this._participantRoleType
				&& (this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactInReview
					|| this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactsTable)
				) {
				// The role has changed since the last time _populateContents
				// was called ... so we need to remove the buttons that are
				// currently being displayed.
				this._destroyVoteCastingButtons();
				isDifferentRoleType = true;
			}
			var doCreateButtons = ((! this._participantRoleType) || isDifferentRoleType);
			var doUpdateButtons = (
				(this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactInReview
					|| this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactsTable)
				&& this._participantRoleType && (! isDifferentRoleType)
			);
			var participantRoleType = this.reviewParser.getRole();
			this._participantRoleType = participantRoleType;
			var artifactStatus = (artifactInfo ? artifactInfo.myArtifactStatusType : null);
			if (doCreateButtons) {
				if (this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactInReview
					|| this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactsTable
					) {
					this._createArtifactVoteCastingButtons(artifactStatus);
				}
				else if (this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.Review) {
					this._createReviewVoteCastingButtons();
				}
			}
			if (doUpdateButtons) {
				this._updateEnablementStateForArtifactVoteCastingButtons(
					artifactStatus, participantRoleType);
			}
		}
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_setReviewParserAttr: function(reviewParser) {
		// summary:
		//    Reset the current value for the reviewParser field and make
		//    appropriate UI updates.
		// description:
		//    When consumers of this class invoke this class's 'attr' function
		//    to set the "reviewParser" field, this function picks up the
		//    request in order to make appropriate UI element updates.
		//
		//    NOTE: This function customizes the default implementation that
		//    would normally put in place by:
		//          _Widget.attr
		//
		this.reviewParser = reviewParser;
	},

	_setDisabledAttr: function(/*Boolean*/value){
		var originalValue = this.disabled;
		if (value != originalValue) {
			this.disabled = value;
			var self = this;
			var setButtonDisabledIfNeeded = function(propertyName) {
				var button = self[propertyName];
				if (button) {
					button.attr("disabled", value);
				}
			};
			setButtonDisabledIfNeeded("_approveButton");
			setButtonDisabledIfNeeded("_disapproveButton");
			setButtonDisabledIfNeeded("_abstainButton");
			setButtonDisabledIfNeeded("_reviewedButton");
			setButtonDisabledIfNeeded("_doneButton");
		}
	},

	_createArtifactVoteCastingButtons: function(/*Optional*/artifactStatus) {
		var self = this;
		var createApproveButton = function(isDisabled) {
			var button = self._createVoteCastingButton(
				self._messages["approve"],
				self._messages["approveWithComments"],
				reviewTypes.ARTIFACT_STATUS_TYPES.Approved,
				isDisabled,
				"approveButtonImage",
				true
			);
			self._approveButton = button;
			return button;
		};
		var createDisapproveButton = function(isDisabled) {
			var button = self._createVoteCastingButton(
				self._messages["disapprove"],
				self._messages["disapproveWithComments"],
				reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved,
				isDisabled,
				"disapproveButtonImage",
				true
			);
			self._disapproveButton = button;
			return button;
		};
		var createAbstainButton = function(isDisabled) {
			var button = self._createVoteCastingButton(
				self._messages["abstain"],
				self._messages["abstainWithComments"],
				reviewTypes.ARTIFACT_STATUS_TYPES.Abstained,
				isDisabled,
				"abstainButtonImage",
				true
			);
			self._abstainButton = button;
			return button;
		};
		var createMarkAsReviewedButton = function(isDisabled) {
			var button = self._createVoteCastingButton(
				self._messages["reviewed"],
				self._messages["reviewedWithComments"],
				reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed,
				isDisabled,
				"reviewedButtonImage",
				true
			);
			self._reviewedButton = button;
			return button;
		};
		var participantRoleType = this.reviewParser.getRole();
		var buttons = [];
		switch (participantRoleType) {
			case reviewTypes.PARTICIPANT_ROLE_TYPES.Approver:
				buttons.push(
					createApproveButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Approved));
				buttons.push(
					createDisapproveButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved));
				buttons.push(
					createAbstainButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained));
				break;
			case reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer:
				buttons.push(
					createMarkAsReviewedButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed));
				buttons.push(
					createAbstainButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained));
				break;
			case reviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer:
				buttons.push(
					createMarkAsReviewedButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed));
				buttons.push(
					createAbstainButton(
						this.disabled
						|| artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained));
				break;
		}
		if (this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactsTable) {
			var span = this._voteCastingButtonsSpan;
			if (span.firstChild) {
				// It must be the case that the user role has changed.  Remove
				// the child node that represents the table (note: the widgets
				// found in this table have already been destroyed via a prior
				// call to _destroyVoteCastingButtons).
				domutils.removeChildNodes(span);
			}
			this._fillSpanWithVerticallyLayedOutButtonTable(span, buttons);
		} else {
			dojo.forEach(buttons, function(button) {
				self._voteCastingButtonsSpan.appendChild(button.domNode);
			});
		}
	},

	_fillSpanWithVerticallyLayedOutButtonTable: function(span, buttons) {
		// summary:
		//    Construct a table with rows representing the passed buttons and
		//    add this table to the passed span.
		// description:
		//    We go out of our way here to avoid GVT issues ... as we want all
		//    buttons to be of a uniform width (which could change depending
		//    on the language being used).
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		var buttonTextNodes = [];
		// Note by SPC: dojo ComboButton's are represented as a table and we're
		// placing these buttons inside a table.  I suspect that due to the fact
		// that we've got a table inside a table, we're unable to ask for what
		// the appropriate offsetWidth value is as it's being added to the DOM
		// (as it's set to 0).  To get around this fact, I'm temporarily adding
		// a span containing the same text that's found within the button.  I am
		// able to then ask for the offsetWidth of the span (which works).
		// Another note: I thought it might be more appropriate, CSS style wise,
		// to temporarily reparent the dojo button to the document body and then
		// ask for the offsetWidth of the node representing the label (thereby
		// avoiding the table within a table issue).  However, I found that this
		// lead to the buttons being slightly too wide (e.g. the div containing
		// the labels for an Approver were set to a width of 70 using this
		// approach, but using the span approach they were set to a width of 63).
		var maxButtonTextOffsetWidth = 0;
		dojo.forEach(
			buttons,
			function(button) {
				// Add a new TR which will hold this button.
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.className = "verticallyAlignedVoteButtonTableData";
				td.appendChild(button.domNode);
				tr.appendChild(td);
				tbody.appendChild(tr);
				// Add the node representing the label to the buttonTextNodes.
				var nodeList = dojo.query(".dijitButtonText", button.domNode);
				var labelNode = nodeList[0];
				buttonTextNodes.push(labelNode);
				// Update maxButtonTextOffsetWidth.
				var label = labelNode.firstChild.nodeValue;
				var offsetWidth = viewerutils.calculateTextWidth(label);
				maxButtonTextOffsetWidth = Math.max(maxButtonTextOffsetWidth, offsetWidth);
			}
		);
		span.appendChild(table);
		var nodeList = dojo.query(".dijitButtonText", table);
		// Resize the buttons so that they are all of a uniform width (equal to
		// the width of the widest button).
		dojo.forEach(
			buttonTextNodes,
			function(node) {
				node.style.width = maxButtonTextOffsetWidth + "px";
			}
		);
	},

	_createReviewVoteCastingButtons: function() {
		var self = this;
		var createDoneButton = function() {
			var button = self._createVoteCastingButton(
				self._messages["done"],
				null,
				reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed,
				false,			// isDisabled
				"", 			// cssClassName
				false			// isOptionalCommentSupported
			);
			self._voteCastingButtonsSpan.appendChild(button.domNode);
			self._doneButton = button;
		};
		createDoneButton();
	},

	_destroyVoteCastingButtons: function() {
		var self = this;
		var destroyButtonIfNeeded = function(propertyName) {
			var button = self[propertyName];
			if (button) {
				button.destroyRecursive();
				self[propertyName] = null;
			}
		};
		destroyButtonIfNeeded("_approveButton");
		destroyButtonIfNeeded("_disapproveButton");
		destroyButtonIfNeeded("_abstainButton");
		destroyButtonIfNeeded("_reviewedButton");
		destroyButtonIfNeeded("_doneButton");
	},

	_updateEnablementStateForArtifactVoteCastingButtons: function(artifactStatus, participantRoleType) {
		if (! this.disabled) {
			switch (participantRoleType) {
				case reviewTypes.PARTICIPANT_ROLE_TYPES.Approver:
					this._approveButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Approved
					);
					this._disapproveButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Disapproved
						);
					this._abstainButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained
					);
					break;
				case reviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer:
					this._reviewedButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed
					);
					this._abstainButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained
					);
					break;
				case reviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer:
					this._reviewedButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Reviewed
					);
					this._abstainButton.attr(
						"disabled",
						artifactStatus === reviewTypes.ARTIFACT_STATUS_TYPES.Abstained
					);
					break;
			}
		}
	},

	_createVoteCastingButton: function(
			label, voteWithCommentsLabel,
			artifactStatusType, isDisabled,
			/*String?*/cssClassName, isOptionalCommentSupported) {
		// summary:
		//    Create a vote casting button that will be used for either voting
		//    on an artifact or voting on the entire review (via "Done").
		var self = this;
		var buttonOrVoteButton = null
		if (isOptionalCommentSupported) {
			buttonOrVoteButton = new VoteButton({
				label: label,
				showLabel: true,
				disabled: isDisabled,
				iconClass: cssClassName,
				voteWithoutCommentsLabel: label,
				voteWithCommentsLabel: voteWithCommentsLabel,
				doAlignVertically: (this.viewType === com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE.ArtifactsTable),
				onVoteWithoutCommentsClick: function() {
					self._castVoteWithoutComments(artifactStatusType);
				},
				onVoteWithCommentsClick: function() {
					self._castVoteWithComments(artifactStatusType);
				}
			});
		} else {
			buttonOrVoteButton = new dijit.form.Button({
				label: label,
				showLabel: true,
				disabled: isDisabled,
				iconClass: cssClassName,
				onClick: function(event) {
					dojo.stopEvent(event);
					self._castVoteWithComments(artifactStatusType);
				}
			});
		}
		dojo.addClass(buttonOrVoteButton.domNode, "voteCastingButton");
		return buttonOrVoteButton;
	},

	_castVoteWithoutComments: function(artifactStatusType) {
		this.onVoteCastHandler(
			artifactStatusType,
			false				// doPromptForComments
		);
	},

	_castVoteWithComments: function(artifactStatusType) {
		this.onVoteCastHandler(
			artifactStatusType,
			true				// doPromptForComments
		);
	}
});


})();


// summary:
//    The different "view types" that VoteCastingButtons supports.
// description:
//    ArtifactInReview
//        Used by ArtifactInReviewViewer.
//    ArtifactsTable
//        Used by the button display to the right of the artifacts
//        table found in the "Participant's Review" accordion pane.
//    Review
//        Used to support the Done button in the title section of
//        the "Participant's Review" accordion pane.
com.ibm.rdm.web.review.VoteCastingButtons.VIEW_TYPE = {
	ArtifactInReview: "ArtifactInReview",
	ArtifactsTable: "ArtifactsTable",
	Review: "Review"
};
