/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.FullTextSearchHandler");

dojo.require("com.ibm.rdm.web.search.AdvancedQuerySection");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search", "FullTextSearchHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.search.QueryBuilder");

var constants = com.ibm.rdm.web.common.MimeTypes;
var uriHelper = com.ibm.rdm.web.common.UriHelper; 
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var QueryBuilder = com.ibm.rdm.web.search.QueryBuilder;

/**
 * This class handles issuing the requests associated with "full text" searching.
 */
dojo.declare(
	"com.ibm.rdm.web.search.FullTextSearchHandler",
	null,
{
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_searchQuery: null,

	_mapContentTypeToQueryParamContentType: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	constructor: function(/* Object */ keywordParameters){
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "FullTextSearchHandler");
		this._searchQuery = keywordParameters.searchQuery;
		this._initContentTypesMapIfNeeded();
	},

	performSearch: function(userQueryString, advancedQuerySection) {
		// summary:
		//    Request that a "full text search" be performed on the server.
		//    Here is an example of the request used for searching for "school"
		//    when the Type is set to Actor:
		//      /jazz/projects/rdm/search?q=((content:%22school%22%20OR%20weighted:%22school%22)%20+%20format:applicationxcomibmrdmbaactorxml)
		com.ibm.rdm.web.search.searchutils.showAsBusy(true);
		var contentType = (
			advancedQuerySection ?
			advancedQuerySection.getContentType() :
			null);
		var queryParams = this._composeSearchURIQueryParam(userQueryString, contentType);
	},

	performJfsMultiFetch: function(userQueryString, advancedQuerySection) {
		// summary:
		com.ibm.rdm.web.search.searchutils.showAsBusy(true);
		var contentType = (
			advancedQuerySection ?
			advancedQuerySection.getContentType() :
			null);
			
		var lastModifiedByUser = (
			advancedQuerySection ?
			advancedQuerySection.getUser() :
			null);
			
		var Query = QueryBuilder;
		Query._contentType = contentType;
		Query._lastModifiedByUser= lastModifiedByUser;
		
		if (this._searchQuery._fullTextCheckBox.checked){
			Query._isFullText = true;
		}
		else{
			Query._isFullText = false;
		}
		
		Query._userQueryString = userQueryString;
		var queryInfo = Query._composeMultiFetchPostData();

		var projectsDeferred = ProjectCache.getUserMemberProjects().addBoth(this, function(projects){
			//terms must be filtered by the projects, which users is a member of
    		var projectURIs = "";
    		for (projectURI in projects) {
    			projectURIs += projectURI + ",";
    		}
    		if (projectURIs) {
    			//remove last comma
    			projectURIs = projectURIs.substring(0, projectURIs.length-1);
    		}
    		queryInfo.projectURI = projectURIs;
    		var deferred = ProjectCache.executeMultiFetch(queryInfo);
			deferred.addCallback(this, this._handleSearchSuccessResponse);
			deferred.addErrback(this, this._handleSearchErrorResponse);
		});
		
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_initContentTypesMapIfNeeded: function() {
		// summary:
		//    Initialize the static field, '_mapContentTypeToQueryParamContentType', if needed.
		if (!this._mapContentTypeToQueryParamContentType) {
			this._mapContentTypeToQueryParamContentType = new Object;
			var contentTypes = [
				constants.CONTENT_TYPE_GLOSSARY,
				constants.CONTENT_TYPE_PROCESS,
				constants.CONTENT_TYPE_USE_CASE,
				constants.CONTENT_TYPE_USE_CASE_DIAGRAM,
				constants.CONTENT_TYPE_ACTOR,
				constants.CONTENT_TYPE_REQUIREMENT,
				constants.CONTENT_TYPE_PART,
				constants.CONTENT_TYPE_SKETCH,
				constants.CONTENT_TYPE_STORY_BOARD,
				constants.CONTENT_TYPE_SCREEN_FLOW,
				constants.CONTENT_TYPE_RICH_TEXT,
				constants.CONTENT_TYPE_TERM,
				constants.CONTENT_TYPE_UPLOADED
			];
			for (var i = 0; i < contentTypes.length; i++) {
				var contentType = contentTypes[i];
				this._mapContentTypeToQueryParamContentType[contentType] =
					this._removeDotsSlashesPlussesAndMinusSigns(contentType);
			}
		}
	},

	_removeDotsSlashesPlussesAndMinusSigns: function(contentType) {
		contentType = contentType.replace(/\./g, "");
		contentType = contentType.replace(/\//g, "");
		contentType = contentType.replace(/\+/g, "");
		contentType = contentType.replace(/\-/g, "");
		return contentType;
	},

	_composeSearchURIQueryParam: function(userQueryString, contentType) {
		var self = this;
		var composeContentTypesORStatement = function() {
			var buf = "";
			var i = 0;
			for (contentType in self._mapContentTypeToQueryParamContentType) {
				if (i !== 0) {
					buf += "+OR+"
				}
				buf += (
					"%22"
					+ self._mapContentTypeToQueryParamContentType[contentType]
					+ "%22"
				);
				i++;
			}
			return buf;
		}
		var encodedUserQueryString = encodeURIComponent(userQueryString);
		return (
			"q=%28%28%28content%3A" + encodedUserQueryString + "*%29+OR+%28weighted%3A" + encodedUserQueryString
			+ "*%29%29%2Bformat%3A"
			+ (contentType ?
			   (this._removeDotsSlashesPlussesAndMinusSigns(contentType) + "%29") :
			   ("%28" + composeContentTypesORStatement() + "%29%29"))
		);
	},

	/*
	 * Callback for a successful query.
	 */
	_handleSearchSuccessResponse: function(response) {
		com.ibm.rdm.web.search.searchutils.showAsBusy(false);
		this._searchQuery.displayQueryResults(response);
	},

	/*
	 * Callback for an unsuccessful query.
	 */
	_handleSearchErrorResponse: function(error) {
		com.ibm.rdm.web.search.searchutils.showAsBusy(false);
		com.ibm.rdm.web.search.searchutils.reportError(
			dojo.string.substitute(this._messages.errorAttemptingSearch, [(error.description || error)])
		);
	}
});

})();
