/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

dojo.provide("com.ibm.rdm.web.search.QueryBuilder");

dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.util.queryutils");

var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var queryutils = com.ibm.rdm.web.util.queryutils;

com.ibm.rdm.web.search.QueryBuilder = {
	// summary:
	//    Contructs a query.

	///////////////////////////////////////////////////////////////////////////
	// Public fields
	queryClause: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields
	_resourceURI: null,
	_isFullText: false,
	_contentType: null,
	_userQueryString: null,
	_lastModifiedByUser: null,
	
	_composeMultiFetchPostData: function() {
		// Create the query clause to be used by multi fetch
		var queryInfo = null;
		var clauseQueryParameters = [];
		var searchParams = {};
		var query = "";
		
		var searchValue = "";
		if (this._userQueryString) {
			searchParams = this.getSearchQueryParams(this._userQueryString, this._isFullText);
			if(this._isFullText){
				searchValue = searchParams.searchQuery;
				if (searchValue) {
					searchValue = searchValue.substring(0, searchValue.length-1) + this._composeFormatClause();
				}
			}
			else{				
				//add non-text search query
				if (searchParams.isLucene) {
					searchValue = searchParams.searchQuery;
				}
				else {
					clauseQueryParameters.push(searchParams.searchQuery);
				}	

				//add author clause
				if(this._lastModifiedByUser){
					clauseQueryParameters.push(this._composeUserClause());
				}
				//add content type clause
				if(this._contentType){
					clauseQueryParameters.push(this._composeTypeClause());
				}
				clauseQueryParameters.push(this._composePartOfClause());
				clauseQueryParameters.push(this._composeSavedFilterClause());
			}		
		}
		
		var numClausesAdded = 0;
		for (var i = 0; i < clauseQueryParameters.length; i++) {
			if (clauseQueryParameters[i]) {
				if (numClausesAdded > 0)
					query += ' and ';
				query += clauseQueryParameters[i];
				numClausesAdded++;
			}
		}
		
		queryInfo = {
			query: query,
			search: searchValue,
			namespaces: 'bpmn20=http://www.ibm.com/xmlns/rrm/1.0/, dc=http://purl.org/dc/terms/, jazz=http://jazz.net/xmlns/foundation/1.0/, nav=http://com.ibm.rdm/navigation#',
			cancelDeferreds: true,
			disablePagination: true
		};
        
		return queryInfo;
    },
	
	
	/**
	 * Adds a Lucene search parameter search=(((weighted:text*))) or a Jene query paremter query=dc:title=text* depending
	 * on the wildcards in the text string. If there are no wildcards or only trailing wildcards the more performant Jena
	 * parameter will be used. If the input string is only supported by Lucene (Lu*ne to match "Lucene") the search= parameter
	 * will be used.
	 * 
	 * Callers should only call this method if their desired search:
	 * 1. Is for all artifacts whose title (regardless of content) begins with the search text.
	 * 2. Does not require results to be ordered by search relevance or have search relevance information (FullTextSearchProperties.SCORE).
	 */
	getSearchQueryParams: function(searchString, isTextSearch) {
		var searchQuery = "";
		var isLucene = null;
		if (searchString) {
			if (this.isRequiresLucene(searchString) || isTextSearch) {
				searchQuery = this.getLuceneSearchQuery(searchString, isTextSearch);
				isLucene = true;
			} else {
				searchQuery = this.getJenaQueryParameter(searchString);
			}
		}
		return {
			searchQuery: searchQuery,
			isLucene: isLucene
		};
	},
	addTrailingWildcardIfNeeded: function(searchString) {
		//add trailing wildcard if it was not specified by the user
		if (searchString[searchString.length-1] !== "*") {
			searchString += "*";
		}
		return searchString;
	},
	isRequiresLucene: function(searchString) {
		//we determine whether or not it is a Jena or Lucene query by checking if the
		//search string contains a wildcard anywhere except last index
		var asteriskIndex = searchString.indexOf('*');
		return asteriskIndex != -1 && asteriskIndex != searchString.length - 1;
	},
	getJenaQueryParameter: function(searchString) {
		return 'dc:title="' + queryutils.escapeQueryStringParameter(this.addTrailingWildcardIfNeeded(searchString)) + '"';
	},
	getLuceneSearchQuery: function(searchString, isTextSearch) {
		var luceneSearchQuery = "";
		var contentSearch = "";
		var weightedSearch = "";
		searchString = queryutils.removeSpecialLuceneChars(searchString);
		if (searchString) {
			var wildcardedTokens = this._extractWildcardedTokens(searchString);
			
			if (isTextSearch) {
				for (var i = 0; i < wildcardedTokens.length; i++) {
					contentSearch += 'content:' + wildcardedTokens[i] + ((i != wildcardedTokens.length-1)?" AND ":"");
				}
				contentSearch = "(" + contentSearch + ") OR ";
			}
			for (var i = 0; i < wildcardedTokens.length; i++) {
				weightedSearch += 'weighted:' + wildcardedTokens[i] + ((i != wildcardedTokens.length-1)?" AND ":"");
			}
			weightedSearch = "(" + weightedSearch + ")";
			luceneSearchQuery = '((' + contentSearch + weightedSearch + '))';
		}
		return luceneSearchQuery;
	},
	_extractWildcardedTokens: function(string) {
		var tokens = string.split(" ");
		var wildcardedTokens = [];
		for (var i = 0; i < tokens.length; i++) {
			var token = tokens[i];
			if (token.length > 0) {
				wildcardedTokens.push(this.addTrailingWildcardIfNeeded(token));
			}
		}
		return wildcardedTokens;
	},
	
	
	
	
	
	
	

	_composeTypeClause: function() {
		var artifactTypesClause = "";
		return 'dc:format="' + this._contentType + '"';
		return artifactTypesClause;
    },
    
    _composeFormatInClause: function() {
		var formatInClause = "";
		formatInClause = 'dc:format in ["application/x-com.ibm.rdm.ba.glossary+xml","application/x-com.ibm.rdm.ba.process+xml","application/x-com.ibm.rdm.ba.usecase.diagram+xml","application/x-com.ibm.rdm.ba.usecase+xml","application/x-com.ibm.rdm.ba.actor+xml","application/x-com.ibm.rdm.requirement+xml","application/x-com.ibm.sid.part+xml","application/x-com.ibm.sid.sketch+xml","application/x-com.ibm.sid.storyboard+xml","application/x-com.ibm.sid.screenFlow+xml","application/x-com.ibm.rdm.richtext+xml","application/x-com.ibm.rdm.ba.term+xml","application/x-com.ibm.rdm.reference+xml","x-com.ibm.rdm.collection/rdf+xml"]';
		return formatInClause;
    },
    
    _composeFormatClause: function() {
		var formatClause = "";
		if (this._contentType) {
			var format = this._contentType;
			format = format.replace(/\//g, '');
			format = format.replace(/\-/g, '');
			format = format.replace(/\./g, '');
			format = format.replace(/\+/g, '');
			formatClause = '+format:("' + format + '"))';
		}
		else {
			formatClause = '+format:("applicationxcomibmrdmbaglossaryxml" OR "applicationxcomibmrdmbaprocessxml" OR "applicationxcomibmrdmbausecasediagramxml" OR "applicationxcomibmrdmbausecasexml" OR "applicationxcomibmrdmbaactorxml" OR "applicationxcomibmrdmrequirementxml" OR "applicationxcomibmsidpartxml" OR "applicationxcomibmsidsketchxml" OR "applicationxcomibmsidstoryboardxml" OR "applicationxcomibmsidscreenFlowxml" OR "applicationxcomibmrdmrichtextxml" OR "applicationxcomibmrdmbatermxml" OR "xcomibmrdmreferencerdfxml" OR "xcomibmrdmcollectionrdfxml"))';
		}
		return formatClause;
    },
    
	_composeNamespaceClause: function() {
		return (
			'namespace in [<' + com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + '>]'
		);
    },
    
    _composeSavedFilterClause: function() {
		return (
			'dc:format!="' + com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_FILTER + '"'
		);
	},
	
	_composePartOfClause: function() {
		return (
			'dc:isPartOf=<' + com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + '>'
		);
	},
    
    _composeUserClause: function() {
    	var usersClause = "";
    	var userName = this._lastModifiedByUser;
		if (userName) {
			usersClause = 'lastModifiedBy=<' + userName + '>';
		}
        return usersClause;
    }
};