/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.SearchQuery");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.rdm.web.search.AdvancedQuerySection");
dojo.require("com.ibm.rdm.web.search.results.SearchResults");
dojo.require("com.ibm.rdm.web.search.DocumentTypes");
dojo.require("com.ibm.rdm.web.search.QueryHandler");
dojo.require("com.ibm.rdm.web.search.FullTextSearchHandler");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search", "SearchQuery");

// Note: As a starting point this code was originally based on Claudio's searching.js file.

dojo.declare(
    "com.ibm.rdm.web.search.SearchQuery",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/templates/SearchQuery.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _queryStringField: null,
        _fullTextCheckBox: null,
        _searchButton: null,
        _moreButton: null,
        _advancedSectionDiv: null,
        _searchResultsDiv: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

		footerHeight: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _advancedQuerySection: null,
        _searchResults: null,
        _documentTypes: null,
        _queryHandler: null,
        _fullTextSearchHandler: null,
        _isSingleSelect: false,
        _isMultiSelect: false,
        _lastQuerySortOrder: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

        search: null,
        text: null,
        more: null,
        less: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._documentTypes = new com.ibm.rdm.web.search.DocumentTypes({searchQuery:this});
            this._queryHandler = new com.ibm.rdm.web.search.QueryHandler({searchQuery:this});
            this._fullTextSearchHandler = new com.ibm.rdm.web.search.FullTextSearchHandler({searchQuery:this});
            this.connect(this._queryStringField, "onkeyup", this._handleKeyUp);
            this.connect(this._queryStringField, "onfocus", this._handleQueryStringFocus);
            this.connect(this._fullTextCheckBox, "onClick", this._handleFullTextCheckBoxClick);
            this.connect(this._searchButton, "onClick", this._handleSearchButtonClick);
            this.connect(this._moreButton, "onClick", this._handleMoreButtonClick);
             // Turn off the context menu (unless we're debugging).
			// if (!djConfig.isDebug) {
			// 		this.connect(dojo.doc, "oncontextmenu", function(event) {dojo.stopEvent(event);return false;} );
			// }
			 this._cshNode.appendChild(new jazz.app.ua.PageHoverHelp({uaKey: com.ibm.rdm.web.help.context.search}).domNode);
        },

        destroy: function() {
            this._documentTypes = null;
            this.inherited("destroy", arguments);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "SearchQuery");
            dojo.forEach(["more", "less", "text", "search"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        setSelectionType: function(isMultiSelect) {
            if (isMultiSelect) {
                this._isMultiSelect = true;
            } else {
                this._isSingleSelect = true;
            }
        },

        doctypeInfoHasBeenUpdated: function() {
            if (this._advancedQuerySection) {
                this._advancedQuerySection.doctypeInfoHasBeenUpdated();
            }
        },

        handlePerformSearchRequest: function(searchText) {
			this._resetDisplayedTextInputValue(searchText);
            this._resetAllOptions();
			if (dojo.trim(searchText) != "") {
				this._makeAppropriateServerRequest();
			}
        },

        handleOpenAdvanced: function(searchText) {
			this._resetDisplayedTextInputValue(searchText);
            if (!this._advancedSectionDiv.firstChild) {
                this._handleMoreButtonClick();
            }
        },

        handleSortedQuery: function(sortBy) {
        	this._makeAppropriateServerRequest(this._getOrderClause(sortBy));
        },

        displayQueryResults: function(doc, filteredPrimaryResourcesObject) {
            var searchResults = this._getSearchResults();
            var secondaryResourceQueryParams = null;
// TODO Remove remove all secondary resource related code when we move over to servlet approach instead
//             if (this._advancedQuerySection) {
//                 secondaryResourceQueryParams =
//                     this._advancedQuerySection.getResourceTypeSpecificQueryParams();
//             }
            searchResults.setSecondaryResourceQueryParams(secondaryResourceQueryParams);
            var userQueryString = this._queryStringField.value;
            if (this._fullTextCheckBox.checked) {
                // We're showing results from a full text based search.
                searchResults.displayQueryResults(doc);
            } else {
                // We're showing results from a query based search.
                // First make user userQueryString ends with a wildcard ...
// TODO Remove this (we're using XQuery in JRS M6 ... so we'll avoid tacking on "*" for now)
//                 if (userQueryString.charAt(userQueryString.length-1) != "*") {
//                     userQueryString += "*";
//                 }
                // and now ask to display the results (note: the presented entries from
                // the doc will ONLY include those that match the userQueryString ...
                // this is somewhat convoluted, but this approach enables us to find
                // artifacts by name regardless of the folder they're found in).
                searchResults.displayQueryResults(doc, userQueryString, filteredPrimaryResourcesObject);
            }
        },

        displayXQueryResults: function(doc) {
            var searchResults = this._getSearchResults();
            var secondaryResourceQueryParams = null;
//             if (this._advancedQuerySection) {
//                 secondaryResourceQueryParams =
//                     this._advancedQuerySection.getResourceTypeSpecificQueryParams();
//             }
//             searchResults.setSecondaryResourceQueryParams(secondaryResourceQueryParams);
            var userQueryString = this._queryStringField.value;
            if (this._fullTextCheckBox.checked) {
                // We're showing results from a full text based search.
                searchResults.displayQueryResults(doc);
            } else {
                // We're showing results from a query based search.
                // First make user userQueryString ends with a wildcard ...
// TODO Remove this (we're using XQuery in JRS M6 ... so we'll avoid tacking on "*" for now)
//                 if (userQueryString.charAt(userQueryString.length-1) != "*") {
//                     userQueryString += "*";
//                 }
                // and now ask to display the results (note: the presented entries from
                // the doc will ONLY include those that match the userQueryString ...
                // this is somewhat convoluted, but this approach enables us to find
                // artifacts by name regardless of the folder they're found in).
                searchResults.displayQueryResults(doc, userQueryString, null);
            }
        },

        advancedQueryHeightHasChanged: function() {
            if (this._searchResults) {
                this._searchResults.searchQueryAreaHeightHasChanged();
            }
        },

        isFullTextSearchingEnabled: function() {
            return this._fullTextCheckBox.checked;
        },
        
        makeSearchRequestAgain: function(sortBy) {
        	this._makeAppropriateServerRequest(this._getOrderClause(sortBy));
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _getSearchResults: function() {
            if (!this._searchResults) {
                var div = this._searchResultsDiv;
                this._searchResults = new com.ibm.rdm.web.search.results.SearchResults(
                    {
                        documentTypes: this._documentTypes,
						footerHeight: this.footerHeight,
                        _searchQuery: this
                    },
                    div
                );
                if (this._isMultiSelect || this._isSingleSelect) {
                    this._searchResults.setSelectionType(this._isMultiSelect);
                }
                this._searchResults.startup();
            }
            return this._searchResults;
        },

        _resetAllOptions: function() {
            if (this._fullTextCheckBox.checked) {
                this._fullTextCheckBox.attr('checked', false);
                this._handleFullTextCheckBoxClick();
            }
            if (this._advancedQuerySection) {
                this._advancedQuerySection.resetAllOptions();
            }
        },

        /**
         * Handle the request for the searching when the Search button is pressed.
         */
        _handleSearchButtonClick: function(e) {
            this._makeAppropriateServerRequest();
        },

        /**
         * Handle the request for the searching when the Search button is pressed.
         */
        _handleFullTextCheckBoxClick: function(e) {
            if (this._advancedQuerySection != null) {
                this._advancedQuerySection.setFullTextEnablement(this._fullTextCheckBox.checked);
            }
        },

        /**
         * When the More button is pressed we toggle visibility to an "advanced section".
         */
        _handleMoreButtonClick: function(event) {
            if (this._advancedSectionDiv.firstChild) {
                // Hide the advanced section.
                if (this._advancedQuerySection && this._advancedQuerySection.domNode.parentNode) {
                    node = this._advancedQuerySection.domNode;
                    node.parentNode.removeChild(node);
                }
                // Change the text on the button to "More"
                this._moreButton.attr("label", this.more);
            } else {
                // Show the advanced section.
                if (!this._advancedQuerySection) {
                    this._advancedQuerySection = new com.ibm.rdm.web.search.AdvancedQuerySection(
                        {searchQuery:this, documentTypes:this._documentTypes},
                        null
                    );
                    this._advancedQuerySection.setFullTextEnablement(this._fullTextCheckBox.checked);
                    this._advancedQuerySection.startup();
                }
                if (this._advancedQuerySection.domNode.parentNode != this._advancedSectionDiv) {
                    this._advancedSectionDiv.appendChild(this._advancedQuerySection.domNode);
                }
                // Change the text on the button to "Less"
                this._moreButton.attr("label", this.less);
            }
            if (this._searchResults) {
                this._searchResults.searchQueryAreaHeightHasChanged();
            }
        },

        /**
         * When the user enters new text in the search text field, this method is called
         * to perform the appropriate search if the Enter key is pressed.
         */
        _handleKeyUp: function(event) {
 			var trimmedInputText = dojo.trim(this._queryStringField.value);
			if (trimmedInputText != "" && event.keyCode == 13) { // Note: 13 is Enter key
				this._makeAppropriateServerRequest();
			}
        },

        _handleQueryStringFocus: function(event) {
            event.preventDefault();
            event.stopPropagation();
            this._queryStringField.select();
        },

		_resetDisplayedTextInputValue: function(searchText) {
            this._queryStringField.value = searchText;
		},
		
        /**
         * Make an appropriate search related request to the server.
         */
        _makeAppropriateServerRequest: function(orderClause) {
        	if (orderClause) {
        		this._lastQuerySortOrder = orderClause;
        	}
            var searchResults = this._getSearchResults();
            searchResults.clearLastResults();
            if (this._searchCriterionSpecified()) {
            	searchResults.informUserThatSearchIsInProgress();            
 				var trimmedInputText = dojo.trim(this._queryStringField.value);
 				// We are using _fullTextSearchHandler for structured and non structured searches
            	if (this._fullTextCheckBox.checked) {
                	this._fullTextSearchHandler.performJfsMultiFetch(
                    	trimmedInputText, this._advancedQuerySection);
            	} else {
                    	this._fullTextSearchHandler.performJfsMultiFetch(
                    	trimmedInputText, this._advancedQuerySection, this._lastQuerySortOrder);
            	}
            } else {
            	searchResults.informUserNoSearchCriterion();            
            }
        },
        
        _searchCriterionSpecified: function() {
 			var trimmedInputText = dojo.trim(this._queryStringField.value);
 			if (trimmedInputText != "") return true;
 			var user = this._advancedQuerySection.getUser();
 			if (user != null) return true;
 			var type = this._advancedQuerySection.getContentType();
 			if (type != null) return true;
 			return false;
        },
        
        _getOrderClause: function(sortBy) {
        	var orderClause = null;
        	switch (sortBy) {
        	case com.ibm.rdm.web.search.SearchQuery.sortBy.NAME:
        		orderClause = com.ibm.rdm.web.search.SearchQuery.orderClause.NAME;
        		break;
        	case com.ibm.rdm.web.search.SearchQuery.sortBy.TYPE:
        		orderClause = com.ibm.rdm.web.search.SearchQuery.orderClause.TYPE;
        		break;
        	case com.ibm.rdm.web.search.SearchQuery.sortBy.USER:
        		orderClause = com.ibm.rdm.web.search.SearchQuery.orderClause.USER;
        		break;
        	case com.ibm.rdm.web.search.SearchQuery.sortBy.DATE:
        		orderClause = com.ibm.rdm.web.search.SearchQuery.orderClause.DATE;
        		break;
        	}
        	return orderClause;
        }
    }
);

com.ibm.rdm.web.search.SearchQuery.sortBy = {
	    NAME: 0,
	    TYPE: 1,
	    USER: 2,
	    DATE: 3
};

com.ibm.rdm.web.search.SearchQuery.orderClause = {
	NAME: "fn:lower-case(*:name)",
	TYPE: "dc:format",
	USER: "prop:resource-last-modified-by",
	DATE: "dc:modified descending"
};

})();
