/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.RecentCommentsOptions");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search", "common");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "RecentCommentsOptions");

dojo.declare(
    "com.ibm.rdm.web.search.results.RecentCommentsOptions",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/RecentCommentsOptions.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _hideResolvedCheckNode: null,
        _applyAnchor: null,
        //_closeAnchor: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        projectURI: null,
        recentCommentResults: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _recentResourcesHeader: null,
        _priorityLabelNode: null,
        _forLabelNode: null,
        _authorLabelNode: null,
        _hideResolvedCheckNode: null,
        _showAllCheckNode: null,
        _showAllLabel: null,
        priorityWidget: null,
        authorWidget: null,
        directedToWidget: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

        hideResolved: null,
        close: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
			this.inherited("postCreate", arguments);
        	// Priority field
        	this.priorityWidget = new com.ibm.rdm.web.common.IconSelectWidget({
            	widgetmode: "full", 
            	iconSelectOptions: com.ibm.rdm.web.commenting.utils.getFilterPriorityOptions()
        	}, this._prioritySelectNode);
			// Accessibility, match label to field
    		this.priorityWidget._htmlSelectNode.setAttribute("id", this.priorityWidget.id);
    		this._priorityLabelNode.setAttribute("for", this.priorityWidget._htmlSelectNode.id);
        	
    		this.directedToWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
    			widgetmode: "full", 
    			forFilter: true,
    			projectURI: this.recentCommentResults.scopingProjectURIs[0]
    		}, 
    		null);
			this._directedToSelectNode.innerHTML = "";   
			this._directedToSelectNode.appendChild(this.directedToWidget.domNode);   
			// Accessibility, match label to field
    		this.directedToWidget._htmlSelectNode.setAttribute("id", this.directedToWidget.id);
    		this._directedToLabelNode.setAttribute("for", this.directedToWidget._htmlSelectNode.id);
   	
    		this.authorWidget = new com.ibm.rdm.web.common.UsersSelectWidget({
    			widgetmode: "full", 
    			forFilter: true,
    			projectURI: this.recentCommentResults.scopingProjectURIs[0]
    		},
    		null);
    		this._authorSelectNode.innerHTML = "";   
    		this._authorSelectNode.appendChild(this.authorWidget.domNode);  
			// Accessibility, match label to field
    		this.authorWidget._htmlSelectNode.setAttribute("id", this.authorWidget.id);
    		this._authorLabelNode.setAttribute("for", this.authorWidget._htmlSelectNode.id);
    		this._connectFormEvents();
        },
        
        startup: function() {
        	this.inherited("startup", arguments);
            document.getElementById(this.id+"_showAllField").checked = true;
            this._disableShowAll();
        },
        
        _connectFormEvents: function() {
        	var formSelectFields = this.domNode.getElementsByTagName("SELECT");
        	for (var i=0; i<formSelectFields.length; i++) {
        		dojo.connect(formSelectFields[i], "onchange", this, "_onFilteringClicked");
        	}
        	dojo.connect(this._hideResolvedCheckNode, "onclick", this, "_onFilteringClicked");
        },

        destroy: function() {
            this.priorityWidget.destroyRecursive();
            this.directedToWidget.destroyRecursive();
            this.authorWidget.destroyRecursive();
            this.inherited(arguments);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "RecentCommentsOptions");
            dojo.forEach(["for", "by", "priority", "hideResolved", "showAll", "close", "showOnlyReview"], function(prop){
                this[prop] = this._messages[prop];
            }, this);
        },

        setRecentResourcesHeader: function(recentResourcesHeader) {
            this._recentResourcesHeader = recentResourcesHeader;
        },

        _onFilteringClicked: function(event) {
        	if (this._isFilterShowAll()) {
            	this._showAllCheckboxNode.checked = true;
                this._disableShowAll();
        	} else {
            	this._showAllCheckboxNode.checked = false;
                this._enableShowAll();
        	}
            this._applyFilter();
        },
        
        _isFilterShowAll: function() {
        	if (this._hideResolvedCheckNode.checked) return false;
        	if (this.directedToWidget.selected != this.directedToWidget.UNSPECIFIED) return false;    	
        	if (this.authorWidget.selected != this.directedToWidget.UNSPECIFIED) return false;    	
        	if (this.priorityWidget.selected != com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED) return false;
        	return true;
        },    
        
        _applyFilter: function() {
        	var showResolved = !this._hideResolvedCheckNode.checked;
        	var forUser = this.directedToWidget.selected;
        	var byUser = this.authorWidget.selected;
            var priority = this.priorityWidget.selected;
            this.recentCommentResults.applyOptions(showResolved, forUser, byUser, priority);
        },
        
        _setDefaultFilterValues: function() {
            this.priorityWidget.select(com.ibm.rdm.web.commenting.PRIORITY_UNSPECIFIED);
            this.directedToWidget.select(this.directedToWidget.UNSPECIFIED);
            this.authorWidget.select(this.authorWidget.UNSPECIFIED);
        	this._hideResolvedCheckNode.checked = false;
        },
        
        _onShowAllClicked: function(event) {
            this._setDefaultFilterValues();
            this._disableShowAll();
            this._applyFilter();
        },
        
        _disableShowAll: function() {
            document.getElementById(this.id+"_showAllField").disabled = true;
    		this._showAllLabel.className = "showall-label-disabled";
        },
        
        _enableShowAll: function() {
            document.getElementById(this.id+"_showAllField").disabled = false;
    		this._showAllLabel.className = "showall-label-enabled";
        },
        
        _onCloseLinkClicked: function(event) {
            event.preventDefault();
            event.stopPropagation();
            this._recentResourcesHeader.closeOptions();
        }
    }
);

})();
