/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.RecentResourcesHeader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Tooltip");
dojo.require("com.ibm.rdm.web.search.domutils");
// No longer necessary, work item 13444
// dojo.require("com.ibm.rdm.web.search.results.RecentResourcesFilter");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search", "common");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "RecentResourcesHeader");

var RecentResourceResults = com.ibm.rdm.web.search.results.RecentResourceResults;

dojo.declare(
    "com.ibm.rdm.web.search.results.RecentResourcesHeader",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/RecentResourcesHeader.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _refreshButton: null,
//      _filterButton: null,
        _expandAllButton: null,
        _collapseAllButton: null,
        _showAllAnchor: null,

        _filterTable: null,
        _filterTwistieImage: null,
        _filterTitle: null,
        _filterSummarySpan: null,
        _filterSectionDiv: null,

        _optionsTable: null,
        _optionsTwistieImage: null,
        _optionsTitle: null,
        _optionsSummarySpan: null,
        _optionsSectionDiv: null,

        _requirementsTable: null,
        _requirementsSummarySpan: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        recentResourceResults: null,
        /** If specified, a Filter option will be presented.  When the user gestures to
            use the filter section, this 'filterSection' will be used to show this
            filtering presentation.  The functions it must implement include only one:
            - removeAllFilters()
            */
        filterSection: null,
        optionsSection: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _commonMessages: null,
        _isFilterOpen: false,
        _isOptionsOpen: false,
        _originalTitle: "",

        ///////////////////////////////////////////////////////////////////////////////////
        // NLS fields requiring translation

//         refresh: null,
        filter: null,
        options: null,
        showAll: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
//             this.connect(this._refreshButton, "onClick", this._handleRefreshButtonClick);
            this.connect(this._expandAllButton, "onClick", this._handleExpandAllButtonClick);
            this.connect(this._collapseAllButton, "onClick", this._handleCollapseAllButtonClick);
            this.connect(this._showAllAnchor, "onclick", this._handleShowAllClick);
            this._showAllAnchor.setAttribute("target", "_blank");
            this._showAllAnchor.setAttribute("href", "none");
            this._showAllAnchor.style.display = "none";
            this._isFilterOpen = false;
            this._isOptionsOpen = false;
            // Only one of the following *table attributes will be shown, we'll start
            // by hiding all of them ...
            this._filterTable.style.display = "none";
            this._optionsTable.style.display = "none";
            this._requirementsTable.style.display = "none";
            // And now we'll unhide the single one that should be shown.
            if (this.filterSection) {
                this._filterTable.style.display = "";
                //this._filterTwistieImage.setAttribute("src", this._getImageSrc(this._isFilterOpen));
                this.connect(this._filterToggleNode, "onClick", this._handleFilterClicked);
            }
            else if (this.optionsSection) {
                this._optionsTable.style.display = "";
                //this._optionsTwistieImage.setAttribute("src", this._getImageSrc(this._isOptionsOpen));
                this.connect(this._filterToggleNode, "onClick", this._handleOptionsClicked);
            }
            else {
                this._requirementsTable.style.display = "";
                this._filterToggleNode.destroyRecursive();
            }
            if (this.recentResourceResults._sidebarSection)
            	this._originalTitle = this.recentResourceResults._sidebarSection.sectionTitle;
            
        },

        startup: function() {
            // Install tool tips on the buttons appearing in this header.
            var addToolTip = function(button, tooltipText) {
                var div = document.createElement("div");
                button.domNode.parentNode.appendChild(div);
                div.appendChild(document.createTextNode(tooltipText));
                var tooltip = new dijit.Tooltip({connectId:[button.id]}, div);
                tooltip.startup();
            };
//             addToolTip(this._refreshButton, this._messages.refreshToolTip);
            //addToolTip(this._expandAllButton, this._messages.expandAllToolTip);
            //addToolTip(this._collapseAllButton, this._messages.collapseAllToolTip);
            this._expandAllButton.attr("title", this._messages.expandAllToolTip);
			this._collapseAllButton.attr("title", this._messages.collapseAllToolTip);
        },

        postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "RecentResourcesHeader");
            this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "common");
            var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search", "common");
//             dojo.forEach(["refresh"], function(prop){
//                 this[prop] = this._commonMessages[prop];
//             }, this);
            dojo.forEach(["filter", "options", "showAll"], function(prop){
                this[prop] = this._messages[prop];
            }, this);

            this.rdmModulePath = "com.ibm.rdm.web";     
        },

        closeFilter: function() {
            this._handleFilterClicked();
        },

        closeOptions: function() {
            this._handleOptionsClicked();
        },

        applyContentTypeFilters: function(contentTypesObject) {
            this.recentResourceResults.applyContentTypeFilters(contentTypesObject);
        },

        removeContentTypeFilters: function() {
            this.recentResourceResults.removeContentTypeFilters();
        },

        informUserThatSearchIsInProgress: function() {
        	this._updateSummaryTitleSpan(this._messages.loading);
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        updateNumberShownInfo: function(numShown, numTotal) {
            var numShownLabel = numShown;
            if (numTotal != numShown) {
	            numShownLabel = dojo.string.substitute(
	                this._messages.showingNofM,[numShown, numTotal]
	            );
            }
            this._updateSummaryTitleSpan(numShownLabel);
			this._showAllAnchor.style.display = (numShown == numTotal ? "none" : "");
        },
		
        _updateSummaryTitleSpan: function(message) {
        	if (this.recentResourceResults._sidebarSection) {
        		var title = dojo.string.substitute(this._messages.summaryTitle, [this._originalTitle, message]);
        		this.recentResourceResults._sidebarSection.setSectionTitleInfo(title);
        	}
        },

        // TODO Maybe move this to a common area (once we know that these icons ARE the
        // same as those in the TimeWindowSection class).
        _getImageSrc: function(isOpen) {
            var bundle = "com.ibm.rdm.web";
            if (isOpen) {
                return dojo.moduleUrl(bundle, "search/images/twistie-open.gif");
            } else { // closed
                return dojo.moduleUrl(bundle, "search/images/twistie-closed.gif");
            }
        },

//         _handleRefreshButtonClick: function(event) {
//             this.informUserThatSearchIsInProgress();
//             this.recentResourceResults.handleRefresh();
//         },

        _handleExpandAllButtonClick: function(event) {
            this.recentResourceResults.expandAll();
        },

        _handleCollapseAllButtonClick: function(event) {
            this.recentResourceResults.collapseAll();
        },

        _handleShowAllClick: function(event) {
            event.preventDefault();
            event.stopPropagation();
            this.filterSection.removeAllFilters();
        },

        _handleFilterClicked: function(event) {
            this._isFilterOpen = (!this._isFilterOpen);
            //this._filterTwistieImage.setAttribute("src", this._getImageSrc(this._isFilterOpen));
            if (!this._filterSectionDiv.firstChild) {
                this.filterSection.setRecentResourcesHeader(this);
                this._filterSectionDiv.appendChild(this.filterSection.domNode);
                this.filterSection.startup();
            } else {
                this.filterSection.domNode.style.display =
                    (this._isFilterOpen ? "" : "none");
            }
            this.recentResourceResults.headerHasChangedHeight();
        },

        _handleOptionsClicked: function(event) {
            this._isOptionsOpen = (!this._isOptionsOpen);
            //this._optionsTwistieImage.setAttribute("src", this._getImageSrc(this._isOptionsOpen));
            if (!this._optionsSectionDiv.firstChild) {
                this.optionsSection.setRecentResourcesHeader(this);
                this._optionsSectionDiv.appendChild(this.optionsSection.domNode);
                this.optionsSection.startup();
            } else {
                this.optionsSection.domNode.style.display =
                    (this._isOptionsOpen ? "" : "none");
            }
            this.recentResourceResults.headerHasChangedHeight();
        }
    }
);

})();
