/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.SearchEntryTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.search.SearchEventTopics");
dojo.require("com.ibm.rdm.web.search.SearchEventConstants");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "SearchEntryTable");

var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;

dojo.declare(
    "com.ibm.rdm.web.search.results.SearchEntryTable",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/SearchEntryTable.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _resourceHyperlink: null,
        _descriptionTD: null,
        _resourcePathTD: null,
        _sizeTD: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        resourceInfo: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _isRCP: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "SearchEntryTable");
            var resourceURI = this.resourceInfo.resourceURI;
            var decodedResourceURI = decodeURI(resourceURI);
            // Add the resource hyperlink.
            var link = ResourceLinksHandler.getResourceLink(resourceURI, this.resourceInfo.displayName, true);
            this._resourceHyperlink.appendChild(link);
            if (com.ibm.rdm.web.search.searchutils.isRCP()) {
                this._isRCP = true;
                this.connect(link, "onmouseover", this._handleLinkMouseOver);
                this.connect(link, "onmouseout", this._handleLinkMouseOut);
            } else {
                this._isRCP = false;
            }
            // Add the description field.
            //TODO No need for this when moving to rich client style search UI
            //var summaryTextNode = this._createDescriptionTextNode();
            //this._descriptionTD.appendChild(summaryTextNode);
            
            // Add the project field.
            var projectBasedPath = com.ibm.rdm.web.search.searchutils.getProjectResourcePath(decodedResourceURI);
            var projectName = this.resourceInfo.projectName;
			if (projectName) {
				var projectSpan = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
	                "search-result-resource-path", projectName);
				this._resourcePathTD.appendChild(projectSpan);
			}
            var size = this.resourceInfo.contentLength;
            if (size) {
                this.addSizeField(size);
            }
        },

        addSizeField: function(size) {
            var sizeSpan = com.ibm.rdm.web.search.domutils.createSpannedTextNode("search-result-resource-path", 
            		dojo.string.substitute(this._messages.size, [size]));
            this._sizeTD.appendChild(sizeSpan);
        },

        // TODO Remove (callers should use use getResourceInfo() instead)
        getLastModifiedDate: function() {
            return this.resourceInfo.lastModifiedDate;
        },

        getResourceInfo: function() {
            return this.resourceInfo;
        },

        updateResourceDisplayName: function() {
            var resourceNode = document.createTextNode(this.resourceInfo.displayName);
            var link = this._resourceHyperlink;
            link.replaceChild(resourceNode, link.firstChild);
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _createDescriptionTextNode: function() {
            var textNode = null;
            var description = this.resourceInfo.description;
            if (description) {
                textNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-entry-description", description);
            } else {
                textNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-entry-empty-description", "<" + this._messages.noDescription + ">");
            }
            return textNode;
        },

        _handleLinkClicked: function(event) {
        	ResourceLinksHandler.handleAnchorClicked(event);
        },

        _handleLinkMouseOver: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OVER);
        },

        _handleLinkMouseOut: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OUT);
        },

        /**
         * Send the status message to the embedded web browser inside the rich client.
         */
        _sendStatus: function(event, eventType) {
            event.preventDefault();
            event.stopPropagation();
            var href = (dojo.isIE ? event.srcElement.href : event.target.href);
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_RESOURCE_ID] = href;
            com.ibm.rdm.web.search.searchutils.triggerEvent(eventType, params);
            return false;
        }
    }
);

})();
