/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.SearchResults");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.search.results.DetailsTable");
dojo.require("com.ibm.rdm.web.search.results.ResourceInfo");
dojo.require("com.ibm.rdm.web.search.results.SearchEntryTable");
dojo.require("com.ibm.rdm.web.search.results.SearchHeader");
dojo.require("com.ibm.rdm.web.search.results.SingleLineEntryTable");
dojo.require("com.ibm.rdm.web.search.results.SelectionHandling");
dojo.require("com.ibm.rdm.web.search.results.SecondaryResourceEntriesTracker");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "SearchResults");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

// Note: As a starting point this code was originally based on Claudio's searching.js file.
var MAX_SEARCH_RESULTS = 300;
dojo.declare(
    "com.ibm.rdm.web.search.results.SearchResults",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/SearchResults.html"),
        widgetsInTemplate: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _searchResultsContent: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        doShowDetails: true,
        documentTypes: null,
        doShowDocTypeInfo: true,
		footerHeight: null,

        viewTypes: {
            LIST: 0,
            DETAILS: 1
        },
        VIEWTYPE_LIST: 0,
        VIEWTYPE_DETAILS: 1,
        
        interval: 1000, //1sec
        scrollOffset: 0.7, //70%

        isFullTextSearch: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _imageTableDataId: "imageTableDataId",
        _detailsLastRowTableDataId: "detailsLastRowTableDataId",
        _header: null,
        _resultsTable: null,
        _numOutstandingPropertyRequests: 0,
        _doIncludeCommentResources: false,
        _doIncludeDirectoryResources: false,
        _atLeastOneRowDataRequestFailed: false,
        _isSingleSelect: false,
        _isMultiSelect: false,
        _selectionHandling: null,
        _itemSelectedClassName: "search-result-row-selected",
        _viewType: null,        // Holds the "current viewtype" enum value (represented by values found in this.viewTypes)
        _secondaryResourceQueryParams: null,
        _secondaryResourceEntriesTracker: null,
        _contentPane: null,
        _searchQuery: null,
        _searchHeader: null,
        _totalResultCount: null,
        _numEntriesDisplayed: null,
        _isLoadingNextPage: null,
        _nextPageURI: null,
        
        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "SearchResults");
            if (this.doShowDocTypeInfo && !this.documentTypes) {
                this.documentTypes = new com.ibm.rdm.web.search.DocumentTypes({});
            }
            this._viewType = this.viewTypes.DETAILS;
            this._header = this._createHeader();
            this._selectionHandling = new com.ibm.rdm.web.search.results.SelectionHandling(
                {itemSelectedClassName: this._itemSelectedClassName}, null);
			this._searchResultsContent.style.marginBottom = this.footerHeight + "px";

			// Hook up events for window resizing.
			this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this.handleWindowResize, force: true});
            
            //this.connect(this._showMoreResultsLink, "onclick", this.showMoreResults);
            //this.connect(this._showAllResultsLink, "onclick", this.showAllResults);
    		//this._showMoreResultsLink.style.display = "none";
			//this._showAllResultsLink.style.display = "none";
        },
        
        destroy: function() {
        	jazz.util.ResizeNotifier.unregister(this._resizeEvent);
            this._secondaryResourceEntriesTracker = null;
            this._secondaryResourceQueryParams = null;
            this.inherited("destroy", arguments);
        },

        startup: function() {
            var docNode = this._searchResultsContent;
            docNode.appendChild(this._header.domNode);
            this._header.startup();
        },

        setSelectionType: function(isMultiSelect) {
            this._doIncludeDirectoryResources = false;
            if (isMultiSelect) {
                this._isMultiSelect = true;
            } else {
                this._isSingleSelect = true;
            }
        },

        informUserThatSearchIsInProgress: function() {
            this._header.informUserThatSearchIsInProgress();
        },
        
		informUserNoSearchCriterion: function() {
            this._header.informUserNoSearchCriterion();
        },
        
        /**
         * Display a search query result set.
         *
	     * @param queryOrSearchDoc the DOM document (xml) representation of a JRS
         *  "query" or "search" response.  May be null -- passed as null whenever
         *  the table is asked to display itself using a different viewType.
         * @param userQueryString passed only in the case of "query" (not
         *  "search").
         * @param filteredPrimaryResourcesObject if the queryOrSearchDoc contains
         *  secondary resources (i.e. it must have been a "query") this object is
         *  consulted to filter out any secondary resources whose containing
         *  primary resource's URI is not found as a property in this object
         */
        displayQueryResults: function(queryOrSearchDoc, userQueryString, filteredPrimaryResourcesObject) {
        	var appendResults = false;
        	if (queryOrSearchDoc) {
        		if (queryOrSearchDoc.doc) {
        			queryOrSearchDoc = queryOrSearchDoc.doc;
        		}
        		//TODO add pagination and compensate for large result sets
	        	//this._nextPageURI = com.ibm.rdm.web.util.queryutils.getNextPageURI(queryOrSearchDoc);
	        	//this._totalResultCount = com.ibm.rdm.web.util.queryutils.getNumberOfEntries(queryOrSearchDoc);
	        	//if (this._totalResultCount > MAX_SEARCH_RESULTS) {
	        		//this._totalResultCount = MAX_SEARCH_RESULTS;
	        	//}
	        }
        	
            var contentNode = this._searchResultsContent;
            var headerNode = this._header.domNode;
            var isDetailsView = (this._viewType == this.viewTypes.DETAILS);
            var secondaryResourceEntriesTracker = null;
			var wrappedResourceURIs = [];
			var tbody;
         
            if (!appendResults) {
	            // Clear out the last results.
//	            contentNode.removeChild(headerNode);
//	            this._clearLastResults();
	            // Add the header, fill it in, and create a table to show the new entries.
//	            contentNode.appendChild(headerNode);
	            var table = document.createElement("table");
	            table.className = (isDetailsView ? "search-results-details-table" : "search-results-list-table");
	            tbody = document.createElement("tbody");
	            table.appendChild(tbody);
	            // Add a content pane.
	            var div = document.createElement("div");
	            contentNode.appendChild(div);
	            // Add a content pane, if needed.
	            var contentPane = null;
	            // TODO If we want to bring back RCP testing pages we'll have to add
	            // another condition here to make sure the content pane is used for
	            // testing cases ...
	            if (com.ibm.rdm.web.search.searchutils.isRCP()) {
	                var div = document.createElement("div");
	                this._searchResultsContent.appendChild(div);
	                contentPane = new dijit.layout.ContentPane({}, div);
	                contentPane.attr('content', table);
	                dojo.addClass(contentPane.domNode, "search-results-content-pane");
	                contentPane.startup();
	            } else {
	                this._searchResultsContent.appendChild(table);
	            }
	            this._contentPane = contentPane;
	            // Fill in the table with the results set.
	            this._resultsTable = table;
	            this._header.setResultsTable(table);
	            this._numEntriesDisplayed = 0;
            } else {
            	tbody = this._resultsTable.firstChild;
            	this._numEntriesDisplayed = tbody.childNodes.length;
            }
            
            if (queryOrSearchDoc) {            	
              
                var entries = queryOrSearchDoc;
                var isXQueryDoc = true;
                secondaryResourceEntriesTracker = new com.ibm.rdm.web.search.results.SecondaryResourceEntriesTracker();
                var mapResourceURIToWhetherVisited = new Object;
                for (var i = 0; i < entries.length; i++) {
                    var entry = entries[i];
                    var resourceURI = entry.resourceURI ? entry.resourceURI : "";
                    var isSecondaryResource = com.ibm.rdm.web.search.searchutils.isSecondaryResource(resourceURI);
                    var primaryResource = (isSecondaryResource ?
                        com.ibm.rdm.web.search.searchutils.getSecondaryResourceParentURI(resourceURI) : resourceURI)
                    var doExcludeEntry = (
                        isXQueryDoc ?
                        false :
                        // Note: full text searches return such things as tags, but
                        // we don't want to include these ... also, full text search
                        // results MAY contain duplicates, which we also want to
                        // avoid including.
                        (com.ibm.rdm.web.search.results.ResourceInfo.isAtomResource(entry)
                         || mapResourceURIToWhetherVisited[resourceURI])
                        // TODO Remove as we're no longer using secondary resources ...
//                         (!this._doIncludeCommentResources
//                             && com.ibm.rdm.web.search.results.ResourceInfo.isCommentResource(entry))
//                         || (userQueryString
//                             && (!com.ibm.rdm.web.search.results.ResourceInfo.isResourceNameMatch(entry, userQueryString)))
//                         || (!this._doIncludeDirectoryResources
//                             && com.ibm.rdm.web.search.results.ResourceInfo.isDirectoryResource(entry))
//                         || (isSecondaryResource && filteredPrimaryResourcesObject
//                             && (!filteredPrimaryResourcesObject[primaryResource]))
//                         || (isSecondaryResource
//                             && secondaryResourceEntriesTracker.isParentResourceAlreadyTracked(resourceURI))
//                         || (com.ibm.rdm.web.search.results.ResourceInfo.isAttributeGroupResource(entry))
                    );
                    mapResourceURIToWhetherVisited[resourceURI] = true;
                    if (isSecondaryResource) {
                        secondaryResourceEntriesTracker.trackSecondaryResource(resourceURI);
                    }
                    if (!doExcludeEntry) {
						var tr = this._createTableRow(entry, tbody, secondaryResourceEntriesTracker, isXQueryDoc);
						if (isDetailsView) {
							tr.className = (this._numEntriesDisplayed%2 ? "search-result-entry-even-row" : "search-result-entry-odd-row");
						}
						this._numEntriesDisplayed++;
                    }
                }
                this._secondaryResourceEntriesTracker = secondaryResourceEntriesTracker;
            } else {
            	var resourceInfos = this._getDisplayedResourceInfos();
                for (var i = 0; i < resourceInfos.length; i++) {
                    var resourceInfo = resourceInfos[i];
                    var tr = this._createTableRow(resourceInfo, tbody, this._secondaryResourceEntriesTracker);
                    if (isDetailsView) {
                        tr.className = (i%2 ? "search-result-entry-even-row" : "search-result-entry-odd-row");
                    }
                }
                this._numEntriesDisplayed = resourceInfos.length;
            }
            this._showLoadingMessage.style.display = "";
			this._fetchWrappersIfNeededAndUpdateTitleAndSummary(
				wrappedResourceURIs, tbody, isDetailsView, secondaryResourceEntriesTracker, isXQueryDoc);
			this._startListeningToScroll();
        },
        
        showMoreResults: function(event) {
    		if( event ) {
    			event.preventDefault();
    			event.stopPropagation();
    		}
    		var deferred = null;
    		if (this._nextPageURI && !this._isLoadingNextPage) {
    			deferred = RDMClient.doGet({uri: this._nextPageURI});
    			deferred.addCallback(this, this.displayQueryResults);
    			deferred.addErrback(this, this.handlePageExpiration);
    			this._isLoadingNextPage = true;
    		}
    		return deferred;
        },

        showAllResults: function(event) {
    		if(event) {
    			event.preventDefault();
    			event.stopPropagation();
    		}
    		var deferred = this.showMoreResults();
    		if (deferred) {
    			deferred.addCallback(this, this.showAllResults);
    			//deferred.addErrback(this, this.handlePageExpiration);
    		}
        },
        
        handlePageExpiration: function() {
        	com.ibm.rdm.web.search.searchutils.showAsBusy(true);
        	this._searchQuery.makeSearchRequestAgain(this._searchHeader.getSortOrder());
        },
        
        performQuery: function(queryParams) {
            // TODO Later pass queryId instead of queryParams
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);;
        },

        showDetails: function(doShowDetails) {
            this.doShowDetails = doShowDetails;
            var tbody = this._resultsTable.firstChild;
            var rows = tbody.childNodes;
            for (var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var td = row.lastChild;
                td.style.display = (doShowDetails ? "" : "none");
            }
        },

        getViewType: function() {
            return this._viewType;
        },

        switchViewType: function(viewType) {
            this._viewType = viewType;
            this.displayQueryResults();
        },
        
        handleSort: function(sortBy) {
        	this._searchQuery.handleSortedQuery(sortBy);
        },

        handleSortHasBeenPerformed: function() {
            var tbody = this._resultsTable.firstChild;
            if (this._viewType == this.viewTypes.DETAILS) {
                var i = 0;
                for (tr = tbody.firstChild; tr != null; tr = tr.nextSibling) {
                    tr.className = (i%2 ? "search-result-entry-even-row" : "search-result-entry-odd-row");
                    i++;
                }
            }
        },

        setSecondaryResourceQueryParams: function(secondaryResourceQueryParams) {
            this._secondaryResourceQueryParams = secondaryResourceQueryParams;
        },

        searchQueryAreaHeightHasChanged: function() {
            this._resizeContentPaneIfNeeded();
        },

        handleWindowResize: function() {
            this._resizeContentPaneIfNeeded();
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _fetchWrappersIfNeededAndUpdateTitleAndSummary: function(wrappedResourceURIs, tbody,
																 isDetailsView, secondaryResourceEntriesTracker, isXQueryDoc) {
			var self = this;
			var updateTitleAndSummary = function() {
				var summaryTitle = self._composeSummaryTitle(self._numEntriesDisplayed, self._totalResultCount);
				self._header.updateSummary(summaryTitle);
				// Set the initial height of the content pane.
				self._resizeContentPaneIfNeeded();
			};
			var numWrapperInfosReceived = 0;
			var onWrapperComplete = function(wrapperInfo) {
				numWrapperInfosReceived++;
				var resourceInfo = new com.ibm.rdm.web.search.results.ResourceInfo({
					wrapperInfo: wrapperInfo
                });
				var tr = self._createTableRow(resourceInfo, tbody, secondaryResourceEntriesTracker, true, true);
				if (isDetailsView) {
					tr.className = (this._numEntriesDisplayed%2 ? "search-result-entry-even-row" : "search-result-entry-odd-row");
				}
				this._numEntriesDisplayed++;
				if (numWrapperInfosReceived === wrappedResourceURIs.length) {
					updateTitleAndSummary();
				}
				self._isLoadingNextPage = false;
	        	if (!self._nextPageURI) {
	        		self._showLoadingMessage.style.display = "none";
	        	}
			};

			updateTitleAndSummary();
			this._isLoadingNextPage = false;
        	if (!this._nextPageURI) {
        		this._showLoadingMessage.style.display = "none";
        	}
			
        },
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Listeners for endless scrolling
        _startListeningToScroll: function () {
        	setTimeout(dojo.hitch(this, this._checkScroll), this.interval);
        },

        _checkScroll: function () {
        	if(!this._nextPageURI ||
        			this._numEntriesDisplayed >= MAX_SEARCH_RESULTS) {
        		return null;
        	}
        	var offset = document.documentElement.scrollTop/document.documentElement.scrollHeight;
        	if(offset > this.scrollOffset) {
        		this.showMoreResults();
        	}
        	this._startListeningToScroll();
        },

        _resizeContentPaneIfNeeded: function() {
            if (this._contentPane) {
                var windowHeight = (window.innerHeight || document.body.clientHeight);
                var windowWidth = (window.innerWidth || document.body.clientWidth);
                var contentPaneTop = this._contentPane.domNode.offsetTop;
                var scrollHeightToUse = (windowHeight - contentPaneTop);
                // TODO Resolve this fudging issue ...
                // The '4' fudge factor in the following statement is needed to
                // make this look right on IE ...
                this._contentPane.resize({ h: scrollHeightToUse, w: windowWidth-4 });
            }
        },

        _getDisplayedResourceInfos: function() {
            var tbody = this._resultsTable.firstChild;
            var resourceInfos = dojo.map(
                tbody.childNodes,
                function(tr) {
                    var entryTable = com.ibm.rdm.web.search.searchutils.getEntryTable(tr);
                    return entryTable.getResourceInfo();
                }
            );
            return resourceInfos;
        },

        clearLastResults: function() {
            var docNode = this._searchResultsContent;
            if (this._resultsTable) {
                var list = dojo.query('[widgetId]', this._resultsTable);
                dojo.forEach(list.map(dijit.byNode), function(widget){ widget.destroyRecursive(); });
            }
            if (this._contentPane) {
                this._contentPane.destroyRecursive();
            }
            docNode.innerHTML = "";
            this._header.destroy();
            this._header = this._createHeader();
            docNode.appendChild(this._header.domNode);
        },

        _convertQueryParamsToQueryParamString: function(queryParams) {
            var buf = "";
            var attrNames = queryParams.attributeNames;
            for (var i = 0; i < attrNames.length; i++) {
                if (i > 0) {
                    buf += "&";
                }
                buf += attrNames[i];
                buf += "=";
                buf += queryParams[attrNames[i]];
            }
            return buf;
        },

        /*
         * Callback for a successful query.
         */
        _handlePerformSearchOrQuerySuccessResponse: function(response) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            this.displayQueryResults(response);
        },

        /*
         * Callback for an unsuccessful query.
         */
        _handlePerformSearchOrQueryErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            com.ibm.rdm.web.search.searchutils.reportError(
                  dojo.string.substitute(this._messages.errorAttemptingToPerformQuery,
                      [(error.description || error)])
            );
        },

        _createHeader: function() {
            var doShowAlternatingRowColors = (!this._isSelectionBasedControl());
            this._searchHeader = (new com.ibm.rdm.web.search.results.SearchHeader({
                searchResults: this,
                doShowDetails: this.doShowDetails,
                doShowAlternatingRowColors: doShowAlternatingRowColors
                }, null));
            return this._searchHeader;
        },

        _composeSummaryTitle: function(numEntries, totalEntries) {
            var summaryTitle;
            var msgKeys = {
                numMatches: numEntries,
                totalMatches: totalEntries
            };
            if (numEntries == 0) {
                summaryTitle = dojo.string.substitute(this._messages.zeroResultsSummaryMsg, msgKeys);
            } else if (numEntries == 1) {
                summaryTitle = dojo.string.substitute(this._messages.singleEntryResultMsg, msgKeys);
            } else if (totalEntries) {
            	summaryTitle = dojo.string.substitute(this._messages.multiplePageEntriesResultMsg, [numEntries, totalEntries]);
            } else {
                summaryTitle = dojo.string.substitute(this._messages.multipleEntriesResultMsg, [numEntries]);
            }
            return summaryTitle;
        },

        // Note: We're asking for HEAD only to obtain the appropriate Size information
        // for each entry.
        _handleGetResourceHeadSuccessResponse: function(ioArgs) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            var contentLocation = ioArgs.xhr.getResponseHeader("Content-Location");
            var resourceURI = contentLocation.substring(0, contentLocation.indexOf("?"));
            var tr = dojo.byId(resourceURI);
            var entryTable = com.ibm.rdm.web.search.searchutils.getEntryTable(tr);
            var resourceInfo = entryTable.getResourceInfo();
            resourceInfo.addResourceHeadInfo(ioArgs);
            if (this._viewType == this.viewTypes.DETAILS) {
                entryTable.addSizeField(resourceInfo.contentLength);
            }
        },

        _handleGetResourceHeadErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            // Note: The only thing missing as a result of not getting a HEAD request
            // back would be that the Size information for a particular resource would
            // not be reported.  We'll just log this and not throw up a error dialog
            // if this occurs.
            console.error("HEAD request returned abnormal response: "
                + (error.description || error));
        },

        /**
         * Return true iff this widget is being used as an embedded control to
         * gather URI selections.
         */
        _isSelectionBasedControl: function() {
            return (this._isSingleSelect || this._isMultiSelect);
        },

        _handleGetResourcePropertiesSuccessResponse: function(response) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            var resourceURI = com.ibm.rdm.web.util.queryutils.getResourceURI(response);
            var tr = dojo.byId(resourceURI);
            if (!tr) {
                console.error("Couldn't find tr with id " + resourceURI);
            } else {
                if (this._isSelectionBasedControl()) {
                    // TODO Do something with tool tips here?
                } else {
                    var entryTable = com.ibm.rdm.web.search.searchutils.getEntryTable(tr);
                    var resourceInfo = entryTable.getResourceInfo();
                    var displayName = resourceInfo.displayName;
                    resourceInfo.addResourceIndexPropertiesInfo(response);
                    if (this._viewType == this.viewTypes.DETAILS) {
                        if (displayName != resourceInfo.displayName) {
                            // Workaround for the fact that full text searches
                            // currently can't return display name info in query
                            // results ...
                            var entryTable = com.ibm.rdm.web.search.searchutils.getEntryTable(tr);
                            entryTable.updateResourceDisplayName(resourceInfo.displayName);
                        }
                        this._addDetailsTable(tr, resourceInfo);
                    } else {
                        entryTable.addSecondaryResourceMatchesIfNeeded();
                    }
                }
            }
            this._decrementOutstandingRequestCount();
        },

        _addDetailsTable: function(tr, resourceInfo) {
        	
        	var fullTextChecked = false;
        	fullTextChecked = this._searchQuery._fullTextCheckBox.checked;
            var detailsTable = new com.ibm.rdm.web.search.results.DetailsTable(
                {
                    resourceInfo:resourceInfo,
                    documentTypes:this.documentTypes,
                    fullTextChecked:fullTextChecked
                },
                null
            );
            tr.appendChild(com.ibm.rdm.web.search.domutils.createTD(detailsTable.domNode));
            detailsTable.startup();
        },

        _handleGetResourcePropertiesErrorResponse: function(error) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(false);
            // Note: We're not immediately reporting the error as it's likely the case
            // that MANY errors would occur rather than just one (as these errors are
            // for individual resource GET requests and there are probably MANY
            // resources contained in a single result set).
            this._atLeastOneRowDataRequestFailed = true;
            console.error("JRS resource properties request returned abnormal response: "
                + (error.description || error));
            this._decrementOutstandingRequestCount();
        },

        _incrementOutstandingRequestCount: function() {
            this._numOutstandingPropertyRequests++;
        },

        _decrementOutstandingRequestCount: function() {
            this._numOutstandingPropertyRequests--;
            if (this._numOutstandingPropertyRequests == 0) {
                try {
                    this._header.handleTableContentsHasChanged();
                } catch(e) {
                    console.error(e);
                }
                if (this._atLeastOneRowDataRequestFailed) {
                    // TODO Provide more useful error information
                    com.ibm.rdm.web.search.searchutils.reportError(this._messages.errorAttemptingToRetrieveTableRowInfo);
                    this._atLeastOneRowDataRequestFailed = false;
                }
            }
        },

        _isResourceInfo: function(something) {
            return (something instanceof com.ibm.rdm.web.search.results.ResourceInfo);
        },

        _createOrGetResourceInfo: function(atomEntryNodeOrResourceInfo, secondaryResourceEntriesTracker, isXQuery, isExtendedResource) {
			var resourceInfo = null;
			if (this._isResourceInfo(atomEntryNodeOrResourceInfo))
				resourceInfo = atomEntryNodeOrResourceInfo;
			else {
                resourceInfo = new com.ibm.rdm.web.search.results.ResourceInfo({
                    atomEntryNode: atomEntryNodeOrResourceInfo,
                    documentTypes: this.documentTypes,
                    secondaryResourceQueryParams: this._secondaryResourceQueryParams,
                    secondaryResourceEntriesTracker: secondaryResourceEntriesTracker,
                    isXQuery: isXQuery,
                    isExtendedResource: (! this.isFullTextSearch)
                });
			}
            return resourceInfo;
        },

        _createTableRow: function(atomEntryNodeOrResourceInfo, tbody, secondaryResourceEntriesTracker, isXQuery, isNewlyCreatedWrapperRow) {

            var resourceInfo = atomEntryNodeOrResourceInfo;
            var resourceURI = resourceInfo.resourceURI;
			var wrappedResourceURI = resourceInfo.wrappedResourceURI;

            // Create the table row and hookup events for the row if it's going to be used
            // for selections.
            var tr = document.createElement("tr");
            tr.id = (wrappedResourceURI || resourceURI);
            if (this._isSelectionBasedControl()) {
                this._addRowEvents(tr);
            }
            tbody.appendChild(tr);

            // Add an image, representing the assoc. resource type, as the first td element.
            var td = document.createElement("td");
            tr.appendChild(td);
            var imageSrc = ((this._viewType == this.viewTypes.DETAILS) ?
            	com.ibm.rdm.web.common.MimeTypeRegistry.getLargeImageURI(resourceInfo.contentType):
                com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(resourceInfo.contentType)
            );
            if (imageSrc) {
                var img = document.createElement("img");
                img.setAttribute("src", imageSrc);
                if (this._viewType == this.viewTypes.DETAILS) {
                    td.className = "search-result-entry-resource-image"; // TODO Rename?
                } else {
                    td.className = "search-result-selection-entry-resource-image"; // TODO Rename
                }
                td.appendChild(img);
            }

            // Add an *EntryTable instance as the second td element.
            var entryTable = null;
            switch (this._viewType) {
                case this.viewTypes.LIST:
                    entryTable = new com.ibm.rdm.web.search.results.SingleLineEntryTable({resourceInfo:resourceInfo}, null);
                    if (this._isMultiSelect || this._isSingleSelect) {
                        entryTable.setSelectionType(this._isMultiSelect);
                    }
                    break;
                case this.viewTypes.DETAILS:
                    entryTable = new com.ibm.rdm.web.search.results.SearchEntryTable({resourceInfo:resourceInfo}, null);
                    break;
            }
            tr.appendChild(com.ibm.rdm.web.search.domutils.createTD(entryTable.domNode));
            entryTable.startup();
            if (this._isResourceInfo(atomEntryNodeOrResourceInfo)) {
                if (this._viewType == this.viewTypes.DETAILS) {
                    this._addDetailsTable(tr, atomEntryNodeOrResourceInfo);
                }
				if (isNewlyCreatedWrapperRow) {
					//this._requestAdditionalRowInformation(wrappedResourceURI);
				}
            } else {
            	//TODO: Art B.
            	//Commenting out a part of the code that sends out a GET request to retrieve more information about a resource.
            	//Collection feed already has the necessary information that needs to be displayed in the table, but may require more in the future.
            	
                // Request more info needed to know such things as the size for every result row.
				//this._requestAdditionalRowInformation(resourceURI);
                //if (isXQuery) {
                	//if (this._viewType == this.viewTypes.DETAILS) {
                		this._addDetailsTable(tr, resourceInfo);
                	//}
                //} else {
                    // Request the information needed to display a DetailsTable instance at the end of every row.
                //    this._requestPropertyInformation(resourceURI);
                //}
            }

            return tr;
        },

        _addRowEvents: function(tr) {
            this._selectionHandling.addNodeSelectionEvents(tr);
            this.connect(tr, "onclick", this._handleRowClicked);
            this.connect(tr, "ondblclick", this._handleRowDoubleClicked);
            this.connect(tr, "onmouseover", this._handleRowMouseOver);
            this.connect(tr, "onmouseout", this._handleRowMouseOut);
        },

        _requestAdditionalRowInformation: function(resourceURI) {
            if (!this._isSelectionBasedControl()) {
                com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            }
        },

        _requestPropertyInformation: function(resourceURI) {
            com.ibm.rdm.web.search.searchutils.showAsBusy(true);
            this._incrementOutstandingRequestCount();
        },

        /**
         * Gets the node text value if the node has a value; otherwise returns "<no value>".
         *
         * @param domNode the DOM node object
         * @return the node string value
         */
        _getNodeValue: function(/* DOM Node */ domNode) {
            var stringValue = "";
            try {
                stringValue = domNode.childNodes[0].nodeValue;
            } catch(e) {
    			var message = dojo.string.substitute(this._messages["errorGetNodeValue"], [e]);
    			ErrorHandler.log(message);
                stringValue = this._messages.noValue;
            }
            return stringValue;
        },

        _handleRowClicked: function(event) {
            if (this._isRCP) {
                this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_SELECTED_TOPIC);
            }
        },

        _handleRowDoubleClicked: function(event) {
            if (this._isRCP) {
                this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_DOUBLE_CLICKED_TOPIC);
            }
        },

        _handleRowMouseOver: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OVER);
        },

        _handleRowMouseOut: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OUT);
        },

        /**
         * Send the status message to the embedded web browser inside the rich client.
         */
        _sendStatus: function(event, eventType) {
            event.preventDefault();
            event.stopPropagation();
            var href = (dojo.isIE ? event.srcElement.href : event.target.href);
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_RESOURCE_ID] = href;
            com.ibm.rdm.web.search.searchutils.triggerEvent(eventType, params);
            return false;
        }
    }
);



})();
