/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.SelectionHandling");

dojo.require("dijit._base");
dojo.require("dijit._Widget");


dojo.declare(
    "com.ibm.rdm.web.search.results.SelectionHandling",
    dijit._Widget,
    {
        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        isSingleSelection: true,
        itemSelectedClassName: "search-result-row-selected",

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _selection: {},
        _nodesEligibleForSelection: {},

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        addNodeSelectionEvents: function(node) {
            this.connect(node, "onclick", this._handleNodeClicked);
            this.connect(node, "ondblclick", this._handleNodeDoubleClicked);
            this._nodesEligibleForSelection[node.id] = 1;
        },

        setSingleSelection: function(isSingleSelection) {
            this.isSingleSelection = isSingleSelection;
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _getSelectionNode: function(clickedNode) {
            var node = clickedNode;
            while (!node.id || this._nodesEligibleForSelection[node.id] != 1) {
                node = node.parentNode;
            }
            return node;
        },

        _removeSelection: function() {
            for (var prop in this._selection) {
                var node = dojo.byId(prop);
                if (node) {
                    dojo.removeClass(node, this.itemSelectedClassName);
                }
            }
            this._selection = {};
        },

        _handleNodeClicked: function(event) {
            var node = this._getSelectionNode(dojo.isIE ? event.srcElement : event.target);
            this._removeSelection();
            this._selection[node.id] = node;
            dojo.addClass(node, this.itemSelectedClassName);
        },

        _handleNodeDoubleClicked: function(event) {
        }
    }
);

})();
