/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.SingleLineEntryTable");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.search.SearchEventTopics");
dojo.require("com.ibm.rdm.web.search.SearchEventConstants");
dojo.require("com.ibm.rdm.web.search.searchutils");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.web.search.results", "SingleLineEntryTable");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;

dojo.declare(
    "com.ibm.rdm.web.search.results.SingleLineEntryTable",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/SingleLineEntryTable.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _contentTypeImageTD: null,
        _resourceHyperlinkTD: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        resourceInfo: null,
        doShowContentTypeImage: false,
        doShowSubSection: true,
        displayContext: null,
        projectURI: null,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields

        _messages: null,
        _isRCP: false,
        _resourceNameAnchor: null,
        _isSingleSelect: false,
        _isMultiSelect: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "SingleLineEntryTable");
            // Add the resource hyperlink field
            var resourceURI = this.resourceInfo.resourceURI;
            var projectResourcePath = com.ibm.rdm.web.search.searchutils.getProjectResourcePath(resourceURI);
            var projectName = this.resourceInfo.projectName;
            var resourceName = this.resourceInfo.displayName;
            if (this.doShowContentTypeImage) {
                var imageSrc = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(this.resourceInfo.contentType);
                if (imageSrc) {
                    var img = document.createElement("img");
                    img.setAttribute("src", imageSrc);
                    this._contentTypeImageTD.appendChild(img);
                }
            } else {
                this._contentTypeImageTD.style.display = "none";
            }
            // TODO Consider using new ResourceAnchor class here ...
            var link = ResourceLinksHandler.getResourceLink(resourceURI, resourceName, true);
            link.classname = "search-result-hyperlink";
            if (com.ibm.rdm.web.search.searchutils.isRCP()) {
                this._isRCP = true;
//                 this.connect(link, "ondblclick", this._handleLinkDoubleClicked);
                this.connect(link, "onmouseover", this._handleLinkMouseOver);
                this.connect(link, "onmouseout", this._handleLinkMouseOut);
            } else {
                this._isRCP = false;
            }
            this._resourceHyperlinkTD.appendChild(link);
            //this._addProject(projectName);
            this._addSecondaryResourceMatchesIfNeeded();
            this._resourceNameAnchor = link;
            
        	if (this.displayContext != 
            	com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.USER_DASHBOARD) {
	            this.connect(this._entryTable, "onmouseover", this._onResourceMouseOver);
	            this.connect(this._entryTable, "onmouseout", this._onResourceMouseOut);
        	}
        },
        
        _onResourceMouseOver: function(event) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
            dojo.addClass(this._entryTable, "highlight");
        },

        _onResourceMouseOut: function(event) {
        	if (event) {
        		event.preventDefault();
    			event.stopPropagation();
        	}
            dojo.removeClass(this._entryTable, "highlight");
        },

        setSelectionType: function(isMultiSelect) {
            if (isMultiSelect) {
                this._isMultiSelect = true;
            } else {
                this._isSingleSelect = true;
            }
        },

        startup: function() {
            var id = (this.resourceInfo.resourceURI + ":link-tooltip");
            this._resourceNameAnchor.id = id;
            //var div = document.createElement("div");
            if (this.doShowSubSection) {
            	this._resourceNameAnchor.parentNode.appendChild(this._createSubSection());
            	dojo.addClass(this._entryTable, "underline-table");
            }
            //div.appendChild(this._createSubSection());
            //var tooltip = new dijit.Tooltip({connectId:[id]}, div);
            //tooltip.startup();
        },

        // TODO Remove (callers should use use getResourceInfo() instead)
        getLastModifiedDate: function() {
            return this.resourceInfo.lastModifiedDate;
        },

        getResourceInfo: function() {
            return this.resourceInfo;
        },

        addSecondaryResourceMatchesIfNeeded: function() {
            this._addSecondaryResourceMatchesIfNeeded();
        },

        ///////////////////////////////////////////////////////////////////////////////////
        // Helpers

        _addProject: function(projectName) {
            var projectSpan = com.ibm.rdm.web.search.domutils.createProjectSpan(
                "search-result-entry-folder", projectName);
            this._resourceHyperlinkTD.appendChild(projectSpan);
        },

        // TODO: nls 
        _addSecondaryResourceMatchesIfNeeded: function() {
            var secondaryResourceMatchesObject = this.resourceInfo.getSecondaryResourceMatchesObject();
            if (secondaryResourceMatchesObject) {
            	for (prop in secondaryResourceMatchesObject) {
                    var label = "(" + dojo.string.substitute(this._messages.propertyLabel, [prop]);
                    this._resourceHyperlinkTD.appendChild(com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                        "search-result-entry-secondary-resource-matches", label));
                    var values = secondaryResourceMatchesObject[prop];
                    com.ibm.rdm.web.search.domutils.appendSecondaryResourceMatches(
                        this._resourceHyperlinkTD, values);
                    this._resourceHyperlinkTD.appendChild(com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                        "search-result-entry-secondary-resource-text", ")"));
                }
            }
        },
        
        _createSubSection: function() {
        	var element = document.createElement("div");
        	var lastModifiedDateDisplayString = dojo.date.locale.format(
        			this.resourceInfo.lastModifiedDate, { formatLength:"medium" });

        	ProjectCache.getProjectUsers(this.projectURI).addCallback(this, 
    			function(users) {
					var userURI = com.ibm.rdm.web.util.queryutils.getFullyQualifiedUriFromRelativeLink(com.ibm.rdm.web.FRONTING_SERVICE_RECENT_FEEDS, this.resourceInfo.lastModifiedBy);
					if(users[userURI]){
						var user = users[userURI].name;
						var userNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode("search-result-user-attribute-value", user);
						element.appendChild(userNode);
						this._createUserHover(user, userNode);
					}
				}
			);
        	
        	var modifiedOnDate = dojo.string.substitute(this._messages.modifiedOnDate, [lastModifiedDateDisplayString])
        	var modifiedOnNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
        			"search-result-last-modified-on-attribute-value", modifiedOnDate);
        	// TODO Consider adding doctype info as well ... but how to display this?
        	element.appendChild(modifiedOnNode);      	
        	element.className = "search-result-sub-section";
            return element;        	
        },

        _createTooltipContent: function() {
            var table = document.createElement("table");
            var lastModifiedDateDisplayString = dojo.date.locale.format(
                this.resourceInfo.lastModifiedDate, { formatLength:"medium" });
            var user = com.ibm.rdm.web.util.queryutils.getTrimmedUserName(
                this.resourceInfo.lastModifiedBy);
            var attrValues = [
                { attr:this._messages.lastUpdated, value:lastModifiedDateDisplayString },
                { attr:this._messages.modifiedBy, value:user }
            ];
            for (var i = 0; i < attrValues.length; i++) {
                var attrValue = attrValues[i];
                var nameNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-tooltip-resource-attribute-name", attrValue.attr + ":");
                var valueNode = com.ibm.rdm.web.search.domutils.createSpannedTextNode(
                    "search-result-tooltip-resource-attribute-value", attrValue.value);
                table.appendChild(com.ibm.rdm.web.search.domutils.createTR([
                    com.ibm.rdm.web.search.domutils.createTD(nameNode),
                    com.ibm.rdm.web.search.domutils.createTD(valueNode)
                ]));
            }
            // TODO Consider adding doctype info as well ... but how to display this?
            return table;
        },
        
        _handleResourceContextClicked: function (event) {
        	ResourceLinksHandler.handleAnchorContextMenuClicked(event);
        },

        _handleLinkClicked: function(event) {
        	ResourceLinksHandler.handleAnchorClicked(event);
        },

        _handleLinkMouseOver: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OVER);
        },

        _handleLinkMouseOut: function(event) {
            this._sendStatus(event, com.ibm.rdm.web.search.SearchEventTopics.RESOURCE_MOUSE_OUT);
        },

        /**
         * Send the status message to the embedded web browser inside the rich client.
         */
        _sendStatus: function(event, eventType) {
            event.preventDefault();
            event.stopPropagation();
            var href = (dojo.isIE ? event.srcElement.href : event.target.href);
            var params = {};
            params[com.ibm.rdm.web.search.SearchEventConstants.JSEVENT_RESOURCE_ID] = href;
            com.ibm.rdm.web.search.searchutils.triggerEvent(eventType, params);
            return false;
        },
        
		_createUserHover: function(user, node){
			var onProjectMembersComplete = function(userPropertyInfos) {
				for (var userUri in userPropertyInfos){	
					if(user == userPropertyInfos[userUri].name){
						var userInfo = userPropertyInfos[userUri];
						var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : this.projectURI}, null);
						userHover.createHoverPopup(node, userInfo);					
					}
				}
			}
			if(this.projectURI){
				var deferred = ProjectCache.getProjectUsersWithRoles(this.projectURI);
				deferred.addCallback(this, onProjectMembersComplete);
			}	
    	}
    }
);

})();
