/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007,2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.search.results.UserDashboardCommentEntry");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rdm.web.commenting.utils");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.requireLocalization("com.ibm.rdm.web.search.results", "UserDashboardCommentEntry");
dojo.require("dojo.i18n");

var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var CommentingUtils = com.ibm.rdm.web.commenting.utils;

dojo.declare(
    "com.ibm.rdm.web.search.results.UserDashboardCommentEntry",
    [dijit._Widget, dijit._Templated],
    {
        templatePath: dojo.moduleUrl("com.ibm.rdm.web", "search/results/templates/UserDashboardCommentEntry.html"),
        widgetsInTemplate: true,

        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points

        _userDBCommentDiv: null,
        _commentBylineDiv: null,
        _commentSubjectDiv: null,
        _commentTextDiv: null,
        _commentButtonDiv: null,
        _commentMoreLink: null,
        _commentArtifactInfoDiv: null,

        _isTextTruncated: null,
        _messages: null,
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Public fields

        comment: null,
        artifactName: null,
        artifactContentType: null,
        isExpanded: false,

        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields


        ///////////////////////////////////////////////////////////////////////////////////
        // API

        postCreate: function() {
    		this._createCommentEntry();
        },
        
   		postMixInProperties: function() {
            this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.search.results", "UserDashboardCommentEntry");
   		},
   		
        _createCommentEntry: function() {
            this._commentBylineDiv.appendChild(com.ibm.rdm.web.commenting.utils.createCommentByline(this.comment));
            if (!this._isExpanded) {
            	this._collapseComment();
            } 
            CommentingUtils.createTitleNode(this._commentSubjectDiv, this.comment);
        	this._isTextTruncated = CommentingUtils.setCommentText(this._commentTextDiv, this.comment);
			com.ibm.rdm.web.commenting.utils.createArtifactNode(this._commentArtifactInfoDiv, this.artifactName, this.artifactContentType, this.comment, true);
        },        
        
        addToolTip: function(){
       		if (this._isTextTruncated){
				com.ibm.rdm.web.commenting.utils.hookupRecentCommentMoreLink(this._commentMoreLink, this.comment, this.artifactName, this.artifactContentType);
			} else {
       			this._commentMoreLink.innerHTML = "";
       			dojo.addClass(this._commentMoreDiv, "comment-no-more-div");
			};
        },
	
		expandCollapse: function() {
			if (this._isExpanded) {
				this._isExpanded = false;
				this._collapseComment();
			} else {
				this._isExpanded = true;
				this._expandComment();
			}
		},
		
        _collapseComment: function() {
            dojo.removeClass(this._commentTextDiv, "showElement");
            dojo.removeClass(this._commentSubjectDiv, "showElement");
            dojo.removeClass(this._commentButtonDiv, "showElement");
            dojo.removeClass(this._commentMoreDiv, "showElement");
            dojo.addClass(this._commentTextDiv, "hideElement");
            dojo.addClass(this._commentSubjectDiv, "hideElement");
            dojo.addClass(this._commentButtonDiv, "hideElement");
            dojo.addClass(this._commentMoreDiv, "hideElement");
            dojo.removeClass(this._commentArtifactInfoDiv, "expanded");
            dojo.addClass(this._commentArtifactInfoDiv, "collapsed");
        },

        _expandComment: function() {
            dojo.removeClass(this._commentTextDiv, "hideElement");
            dojo.removeClass(this._commentSubjectDiv, "hideElement");
            dojo.removeClass(this._commentButtonDiv, "hideElement");
            dojo.removeClass(this._commentMoreDiv, "hideElement");
            dojo.addClass(this._commentTextDiv, "showElement");
            dojo.addClass(this._commentSubjectDiv, "showElement");
            dojo.addClass(this._commentButtonDiv, "showElement");
            dojo.addClass(this._commentMoreDiv, "showElement");
            dojo.removeClass(this._commentArtifactInfoDiv, "collapsed");
            dojo.addClass(this._commentArtifactInfoDiv, "expanded");
        },
        
        _handleCommentArtifactLinkClicked: function(event, commentURI) {
        	ResourceLinksHandler.handleCommentArtifactAnchorClicked(event, commentURI);
        },
        
        _replyToClicked: function(event) {
			if (dojo.hasClass(this._replyToAnchor, "enabled")) {
    			event.preventDefault();
				event.stopPropagation();
				com.ibm.rdm.web.commenting.utils.replyToComment(this.comment);
			}	
        }
    }
);

})();
