 /* Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
  *
  * US Government Users Restricted Rights - Use, duplication or disclosure
  * restricted by GSA ADP Schedule Contract with IBM Corp.
  */
(function() {
dojo.provide("com.ibm.rdm.web.server.cache.RDMCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var UpdateContext = com.ibm.rdm.web.common.UpdateContext;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var Headers = com.ibm.rdm.web.server.Headers;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

com.ibm.rdm.web.server.cache.RDMCache = {
    // summary:
    //    This static class

    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
	_deferredsMap: {},
	_etagsMap: {},
	_responsesMap: {},
	_updateContextMap: {},

    ///////////////////////////////////////////////////////////////////////////////////
    // API
	
	createCachedDeferred: function(request, args, callbackParser, cacheType) {	
		var uri = args.url;
		var cacheType = args._cacheType;
		var uriMapKey = args._requestMethod + " " + uri + ((args.postData)?" " + args.postData:"");
		
		if (args._cancelDeferreds) {
			this._cancelDeferredsInCurrentUpdateContext(true);
		}
		
		var newDeferred = new dojo.Deferred();
		var deferred = this._getCachedDeferred(uriMapKey);		
		
		if (!deferred || !this._isValidUpdateContext(deferred) || args._skipCache) {
			ErrorHandler.info("Cache missed: " + args.headers[Headers.CORRELATION_ID] + " " + uriMapKey);
			var etag = this._getCachedDeferredEtag(uriMapKey);
			if (etag) {
				args.headers["If-None-Match"] = etag;
			}
			if (etag && args._cacheType == RDMClient.CACHE_TYPE_HEAD_ETAG) {
				var headDeferred = RDMClient.doHead({uri:uri});
				headDeferred.addBoth(this, function(response){
					var headEtag = XhrUtils.getResponseEtag(response, true);
					if (headEtag == etag) {
						newDeferred.callback(this._getCachedDeferredResponse(uriMapKey));
					}
					else {
						deferred = this._doRequest(request, args, uriMapKey, newDeferred, callbackParser);
					}
				});
			}
			else {			
				deferred = this._doRequest(request, args, uriMapKey, newDeferred, callbackParser);
			}
		}
		else {
			ErrorHandler.info("Cache hit: " + uriMapKey);
			deferred.addBoth(this, function(response){
				var cachedResponse = this._getCachedDeferredResponse(uriMapKey);
				if (cachedResponse) {
					newDeferred.callback(cachedResponse);
				}
				else {
					newDeferred.errback();
				}
				
			});
		}
		return newDeferred;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
    // PRIVATE API
    
	_doRequest: function(request, args, uriMapKey, newDeferred, callbackParser) {
    	var deferred = dojo.hitch(RDMClient, request)(args);
		deferred.addBoth(this, function(response){
			var cachedResponse = response;
			var isError = (response instanceof Error);
			var responseCode = XhrUtils.getResponseCode(response);
			if ((!isError) && responseCode != 410) { // Note: 410 is "Gone"
				if (responseCode != 304) {            // Note: 304 is "Not Modified"
					if (callbackParser) {
						if (dojo.isFunction(callbackParser)) {
							try {
								cachedResponse = dojo.hitch(this, callbackParser)(response);
							}
							catch(parsingError) {
								ErrorHandler.error("There was an error while parsing a response from: " + uriMapKey);
								ErrorHandler.error(parsingError);
							}
						}
						else if (dojo.isObject(callbackParser)) {
							callbackParser.init(response);
							cachedResponse.doc = callbackParser;
						}
						else {
							throw new Error("unknown parser");
						}
					}
					this._setCachedDeferredEtag(uriMapKey, response);
				}
				else if (this._getCachedDeferredResponse(uriMapKey)){
					cachedResponse = this._getCachedDeferredResponse(uriMapKey);
				}
				this._setCachedDeferredResponse(uriMapKey, cachedResponse);
				newDeferred.callback(cachedResponse);
			}
			else {
				ErrorHandler.info("Cache error: " + uriMapKey);
				if (responseCode == 410) { // Note: 410 is "Gone"
					// We take care here to "null out" the previously cached
					// value (if there is one) so that the next request for
					// this same resource will get an errback instead of
					// a callback.
					this._setCachedDeferredResponse(uriMapKey, null);
				}
				newDeferred.errback(cachedResponse);
				this._removeCachedDeferred(uriMapKey);
			}
			return cachedResponse;
		});
		
		deferred._cacheMapKey = uriMapKey;
		
		if (!args._skipCache) {
			this._setCachedDeferred(uriMapKey, deferred);
		}
		this._setUpdateContextDeferred(deferred, args._cancelDeferreds);
		
		return deferred;
	},
	
	_getCachedDeferred: function(uri) {
    	var cachedDeferred = this._deferredsMap[uri];
    	if (!cachedDeferred) {
    		cachedDeferred = null;
    	}
		return cachedDeferred
	},
    _setCachedDeferred: function(uri, deferred) {
    	var deferredUpdateContext = com.ibm.rdm.web.common.UpdateContext.getValue();
    	deferred._updateContext = deferredUpdateContext;
		this._deferredsMap[uri] = deferred;
	},
	_removeCachedDeferred: function(uri) {
		this._deferredsMap[uri] = undefined;
	},
	_setUpdateContextDeferred: function(deferred, isQualifiedForCancellation) {
		var deferredUpdateContext = com.ibm.rdm.web.common.UpdateContext.getValue();
		if (!this._updateContextMap[deferredUpdateContext])
    		this._updateContextMap[deferredUpdateContext] = [];
    	deferred._isQualifiedForCancellation = isQualifiedForCancellation;
		this._updateContextMap[deferredUpdateContext].push(deferred);
	},
	_isValidUpdateContext: function(cachedDeferred) {
		return (cachedDeferred._updateContext == com.ibm.rdm.web.common.UpdateContext.getValue());
	},
	_setCachedDeferredEtag: function(uri, response) {
		var deferred = this._getCachedDeferred(uri);		
		this._etagsMap[uri] = XhrUtils.getResponseEtag(response, true);
	},
	_getCachedDeferredEtag: function(uri) {		
		return this._etagsMap[uri];
	},
	_setCachedDeferredResponse: function(uri, response) {		
		this._responsesMap[uri] = response;
	},
	_getCachedDeferredResponse: function(uri) {		
		return this._responsesMap[uri];
	},
	_cancelDeferredsInCurrentUpdateContext: function(cancelQualifiedDeferredsOnly) {
		cancelQualifiedDeferredsOnly = (cancelQualifiedDeferredsOnly)?true:false;
		var currentUpdateContext = com.ibm.rdm.web.common.UpdateContext.getValue();
		var deferredsArray = this._updateContextMap[currentUpdateContext];
		if (deferredsArray) {
			for (var i=0; i<deferredsArray.length; i++) {
				if (deferredsArray[i].fired == -1) {
					if (cancelQualifiedDeferredsOnly && !deferredsArray[i]._isQualifiedForCancellation)
						continue;
					ErrorHandler.warn("Cancelling deferred: " + deferredsArray[i]._cacheMapKey);
					this._removeCachedDeferred(deferredsArray[i]._cacheMapKey);
					deferredsArray[i].cancel();
				}
			}
			this._updateContextMap[currentUpdateContext] = [];
		}
	}
};

})();
