/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.server.cache.RDMCache");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.util.queryutils");

dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.proxy");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var RDMCache = com.ibm.rdm.web.server.cache.RDMCache;
var Headers = com.ibm.rdm.web.server.Headers;
var JazzClient = jazz.client;


var DEFAULT_REQUEST_TIMEOUT = 120000;

com.ibm.rdm.web.server.client.RDMClient = {
	///////////////////////////////////////////////////////////////////////////////////
    // PUBLIC FIELDS
    HTTP_METHOD_GET: 		"GET",
	HTTP_METHOD_POST: 		"POST",
	HTTP_METHOD_HEAD: 		"HEAD",
	HTTP_METHOD_PUT: 		"PUT",
	HTTP_METHOD_DELETE: 	"DELETE",
	
	CACHE_TYPE_UPDATECONTEXT:	"UpdateContext",	//default cache that only relies on UpdateContext value when no etag found in the response header
	CACHE_TYPE_IFNONEMATCH_ETAG:"IfNoneMatch",		//this is a desired cache is used by default if an etag was found in the response header
	CACHE_TYPE_HEAD_ETAG:		"HeadEtag",			//cache for resources that do not support "IfNoneMatch" header. It will send out a HEAD request to check for etag (should become deprecated going forward
    
    ///////////////////////////////////////////////////////////////////////////////////
    // API
    
    doGet: function(args) {
    	/*args: {
			uri,			String - requested resource uri
			revision,		String - resource revision id
			projectSnapshotURI, 	String - project snapshot (fully qulified URI) (revision required)
			projectURI,		String - fully qualified projct uri or comma separated list of project uris
			xhrArgs,		dojo.__XhrArgs - additional arguments to be overriden
			parser,			Class or Function
			skipCache,		Boolean - flag to skip internal cache
			skipErrors,		Boolean - flag to skip error messages in the error section
			cacheType,		RDMClient.CACHE_TYPE,
			cancelDeferreds	Boolean - flag to cancel all deferreds in the current update context
		 }*/
		// If this request is for the converter webapp, we need to convey two
		// oauth related tokens as query params (so that the converter can then
		// make use of them as cookies to enable authorization in subsequent
		// HttpMethod calls that it will make is servicing our request).
		if (args.uri && args.uri.substring(0, 10) // Note: 10 is length of "/converter"
			=== UriHelper.WEB_APP_CONVERTER
			) {
			var delimiter = (args.uri.lastIndexOf("?") === -1 ? "?" : "&");
			args.uri += delimiter + UriHelper.getOAuthQueryParams();
		}
        var getArgs = this._overrideDefaultArgs(args);
        return this._createXhrDeferred(this.HTTP_METHOD_GET, getArgs, args.parser);
    },
    
    doPost: function(args) {
    	/*args: {
			uri,			String - requested resource uri
			projectURI,		String - fully qualified projct uri or comma separated list of project uris
			postData,		String - post data
			xhrArgs,		dojo.__XhrArgs - additional arguments to be overriden
			parser,			Class or Function
			skipCache,		Boolean - flag to skip internal cache
			skipErrors		Boolean - flag to skip error messages in the error section,
			cancelDeferreds	Boolean - flag to cancel all deferreds in the current update context
		 }*/
    	args.skipCache = (args.skipCache == undefined)?true:args.skipCache;
        var postArgs = this._overrideDefaultArgs(args);
        return this._createXhrDeferred(this.HTTP_METHOD_POST, postArgs, args.parser);
    },
    
    doPut: function(args) {
    	/*args: {
			uri,			String - requested resource uri
			projectURI,		String - fully qualified projct uri or comma separated list of project uris
			putData,		String - put data
			xhrArgs,		dojo.__XhrArgs - additional arguments to be overriden
			parser,			Class or Function
			skipCache,		Boolean - flag to skip internal cache
			skipErrors		Boolean - flag to skip error messages in the error section,
			cancelDeferreds	Boolean - flag to cancel all deferreds in the current update context
		 }*/
    	args.skipCache = (args.skipCache == undefined)?true:args.skipCache;
        var putArgs = this._overrideDefaultArgs(args);
        return this._createXhrDeferred(this.HTTP_METHOD_PUT, putArgs, args.parser);
    },
    
    doDelete: function(args) {
    	/*args: {
			uri,			String - requested resource uri
			projectURI,		String - fully qualified projct uri or comma separated list of project uris
			putData,		String - put data
			xhrArgs,		dojo.__XhrArgs - additional arguments to be overriden
			parser,			Class or Function
			skipCache,		Boolean - flag to skip internal cache
			skipErrors		Boolean - flag to skip error messages in the error section,
			cancelDeferreds	Boolean - flag to cancel all deferreds in the current update context
		 }*/
    	args.skipCache = (args.skipCache == undefined)?true:args.skipCache;
        var deleteArgs = this._overrideDefaultArgs(args);
        return this._createXhrDeferred(this.HTTP_METHOD_DELETE, deleteArgs, args.parser);
    },
    
    doHead: function(args) {
    	/*args: {
			uri,			String - requested resource uri
			projectURI,		String - fully qualified projct uri or comma separated list of project uris
			xhrArgs,		dojo.__XhrArgs - additional arguments to be overriden
			skipCache,		Boolean - flag to skip internal cache
			skipErrors		Boolean - flag to skip error messages in the error section,
			cancelDeferreds	Boolean - flag to cancel all deferreds in the current update context
		 }*/
    	args.skipCache = (args.skipCache == undefined)?true:args.skipCache;
        var headArgs = this._overrideDefaultArgs(args);
        return this._createXhrDeferred(this.HTTP_METHOD_HEAD, headArgs, args.parser);
    },
    
    doMultiGet: function(argsArray) {
    	var multiGetPostData = '<rdf:RDF \n'+
				'xmlns:rrm="http://www.ibm.com/xmlns/rrm/1.0/" \n'+
				'xmlns:rrmMulti="http://com.ibm.rdm/multi-request#" \n'+
				'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"> \n';
    	
    	for (var i=0; i<argsArray.length; i++) {
    		var gerArgs = this._overrideDefaultArgs(argsArray[i]);
    		multiGetPostData += '	<rrmMulti:MultiRequest rdf:about="' + gerArgs.url + '"> \n';
    		for (var headerName in gerArgs.headers) {
    			multiGetPostData += '		<rrmMulti:httpHeader rdf:parseType="Resource"> \n';
    			multiGetPostData += '			<rrmMulti:httpHeaderValue>' + gerArgs.headers[headerName] + '</rrmMulti:httpHeaderValue> \n';
    			multiGetPostData += '			<rrmMulti:httpHeaderName>' + headerName + '</rrmMulti:httpHeaderName> \n';
    			multiGetPostData += '		</rrmMulti:httpHeader> \n';
    		}
    		multiGetPostData += '	<rrmMulti:httpMethod>GET</rrmMulti:httpMethod> \n';
    		multiGetPostData += '</rrmMulti:MultiRequest> \n';
    	}
    	multiGetPostData += '</rdf:RDF>';
    	
		return this.doPost({
			skipCache: false,
    		uri: com.ibm.rdm.web.FRONTING_SERVICE_MULTI_REQUEST,
    		postData: multiGetPostData		
    	});
    },
    
    cancelDeferredsInCurrentUpdateContext: function() {
    	RDMCache._cancelDeferredsInCurrentUpdateContext();
    },
    
    ///////////////////////////////////////////////////////////////////////////////////
    // Private API
    
    _createXhrDeferred: function(requestMethod, args, parser, cacheType) {
		var deferred;
		args._requestMethod = requestMethod;
		switch (requestMethod) {
        	case this.HTTP_METHOD_GET:
        		deferred = RDMCache.createCachedDeferred(JazzClient.xhrGet, args, parser);
        		break;
        	case this.HTTP_METHOD_POST:
        		deferred = RDMCache.createCachedDeferred(JazzClient.rawXhrPost, args, parser);
        		break;
			case this.HTTP_METHOD_HEAD:
				deferred = JazzClient.xhrHead(args);
				break;
			case this.HTTP_METHOD_PUT:
				deferred = JazzClient.rawXhrPut(args);
				break;
			case this.HTTP_METHOD_DELETE:
				deferred = JazzClient.xhrDelete(args);
				break;
		}
		return deferred;
	},
    
    _overrideDefaultArgs: function(args) {
    	//add revision and snapshot query parameters
		if (args.revision) {
			args.uri = args.uri + UriHelper.getResourceRevisionQueryAttribute(args.revision, true);
			if (args.projectSnapshotURI) {
				var rrcBaseline = queryutils.getLastPathSegment(args.projectSnapshotURI);
				args.uri = args.uri + "&rrc:baseline=" + rrcBaseline;
			}
		}
		//add project uri header
		if (args.projectURI) {
			if (!args.xhrArgs) {
				args.xhrArgs = {};
			}
			if (!args.xhrArgs.headers) {
				args.xhrArgs.headers = {};
			}
			args.xhrArgs.headers[Headers.OWNING_CONTEXT] = args.projectURI;
		}
    	var xhrArgs = {
    		url: args.uri,
            handleAs: "xml",
            preventCache: true,
            timeout: DEFAULT_REQUEST_TIMEOUT,
            headers: {
                "Content-Type":	"text/plain",
                "Accept": "none",
                "X-com-ibm-rdm-request-correlation-id": "RRC_WEBUI_" + Math.random()
            },
            load: function(response, ioArgs) {
            	return {
                    doc: response,
                    ioArgs: ioArgs
                };
            },
            error: function(response, ioArgs) {
            	if (response.dojoType && response.dojoType == "cancel") {
            		ErrorHandler.warn("Deferred cancelled: " + args.uri);
            		return response;
            	}
				if (!ioArgs.args._skipErrors) {
					var errorSectionWidget = ErrorHandler.getErrorSectionWidget();
					if (errorSectionWidget)
						errorSectionWidget.showXhrError(response, ioArgs);
					//else
					//	alert((response.message)?response.message:response);
				}
				console.error(response);
            	response.ioArgs = ioArgs;
                return response;
		    }
        };

        for (var argName in args.xhrArgs) {
        	if (argName == "headers") {
        		for (var header in args.xhrArgs[argName]) {
        			xhrArgs[argName][header] = args.xhrArgs[argName][header];
        		}
        	}
        	else {
        		xhrArgs[argName] = args.xhrArgs[argName];
        	}
        }
        if (args.postData) {
        	xhrArgs.postData = args.postData;
        }
        if (args.putData) {
        	xhrArgs.putData = args.putData;
        }
        if (args.putData) {
        	xhrArgs.putData = args.putData;
        }

        xhrArgs._skipErrors = (args.skipErrors)?true:false;
        xhrArgs._skipCache = (args.skipCache)?true:false;
        xhrArgs._cacheType = (args.cacheType)?args.cacheType:null;
        xhrArgs._cancelDeferreds = (args.cancelDeferreds)?true:false;
        
         
        return xhrArgs;
    }
};

})();
