/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.storyboard.FrameListView");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojox.collections.Dictionary");
dojo.requireLocalization("com.ibm.rdm.web.storyboard", "FrameListView");


var domutils = com.ibm.rdm.web.util.domutils;


dojo.declare(
	"com.ibm.rdm.web.storyboard.FrameListView",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "storyboard/templates/FrameListView.html"),
	widgetsInTemplate: false,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points

	_contentPaneDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

    storyboardViewer: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_contentPane: null,
	_frameList: null,
	_frameInfos: null,
	_connectHandles: null,
    _idNodesMap: null,

	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.storyboard", "FrameListView");
		this._connectHandles = null;
		this._frameList = document.createElement("ol");
		this._contentPane = new dijit.layout.ContentPane({}, this._contentPaneDiv);
		this._contentPane.attr('content', this._frameList);
		this._frameInfos = null;
		this._idNodesMap = new dojox.collections.Dictionary();
	},

	startup: function() {
		this.inherited("startup", arguments);
	},
	
	destroy: function() {
		this._clearConnectHandles();
		this.inherited("destroy", arguments);
	},

	setFrameInfos: function(frameInfos, isAtLeastOneFrameBasedOnSketchInDifferentRepo) {
		this._contentPane.attr('content', this._frameList);
		this._clearExistingContentsIfNeeded();
		this._populateContents(
			frameInfos, isAtLeastOneFrameBasedOnSketchInDifferentRepo);
		this._frameInfos = frameInfos;
	},

	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.
        this._contentPane.attr('content', htmlContent);
	},

	updateInheritedFromArtifactNames: function(mapResourceURIToArtifactName) {
		for (var i = 0; i < this._frameInfos.length; i++) {
			var frameInfo = this._frameInfos[i];
			if (frameInfo.pathlessMasterResourceURI) {
				var resourceURI = this.storyboardViewer.pathlessResourceURIToFullResourceURI(
					frameInfo.pathlessMasterResourceURI
				);
				var artifactName = mapResourceURIToArtifactName[resourceURI];
				this._updateListItem(frameInfo, artifactName);
			}
		}
	},

	resize: function(args) {
		// summary:
		//    This is based on the version of 'resize' found in _LayoutWidget.js.
		this._contentPane.resize(args);
	},
    
    getRootArtifactNode: function() {
        var r = this._contentPaneDiv;
        return r;
    },
	
	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_clearConnectHandles: function() {
		if (this._connectHandles) {
			dojo.forEach(this._connectHandles, function(handle) {
				dojo.disconnect(handle);
			});
			delete this._connectHandles;
			this._connectHandles = null;
		}
	},

	_clearExistingContentsIfNeeded: function() {
		this._clearConnectHandles();
		if (this._frameList.firstChild != null) {
			com.ibm.rdm.web.util.domutils.removeChildNodes(this._frameList);
		}
	},

	_populateContents: function(frameInfos, isAtLeastOneFrameBasedOnSketchInDifferentRepo) {
		this._connectHandles = [];
		for (var i = 0; i < frameInfos.length; i++) {
			var frameInfo = frameInfos[i];
			var masterResourceURI = null;
			if (frameInfo.pathlessMasterResourceURI) {
				masterResourceURI = this.storyboardViewer.pathlessResourceURIToFullResourceURI(
					frameInfo.pathlessMasterResourceURI
				);
			}
			var isBasedOnSketchFromExternalRepo = frameInfo.externalMasterResourceURI;
			this._addListItem(
				frameInfo,
				masterResourceURI,
				isBasedOnSketchFromExternalRepo,
				isAtLeastOneFrameBasedOnSketchInDifferentRepo);
		}
	},
	
	getNode: function(frameLocationId) {
		return this._idNodesMap.item(frameLocationId);
	},
	
	getFrameNameForLocation: function(frameLocationId) {
		var node = this._idNodesMap.item(frameLocationId);
		if (!node) return null;
		var frameNodes = node.parentNode.childNodes;
		var found = false;
		var i = 0;
		for (i; i<frameNodes.length; i++) {
			if (frameNodes[i] === node) {
				found = true;
				break;
			}
		}
		if (!found) return null;
		return this._frameInfos[i].frameName;
	},
	
	getFrameNameForIndex: function(index) {
		return this._frameInfos[index].frameName;
	},
	
	getFrameLocationIdForIndex: function(index) {
		return this._frameInfos[index].key;
	},
	
	getFrameNodes: function() {
		return this._idNodesMap.getValueList();
	},
	
	getNoFrameTitleDisplayName: function() {
		return "&lt;" + this._messages["noFrameTitleSpecified"] + "&gt;";
	},
	
	_addListItem: function(frameInfo, masterResourceURI,
						   isBasedOnSketchFromExternalRepo, isAtLeastOneFrameBasedOnSketchInDifferentRepo) {
		var frameIndex = frameInfo.frameIndex;
		var li = document.createElement("li");
		var isFrameTitleMissing = ((!frameInfo.frameName) || frameInfo.frameName === "");
		var frameName = (
			isFrameTitleMissing ?
			this.getNoFrameTitleDisplayName() :
			frameInfo.frameName
		);
		var frameTitleHtmlContent = (
			isAtLeastOneFrameBasedOnSketchInDifferentRepo ?
			(
				"<span class=\""
			    + (isFrameTitleMissing ?
				    "storyboard-frame-link-that-is-not-navigable-with-missing-title" :
				 	"storyboard-frame-link-that-is-not-navigable-with-title-specified")
			    + "\">" + frameName + "</span>"
			) :
			(
				"<a class=\""
				+ (isFrameTitleMissing ?
				   "storyboard-frame-link-with-missing-title" :
				   "storyboard-frame-link-with-title-specified")
				+ "\">" + frameName + "</a>"
			 )
		);
		if (masterResourceURI) {
			li.innerHTML = (
				"<a class=\"storyboard-frame-inherits-from-link\" target=\"_blank\">...</a>"
				+ frameTitleHtmlContent
			);
		} else if (isBasedOnSketchFromExternalRepo) {
			li.innerHTML = (
				"<span class=\"storyboard-frame-based-on-external-sketch\">"
				+ "&lt;" + this._messages["externalMasterLink"] + "&gt;"
				+ "</span>"
				+ frameTitleHtmlContent
			);
		} else {
			li.innerHTML = frameTitleHtmlContent;
		}
		var self = this;
		if (! isAtLeastOneFrameBasedOnSketchInDifferentRepo) {
			var frameAnchor = li.lastChild;
			this._connectHandles.push(dojo.connect(frameAnchor, "onclick", function(event) {
				dojo.stopEvent(event);
		    	self._selectListItem(frameIndex);
			}));
		}
		if (li.childNodes.length == 2 && !isBasedOnSketchFromExternalRepo) {
			var inheritsFromAnchor = li.firstChild;
			this._connectHandles.push(dojo.connect(inheritsFromAnchor, "onclick", function(event) {
				dojo.stopEvent(event);
		    	self.storyboardViewer.openInheritsFromArtifact(masterResourceURI);
			}));
		}
    	dojo.addClass(li, "frameListItem");
    	li.id = frameInfo.key;
        this._idNodesMap.add(frameInfo.key, li);
		this._frameList.appendChild(li);
	},

	_updateListItem: function(frameInfo, inheritedFromArtifactName) {
		var li = this._frameList.childNodes[frameInfo.frameIndex];
		var inheritsFromAnchor = li.firstChild;
		inheritsFromAnchor.removeChild(inheritsFromAnchor.firstChild);
		inheritsFromAnchor.appendChild(document.createTextNode(inheritedFromArtifactName));
	},

	_selectListItem: function(frameIndex) {
		this.storyboardViewer.selectFrame(frameIndex);
	}
});

})();
