/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.storyboard.StoryboardViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.commenting.ResourceViewerWidget");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.storyboard.FrameListView");
dojo.require("com.ibm.rdm.web.storyboard.StoryboardParser");
dojo.require("com.ibm.rdm.web.storyboard.ThumbnailStackView");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.storyboard", "StoryboardViewer");


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var StoryboardParser = com.ibm.rdm.web.storyboard.StoryboardParser;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var domutils = com.ibm.rdm.web.util.domutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;

var THUMBNAIL_STACK_HEIGHT = 100;


dojo.declare(
	"com.ibm.rdm.web.storyboard.StoryboardViewer",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a viewer which displays Storyboard artifacts in a similar
	//    manner to the Storyboard editor found in the native RCP client.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "storyboard/templates/StoryboardViewer.html"),
	widgetsInTemplate: false,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_frameViewerDiv: null,
	_thumbnailStackDiv: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceURI: null,
	resourceViewer: null,
	htmlContentToUseWhenLoading: null,
	// onFrameSelected: Callback
	//    Every time a new frame is selected, this callback will be invoked with
	//    a single param, the new "frame title" to use within the calling class.
	onFrameSelected: null,

	STORY_NAMESPACE: "http://jazz.net/xmlns/alm/rm/Storyboard/v0.1",
	STORY_QUALIFIER: "story",

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_stackContainer: null,
	_resourceViewerWidget: null,
	_thumbnailStackView: null,
	_revision: null,
	_frameInfos: null,
	_mapResourceURIToArtifactName: null,
	_currentlyDisplayedFrameIndex: null,
	_selectedNode: null,
	_subs: [],
	_connectHandles: null,
	_documentHasBeenLoaded: null,
	_reviewParser: null,
		
	_STORYBOARD_CONTENT_TYPE: com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD.replace("+", "%2b"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited("postCreate", arguments);
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.storyboard", "StoryboardViewer");
		this._initSubscriptions();
		this._connectHandles = new Array();
		this._documentHasBeenLoaded = false;
		var self = this;
		this._stackContainer = new dijit.layout.StackContainer(
			{},
			this._frameViewerDiv
		);
		this._frameListView = new com.ibm.rdm.web.storyboard.FrameListView(
			{
				storyboardViewer: this
			},
			null
		);
		this._resourceViewerWidget = new com.ibm.rdm.web.commenting.ResourceViewerWidget(
			{
				resourceURI: this.resourceURI,
				resourceViewer: this.resourceViewer
			},
			null
		);
		this._stackContainer.addChild(this._frameListView);
		this._stackContainer.addChild(this._resourceViewerWidget);
		this._thumbnailStackView = new com.ibm.rdm.web.storyboard.ThumbnailStackView(
			{
				resourceURI: this.resourceURI,
				onThumbnailClicked: function(frameIndex) {
					self._viewFrame(frameIndex);
				},
				onLoadThumbnail: function(frameIndex, thumbnailWidth, thumbnailHeight, onInstallThumbnail) {
					self._loadThumbnail(frameIndex, thumbnailWidth, thumbnailHeight, onInstallThumbnail);
				}
			},
			this._thumbnailStackDiv
		);
		this._currentlyDisplayedFrameIndex = null;
	},

	startup: function() {
		this.inherited("startup", arguments);
 		this._stackContainer.startup();
	},
	
	destroy: function() {
		for (var i = 0; i < this._subs.length; i++) {
			dojo.unsubscribe(this._subs[i]);
			this._subs[i] = null;
		}
		this.inherited("destroy", arguments);
	},
	
	getElementDisplayName: function(elementId) {
		var displayName = "";
		var name = this._frameListView.getFrameNameForLocation(elementId);
		if (name) {
			displayName = name;
		} else {
			var locationResource = this._resourceViewerWidget.idNodesMap[elementId];
			if (locationResource) {
				displayName = locationResource.getAttribute("controlLabel");
				if (displayName) {
					displayName = com.ibm.rdm.web.util.queryutils.decode(displayName);
				}
				if (!displayName || displayName.length === 0) {
					var controlType = locationResource.getAttribute("controlType");
					if (controlType) {
						displayName = com.ibm.rdm.web.common.controlTypes.getControlTypeDisplayName(controlType);
					}
				}
			}
		}
		return displayName;
	},
	
	loadResource: function(contentType, revision) {
		var deferred = RDMClient.doGet({
				uri: this.resourceURI,
				revision: revision,
				projectSnapshotURI: this.resourceViewer.projectSnapshotURI
			});
		deferred.addCallback(this, this._handleLoadResponse);
		return deferred;
	},
	
	subscribe: function(topic, method) {
		var topic = EventTopics.getTopic(topic, this.resourceURI);
		ErrorHandler.debug("ResourceViewerWidget: subscribe to event -> " + topic);
		this._subscriptions.push(
				dojo.subscribe(
						topic, 
						this, 
						method));
	},
	
	publish: function(topic, params) {
		var topic = EventTopics.getTopic(topic, this.resourceURI);
		ErrorHandler.debug("StoryboardViewer: fire event -> " + topic);
		dojo.publish(
				topic, 
				params);
	},

	isFrameDisplayed: function() {
		// Return whether or not a frame vs. the framelist is being displayed in the
		// viewer.
		return this._currentlyDisplayedFrameIndex != null;
	},

	getActiveFrameName: function() {
		// Return the name of the frame that is being displayed.
		if (!this.isFrameDisplayed) return null;
		return this._frameListView.getFrameNameForIndex(this._currentlyDisplayedFrameIndex);
	},

	getActiveFrameLocationId: function() {
		// Return the element id for the frame that is being displayed.
		if (!this.isFrameDisplayed) return null;
		return this._frameListView.getFrameLocationIdForIndex(this._currentlyDisplayedFrameIndex);
	},
	
	select: function(node) {
		if (this.isSelected(node))
			return;
		dojo.addClass(node, "frameSelected");
		this.unselect(this._selectedNode);
		this._selectedNode = node;        
	},

	unselect: function(node) {
		if (!this.isSelected(node))
			return;
		dojo.removeClass(node, "frameSelected");
		this._selectedNode = null;
	},

	hoverOver: function(node /*Node*/) {    
		if (node == this._selectedNode) return;    
		dojo.addClass(node, "commentHover");
	},

	hoverOut: function(node /*Node*/) {
		dojo.removeClass(node, "commentHover");
	},

	isSelected: function(node /*Node */) {
		return node && this._selectedNode && this._selectedNode == node;
	},
		
	setDimensions: function(width, height) {
		var stackContainerHeight = height - THUMBNAIL_STACK_HEIGHT;
		this.domNode.style.height = height + "px";
		this.domNode.style.width = width + "px";
		this._stackContainer.resize({
			w: width,
			h: stackContainerHeight
		});
		// TODO Consider changing SidebarLayout's API to use resize instead of
		// setDimensions ... as this would allow a uniform resize API for
		// classes like ResourceViewerWidget that are housed in a StackContainer
		// as well as a SidebarLayout.
		this._resourceViewerWidget.setDimensions(width, stackContainerHeight);
		// Resize the thumbnail stack viewer.
		this._thumbnailStackView.domNode.style.height = THUMBNAIL_STACK_HEIGHT + "px";
		this._thumbnailStackView.setDimensions(width, THUMBNAIL_STACK_HEIGHT);
	},

	showFrameList: function() {
		this._viewFrameList();
	},

	selectFrame: function(frameIndex) {
		if (this._selectedNode) {
			this.unselect(this._selectedNode);
		}
		this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [null, null]);
		this._viewFrame(frameIndex, true);
	},

	openInheritsFromArtifact: function(masterResourceURI) {
		viewerutils.openResourceUsingAppropriateViewer(
			masterResourceURI,
			this.resourceViewer.reviewId,
			this.resourceViewer.projectSnapshotURI
		);
	},

	setHtmlContent: function(htmlContent) {
		// summary:
		//    Reset the displayed HTML content to be the passed htmlContent.
		//    This method is normally called during initial opening or
		//    subsequent refreshing of the artifact under view.
		var currentlyDisplayedWidget = this._stackContainer.selectedChildWidget;
		if (currentlyDisplayedWidget) {
			var setHtmlContentFunction = currentlyDisplayedWidget.setHtmlContent;
			if (setHtmlContentFunction) {
				setHtmlContentFunction.call(
					currentlyDisplayedWidget, htmlContent
				);
			}
		}
	},

	// TODO This is needed as ResourceViewer calls this function ... but we should
	// probably refactor ResourceViewer to NOT always make this call.
	highlightComments: function() {
	},

	pathlessResourceURIToFullResourceURI: function(pathlessResourceURI) {
		var lastSlashIndex = this.resourceURI.lastIndexOf("/");
		return (
			this.resourceURI.substring(0, lastSlashIndex+1)
			+ pathlessResourceURI
		);
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_initSubscriptions: function() {
		this._subscriptions = [];
		this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_SELECTED_TOPIC, "_onLocationPaneSelected");
//    	this.subscribe(EventTopics.LOCATE_COMMENTED_LOCATIONS_IN_RESOURCE, "_onHighlightComments");
		this.subscribe(EventTopics.RESOURCE_LOCATION_PANE_HOVER_TOPIC, "_onLocationPaneHover");
	},
	
	_getRootArtifactNode: function() {
		var r = this._frameListView.getRootArtifactNode();
		return r;
	},
	
	_onLocationPaneHover: function(resourceLocationId, isHoverOver) {
		var node = null;
		if (resourceLocationId=="")
			node = this._getRootArtifactNode();
		else 
			node = this._frameListView.getNode(resourceLocationId);
		
		if (node) {
			if (isHoverOver)
				this.hoverOver(node);
			 else 
				this.hoverOut(node);
		}
	},

	_onLocationPaneSelected: function(resourceLocationId, resourceUri) {
		var node = null;
		if (resourceLocationId=="")
			node = this._getRootArtifactNode();
		else 
			node = this._frameListView.getNode(resourceLocationId);

		if (node) {
			this.select(node);
			com.ibm.rdm.web.util.autoscroller.scrollToNode(node, this._getRootArtifactNode());
		}
		else {
			this.unselect(this._selectedNode);
		}
	},
	
	_onElementClicked: function(event, /*Optional*/targetElement) {
		// summary:
		//    Toggle selection and publish a selection event.
		// event:
		//    A DOM click event.
		// targetElement:
		//    If passed, this element will be used for selection rather than
		//    the element found in event.currentTarget.
		dojo.stopEvent(event);
		var node = (targetElement || event.currentTarget);
		var selectedNodeId = null;
		var unselectedNodeId = null;
		if (this._selectedNode) {
			unselectedNodeId = this._selectedNode.id;
		}
		this.select(node);
		if (this._selectedNode) {
			selectedNodeId = this._selectedNode.id;
		}
		this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [selectedNodeId, unselectedNodeId]);
	},

	_viewFrame: function(frameIndex, /*Optional*/ doSelectThumbnail) {
		if (this._selectedNode) {
			this.unselect(this._selectedNode);
		}
		this.publish(EventTopics.COMMENT_SELECTED_TOPIC, [null, null]);
		this._showResourceViewer();
		var frameName = (this._frameInfos[frameIndex].frameName || this._frameListView.getNoFrameTitleDisplayName());
		var frameDescription = (this._frameInfos[frameIndex].frameDescription || "");
		var indexString = (frameIndex + 1).toString();
		var frameTitleToUse = dojo.string.substitute(this._messages.frameNumAndTitle, [indexString, frameName]);
		var pathlessMasterResourceURI = this._frameInfos[frameIndex].pathlessMasterResourceURI;
		var inheritsFromResourceURI = (
			pathlessMasterResourceURI ?
			this.pathlessResourceURIToFullResourceURI(pathlessMasterResourceURI) :
			null);
		var inheritsFromDisplayName = (
			inheritsFromResourceURI ?
			this._mapResourceURIToArtifactName[inheritsFromResourceURI] :
			null);
		this.onFrameSelected(
			frameTitleToUse, frameDescription,
			inheritsFromResourceURI, inheritsFromDisplayName);
		this._resourceViewerWidget.setHtmlContent(this.htmlContentToUseWhenLoading);
		this._resourceViewerWidget.loadResource(
			com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD,
			this._revision,
			frameIndex
		);
		if (doSelectThumbnail) {
			this._thumbnailStackView.selectThumbnail(frameIndex);
		}
		this._currentlyDisplayedFrameIndex = frameIndex;
	},

	_viewFrameList: function() {
		this._showFrameListView();
		this._thumbnailStackView.clearSelection();
		this._currentlyDisplayedFrameIndex = null;
	},

	_showResourceViewer: function() {
		if (this._stackContainer.selectedChildWidget !== this._resourceViewerWidget) {
			this._stackContainer.selectChild(this._resourceViewerWidget);
		}
	},

	_showFrameListView: function() {
		if (this._stackContainer.selectedChildWidget !== this._frameListView) {
			this._stackContainer.selectChild(this._frameListView);
		}
	},

	_clearLastResultsIfNeeded: function() {
		this._selectedNode = null;
		for (var i = 0; i < this._connectHandles.length; i++) {
			this.disconnect(this._connectHandles[i]);
		}
		this._connectHandles = new Array();
	},

	_handleLoadResponse: function(response) {
		this._clearLastResultsIfNeeded();
		var self = this;
		var getFrameIndexToDisplay = function() {
			var indexToDisplay = null;
			var currentlyDisplayedFrameKey = (
				self._currentlyDisplayedFrameIndex != null ?
				self._frameInfos[self._currentlyDisplayedFrameIndex].key :
				null
			);
			if (currentlyDisplayedFrameKey) {
				for (var i = 0; i < frameInfos.length; i++) {
					var frameInfo = frameInfos[i];
					if (frameInfo.key == currentlyDisplayedFrameKey) {
						indexToDisplay = frameInfo.frameIndex;
						break;
					}
				}
			}
			return indexToDisplay;
		}
		var isAtLeastOneFrameBasedOnSketchInDifferentRepo = function(frameInfos) {
			for (var i = 0; i < frameInfos.length; i++) {
				if (frameInfos[i].externalMasterResourceURI) {
					return true;
				}
			}
			return false;
		};
		var doc = response.doc;
		this._revision = XhrUtils.getResponseEtag(response);
		var storyboardParser = new StoryboardParser({
			doc: doc
		});
		var frameInfos = storyboardParser.getFrameInfos(doc);
		var frameIndexToDisplay = getFrameIndexToDisplay();
		this._frameInfos = frameInfos;
		if (frameIndexToDisplay != null) {
			this._viewFrame(frameIndexToDisplay, false);
		} else {
			this._viewFrameList();
		}
		var isAtLeastOneFrameBasedOnSketchInDifferentRepo =
			isAtLeastOneFrameBasedOnSketchInDifferentRepo(frameInfos);
		this._frameListView.setFrameInfos(
			frameInfos, isAtLeastOneFrameBasedOnSketchInDifferentRepo);
		if (isAtLeastOneFrameBasedOnSketchInDifferentRepo) {
			var htmlContent = (
				"<div class=\"unable-to-load-thumbnails-div\">"
				+ this._messages["unableToLoadFramesDueToMoreThanOneRepo"]
				+ "</div>"
			);
			this._thumbnailStackView.setHtmlContent(htmlContent);
		} else {
			this._loadThumbnailStackViewWithThumbnailContent(frameInfos);
		}
		this._updateArtifactNamesMap(frameInfos);
		var frameNodes = this._frameListView.getFrameNodes();
		for (var i = 0; i < frameNodes.length; i++) {
			// add the event handler to the node
			self._connectHandles.push(
				self.connect(frameNodes[i], "onclick", self._onElementClicked));
		}
		this._resourceViewerWidget._documentHasBeenLoaded = true;
		this._documentHasBeenLoaded = true;
		return response;
	},

	_loadThumbnailStackViewWithThumbnailContent: function(frameInfos) {
		// summary:
		//    Load the thumbnail images found in the timeline view at the bottom.
		// description:
		//    This function FIRST makes sure that this._reviewParser is set to an
		//    appropriate value BEFORE loading these thumbnails.  The reason for
		//    this is that each thumbnail section will call this class back to
		//    invoke the _loadThumbnail function, and this function requires a
		//    reviewParser if the storyboard viewer is being shown in the case of
		//    a formal review.
		var self = this;
		var loadThumbnails = function() {
			self._thumbnailStackView.setFrameInfos(frameInfos);
		}
		// If it's associated with a formal review, we must set
		// this._reviewParser BEFORE loading the thumbnails.
		if (this.resourceViewer.reviewId && this.resourceViewer.revision) {
			var onReviewLoaded = function(response) {
				reviewParser = response.doc;
				self._reviewParser = reviewParser;
				loadThumbnails();
			}
			var deferred = ReviewCache.getReview(this.resourceViewer.reviewId);
			deferred.addCallback(onReviewLoaded);
		} else {
			loadThumbnails();
		}
	},

	_updateArtifactNamesMap: function(frameInfos) {
		// summary:
		//    Fill this._mapResourceURIToArtifactName with appropriate
		//    content.
		var self = this;
		var mapResourceURIToArtifactName = new Object();
		// Collect all "master" resource URIs into resourceURIs.
		var resourceURIs = [];
		for (var i = 0; i < frameInfos.length; i++) {
			var frameInfo = frameInfos[i];
			var pathlessMasterResourceURI = frameInfo.pathlessMasterResourceURI;
			if (pathlessMasterResourceURI != null) {
				resourceURIs.push(
					this.pathlessResourceURIToFullResourceURI(pathlessMasterResourceURI));
			}
		}
		// Note: We only attempt to get the HEAD version of the "inherited from"
		// artifact names (regardless of whether this storyboard is being viewed
		// from Open History).  This is the way the native client does it and
		// they've informed us that it's by design (from UX team).
		var updateArtifactNames = function(resourceURIs) {
			if (resourceURIs.length > 0) {
				var deferred = ProjectCache.getBatchResourceDescriptionsMultiFetch(resourceURIs);
				deferred.addCallback(function(resourceDescriptions) {
					for (var i = 0; i < resourceURIs.length; i++) {
						var resourceURI = resourceURIs[i];
						mapResourceURIToArtifactName[resourceURI]
							= resourceDescriptions[resourceURI].name;
					}
					self._frameListView.updateInheritedFromArtifactNames(
						mapResourceURIToArtifactName);
					return resourceDescriptions;
				});
			}
		};
		this._mapResourceURIToArtifactName = mapResourceURIToArtifactName;
		// Get the artifact names for these resources and inform the frame list
		// about these names.
		updateArtifactNames(resourceURIs);
	},

	_loadThumbnail: function(storyboardFrameIndex, thumbnailWidth, thumbnailHeight, onInstallThumbnail) {
		var onGetSuccess = function(response) {
			var htmlContent = response.ioArgs.xhr.responseText;
			var imageSrc = viewerutils.getUrlToUseToForBackgroundImage(htmlContent);
			onInstallThumbnail(imageSrc, storyboardFrameIndex);
			return response;
		};
		var onGetError = function(response) {
			onInstallThumbnail(null, storyboardFrameIndex, response);
			return response;
		};
		var urlToUse = (
			UriHelper.getGefDiagramConversionUri()
			+ "?resourceURL=" + encodeURIComponent(this.resourceURI) + ".html"
			+ "&width=" + thumbnailWidth
			+ "&height=" + thumbnailHeight
			+ "&doSuppressCommentingDivCreation=true"
			+ "&contentType=" + this._STORYBOARD_CONTENT_TYPE
			+ "&revision=" + this._revision
			+ "&storyboardFrameIndex=" + storyboardFrameIndex
			+ UriHelper.getReviewQueryParams(this._reviewParser)
			+ UriHelper.getBaselineQueryParams(this.resourceViewer.projectSnapshotURI)
		);
		var deferred = RDMClient.doGet({
			uri: urlToUse,
			skipErrors: true
		});
		deferred.addCallback(onGetSuccess);
		deferred.addErrback(onGetError);
	}
});


})();
