/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2008, 2009.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
dojo.provide("com.ibm.rdm.web.storyboard.ThumbnailStackView");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.layout.ContentPane");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");


var UriHelper = com.ibm.rdm.web.common.UriHelper;


dojo.declare(
	"com.ibm.rdm.web.storyboard.ThumbnailStackView",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Shows an animated Storyboard view similar to the "TimelineViewer" found
	//    in the native RCP client.  We take care in this class to avoid making
	//    direct calls to the viewer specific classes (like StoryboardViewer) as
	//    we might like to make use of this class in the future for other viewer-
	//    related classes.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "storyboard/templates/ThumbnailStackView.html"),

	///////////////////////////////////////////////////////////////////////////
	// Attach points

 	_contentPane: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	resourceURI: null,
	onThumbnailClicked: null,
	onLoadThumbnail: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	// _frameInfos: Array
	//    See getFrameInfos in StoryboardParser for a description of the
	//    property lists found in this array.
	//    Note: We add a "loaded" property to each of these property lists
	//    whenever a request for the associated thumbnail image has been
	//    performed.
	_frameInfos: null,
	// _frameTable: an HTML table element
	//    The cells in this table represent thumbnails.
	//    Note: this table will ONLY be displayed when the _messageDiv
	//    field is set to hidden.
	_frameTable: null,
	// _messageDiv: a div element
	//    This div element is used to present a "message" when conditions
	//    warrant.
	//    Note: this div element will ONLY be displayed when the _frameTable
	//    field is set to hidden.
	_messageDiv: null,
	// _lastSelectedFrameIndex: Integer or null
	//    Whenever selection changes, this field holds the index of the newly
	//    selected index.  If selection is removed, it's reset to null.
	_lastSelectedFrameIndex: null,
	_connectHandles: null,
	_leftEndpointScroller: null,
	_rightEndpointScroller: null,
	
	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
		var rootDiv = document.createElement("div");
		rootDiv.className = "root-div";
		this._contentPane = this._contentPaneDiv;
		this._contentPane.domNode = this._contentPane;
		this._contentPane.domNode.appendChild(rootDiv);
		// Add a thumbnail frame table.
		this._frameTable = document.createElement("table");
		rootDiv.appendChild(this._frameTable);
		// Add a div in which we'll add a message when thumbnails are not being
		// displayed.
		this._messageDiv = document.createElement("div");
		this._messageDiv.style.display = "none";
		rootDiv.appendChild(this._messageDiv);
		// Other initializations ...
		this._connectHandles = null;
		this._leftEndpointScroller = new com.ibm.rdm.web.storyboard._EndpointScroller({
			thumbnailStackView: this,
			endpointType: ENDPOINT_TYPE.LEFT_SIDE
		});
		this._rightEndpointScroller = new com.ibm.rdm.web.storyboard._EndpointScroller({
			thumbnailStackView: this,
			endpointType: ENDPOINT_TYPE.RIGHT_SIDE
		});
		this.domNode.appendChild(this._leftEndpointScroller.domNode);
		this.domNode.appendChild(this._rightEndpointScroller.domNode);
		this._leftEndpointScroller.startup();
		this._rightEndpointScroller.startup();
		this._lastSelectedFrameIndex = null;
	},

	startup: function() {
		this.inherited(arguments);
	},
	
	destroy: function() {
		this._clearConnectHandles();
		if (this._leftEndpointScroller) {
			this._leftEndpointScroller.destroyRecursive();
		}
		if (this._rightEndpointScroller) {
			this._rightEndpointScroller.destroyRecursive();
		}
		this.inherited(arguments);
	},

	setDimensions: function(width, height) {
		this.domNode.style.width = width + "px";
		this.domNode.style.height = height + "px";
		if (this._frameInfos) {
			this._installAnyMissingThumbnailsWithinViewportRegion();
			this._scrollViewportToTheRightIfNeeded();
		}
		if (this._rightEndpointScroller) {
			this._rightEndpointScroller.thumbnailStackDimensionsHaveChanged();
		}
	},

	setFrameInfos: function(frameInfos) {
		var frameKeyToSelect = null;
		if (this._lastSelectedFrameIndex != null) {
			frameKeyToSelect = this._frameInfos[this._lastSelectedFrameIndex].key;
		}
		this._frameInfos = frameInfos;
		this._clearSelection();
		this._clearExistingContentsIfNeeded();
		this._showThumbnails(frameKeyToSelect);
	},

	setHtmlContent: function(htmlContent) {
		this._showMessage(htmlContent);
	},

	scrollForward: function() {
 		this._scrollOnePage(true);
	},

	scrollBackward: function() {
		this._scrollOnePage(false);
	},

	isShowingThumbnails: function() {
		return this._frameTable.style.display != "none";
	},

	isViewportAtStart: function() {
		return (
			this._contentPane.domNode.scrollLeft === 0
		);
	},

	isViewportAtEnd: function() {
		return (
			this._frameTable.firstChild === null
			||
			(
			 	this._frameTable.style.display != "none"
				&& (
					this._contentPane.domNode.scrollLeft
					=== (this._contentPane.domNode.scrollWidth
						- this._contentPane.domNode.clientWidth)
				)
			)
		);
	},

	selectThumbnail: function(frameIndex) {
		var thumbnailLeft = THUMBNAIL_CELL_WIDTH * frameIndex;
		var thumbnailRight = THUMBNAIL_CELL_WIDTH * (frameIndex + 1);
		var viewportLeft = this._contentPane.domNode.scrollLeft;
		var viewportRight = viewportLeft + this._contentPane.domNode.clientWidth;
		var scrollPixelDelta = 0;
		this._selectThumbnail(frameIndex);
		if (thumbnailLeft < viewportLeft) {
			// scroll left
			this._scroll(thumbnailLeft - viewportLeft - ENDPOINT_SCROLLER_WIDTH);
		} else if (thumbnailRight > viewportRight) {
			// scroll right
			this._scroll(thumbnailRight - viewportRight + ENDPOINT_SCROLLER_WIDTH);
		}
	},

	clearSelection: function() {
		this._clearSelection();
	},

	///////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_scrollViewportToTheRightIfNeeded: function() {
		// summary:
		//    Make sure that "empty space" does not appear to the right after
		//    this widget's width has been changed.
		// description
		//    This widget's width has just changed.  If it's now wider than it
		//    used to be, we might need to move the viewport to the left to
		//    avoid showing empty space on the right.  This function takes
		//    care of moving the viewport appropriately if this situation has
		//    occurred.
		var pageWidth = this._contentPane.domNode.clientWidth;
		if ((this._contentPane.domNode.scrollLeft + this._contentPane.domNode.clientWidth)
			> this._contentPane.domNode.scrollWidth) {
			// There's now empty space available on the right to fill with
			// additional frames.  Scroll the viewport to the left to
			// accommodate filling this space.
			var newScrollLeft = (
				this._contentPane.domNode.scrollWidth
				- this._contentPane.domNode.clientWidth
			);
			var scrollPixelDelta =
				this._contentPane.domNode.scrollLeft - newScrollLeft;
			this._scroll(scrollPixelDelta);
		}
	},

	_getTableDataForFrame: function(frameIndex) {
		// target TD is found at the n'th TD within the first TR of the _frameTable's TBODY.
		return this._frameTable.firstChild.firstChild.childNodes[frameIndex];
	},

	_scrollOnePage: function(isForward) {
		var pageWidth = this._contentPane.domNode.clientWidth;
		var scrollDeltaToUse = Math.floor(pageWidth / 2.0);
		this._scroll(isForward ? scrollDeltaToUse : -scrollDeltaToUse);
	},

	_scroll: function(pixelDelta) {
		var currentScrollLeft = this._contentPane.domNode.scrollLeft;
		var targetScrollLeft = currentScrollLeft + pixelDelta;
		this._installAnyMissingThumbnailsWithinViewportRegion(targetScrollLeft);
		var self = this;
		// Note by SPC: We do some weird stuff here with CSS to avoid what
		// appears to be an IE bug ... we make sure to tweak the CSS class
		// of the _contentPane during every frame of the animation.  Without
		// this work-around, IE for some reason thinks that a repaint is
		// not needed and the end result is that the _contentPane's contents
		// would appear to still be at their original scrollLeft position
		// (but if you occlude the window and then bring it to the foregound
		// you'd see that the _contentPane has appropriately scrolled to the
		// new target scrollLeft).
		var ieDummyClass = "temp";
		var ieDoAddClass = false;
		var anim = new dojo._Animation({
			curve: [currentScrollLeft, targetScrollLeft],
			onEnd: function() {
				self._leftEndpointScroller.scrollingHasOccurred();
				self._rightEndpointScroller.scrollingHasOccurred();
				if (dojo.isIE) {
					dojo.removeClass(self._contentPane.domNode, ieDummyClass);
				}
			},
			onAnimate: function(offset) {
				self._contentPane.domNode.scrollLeft = Math.floor(offset);
				if (dojo.isIE) {
					if (ieDoAddClass) {
						dojo.addClass(self._contentPane.domNode, ieDummyClass);
					} else {
						dojo.removeClass(self._contentPane.domNode, ieDummyClass);
					}
					ieDoAddClass = !ieDoAddClass;
				}
			},
			duration: 300
		});
		anim.play();
	},

	_installAnyMissingThumbnailsWithinViewportRegion: function(/*Optional*/targetScrollLeft) {
		if (! targetScrollLeft) {
			targetScrollLeft = this._contentPane.domNode.scrollLeft;
		}
		var left = targetScrollLeft;
		var right = targetScrollLeft + this._contentPaneDiv.offsetWidth;
		var firstVisibleThumbnail = Math.max(
			0,
			Math.floor(left / THUMBNAIL_CELL_WIDTH)
		);
		var lastVisibleThumbnail = Math.min(
			this._frameInfos.length - 1,
			Math.ceil(right / THUMBNAIL_CELL_WIDTH) - 1
		);
		for (var i = firstVisibleThumbnail; i <= lastVisibleThumbnail; i++) {
			if (! this._frameInfos[i].loaded) {
				this._requestThumbnailLoadIfNeeded(i, false);
			}
		}
	},

	_clearConnectHandles: function() {
		if (this._connectHandles) {
			dojo.forEach(this._connectHandles, function(handle) {
				dojo.disconnect(handle);
			});
			delete this._connectHandles;
			this._connectHandles = null;
		}
	},

	_clearExistingContentsIfNeeded: function() {
		this._clearConnectHandles();
		if (this._frameTable.firstChild != null) {
			com.ibm.rdm.web.util.domutils.removeChildNodes(this._frameTable);
		}
	},

	_showMessage: function(htmlContent) {
		this._frameTable.style.display = "none";
		this._messageDiv.style.display = "";
		this.domNode.style.background = THUMBNAIL_MESSAGE_BACKGROUND_COLOR;
		this._contentPaneDiv.style.overflow = "auto";
		this._messageDiv.innerHTML = htmlContent;
		this._leftEndpointScroller.framesHaveBeenRefreshed();
		this._rightEndpointScroller.framesHaveBeenRefreshed();
	},

	_showThumbnails: function(frameKeyToSelect) {
		this._frameTable.style.display = "";
		this._messageDiv.style.display = "none";
		this.domNode.style.background = THUMBNAIL_NORMAL_BACKGROUND_COLOR;
		this._contentPaneDiv.style.overflow = "hidden";
		this._connectHandles = [];
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		var loadingMessage = messages["loading"];
		tbody.appendChild(tr);
		this._frameTable.appendChild(tbody);
		var selectedFrameIndex = null;
		for (var i = 0; i < this._frameInfos.length; i++) {
			var frameIndex = this._frameInfos[i].frameIndex;
			var frameName = this._frameInfos[i].frameName;
			var doAddAsSelected = false;
			if (frameKeyToSelect) {
				var key = this._frameInfos[i].key;
				if (key == frameKeyToSelect) {
					selectedFrameIndex = frameIndex;
				}
			}
			this._addFrame(tr, frameIndex, frameName, doAddAsSelected, loadingMessage);
		}
		if (selectedFrameIndex != null) {
			for (var i = 0; i <= selectedFrameIndex; i++) {
				var doAddAsSelected = (i === selectedFrameIndex);
				this._requestThumbnailLoadIfNeeded(i, doAddAsSelected);
			}
		}
		var right = (
			this._contentPane.domNode.scrollLeft
			+ this._contentPaneDiv.offsetWidth
		);
		var lastVisibleThumbnail = Math.min(
			this._frameInfos.length - 1,
			Math.ceil(right / THUMBNAIL_CELL_WIDTH) - 1
		);
		for (i = (selectedFrameIndex ? selectedFrameIndex : 0);
			 i <= lastVisibleThumbnail;
			 i++) {
			this._requestThumbnailLoadIfNeeded(i, false);
		}
		this._leftEndpointScroller.framesHaveBeenRefreshed();
		this._rightEndpointScroller.framesHaveBeenRefreshed();
	},

	_addFrame: function(tr, frameIndex, frameName, doAddAsSelected, loadingMessage) {
		var td = document.createElement("td");
		tr.appendChild(td);
		var div = document.createElement("div");
		div.className = "thumbnail-loading";
		var textNode = document.createTextNode(loadingMessage);
		div.appendChild(textNode)
		td.appendChild(div);
		td.title = frameName;
		var self = this;
		this._connectHandles.push(dojo.connect(td, "onclick", td, function() {
			self._selectThumbnail(frameIndex);
		}));
	},

	_requestThumbnailLoadIfNeeded: function(frameIndex, /*Optional*/doAddAsSelected) {
		if (! this._frameInfos[frameIndex].loaded) {
			var td = this._getTableDataForFrame(frameIndex);
			var self = this;
			this.onLoadThumbnail(
				frameIndex,
				THUMBNAIL_IMAGE_WIDTH,
				THUMBNAIL_IMAGE_HEIGHT,
				function(imageSrc, frameIndex, errorResponse) {
					self._installThumbnailContent(imageSrc, frameIndex, td, errorResponse);
					if (doAddAsSelected) {
						self._selectThumbnail(
							frameIndex,
							td,
							true // doSuppessSelectionCallback
						);
					}
				}
			);
			this._frameInfos[frameIndex].loaded = true;
		}
	},

	_installThumbnailContent: function(imageSrc, frameIndex, td, /*Optional*/errorResponse) {
		var contentNode = null;
		// Set contentNode to an appropriate value (a "div" containing an error message
		// if errorResponse has been passed, but otherwise [the normal case] an "img"
		// that the converter has produced).
		if (errorResponse) {
			var msg = com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().getErrorMessage(errorResponse);
			var div = document.createElement("div");
			div.style.width = THUMBNAIL_IMAGE_WIDTH + "px";
			div.style.height = THUMBNAIL_IMAGE_HEIGHT + "px";
			div.className = "thumbnail-error-div";
			div.innerHTML = (
				"<span>"
				+ "&lt;" + msg + "&gt;"
				+ "</span>"
			);
			contentNode = div;
		} else {
			var img = document.createElement("img");
			img.className = "thumbnail-image";
			dojo.addClass(img, "thumbnail-unselected");
			img.title = this._frameInfos[frameIndex].frameName;
			img.src = imageSrc;
			contentNode = img;
		}
		// Remove the Loading... placeholder.
		com.ibm.rdm.web.util.domutils.removeChildNodes(td);
		// Add the thumbnail image.
		td.appendChild(contentNode);
		// Add a frame number.  Please note that we're using absolute
		// positioning to add the span containing this number.
		var span = document.createElement("span");
		var frameNumberTextNode = document.createTextNode(frameIndex + 1);
		span.className = "thumbnail-frame-number";
		span.appendChild(frameNumberTextNode);
		td.appendChild(span);
		var cellCenterOffset = Math.floor(THUMBNAIL_CELL_WIDTH / 2.0);
		span.style.left = (
			(((THUMBNAIL_CELL_WIDTH * frameIndex) + cellCenterOffset)
				- Math.floor(span.clientWidth / 2.0))
			+ "px"
		);
		span.style.bottom = THUMBNAIL_FRAME_NUMBER_BOTTOM_OFFSET + "px";
	},

	_setSelectionState: function(td, doSelect) {
		var classToAdd = (doSelect ? "thumbnail-selected" : "thumbnail-unselected");
		var classToRemove = (doSelect ? "thumbnail-unselected" : "thumbnail-selected");
		dojo.removeClass(td.firstChild, classToRemove);
		dojo.addClass(td.firstChild, classToAdd);
	},

	_selectThumbnail: function(frameIndex, /*Optional*/td, /*Optional*/doSuppessSelectionCallback) {
		if (frameIndex != this._lastSelectedFrameIndex) {
			var td = (td || this._getTableDataForFrame(frameIndex));
			// Remove the last selection.
			this._clearSelection();
			// Add the new selection.
			this._setSelectionState(td, true);
			this._lastSelectedFrameIndex = frameIndex;
			// Inform the controlling class that a the thumbnail selection has
			// changed.
			if (!doSuppessSelectionCallback) {
				this.onThumbnailClicked(frameIndex);
			}
		}
	},

	_clearSelection: function() {
		if (this._lastSelectedFrameIndex != null) {
			var td = this._getTableDataForFrame(this._lastSelectedFrameIndex);
			this._setSelectionState(td, false);
		}
		this._lastSelectedFrameIndex = null;
	}
});

// TODO Interrogate CSS for these values ...
var THUMBNAIL_IMAGE_WIDTH = 75;
var THUMBNAIL_IMAGE_HEIGHT = 90;
var THUMBNAIL_MARGIN = 4;
var THUMBNAIL_FRAME_NUMBER_BOTTOM_OFFSET = 10;
var THUMBNAIL_CELL_WIDTH = (THUMBNAIL_MARGIN + THUMBNAIL_IMAGE_WIDTH + THUMBNAIL_MARGIN);
var THUMBNAIL_CELL_HEIGHT = (
	THUMBNAIL_MARGIN + THUMBNAIL_IMAGE_HEIGHT + THUMBNAIL_MARGIN
	+ 2 // FIXME Why do we need to add 2 here?
	);
var ENDPOINT_SCROLLER_WIDTH = 16;
var ENDPOINT_TYPE = {
	LEFT_SIDE: 0,
	RIGHT_SIDE: 1
};
var THUMBNAIL_NORMAL_BACKGROUND_COLOR = "#cccccc";
var THUMBNAIL_MESSAGE_BACKGROUND_COLOR = "white";

dojo.declare(
	"com.ibm.rdm.web.storyboard._EndpointScroller",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//    Provides a "hot area" on the end of the ThumbnailStackView (either the
	//    right side or the left side).  When the user moves over this hot
	//    region feedback is provided to let them know that clicking will either
	//    slide the contents to the left (when this class is used on the right
	//    hand side) or right (when this class is used on the left hand side).

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "storyboard/templates/EndpointScroller.html"),

	///////////////////////////////////////////////////////////////////////////
	// Attach points

 	_rootDiv: null,
	_arrowIcon: null,

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	thumbnailStackView: null,
	// endpointType: ENDPOINT_TYPE
	//    The value of this field should be one of the values found in ENDPOINT_TYPE.
	endpointType: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_ARROW_TOP: Math.floor(THUMBNAIL_CELL_HEIGHT / 2.0) - 8, // Note: Our icon is 16x16 so half of height is 8

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
		if (this.endpointType === ENDPOINT_TYPE.RIGHT_SIDE) {
			this._arrowIcon.src = "com.ibm.rdm.web/common/images/scroll_rt.gif";
			this._arrowIcon.style.marginLeft = "3px";
			this._arrowIcon.style.marginTop = this._ARROW_TOP + "px";
		} else if (this.endpointType === ENDPOINT_TYPE.LEFT_SIDE) {
			this._arrowIcon.src = "com.ibm.rdm.web/common/images/scroll_lft.gif";
			this._arrowIcon.style.marginTop = this._ARROW_TOP + "px";
			this._arrowIcon.style.marginLeft = "-3px";
		}
		this.connect(this._rootDiv, "onmouseover", this._onMouseOver);
		this.connect(this._rootDiv, "onmouseout", this._onMouseOut);
		this.connect(this._rootDiv, "onclick", this._onClick);
	},

	startup: function() {
		this.inherited(arguments);
		this._rootDiv.style.height = THUMBNAIL_CELL_HEIGHT + "px";
		if (this.endpointType === ENDPOINT_TYPE.RIGHT_SIDE) {
			this.domNode.style.right = "0px";
			this.domNode.style.display = "none";
		} else if (this.endpointType === ENDPOINT_TYPE.LEFT_SIDE) {
			this.domNode.style.left = "0px";
			this.domNode.style.display = "none";
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
	},

	framesHaveBeenRefreshed: function() {
		this._changeVisibilityIfNeeded();
	},

	scrollingHasOccurred: function() {
		this._changeVisibilityIfNeeded();
	},

	thumbnailStackDimensionsHaveChanged: function() {
		this._changeVisibilityIfNeeded();
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_changeVisibilityIfNeeded: function() {
		var isShowingMessage = (! this.thumbnailStackView.isShowingThumbnails());
		if (this.endpointType === ENDPOINT_TYPE.RIGHT_SIDE) {
			this.domNode.style.display = (
				(isShowingMessage || this.thumbnailStackView.isViewportAtEnd()) ?
				"none" :
				""
			);
		} else if (this.endpointType === ENDPOINT_TYPE.LEFT_SIDE) {
			this.domNode.style.display = (
				(isShowingMessage || this.thumbnailStackView.isViewportAtStart()) ?
				"none" :
				""
			);
		}
	},

	_onMouseOver: function() {
		dojo.removeClass(this._rootDiv, "endpoint-unhighlighted");
		dojo.addClass(this._rootDiv, "endpoint-highlighted");
	},

	_onMouseOut: function() {
		dojo.removeClass(this._rootDiv, "endpoint-highlighted");
		dojo.addClass(this._rootDiv, "endpoint-unhighlighted");
	},

	_onClick: function() {
		if (this.endpointType === ENDPOINT_TYPE.RIGHT_SIDE) {
			this.thumbnailStackView.scrollForward();
		} else if (this.endpointType === ENDPOINT_TYPE.LEFT_SIDE) {
			this.thumbnailStackView.scrollBackward();
		}
	}
});

})();
