/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.EmbeddedDiagramViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedResourceViewer");
dojo.require("com.ibm.rdm.web.viewer.ScreenFlowUIActivityLoader");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var constants = com.ibm.rdm.web.common.MimeTypes;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

dojo.declare(
	"com.ibm.rdm.web.viewer.EmbeddedDiagramViewer",
	[com.ibm.rdm.web.viewer.EmbeddedResourceViewer, com.ibm.rdm.web.viewer.ScreenFlowUIActivityLoader],
{
	// summary:
	//    A widget used for displaying simple GEF-based artifact types in an
	//    "embedded" manner.
	// description:
	//    This widget can be used to display an artifact of type User Interface
	//    Sketch, User Interface Part, Screen Flow, Use-Case Diagram or
	//    Business Process Diagram.  Rich text style artifacts make use of this
	//    widget to display these simple GEF-based artifacts as embedded
	//    viewers.  In addition, at some point we'll probably also have popups
	//    that will leverage this widget display an embedded control when
	//    needed (e.g. perhaps hovering over a Screen Flow in a artifact
	//    listing will result in displaying this Screen Flow by adding an
	//    appropriate instance of this widget to a popup's content area).

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this.inherited(arguments);
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	// Superclass override functions ...

	_populateContents: function() {
		// Note: this is an override of a superclass function.
		this._loadImageConversionBasedContent();
	},

	// Normal (non-override) functions ...

	_loadImageConversionBasedContent: function() {
		var self = this;
		var isScreenFlow = (self.contentType === constants.CONTENT_TYPE_SCREEN_FLOW);
		var onGetSuccess = function(response) {
			self._loadingDiv.style.display = "none";
			var htmlContent = viewerutils.transformImageGenerationDocument(
				response.ioArgs.xhr.responseText);
			viewerutils.appendDivWithHtmlContent(
				self._artifactContentDiv, htmlContent);
			if (isScreenFlow) {
 				self._artifactContentDiv.style.position = "relative";
				self._loadUIActivities(
					self._artifactContentDiv, self.reviewParser, self.projectSnapshotURI);
			}
			var div = viewerutils.getBackgroundImageDiv(self._artifactContentDiv);
			var imageWidth = div.clientWidth;
			var titleWidth = self._titleSpan.offsetWidth;
			var contentWidth = Math.max(titleWidth, imageWidth);
			self.domNode.style.width =
				contentWidth + self._contentPaddingLeft + self._contentPaddingRight + "px";
			return response;
		};
		var onGetError = function(response) {
			self._loadingDiv.style.display = "none";
			var msg = com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().getErrorMessage(response);
			var htmlContent =
			   	("<span>" + "&lt;" + msg + "&gt;" + "</span>");
			viewerutils.appendDivWithHtmlContent(
				self._artifactContentDiv, htmlContent);
			var div = self._artifactContentDiv.firstChild;
			var span = div.firstChild;
			div.style.background = "#FFDEDE";
			div.style.fontSize = "9pt";
			div.style.color = "#999999";
			var titleWidth = self._titleSpan.offsetWidth;
			var contentWidth = Math.max(titleWidth, span.offsetWidth);
			self.domNode.style.width =
				contentWidth + self._contentPaddingLeft + self._contentPaddingRight + "px";
			return response;
		};
		var urlToUse = (
			UriHelper.getGefDiagramConversionUri()
			+ "?resourceURL=" + encodeURIComponent(this.resourceURI) + ".html"
			+ "&contentType=" + this.contentType.replace("+", "%2b")
			+ "&revision=" + this.revision
			+ "&doSuppressCommentingDivCreation=" + (isScreenFlow ? "false" : "true")
			+ UriHelper.getReviewQueryParams(this.reviewParser)
			+ UriHelper.getBaselineQueryParams(this.projectSnapshotURI)
		);
		var deferred = RDMClient.doGet({
			uri: urlToUse,
			skipErrors: true
		});
		deferred.addCallback(onGetSuccess);
		deferred.addErrback(onGetError);
	}
});


})();
