/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.EmbeddedStoryboardViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.form.Button");
dojo.require("dojo.i18n");

dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.storyboard.StoryboardParser");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedResourceViewer");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");

dojo.requireLocalization("com.ibm.rdm.web.viewer", "EmbeddedStoryboardViewer");

var StoryboardParser = com.ibm.rdm.web.storyboard.StoryboardParser;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var constants = com.ibm.rdm.web.common.MimeTypes;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;

dojo.declare(
	"com.ibm.rdm.web.viewer.EmbeddedStoryboardViewer",
	com.ibm.rdm.web.viewer.EmbeddedResourceViewer,
{
	// summary:
	//    A widget used to display Storyboard artifacts.

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	// _frameInfos: an array of property lists
	//    Please refer to the StoryboardParser class for details on what to
	//    expect in the property lists held by this array.
	_frameInfos: null,

	_frameTitleSpan: null,
	_currentFrameIndex: 0,
	_previousButton: null,
	_nextButton: null,
	_frameLoadingDiv: null,
	_messages: null,
	_deferredArray: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	postCreate: function() {
		this._messages = dojo.i18n.getLocalization(
			"com.ibm.rdm.web.viewer", "EmbeddedStoryboardViewer");
		this._currentFrameIndex = 0;
		this.inherited(arguments);
	},

	///////////////////////////////////////////////////////////////////////////
	// Helpers

	// Superclass override functions ...

	_populateContents: function() {
		// summary:
		//    This is an override of a superclass function.
		this._addForwardAndBackButtons();
		this._addFrameTitleSpan();
		this._loadStoryboardContent();
	},

	// Normal (non-override) functions ...

	_addForwardAndBackButtons: function() {
		// Note: We need to override some of the CSS that EmbeddedResourceViewer
		// defines.  We achieve this by introducing a new CSS class to the
		// _artifact DIV so that our CSS rule can be more specific (to achieve
		// override).
		dojo.addClass(this._artifactDiv, "storyboardDiv");
		var buttonsSpan = document.createElement("span");
		buttonsSpan.className = "previousAndNextButtonsSpan";
		var self = this;
		var previousButton = new dijit.form.Button({
			label: this._messages["previousFrame"],
			iconClass: "previousIconDisabled",
			showLabel: false,
			onClick: dojo.hitch(self, self._onPreviousFrameClicked)
		});
		var nextButton = new dijit.form.Button({
			label: this._messages["nextFrame"],
			iconClass: "nextIconDisabled",
			showLabel: false,
			onClick: dojo.hitch(self, self._onNextFrameClicked)
		});
		buttonsSpan.appendChild(previousButton.domNode);
		buttonsSpan.appendChild(nextButton.domNode);
		dojo.place(buttonsSpan, this._artifactTypeImage, "before");
		var dividerSpan = this._createTitleDividerSpan();
		dojo.place(dividerSpan, buttonsSpan, "after");
		this._previousButton = previousButton;
		this._nextButton = nextButton;
	},

	_addFrameTitleSpan: function() {
		var frameTitleSpan = document.createElement("span");
		frameTitleSpan.className = "frameTitleSpan";
		var dividerSpan = this._createTitleDividerSpan();
		dojo.place(dividerSpan, this._artifactNameSpan, "after");
		dojo.place(frameTitleSpan, dividerSpan, "after");
		this._frameTitleSpan = frameTitleSpan;
	},

	_loadStoryboardContent: function() {
		var storyboardParser = new StoryboardParser({
			doc: this.docAsXml
		});
		this._frameInfos = storyboardParser.getFrameInfos();
		this._setEnablementsForPreviousAndNextButtons();
		this._viewFrame(this._currentFrameIndex);
	},

	_createTitleDividerSpan: function() {
		var dividerSpan = document.createElement("span");
		dividerSpan.innerHTML = "|";
		dividerSpan.className = "titleDividerSpan";
		return dividerSpan;
	},

	_onPreviousFrameClicked: function(event) {
		dojo.stopEvent(event);
		if (this._currentFrameIndex !== 0) {
			this._currentFrameIndex--;
			this._setEnablementsForPreviousAndNextButtons();
			this._viewFrame(this._currentFrameIndex);
		}
	},

	_onNextFrameClicked: function(event) {
		dojo.stopEvent(event);
		if (this._currentFrameIndex !== (this._frameInfos.length-1)) {
			this._currentFrameIndex++;
			this._setEnablementsForPreviousAndNextButtons();
			this._viewFrame(this._currentFrameIndex);
		}
	},

	_setEnablementsForPreviousAndNextButtons: function() {
		var isStartFrame = (this._currentFrameIndex === 0);
		var isEndFrame = (this._currentFrameIndex === (this._frameInfos.length - 1));
		this._previousButton.attr(
			"iconClass", (isStartFrame ? "previousIconDisabled" : "previousIconEnabled"));
		this._nextButton.attr(
			"iconClass", (isEndFrame ? "nextIconDisabled" : "nextIconEnabled"));
	},

	_viewFrame: function(frameIndex) {
		this._cancelOutstandingDeferreds();
		this._deferredArray = [];
		var self = this;
		var setFrameTitle = function() {
			var textNode = self._frameTitleSpan.firstChild;
			if (textNode) {
				self._frameTitleSpan.removeChild(textNode);
			}
			// TODO If frame title is not specified, should we display something like
			// "<No frame title specified>" in the header area?
			var frameName = (self._frameInfos[frameIndex].frameName || "");
			var frameTitleToUse = (frameIndex+1) + ". " + frameName;
			self._frameTitleSpan.appendChild(
				document.createTextNode(frameTitleToUse));
		};
		var onGetSuccess = function(response) {
			self._loadingDiv.style.display = "none";
			self._artifactContentDiv.innerHTML = "";
			// TODO Consider changing this so that we make sure that the browser
			// cached images ARE used.  For example if I flip from frame 1 to 2
			// and then back to 1, we should probably just use the previously
			// cached version of frame 1's content.  As it stands today we're
			// getting brand new content ...
			var htmlContent = viewerutils.transformImageGenerationDocument(
				response.ioArgs.xhr.responseText);
			viewerutils.appendDivWithHtmlContent(
				self._artifactContentDiv, htmlContent);
			var div = viewerutils.getBackgroundImageDiv(self._artifactContentDiv);
			var imageWidth = div.clientWidth;
			var imageHeight = div.clientHeight;
			var titleWidth = self._titleSpan.offsetWidth;
			var contentWidth = Math.max(titleWidth, imageWidth);
			// TODO Need to work a little on resize issues when flipping through frames.
			// Should the first frame set the dimensions fixed for all subsequent frames
			// or should we dynamically resize for each frame?
			self.domNode.style.width =
				contentWidth + self._contentPaddingLeft + self._contentPaddingRight + "px";
			self._artifactContentDiv.style.height =
				self._artifactContentDiv.scrollHeight + "px";
			return response;
		};
		var onGetError = function(response) {
			self._loadingDiv.style.display = "none";
			self._artifactContentDiv.innerHTML = "";
			var msg = com.ibm.rdm.web.common.ErrorHandler.getErrorSectionWidget().getErrorMessage(response);
			var htmlContent =
			   	("<span>" + "&lt;" + msg + "&gt;" + "</span>");
			viewerutils.appendDivWithHtmlContent(
				self._artifactContentDiv, htmlContent);
			var div = self._artifactContentDiv.firstChild;
			var span = div.firstChild;
			div.style.background = "#FFDEDE";
			div.style.fontSize = "9pt";
			div.style.color = "#999999";
			var titleWidth = self._titleSpan.offsetWidth;
			var contentWidth = Math.max(titleWidth, span.offsetWidth);
			self.domNode.style.width =
				contentWidth + self._contentPaddingLeft + self._contentPaddingRight + "px";
			return response;
		};
		var urlToUse = (
			UriHelper.getGefDiagramConversionUri()
			+ "?resourceURL=" + encodeURIComponent(this.resourceURI) + ".html"
			+ "&contentType=" + this.contentType.replace("+", "%2b")
			+ "&revision=" + this.revision
			+ "&doSuppressCommentingDivCreation=true"
			+ "&storyboardFrameIndex=" + frameIndex
			+ UriHelper.getReviewQueryParams(this.reviewParser)
			+ UriHelper.getBaselineQueryParams(this.projectSnapshotURI)
		);
		setFrameTitle();
		if (this._artifactContentDiv.firstChild) {
			this._replaceContentWithLoadingMessage();
		}
		var deferred = RDMClient.doGet({
			uri: urlToUse,
			skipErrors: true
		});
		deferred.addCallback(onGetSuccess);
		deferred.addErrback(onGetError);
		this._deferredArray.push(deferred);
	},

	_replaceContentWithLoadingMessage: function() {
		if (this._artifactContentDiv.firstChild) {
			var div = document.createElement("div");
			div.className = "loadingFrameDiv";
			div.innerHTML = (
				"<span class=\"loadingFrameSpan\">"
				+ this.loading
				+ "</span>"
			);
			this._artifactContentDiv.innerHTML = "";
			this._artifactContentDiv.appendChild(div);
		}
	},

	// TODO Note, this deferred logic is not quite working yet as
	// _onNextFrameClicked/_onPreviousFrameClicked do not appear to be
	// called until after the frame content has loaded (i.e. the rendered
	// artifact image is displayed).  Need to figure out why ...
	_cancelOutstandingDeferreds: function() {
        if (this._deferredArray) {
            for (var i = 0; i < this._deferredArray.length; i++) {
                var deferred = this._deferredArray[i];
                if (deferred.fired == -1) {
                    deferred.cancel();
                }
            }
        }
		this._deferredArray = null;
	}
});


})();
