/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.RecentlyViewedSection");

dojo.require("dojox.collections.Dictionary");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.requireLocalization("com.ibm.rdm.web.pages", "ResourcesPage");


var UriHelper = com.ibm.rdm.web.common.UriHelper;
var MAX_RECENTLY_VIEWED_ITEMS = 10;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ResourcesPage");

// TODO Consider adding "Show More" "Show Less" links to expand/shrink the
// number of items displayed in this section ... as it stands right now
// this class will display up to MAX_RECENTLY_VIEWED_ITEMS and when an item
// is added that would exceed this amount, the oldest item is removed from
// the listing.

com.ibm.rdm.web.viewer.RecentlyViewedSection = {

	
    _recentResourcesDictionary: new dojox.collections.Dictionary(),

    addNewResource: function(resourceURI, contentType, displayName, /*String?*/revision, /*String?*/reviewId) {
    	if (!this._recentResourcesDictionary.containsKey(resourceURI)) {
	    	this._recentResourcesDictionary.add(
					resourceURI,
					{
						resourceURI:resourceURI, 
						contentType:contentType, 
						displayName:displayName, 
						revision:revision, 
						reviewId:reviewId
					}
				);
    	}
    },

    _menuProvider: function() {
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode,"jazz-app-BannerButton-menu");
		if (this._recentResourcesDictionary.count == 0) {
			menu.addChild(new dijit.MenuItem({label: messages.empty}));
		}
		else {
			this._recentResourcesDictionary.forEach(
					function(recentResourceNVP) {
						var recentResource = recentResourceNVP.value;
						var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(recentResource.contentType);
						var menuItem = new dijit.MenuItem({
							label: recentResource.displayName, 
							/*iconClass: 'helpIcon',*/ 
							onClick: dojo.hitch(this, this._handleResourceClicked, recentResource)})
						menuItem.iconNode.style.backgroundImage="url(" + imageURI + ")";
						menu.addChild(menuItem);
			        },
			        this
			)
		}
		return menu;
	},

    _handleResourceClicked: function(resource, event) {
		var href;
		if (resource.reviewId) {
			// The resource is either a review or an artifact under review.
			if (0 == resource.resourceURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL)) {
				// It's a review.
				href = UriHelper.getResourceJafLink(resource.resourceURI);
			} else {
				// It's an artifact under review
				href = UriHelper.getResourceInReviewContextJafLink(resource.resourceURI, resource.revision, resource.reviewId);
			}
		} else {
			// The resource is NOT a review or an artifact under review.
			href = UriHelper.getResourceJafLink(resource.resourceURI);
		}
		window.location.href = href;
    }
}

})();
