/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.ResourceViewer");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.Menu");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.rdm.web.attributes.OverviewSection");
dojo.require("com.ibm.rdm.web.collections.CollectionViewer");
dojo.require("com.ibm.rdm.web.commenting.CommentingWidget");
dojo.require("com.ibm.rdm.web.commenting.EventTopics");
dojo.require("com.ibm.rdm.web.commenting.ResourceViewerWidget");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.controlTypes");
dojo.require("com.ibm.rdm.web.glossary.GlossaryWidget");
dojo.require("com.ibm.rdm.web.glossary.TermWidget");
dojo.require("com.ibm.rdm.web.history.HistoryViewer");
dojo.require("com.ibm.rdm.web.history.RevisionsCache");
dojo.require("com.ibm.rdm.web.links.LinkCache");
dojo.require("com.ibm.rdm.web.links.LinksWidget");
dojo.require("com.ibm.rdm.web.pages.WebUIinitializer");
dojo.require("com.ibm.rdm.web.review.ApprovalEditor");
dojo.require("com.ibm.rdm.web.review.ArtifactInReviewViewer");
dojo.require("com.ibm.rdm.web.review.ReviewArtifactsCommentWidget");
dojo.require("com.ibm.rdm.web.review.ReviewCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.storyboard.StoryboardViewer");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");
dojo.require("com.ibm.rdm.web.wrapper.WrappedResourceViewer");

dojo.requireLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.common", "ErrorSectionWidget");


var ArtifactInReviewViewer = com.ibm.rdm.web.review.ArtifactInReviewViewer;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ReviewCache = com.ibm.rdm.web.review.ReviewCache;
var RevisionsCache = com.ibm.rdm.web.history.RevisionsCache;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var controlTypes = com.ibm.rdm.web.common.controlTypes;
var domutils = com.ibm.rdm.web.util.domutils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;

/* In truncation situations for the folders section or the public tags or
   private tags sections, the minimum number of characters to display before
   using ellipses. */
var MIN_FOLDERS_AND_TAGS_TEXT_BEFORE_ELLIPSES = 4;
/* In truncation situations for the artifact name title appearing in the drop
   down button, the minimum number of characters to display before using
   ellipses. */
var MIN_TITLE_TEXT_BEFORE_ELLIPSES = 7;


/**
 * Displays a single RPC resource (e.g. Requirement, Rich Text Doc, etc.).
 */
dojo.declare("com.ibm.rdm.web.viewer.ResourceViewer", [dijit._Widget, dijit._Templated], {

    templatePath: dojo.moduleUrl("com.ibm.rdm.web", "viewer/templates/ResourceViewer.html"),
    widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_resourceHeaderDiv: null,
	_resourceImageDiv: null,
	_goToFrameListTD: null,
	_goToFrameListAnchor: null,
	_titleSpan: null,
	_idSpan: null,
	_artifactTitleDropDown: null,
	_openInRichClientMenuItem: null,
	_openHistoryMenuItem: null,
	_openCurrentVersionMenuItem: null,
	_modifiedOnLabel: null,
	_modifiedOnSpan: null,
	_modifiedBySpan: null,
	_refreshButton: null,
	_editResourceButton: null,
	_sidebarLayoutDiv: null,
	_reqLinksWidget: null,

	_commentReqLinkMenu: null,
	_addCommentNode: null,
	_openLinkSectionNode: null,
	_openRequirementSectionNode: null,
	_refreshNode: null,

	_projectNameAndFoldersSpan: null,
	_tagSectionSpan: null,
	_tagImageOnSpan: null,
	_tagOnSpan: null,
	_privateTagOnSpan: null,
	_descriptionOnSpan: null,
	_selectCommentURI: null,
	_storyboardInheritsFromSpan: null,
	_inheritsFromAnchor: null,
	_inheritsFromArtifactName: null,

	_isInFormalReviewMode: null,
	_isInInformalReviewMode: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields

	resourcesPage: null,
    resourceURI: null,
	footerHeight: 0,
	// viewType: enum value
	//    The value of this field must be one of the values found in the
	//    com.ibm.rdm.web.viewer.ResourceViewer.viewType property list.
	viewType: null,
	// revision: string
	//    A artifact revision, such as "_WeXBQhr2Ed6zUeOnEVe9DA".  This value
	//    must be passed whenever the viewType is set to either:
	//        com.ibm.rdm.web.viewer.ResourceViewer.ShowArtifactHistory
	//    or:
	//        com.ibm.rdm.web.viewer.ResourceViewer.ShowArtifactInReviewContext
	revision: null,
	// reviewId: string
	//    A review rrc id, such as "_WeXBQhr2Ed6zUeOnEVe9DA".  This value
	//    must be passed whenever the viewType is set to:
	//        com.ibm.rdm.web.viewer.ResourceViewer.ShowArtifactInReviewContext
	reviewId: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_commonMessages: null,
	_errorSectionWidgetMessages: null,
	_contentType: null,
	_overviewSection: null,
	_commentingWidget: null,
	_linksWidget: null,
	_reviewArtifactsCommentsWidget: null,
	_resourceViewerWidget: null,
	_sidebarLayout: null,
	
	_folderPath: null,
	_projectName: null,
	// _projectNameAndFolders: String
	//    A string such as "Classics CD | Storyboards | Images"
	_projectNameAndFolders: null,
	_projectURI: null,
	_description: null,
	_refreshInProgress: null,
	_lastModifiedByUserUri: null,

	// _originalTitle: String
	//    The artifact name WITHOUT the artifact ID.
	//    For example, this might be set to "CD Details".
	_originalTitle: null,
	_isWrappedResource: false,
	_inheritsFromConnectHandle: null,

	// _artifactTitle: String
	//    A string such as "Doc1 (142)" representing the artifact name followed
	//    by the artifact RRC id.
	_artifactTitle: null,
	_artifactTitleWithoutRRCId: null,

	_resizeControlsInfo: {
		projectNameAndFoldersSpanWidth: null,
		publicTagsSpanWidth: null,
		privateTagsSpanWidth: null,
		lastPageWidth: null
	},
	_resizeTimoutHandle: null,
	_resizeEvent: null,

	// _publicTags: String
	//    A string represented the ";" deliminated public tags.
	_publicTags: null,
	// _privateTags: String
	//    A string represented the ";" deliminated private tags.
	_privateTags: null,
				
	///////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	modifiedOn: null,
	modifiedBy: null,
	more: null,
	less: null,
	loading: null,

	///////////////////////////////////////////////////////////////////////////
	// API

	initialize: function(ioArgs) {
		this._refreshInProgress = false;
		this.revision = this._getETag(ioArgs);
		this.connect(this._refreshButton, "onClick", this.handleRefresh);
		this.connect(this._editResourceButton, "onClick", this._handleEditResourceClicked);	
		this._createSidebarSections(ioArgs);
		this._sidebarLayout = new com.ibm.rdm.web.sidebar.SidebarLayout(
		{
			bottomOffset: this.footerHeight
		},
		this._sidebarLayoutDiv);

		// we do not need sidebar panes when we're in history mode
		if (this.viewType !== com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {		
			this.connect(this._openLinkSectionNode, "onClick", this._expandLinkSection);
			this.connect(this._openRequirementSectionNode, "onClick", this._expandReqSection);
			this._tagFolderMenu.style.display = "";
			// Note: Review artifacts don't show with the Tags, Links or
			// Requirements header treatment.
			if (this._contentType === com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
				this._foldersSectionSpan.style.display = "none";
				this._tagSectionSpan.style.display = "none";
				this._openLinkSectionNode.domNode.style.display = "none";
				this._openRequirementSectionNode.domNode.style.display = "none";
			}
			this._commentReqLinkMenu.style.display = "";
		}
		var helpContext = com.ibm.rdm.web.help.context.artifact;
		if (this._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
			helpContext = com.ibm.rdm.web.help.context.review;
		}
		else if (this._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION) {
			helpContext = com.ibm.rdm.web.help.context.collection;
		}
		this._cshNode.appendChild(new jazz.app.ua.PageHoverHelp({uaKey: helpContext}).domNode);
		
		var self = this;
		this.connect(
			this._refreshNode,
			"onClick",
			function() {
				self.resourcesPage.handleRefresh();
			}
		);
	},

	postCreate: function() {
		this._projectNameAndFolders = "";
		this._publicTags = "";
		this._privateTags = "";
		this._inheritsFromArtifactName = "";
		this._originalTitle = "";
		this._artifactTitleWithoutRRCId = "";
		// Use a different gray treatment in the header if this viewer is
		// pinned to a particular revision (rather than just displaying the
		// HEAD version).
		if (this.viewType
			=== com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext
			) {
			if (this.revision) {
				this._isInFormalReviewMode = true;
			} else {
				this._isInInformalReviewMode = true;
			}
		}
		if (this.revision) {
			this._setRevision(this.revision);
			dojo.addClass(this._resourceHeaderDiv, "resource-header-for-bound-revision");
		}
		
		var deferredHeadRequest =
			RDMClient.doHead({
					uri: this.resourceURI,
					revision: this.revision,
					projectSnapshotURI: this.projectSnapshotURI
			});
		deferredHeadRequest.addCallback(this, function(response){
			this._contentType = response.ioArgs.xhr.getResponseHeader("Content-Type");
			// Remove the "; charset=UTF-8", if any, after Content-Type
			if (this._contentType && this._contentType.indexOf(";")!=-1) {
				this._contentType = this._contentType.split(";")[0]; 
			}
			if (this.viewType == com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory ||
				this._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION) {
				dojo.addClass(this._resourceHeaderDiv, "historical");
			}
			
			// If we are in the history viewer, we first need to get artifact revisions
			// so that the filter parameters could be passed to sidebar panes constructors.
			if (this.viewType == com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
				RevisionsCache.getRevisions(this.resourceURI).addCallback(this, function(revisions){
						this._revisions = revisions.doc._revisionInfosMap;
						this.processHeadRequest(response);
					}
				);
			}
			else {
				this.processHeadRequest(response);
			}
		});

		// Add the DropDownButton menu items.
		var doHideOpenHistoryMenuItem = false;
		var doHideOpenCurrentVersionMenuItem = false;
		switch (this.viewType) {
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifact:
				if (0 == this.resourceURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL)) {
					doHideOpenHistoryMenuItem = true;
				}
				doHideOpenCurrentVersionMenuItem = true;
				break;
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext:
				doHideOpenHistoryMenuItem = true;
				doHideOpenCurrentVersionMenuItem = true;
				break;
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory:
				doHideOpenHistoryMenuItem = true;
				doHideOpenCurrentVersionMenuItem = false;
				break;
		}
		if (doHideOpenHistoryMenuItem) {
			this._openHistoryMenuItem.domNode.style.display = "none";
		}
		if (doHideOpenCurrentVersionMenuItem) {
			this._openCurrentVersionMenuItem.domNode.style.display = "none";
		}
		
		this._snapshotInfo.style.display = "none";
		this._collectionSnapshotInfoNode.style.display = "none";
		
		this.connect(this._openInRichClientMenuItem, "onClick", this._handleOpenInRichClientLinkClick);
		this.connect(this._openHistoryMenuItem, "onClick", this._handleHistoryLinkClick);
		this.connect(this._openCurrentVersionMenuItem, "onClick", this._handleOpenCurrentVersionLinkClick);
		
		this._addCommentNode.attr('title', this._messages.comments);
		this._openLinkSectionNode.attr('title', this._messages.links);
		this._openRequirementSectionNode.attr('title', this._messages.req);

		this._resizeControlsInfo = {
			projectNameAndFoldersSpanWidth: null,
			publicTagsSpanWidth: null,
			privateTagsSpanWidth: null,
			lastPageWidth: null
		};
		this._resizeEvent = jazz.util.ResizeNotifier.register(
			{context: this, callback: this._resizeHeaderControlsIfNeeded, force: true});
	},

	processHeadRequest: function(response) {
		if (response.ioArgs.xhr.status != 200)
			return;

		this.initialize(response.ioArgs);
		
		switch (this._contentType) {
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION:
				this._resourceViewerWidget = new com.ibm.rdm.web.collections.CollectionViewer({
		            projectURI: this.resourceURI,
		            resourceURI: this.resourceURI,
					footerHeight: this.footerHeight,
					resourceViewer: this
		        });
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY:
				this._resourceViewerWidget = new com.ibm.rdm.web.glossary.GlossaryWidget({
						resourceURI: this.resourceURI,
						resourceViewer: this
					});
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_TERM:
				this._resourceViewerWidget = new com.ibm.rdm.web.glossary.TermWidget({
						resourceURI: this.resourceURI,
						resourceViewer: this
					});
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD:
				var self = this;
				this.connect(this._goToFrameListAnchor, "onclick", function() {
					self._resourceViewerWidget.showFrameList();
					self._goToFrameListTD.style.display = "none";
					self._updateTitle(self._originalTitle);
					self.setDescription(self._originalDescription);
					self._resizeHeaderControlsIfNeeded();
				});
				this._resourceViewerWidget = new com.ibm.rdm.web.storyboard.StoryboardViewer({
					resourceURI: this.resourceURI,
					resourceViewer: this,
					htmlContentToUseWhenLoading: this._getHtmlContentToUseWhenLoadingViewers(),
					onFrameSelected:
						function(frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName) {
							self._storyboardFrameSelected(
								frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName);
						}
				});
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW:
				this._resourceViewerWidget = new com.ibm.rdm.web.review.ApprovalEditor({
						resourceURI: this.resourceURI, 
						resourceViewer: this
					});
				break;
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_ACTOR:
			case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE:
				this._resourceViewerWidget = new com.ibm.rdm.web.commenting.ResourceViewerWidget({
					resourceURI: this.resourceURI,
					resourceViewer: this
				});
				break;
			//any other resource opens up in the wrapper resource viewer
			default:
				this._resourceViewerWidget = new com.ibm.rdm.web.wrapper.WrappedResourceViewer({
						resourceURI: this.resourceURI, 
						resourceViewer: this
					});
				break;
		}
		
		
//		else { // viewType is ShowArtifact or ShowArtifactInReviewContext
			this.contentAreaViewer = null;
			if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifact) {
				this.contentAreaViewer = this._resourceViewerWidget;
			}
			else if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext) {
				this.contentAreaViewer = new ArtifactInReviewViewer({
					resourceViewer: this,
					resourceURI: this.resourceURI,
					revision: this.revision,
					resourceViewerWidget: this._resourceViewerWidget,
					reviewId: this.reviewId
				});
			}
			else if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
				this.contentAreaViewer = new com.ibm.rdm.web.history.HistoryViewer(
				{
					footerHeight: this.footerHeight, 
					resourceURI: this.resourceURI,
					resourceViewer: this,
					_resourceViewerWidget: this._resourceViewerWidget
				});
				this.contentAreaViewer.showRevision(this.revision, this.projectSnapshotURI);
			}
			 
			this._sidebarLayout.setContent(this.contentAreaViewer.domNode);
			if (this._overviewSection) {
				this._sidebarLayout.addSidebarSection(this._messages.overview, this._overviewSection.domNode, true);
			}
			this._commentingWidget._sidebarSection =
				this._sidebarLayout.addSidebarSection(
					this._messages.comments, this._commentingWidget.domNode, this._selectCommentURI != null);
			// Note: Review artifacts do not contain a Requirements section.
			if (this._reqLinksWidget) {
				this._reqLinksWidget.setSidebarSection(
					this._sidebarLayout.addSidebarSection(this._messages.req, this._reqLinksWidget.domNode, false));
			}
			// Note: Review artifacts do not contain a Links section.
			if (this._linksWidget) {
				this._linksWidget.setSidebarSection(
					this._sidebarLayout.addSidebarSection(this._messages.links, this._linksWidget.domNode, false));
			}
			// Note: Only reviews contain an artifacts comments section.
			if (this._reviewArtifactsCommentsWidget) {
				this._reviewArtifactsCommentsWidget.setSidebarSection(
					this._sidebarLayout.addSidebarSection(this._messages.reviewArtifactsComments, this._reviewArtifactsCommentsWidget.domNode, false));
			}
			if (this.viewType != com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
				this.loadResource(this.revision);
			}
//		}
	},
	
	loadResource: function(revision) {
		this._setRevision(revision);
		this.displayResource(revision);
	},
	
	displayResource: function(revision) {
		this._showLoadingMessageIfSupported();
		var imageURI = com.ibm.rdm.web.common.MimeTypeRegistry.getHeaderImageURI(this._contentType);
		var deferredLoadRequest = this._resourceViewerWidget.loadResource(
			this._contentType, revision);
		if (this.reviewId
			&& this._contentType !== com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW
			) {
			// Show the header resource type icon, but with a "Review" overlay
			// positioned slightly above and to the left of this icon.
			this._resourceImageDiv.className = "overlay-div";
			this._resourceImageDiv.innerHTML =
				"<img src=\"" + imageURI + "\" class=\"overlaid-resource-type-image\"></img>"
				+ "<img src=\"com.ibm.rdm.web/review/images/icons/ovr16/review-ovr-hdr.gif\" class=\"overlay-image\"></img>";
		}
		else if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
			this._resourceImageDiv.className = "overlay-div";
			if (this.projectSnapshotURI) {
				this._resourceImageDiv.innerHTML =
					"<img src=\"" + imageURI + "\" class=\"overlaid-resource-type-image\"></img>"
					+ "<img src=\"com.ibm.rdm.web/history/images/baseline-ovr-hdr.gif\" class=\"overlay-image\"></img>";
			}
			else {
				this._resourceImageDiv.innerHTML =
					"<img src=\"" + imageURI + "\" class=\"overlaid-resource-type-image\"></img>"
					+ "<img src=\"com.ibm.rdm.web/history/images/version-ovr-hdr.gif\" class=\"overlay-image\" style=\"left:34px\"></img>";
			}
		}
		else {
			// Show the header resource type icon (without any overlay).
			this._resourceImageDiv.innerHTML =
				"<img src=\"" + imageURI + "\" class=\"resource-type-image\"></img>";
		}
		deferredLoadRequest.addCallback(this, function(response) {
			this._parseLoadResourceResults(response, false);
			return response;
		});
		deferredLoadRequest.addErrback(this, function(e){
			console.error("Error performing load request in ResourceViewer.processHeadRequest\n" + e);
			this._markRefreshAsDone();
			return e;
		});
	},
	
	_parseLoadResourceResults: function(response) {
		// summary:
		//    Parse the 'response' associated with a deferred returned by
		//    loadResource and update the UI appropriately to show the
		//    content found within this 'response'.
		var doc = response.doc;
		this._revisionDate = dojo.date.stamp.fromISOString(dojo.date.stamp.toISOString(new Date(response.ioArgs.xhr.getResponseHeader("Last-Modified"))));
		this._lastModifiedByUserUri = response.ioArgs.xhr.getResponseHeader("X-Last-Modified-User");
		var description = com.ibm.rdm.web.util.queryutils.getDescriptionFromDoc(doc, this._contentType);
		var title = com.ibm.rdm.web.util.queryutils.getArtifactNameFromArtifactContent(doc, this._contentType);
		
		var managedRequirement = domutils.getElementsByTagNameNS(
			doc, "managedRequirement",
			queryutils.REQUIREMENT_NAMESPACE, queryutils.REQUIREMENT_QUALIFIER);
			
		if (managedRequirement.length > 0){
			this._updateManagedReqInfo(managedRequirement);
		}
				
		this._originalTitle = title;
		this._originalDescription = description;
		this._updateTitle(title);
		this.setDescription(description);
		this._resizeHeaderControlsIfNeeded();

		if (this._overviewSection) {
			this._overviewSection.updateAttributeGroupSections(doc);
			this._overviewSection.updateNameField(title);
			this._overviewSection.updateModifiedDateField(this._revisionDate);
		}
		
		// Passing the current document to the links widget for the processing of links
		if (this._linksWidget) {
			this._linksWidget.attr("doc", doc);
		}
		if (this._reqLinksWidget) {
			this._reqLinksWidget.attr("doc", doc);
		}

		if (this._refreshInProgress
			&& this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext
			) {
			this.contentAreaViewer.viewHasBeenRefreshed();
		}

		if (this._commentingWidget.hasStarted) {
			var successorRevisionDate =
				(this._successorRevision) ? this._successorRevision.revisionDate : null;
			this._commentingWidget.setRevisionDate(successorRevisionDate);
		}

		var deferred = ProjectCache.getResourceDescriptionMultiFetch(this.resourceURI, this.projectSnapshotURI, true);
		deferred.addCallback(this, function(resourceDescription) {
			if (resourceDescription) {
				this._projectURI = resourceDescription.projectURI;
				this._updateHeaderOverviewAndRecentlyViewedSection(resourceDescription);
			} else if (this._isInFormalReviewMode) {
				// Note: It must be the case that the user is attempting to open
				// an artifact within a formal review and that this artifact has
				// been deleted (since originally being added to the formal
				// review).  In this case we show an appropriate error at the top
				// but we still attempt to show a normal presentation otherwise.
				// There are a few exceptions though: the header area folder and
				// tags sections will not be filled in and the "Created on"
				// field will not be filled in (as we require describe results
				// to fill these fields in ... but it's not possible to use this
				// API when the artifact in question has been deleted).
				this._performHeadRequestAndTellUserIfArtifactNoLongerExists(this.resourceURI);
				var loadReviewDeferred = ReviewCache.getReview(this.reviewId);
				loadReviewDeferred.addCallback(this, function(response) {
					var reviewParser = response.doc;
					this._projectURI = reviewParser.getProjectUri();
					this._updateHeaderOverviewAndRecentlyViewedSection(resourceDescription);
					return response;
				});
			}
		});
		deferred.addErrback(this, function(response) {
			var doc = response.doc;
			console.error("ResourceViewer._updateHeaderOverviewAndRecentlyViewedSection() error getting resource info: " + doc);
			this._markRefreshAsDone();
			return response;
		});
	},

	_performHeadRequestAndTellUserIfArtifactNoLongerExists: function(resourceURI) {
		var headDeferred = RDMClient.doHead({
			uri: resourceURI,
			skipCache: true,
			skipErrors: true
		});
		headDeferred.addErrback(this, function(response){
			if (response.ioArgs.xhr.status === 410) { // Note: 410 is "Gone"
				this._showArtifactNoLongerExistsError();
			}
			return response;
		});
	},

	_showArtifactNoLongerExistsError: function() {
		ErrorHandler.getErrorSectionWidget().showErrorMessage(
			this._errorSectionWidgetMessages["Error_410"]);
	},

	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.viewer", "ResourceViewer");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this._errorSectionWidgetMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "ErrorSectionWidget");
		dojo.forEach(["modifiedOn", "modifiedBy", "more", "less", "loading", "notLoaded", "managedRequirement"], function(prop){
			this[prop] = this._messages[prop];
		}, this);
	},

	destroy: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeEvent);
		this._resizeEvent = null;
		if (this._sidebarLayout) {
			this._sidebarLayout.destroyRecursive();
			this._sidebarLayout = null;
		}
		if (this._resizeTimoutHandle) {
			clearTimeout(this._resizeTimoutHandle);
		}
		this.inherited("destroy", arguments);
	},

	getContentType: function() {
		return this._contentType;
	},
		
	setDescription: function(description) {
		this._description = description;
		this._updateDescription();
	},

	isInReviewMode: function() {
		return (
			this.viewType
			=== com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext
		);
	},

	isInFormalReviewMode: function() {
		return this._isInFormalReviewMode;
	},

	isInInformalReviewMode: function() {
		return this._isInInformalReviewMode;
	},
	
	isStoryboardViewer: function() {
		return this._contentType === com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD;
	},
	
	handleRefresh: function() {
		if (!this._refreshInProgress) {
			this._refreshInProgress = true;
			var self = this;
			var loadResourceAndLoadCommentsIfNeeded = function(revision) {
				var deferredLoadRequest = self._resourceViewerWidget.loadResource(
					self._contentType, revision);
				deferredLoadRequest.addCallback(function(result) {
 					self._parseLoadResourceResults(result, true);
					return result;
				});
				deferredLoadRequest.addErrback(function(e){
					console.error("Error performing head request in ResourceViewer.handleRefresh\n" + e);
					self._markRefreshAsDone();
					return e;
				});
				if (self._reviewArtifactsCommentsWidget && self._reviewArtifactsCommentsWidget.hasStarted) {
					self._reviewArtifactsCommentsWidget.loadComments();
				}
			};
			// Note: If we're viewing an artifact in a formal review context we
			// make sure here to NOT stop normal processing if the HEAD request
			// fails (but we'd still alert the user to the fact that the
			// artifact has been deleted).
			var deferredHeadRequest =
				RDMClient.doHead({
					uri: this.resourceURI,
					skipErrors: this._isInFormalReviewMode // See above Note ...
				});
			deferredHeadRequest.addCallback(this, function(response) {
				if (! this._isInFormalReviewMode) {
					this.revision = this._getETag(response.ioArgs);
				}
				loadResourceAndLoadCommentsIfNeeded(this.revision);
				return response;
			});
			deferredHeadRequest.addErrback(this, function(response){
				if (this._isInFormalReviewMode && response.ioArgs.xhr.status === 410) { // Note: 410 is "Gone"
					// See above Note ... we still want to continue loading (after
					// alerting the user to the fact that the artifact has been
					// deleted).
					this._showArtifactNoLongerExistsError();
					loadResourceAndLoadCommentsIfNeeded(this.revision);
				}
				else {
					console.error("Error performing head request in ResourceViewer.handleRefresh\n" + response);
					this._markRefreshAsDone();
				}
				return response;
			});
			this._showLoadingMessageIfSupported();
		} else { //Wait for refresh to finish
			ErrorHandler.debug("refreshing");
		}
	},
	
	updateBrowserTitle: function() {
		// Note: Method updateBrowserTitle is called from a Deferred.
		// It is possible that a user has left the resource view at the time the 
		// Deferred executes.  So it is necessary to check if this dijit is still
		// in the DOM tree.  If not, there is no need to change browser title.
		if (document.getElementById(this.id)) {
			var workbenchTitle = dojo.string.substitute(
				this._messages.contentDisplayName,
				[
					com.ibm.rdm.web.common.MimeTypeRegistry.getMimeTypeDisplayName(this._contentType),
					this._artifactTitle
				]
			);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(workbenchTitle);
		}
	},

	viewHasBeenRestored: function() {
		if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext) {
			// Tell the ArtifactInReviewViewer that the view has been restored.
 			this.contentAreaViewer.viewHasBeenRestored();
		}
		this._resizeHeaderControlsIfNeeded();
	},

	handleWindowResize: function() {
		if (this._sidebarLayout) {
			this._sidebarLayout.handleWindowResize();
		}
	},

	getElementDisplayName: function(elementId) {
		// summary:
		//    Get the display name for the given element within the viewer.
		// description:
		//    Gets the display name that should be used in viewers when the
		//    elementId (corresponding to an element in the viewer's DOM) is
		//    shown to the user.  For example, the commenting code uses this
		//    display name to compose the
		//        "Create a new comment for: <display name>"
		//    menu item.
		// returns: String or null
		//    If the element exists within the viewer, its display name is
		//    returned.  Otherwise, if the element no longer exists, null is
		//    returned.
		var displayName = null;
		if (this._contentType == com.ibm.rdm.web.common.MimeTypes.STORY_BOARD.contentType) {
			return this._resourceViewerWidget.getElementDisplayName(elementId);
		} else if (this._resourceViewerWidget._documentHasBeenLoaded && this._resourceViewerWidget.idNodesMap) {
			var locationResource = this._resourceViewerWidget.idNodesMap[elementId];
			if (locationResource) {
				switch(this._contentType) {
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_RICH_TEXT:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REQUIREMENT:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_ACTOR:
						var type = locationResource.getAttribute("rdmtype");	
						if (type == "actor")
							displayName = this._messages.embeddedActor;
						else if (type == "usecase")	
							displayName = this._messages.embeddedUsecase;
						else if (type == "richtext")
							displayName = this._messages.embeddedDocument;
						else if (type == "requirement")
							displayName = this._messages.embeddedRequirement;
                        else if (type == controlTypes.CONTROL_TYPES.Paragraph)
							displayName = controlTypes.getControlTypeDisplayName(controlTypes.CONTROL_TYPES.Paragraph);
						else	
							displayName = dojo.isIE ? locationResource.innerText : locationResource.textContent;
						break;
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SKETCH:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PART:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_SCREEN_FLOW:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_PROCESS:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_USE_CASE_DIAGRAM:
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_STORY_BOARD:
						// TODO As implemented here, for brevity we're showing ONLY the
						// control label, if it's available, otherwise we're showing
						// the control type (when the label isn't available).  In today's
						// native client, the BAW editors seem to use this approach,
						// but sketching editors use a control type PLUS control label
						// approach.  We need direction on which way to go ...
						displayName = locationResource.getAttribute("controlLabel");
						if (displayName) {
							displayName = com.ibm.rdm.web.util.queryutils.decode(displayName);
						}
						if (!displayName || displayName.length === 0) {
							var controlType = locationResource.getAttribute("controlType");
							if (controlType) {
								displayName = controlTypes.getControlTypeDisplayName(controlType);
							}
						}
						break;
					case com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_GLOSSARY:
						var termWidget = dijit.byNode(locationResource.firstChild);
						displayName = termWidget.getTitle();
						break;
				}
			}
		}
		return displayName;
	},
	
	setCommentToSelect: function(commentURI) {
		this._selectCommentURI = commentURI;
	},
	
	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_createSidebarSections: function(ioArgs) {
		this._overviewSection = new com.ibm.rdm.web.attributes.OverviewSection({
			resourceURI: this.resourceURI,
			contentType: this._contentType
		});
		this._commentingWidget = new com.ibm.rdm.web.commenting.CommentingWidget({
			viewer: this,
			resourceURI: this.resourceURI,
			resourceContentType: this._contentType,
			displayContext: com.ibm.rdm.web.commenting.CommentingWidget.displayContext.SIDEBAR_SECTION,
			selectCommentURI: this._selectCommentURI,
			revisionDate: (this._successorRevision)?this._successorRevision.revisionDate:null
		});
		// Note: Review artifacts have a special section for showing comments for artifacts in the review.
		if (this._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
			this._reviewArtifactsCommentsWidget = new com.ibm.rdm.web.review.ReviewArtifactsCommentWidget({
				viewer: this, 
				resourceURI: this.resourceURI,
				ioArgs: ioArgs
			});
		}
		// Note: Review artifacts do not show a Links section.
		if (this._contentType !== com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
			this._linksWidget = new com.ibm.rdm.web.links.LinksWidget({
				viewer: this, 
				resourceURI: this.resourceURI,
				ioArgs: ioArgs,
				viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.ARTIFACTS
			});
		}
		// Note: Review artifacts do not show a Requirements section.
		if (this._contentType !== com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW &&
			this._contentType !== com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION) {
			this._reqLinksWidget = new com.ibm.rdm.web.links.LinksWidget({
				viewer: this, 
				resourceURI: this.resourceURI,
				ioArgs: ioArgs,
				viewType: com.ibm.rdm.web.project.DashboardTabPage.viewType.REQUIREMENTS
			});
		}
	},

	_storyboardFrameSelected: function(frameTitle, frameDescription, inheritsFromResourceURI, inheritsFromDisplayName) {
		var self = this;
		if (this._inheritsFromConnectHandle != null) {
			self.disconnect(this._inheritsFromConnectHandle);
		}
		this._inheritsFromConnectHandle = [];
		var createInheritsFromLink = function(resourceURI, displayName) {
			var textNode =
			    domutils.createSpannedTextNode("banner-right-text", self._messages["inheritsFrom"]);
			var anchor =
			    domutils.createAnchor(displayName, resourceURI, "inherits-from-link");
			var image = document.createElement("img");
			image.src = "com.ibm.rdm.web/storyboard/images/icons/inherits-link.gif";
			image.className = "banner-right-image";
			self._storyboardInheritsFromSpan.appendChild(textNode);
			self._storyboardInheritsFromSpan.appendChild(image);
			self._storyboardInheritsFromSpan.appendChild(anchor);
			self._inheritsFromConnectHandle = (
				self.connect(anchor, "onclick", function(event) {
					ResourceLinksHandler.handleAnchorClicked(event);
				})
			);
			anchor.title = displayName;
			return anchor;
		};
		domutils.removeChildNodes(this._storyboardInheritsFromSpan);
		this._goToFrameListTD.style.display = "";
		this._updateTitle(frameTitle, true);
		this.setDescription(frameDescription);
		if (inheritsFromResourceURI != null) {
			var anchor = createInheritsFromLink(inheritsFromResourceURI, inheritsFromDisplayName);
			this._inheritsFromArtifactName = inheritsFromDisplayName;
			this._inheritsFromAnchor = anchor;
		}
		this._resizeHeaderControlsIfNeeded();
	},

	_updateHeaderOverviewAndRecentlyViewedSection: function(resourceDescription) {
		// summary:
		//    Update the artifact header area, the sidebar Overview section and the
		//    recently viewed section.
		// resourceDescription: Object
		//    If the artifact being viewed hasn't been deleted, this parameter
		//    will be a "multi-fetch" based resource description property list.
		//    Please note, however, that in the case of formal reviews, this
		//    parameter MAY be undefined or null (if the artifact in question
		//    has been deleted since originally being added to the formal
		//    review).
		this._folderPath = new Array();
		var self = this;
		if (resourceDescription) {
			this.parentResourceURI = resourceDescription.parentResourceURI;
		}

		this._updateOverviewSectionModifiedByField();

		com.ibm.rdm.web.viewer.RecentlyViewedSection.addNewResource(
				this.resourceURI,
				this._contentType,
				this._originalTitle,
				this.revision,
				this.reviewId
		);
		
		if (this._overviewSection) {
			this._overviewSection.setProjectURI(this._projectURI);
			if (resourceDescription) {
				// TODO: For a deleted artifact (in the case of a formal review)
				// the resourceDescription will be null and therefore we don't
				// know what date should be passed to the overviewSection's
				// updateCreatedDateField function ... is there some other way
				// to obtain this created date for a deleted artifact?
				this._overviewSection.updateCreatedDateField(resourceDescription.createdDate);
			}
		}
		
		this._commentingWidget.setResourceInfo({"owningTeam": this._projectURI, "displayName": this._originalTitle});
		if (this._reqLinksWidget) {
			this._reqLinksWidget.setProjectURI(this._projectURI);
		}
		if (this._reviewArtifactsCommentsWidget) {
			this._reviewArtifactsCommentsWidget.setProjectURI(this._projectURI);
		}
		
		// some of the sidebar sections need the projectURI set for loading, so startup the 
		// sidebars here
		this._sidebarLayout.startup();
		
		// Note: Review artifacts do not show tags in the header.
		if (this._contentType !== com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW) {
			var tags = (resourceDescription ? resourceDescription.tags : []);
			this._getTags(tags);
		}
		if (this._contentType == com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_COLLECTION) {
			this._resourceViewerWidget.setProjectURI(this._projectURI);	
		}
		
		//process associated project snapshot
		this._snapshotInfo.innerHTML = "";
		this._snapshotInfo.style.display = "none";
		if (this.projectSnapshotURI) {
			var projectSnapshotDeferred = ProjectCache.getProjectSnapshot(this.projectSnapshotURI);
			projectSnapshotDeferred.addCallback(this, function (projectSnapshot){
				if (projectSnapshot) {
					this._snapshotInfo.style.display = "";
					this._snapshotInfo.appendChild(document.createTextNode(this._messages.snapshotLabel + projectSnapshot.name));
				}
			});
		}
					
		ProjectCache.getProject(this._projectURI).addCallback(this, 
			function(project) {
				this._projectName = project.name;
				this._getResourceFolder();
				this._updateOverviewProjectField();
				return project;
		});
		// enable comment creation from the header if allowed
		if (this.viewType !== com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
			// header buttons are not there for history view
			var projectOperations = com.ibm.rdm.web.AUTHENTICATED_USER.permissions.item(this._projectURI);	
			var commentingIsAllowed = projectOperations.item(ProjectCache.MODIFY_COMMENT);
			if (this.isInReviewMode() && commentingIsAllowed) {
				var onReviewLoaded = function(response) {
					reviewParser = response.doc;
					var doDisable = reviewParser.getReviewState() == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Paused
						|| reviewParser.getReviewState() == com.ibm.rdm.web.review.reviewTypes.REVIEW_STATE_TYPES.Finalized;
					commentingIsAllowed = !doDisable;
					if (commentingIsAllowed) {
						self.connect(self._addCommentNode, "onClick", self._addCommentToArtifact);
					} else {
						self.connect(self._addCommentNode, "onClick", self._expandCommentsSection);
					}
					return response;
				};
				var reviewDeferred = ReviewCache.getReview(this.reviewId);
				reviewDeferred.addCallback(onReviewLoaded);
			} else {
				if (commentingIsAllowed) {
					this.connect(this._addCommentNode, "onClick", this._addCommentToArtifact);
				} else {
					this.connect(this._addCommentNode, "onClick", this._expandCommentsSection);
				}
			}
		}
	},

	_updateOverviewSectionModifiedByField: function() {
		var authorUri = "";
		if (this.viewType === com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory) {
			authorUri = this._revisions[this.revision].authorURI;
		}
		else {
			authorUri = this._lastModifiedByUserUri;
		}
		ProjectCache.getProjectUsers(this._projectURI).addCallback(this, 
			function(users) {
				var authorName = "";
				var authorUserId = "";
				var user = users[authorUri];
				if (user) {
					authorName = user.name;
					authorUserId = user.userId;
				}
				else {
					// Handle the case of the "rrs-daemon" user.  This user isn't
					// actually found in the project user listing.  However, when
					// displaying a review you might find that the user that last
					// modified the review is "rrs-daemon" (as the R/A service
					// sometimes moves the review from one state to another).
					var usersPrefix = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS;
					authorName = authorUri.substring(usersPrefix.length + 1);
					authorUserId = authorName;
				}
				if (this._overviewSection) {
					this._overviewSection.updateModifiedByField(authorName, authorUserId);
				}
			}
		);
	},

	_composeShortDate: function(date) {
		return dojo.date.locale.format(date, { formatLength: "short" });
	},

	_updateTitle: function(artifactOrFrameName, /*Boolean?*/isStoryboardFrameName) {
		// summary:
		//    Update the displayed title.
		// description:
		//    In addition to updating the title that appears in the title area
		//    drop down button, this function also updates the browser title
		//    unless 'isStoryboardFrameName' is passed as true.
		//    Please note that the title presented is normally an artifact name,
		//    but it will instead be set to a frame name within a Storyboard
		//    artifact when the user selects a particular frame from within a
		//    Storyboard.
		var encodeGreaterThanAndLessThan = function(text) {
			text = text.replace(/</g, "&lt;");
			text = text.replace(/>/g, "&gt;");
			return text;
		};
		var decodeGreaterThanAndLessThan = function(text) {
			text = text.replace(/&lt;/g, "<");
			text = text.replace(/&gt;/g, ">");
			return text;
		};
		// Note: Artifact names can't have "<" or ">" chars in them, but these
		// characters _are_ legal in Storyboard frame names.  We need to account
		// for this fact here: for example, if we didn't, the normal "<No frame
		// title specified>" frame title (used when the title is not specified)
		// would display as "&lt;No frame title specified&gt;" when the user
		// hovers over the title.
		this._artifactTitleDropDown.attr(
			"label",
			(isStoryboardFrameName ?
			 encodeGreaterThanAndLessThan(artifactOrFrameName) :
			 artifactOrFrameName)
		);
		this._artifactTitleDropDown.attr(
			"title",
			(
				(isStoryboardFrameName ?
				 decodeGreaterThanAndLessThan(artifactOrFrameName) :
				 artifactOrFrameName)
				+ " (" + this._messages.clickForActions + ")"
			)
		);
		var getOrCreateRRCIdSpan = function(artifactTitleDropDown) {
			var rrcIdSpan = null;
			var nextSiblingAfterLabel = artifactTitleDropDown.containerNode.nextSibling;
			if (nextSiblingAfterLabel && nextSiblingAfterLabel.className === "resource-id") {
				rrcIdSpan = nextSiblingAfterLabel;
			} else {
				var span = document.createElement("span");
				span.className = "resource-id";
				dojo.place(span, artifactTitleDropDown.containerNode, "after");
				rrcIdSpan = span;
			}
			return rrcIdSpan;
		};
		var rrcIdSpan = getOrCreateRRCIdSpan(this._artifactTitleDropDown);
        var artifactId = UriHelper.getArtifactId(this.resourceURI);
        if (artifactId != null) {
			rrcIdSpan.innerHTML = "(" + artifactId + ")";
			// Reset _artifactTitle so that the updateBrowserTitle function can pick this up.
			this._artifactTitle = artifactOrFrameName + " (" + artifactId + ")";
		} else {
			rrcIdSpan.innerHTML = "";
			this._artifactTitle = artifactOrFrameName;
		}
		this._artifactTitleWithoutRRCId = artifactOrFrameName;
		if (!isStoryboardFrameName) {
			this.updateBrowserTitle();
		}
		this._resizeHeaderControlsIfNeeded();
	},

	_updateLastModifiedDate: function(dateString) {
		this._modifiedOnLabel.style.display = "";
		var child = this._modifiedOnSpan.firstChild
		if (child) {
			this._modifiedOnSpan.removeChild(child);
		}
		this._modifiedOnSpan.appendChild(document.createTextNode(dateString));
	},

	_updateDescription: function() {
		// summary:
		//    Updates the description text presented below the artifact title area.
		domutils.removeChildNodes(this._descriptionOnSpan);
		this._descriptionOnSpan.appendChild(document.createTextNode(this._description));
		this._descriptionOnSpan.title = this._description;
	},
		
	_expandCommentsSection: function() {
		if (this._commentingWidget) {
			this._commentingWidget.expandSidebar();
		}
	},
		
	_expandLinkSection: function() {
		if (this._linksWidget) {
			this._linksWidget.expandSidebar();
		}
	},
	
	_addCommentToArtifact: function() {
		this._commentingWidget.onCreateNewCommentForArtifact();
	},
	
	_expandReqSection: function() {
		if (this._reqLinksWidget) {
			this._reqLinksWidget.expandSidebar();
		}
	},
	
	////////////////////////////////////////////////////////////////////
	// Performs an xquery that will return tag names of current resource
	_getTags: function(resourceTags) {
		if (!this._projectURI)
			return;
		
		/* Remove all tags and adding Loading message */
		domutils.removeChildNodes( this._tagOnSpan );
		domutils.removeChildNodes( this._privateTagOnSpan );
		this._tagOnSpan.appendChild(document.createTextNode( this.loading ));

		ProjectCache.getResourceTags(this.resourceURI).addCallback(this, function(resourceTags) {
			//create arrary containing only resource tags objects
			var tags = [];
			for (var tagResourceURI in resourceTags) {
				tags.push(resourceTags[tagResourceURI]);
			}
			
			//this.connect( this._tagImageOnSpan, "onmouseover", this._addTag );
			/* Remove all tags to prevent addition of old tags and to order alphabetically*/
			domutils.removeChildNodes(this._tagOnSpan);
			domutils.removeChildNodes(this._privateTagOnSpan);
	
			var publicTagsString = "";
			var privateTagsString = "";
			
			var tagRow = "";
			for (var i = 0; i < tags.length; i++) {
				tagRow = tagRow + tags[i].title;
				if (tags[i].scope == "public") {
					if (publicTagsString.length > 0) {
						publicTagsString += this._messages.tagSeperator;
					}
					publicTagsString += tags[i].title;
					onePublicTagAdded = true;
				}
				else if (tags[i].scope == "private") {
					if (privateTagsString.length > 0) {
						privateTagsString += this._messages.tagSeperator;
					}
					privateTagsString += tags[i].title;
					onePrivateTagAdded = true; 
				}
			}

			// If appropriate, manipulate the DOM to add public tag info.
			if (publicTagsString.length > 0) {
				// TODO: Add nls string to prepend the following node title with something
				// like "public tags: ".
				this._tagOnSpan.title = publicTagsString;
				// add a delimiter between the public and private tags only if there is
				// atleast one public and private tag
				if (publicTagsString.length > 0 && privateTagsString.length > 0) {
					publicTagsString += this._messages.tagSeperator;
				}
				this._tagOnSpan.appendChild(document.createTextNode(publicTagsString));
			}
			// If appropriate, manipulate the DOM to add private tag info.
			if (privateTagsString.length > 0) {
				// TODO: Add nls string to prepend the following node title with
				// something like "private tags: ".
				this._privateTagOnSpan.title = privateTagsString;
				this._privateTagOnSpan.appendChild(document.createTextNode(privateTagsString));
			}
			// Only show the tags section if at least one tag is associated with the
			// artifact.
			if (publicTagsString.length === 0 && privateTagsString.length === 0) {
				this._tagSectionSpan.style.display = "none";
			} else {
				this._tagSectionSpan.style.display = "";
			}
			// Initialize some fields and resize the controls if needed.
			this._publicTags = publicTagsString;
			this._privateTags = privateTagsString;
			this._resizeControlsInfo.publicTagsSpanWidth =
				dojo.coords(this._tagOnSpan).w;
			this._resizeControlsInfo.privateTagsSpanWidth =
				dojo.coords(this._privateTagOnSpan).w;
			this._resizeHeaderControlsIfNeeded();
	/*        return response;*/
		});
	},
	
	////////////////////////////////////////////////////////////////////////////////////////
	// Performs the GET on public tag list feed
	// It does not perform the action of getting the folder hierarchy
	_getResourceFolder: function() {
		var self = this;
		if (!this.parentResourceURI) {
			this._markRefreshAsDone();
			return;
		}
		/* Remove all folders and adding Loading message */
		domutils.removeChildNodes(this._projectNameAndFoldersSpan);
		
		this._projectNameAndFoldersSpan.appendChild(document.createTextNode(this.loading));
		// Note: If this.resourceURI represents a wrapper resource, we need to
		// make sure to use the _wrapped_ resource when getting at folder info.
		/*var resourceURIToUse = (
			this._wrapperInfo ?
			this._wrapperInfo.rscURI :
			this.resourceURI);*/

		ProjectCache.getProjectFolders(this._projectURI).addCallback(this, this._createFolderHierarchy);
	},

	//////////////////////////////////////////////////////////////////////////////////////////
	// Function is responsible for creating the heirarchy of folders
	// Need to pass in the response from an xqeury and it gets the title and parent folder 
	// information recurssively
	_createFolderHierarchy: function(folderHierarchyMap) {
		var SELF = com.ibm.rdm.web.common.PublicTagsParser.SELF;
		var folderObj = folderHierarchyMap[this.parentResourceURI][SELF];
		while (folderObj.title != 'root') {
			this._folderPath.push(folderObj.title);	
			folderObj = folderHierarchyMap[folderObj.parentURI][SELF];
		}
		this._displayFolderPath();
	},

	_updateOverviewProjectField: function() {
		if (this._overviewSection) {
			this._overviewSection.updateProjectField(this._projectName);
		}
	},
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Updates the folder span by removing all of the old folder list and placing new ones 
	// that it gets from _folderPath
	// _folderPath is an array of strings that holds the folder hierarchy starting with the
	// folder in which resource resides and moving up in the hierarchy
	// refreshInProgress Variable is set to false here
	_displayFolderPath: function() {
		/* Remove all folders to prevent addition of old folders */
		domutils.removeChildNodes(this._projectNameAndFoldersSpan);
		var projectNameAndFolders = this._projectName;
		for (var i = this._folderPath.length - 1; i >= 0; i--) {
			projectNameAndFolders += (" | " + this._folderPath[i]);
		}
		this._projectNameAndFoldersSpan.appendChild(document.createTextNode(projectNameAndFolders));
		this._projectNameAndFoldersSpan.title = projectNameAndFolders;
		this._projectNameAndFolders = projectNameAndFolders;
		this._resizeControlsInfo.projectNameAndFoldersSpanWidth =
			dojo.coords(this._projectNameAndFoldersSpan).w;
		this._markRefreshAsDone();
		this._resizeHeaderControlsIfNeeded();
	},
	
	_markRefreshAsDone: function() {
		this._refreshInProgress = false;
	},

	_getETag: function(ioArgs) {
		var stripLeadingAndTrailingDoubleQuotes = function(etag) {
			var len = etag.length;
			return etag.substring(1, len-1);
		};
		return stripLeadingAndTrailingDoubleQuotes(
			ioArgs.xhr.getResponseHeader("Etag"));
	},

	_getHtmlContentToUseWhenLoadingViewers: function() {
		return (
			"<p style=\"color:#999999; margin:20px;\">"
			+ this._messages["waitForContentMessage"]
			+ "</p>"
		);
	},

	_showLoadingMessageIfSupported: function() {
		var setHtmlContentFunction = this._resourceViewerWidget.setHtmlContent;
		if (setHtmlContentFunction) {
			setHtmlContentFunction.call(
				this._resourceViewerWidget,
				this._getHtmlContentToUseWhenLoadingViewers()
			);
		}
	},

	_updateCommentsCounterInHeader: function() {
		//var countText = dojo.string.substitute(this._messages.count, [com.ibm.rdm.web.util.queryutils.getCommentsCount(doc)]);
		this._commentingWidget.getCommentCount(function(commentsCount){
			this._addCommentNode.attr('label', "(" + commentsCount + ")");
			this._addCommentNode.attr('title', dojo.string.substitute(this._messages.numComments, [commentsCount]));
		},
		this);
	},
	_updateOutgoingLinksCounterInHeader: function(count){
		this._openLinkSectionNode.attr('label', count);
		this._openLinkSectionNode.attr('title', dojo.string.substitute(this._messages.numLinksInArtifact, [count]));
	},
	_updateRequirementsLinksCounterInHeader: function (count){
		this._openRequirementSectionNode.attr('label', count);
		this._openRequirementSectionNode.attr('title', dojo.string.substitute(this._messages.numReqsInArtifact, [count]));
	},
	_handleAddCommentClicked: function() {
		dojo.publish(
			com.ibm.rdm.web.commenting.EventTopics.EDIT_COMMENT_FORM_FOR_LOCATION_REQUESTED_TOPIC,
			[this._resourceViewerWidget._selectedNode.id]);
	},
	
	_handleEditResourceClicked: function() {
		ResourceLinksHandler.showExternalResource(nativeURI);
	},
	
	_handleHistoryLinkClick: function() {
		ResourceLinksHandler.showInternalResource(this.resourceURI, true, true);
	},

	_handleOpenCurrentVersionLinkClick: function() {
		ResourceLinksHandler.showInternalResource(this.resourceURI, true);
	},

	_handleOpenInRichClientLinkClick: function() {
		var nativeURL = null;
		switch (this.viewType) {
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifact:
				nativeURL = viewerutils.getNativeClientUrlForOpenArtifact(
					this.resourceURI);
				break;
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactHistory:
				nativeURL = viewerutils.getNativeClientUrlForOpenHistory(
					this.resourceURI, this.revision, this.projectSnapshotURI);
				break;
			case com.ibm.rdm.web.viewer.ResourceViewer.viewType.ShowArtifactInReviewContext:
				nativeURL = viewerutils.getNativeClientUrlForOpenArtifactInReviewContext(
					this.resourceURI, this.reviewId, this.revision);
				break;
		}
		if (nativeURL) {
			window.location = nativeURL;
		}
	},
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// Function return true if document has parent element otherwise returns false
	_hasParent: function( doc ) {
		var nodes = doc.getElementsByTagName("parent");
		for (var i = 0; i < nodes.length; i++) {
			var node = nodes[i];
			var namespaceURI = node.namespaceURI;
			if (namespaceURI && (namespaceURI === "http://com.ibm.rdm/navigation#" 
				|| namespaceURI === "http://com.ibm.rdm/navigation") ) {
				if (node.firstChild) {
					return true;
				}
			}
		}
		return false;
	},
	
	_setRevision: function(revision) {
		this.revision = revision;		
		this._successorRevision = null;
		if (this._revisions && this._revisions[this.revision]) {
			var successorRevisionId = this._revisions[this.revision]["http://jazz.net/xmlns/v0.6#successor"];
			if (this._revisions[successorRevisionId])
				this._successorRevision = this._revisions[successorRevisionId];
		}
	},
	
	_updateManagedReqInfo: function(ManagedReqNode) {
		
		var linkToReqPro = domutils.getAttributeNS(ManagedReqNode[0],
			"linkToReqPro", queryutils.REQUIREMENT_NAMESPACE ,queryutils.REQUIREMENT_QUALIFIER)
		var reqProTag = domutils.getAttributeNS(ManagedReqNode[0],
			"reqProTag", queryutils.REQUIREMENT_NAMESPACE ,queryutils.REQUIREMENT_QUALIFIER)
	
		var managedDiv = document.createElement("div");
		var link = ResourceLinksHandler.getResourceLink(decodeURIComponent(linkToReqPro), reqProTag, false);
		managedDiv.appendChild(document.createTextNode(this._messages["managedRequirement"]));
		managedDiv.appendChild(link);
		dojo.addClass(managedDiv, "managed-requirement");
		this._commentReqLinkMenu.appendChild(managedDiv);

	},

	_resizeHeaderControlsIfNeeded: function() {
		// summary:
		//    This function is called to perform appropriate truncation within
		//    the header UI elements so that everything fits appropriately in
		//    the width available.
		// description:
		//    This function is called in three different situations:
		//    (1) when the UI elements are initialy populated (i.e. the artifact
		//        viewer is first opened)
		//    (2) whenever the user resizes the browser
		//    (3) whenever the user restores a view from our view cache
		//    In each of these situations, displaying all of the "displayable"
		//    artifact specific text (e.g. artifact name, description, etc.)
		//    could lead to the header becoming too wide.
		//    This function is called to make sure that the header area doesn't
		//    become too wide by performing appropriate truncation on the
		//    resource specific text areas.
		//    With UX guideance, the algorithm we use when narrowing is:
		//    - Truncate the "Inherits from" artifact name, if present (this
		//      applies only to Storyboard frames)
		//    Then, if the header area is still too wide:
		//    - Trucate the artifact title and description area
		//    Then, if the header area is still too wide:
		//    - Evenly truncate the three areas represented by the project-
		//      name/folders section, the public tags section and the private
		//      tags section
		//    The algorithm to use for widening is the reverse of the one used
		//    for narrowing: e.g. project-name/folders section and public/-
		//    private tags sections are widened before the title/description
		//    section.
		//
		var self = this;
		//
		// **** Utility Functions ****
		//
		var getSpanOrAnchorText = function(spanOrAnchor) {
			return (spanOrAnchor.firstChild ? spanOrAnchor.firstChild.nodeValue : "");
		};
		var isInheritsFromTruncated = function() {
			var isTruncated = false;
			if (self._inheritsFromAnchor) {
				var text = getSpanOrAnchorText(self._inheritsFromAnchor);
				isTruncated = (text.length > 0 && text !== self._inheritsFromArtifactName);
			}
			return isTruncated;
		};
		var isTitleTruncated = function() {
			var text = self._artifactTitleDropDown.attr("label");
			return (text.length > 0 && text !== self._artifactTitleWithoutRRCId);
		};
		var isDescriptionTruncated = function() {
			var text = getSpanOrAnchorText(self._descriptionOnSpan);
			return (text.length > 0 && text !== self._description);
		};
		var isProjectNameAndFoldersSectionTruncated = function() {
			var text = getSpanOrAnchorText(self._projectNameAndFoldersSpan);
			return (text.length > 0 && text !== self._projectNameAndFolders);
		};
		var isPublicTagsTruncated = function() {
			var text = getSpanOrAnchorText(self._tagOnSpan);
			return (text.length > 0 && text !== self._publicTags);
		};
		var isPrivateTagsTruncated = function() {
			var text = getSpanOrAnchorText(self._privateTagOnSpan);
			return (text.length > 0 && text !== self._privateTags);
		};
		var getTitleLineWidth = function() {
			var rightSideControlCoords = dojo.coords(self._cshNode);
			return (rightSideControlCoords.l + rightSideControlCoords.w);
		};
		var getSpanAnchorOrDropDownWidth = function(spanOrAnchor, dropDown) {
			return dojo.coords(spanOrAnchor ? spanOrAnchor : dropDown.domNode).w
		};
		var resetTextInSpanAnchorOrDropDown = function(newText, spanOrAnchor, dropDown) {
			if (spanOrAnchor) {
				spanOrAnchor.innerHTML = newText;
			} else {
				dropDown.attr("label", newText);
			}
		};
		var removeTextToAccommodateWidthInSpanAnchorOrDropDown = function(widthToUse, spanOrAnchor, dropDown) {
			// Note: A precondition is that only one of 'spanOrAnchor' or 'dropDown'
			// will be non-null.  Remove enough text in 'spanOrAnchor' or 'dropDown'
			// (whichever is passed) so that it is no wider than 'widthToUse'.
			var currentText = (
				dropDown ?
				self._artifactTitleDropDown.attr("label") :
				spanOrAnchor.firstChild.nodeValue
			);
			var numCharsToRemove = 1;
			var newText = currentText;
			var ellipses = self._messages.tagEtc;
			var minNewTextLength = (
				(dropDown ? MIN_TITLE_TEXT_BEFORE_ELLIPSES : MIN_FOLDERS_AND_TAGS_TEXT_BEFORE_ELLIPSES)
				+ ellipses.length);
			while (getSpanAnchorOrDropDownWidth(spanOrAnchor, dropDown) > widthToUse
				   && newText.length > minNewTextLength
				   ) {
				newText = (
					currentText.substring(
						0,
						currentText.length - ellipses.length - numCharsToRemove
					)
					+ ellipses
				);
				resetTextInSpanAnchorOrDropDown(newText, spanOrAnchor, dropDown);
				numCharsToRemove++;
			}
		};
		var addTextToAccommodateWidthInSpanAnchorOrDropDown = function(widthToUse, fullText, spanOrAnchor, dropDown) {
			// Note: A precondition is that only one of 'spanOrAnchor' or 'dropDown'
			// will be non-null.  Add text (found in 'fullText') to the passed
			// 'spanOrAnchor' or 'dropDown' such that the new width is closest to
			// but no greater than 'widthToUse'.
			var currentText = (
				dropDown ?
				self._artifactTitleDropDown.attr("label") :
				spanOrAnchor.firstChild.nodeValue
			);
			var ellipses = self._messages.tagEtc;
			var originalTextLengthWithoutEllipses = currentText.length - ellipses.length;
			var numCharsToAdd = 0;
			var newText = currentText;
			var lastNewText = null;
			while (getSpanAnchorOrDropDownWidth(spanOrAnchor, dropDown) < widthToUse
				   && newText.length < fullText.length
				   ) {
				lastNewText = newText;
				if (fullText.length
					=== originalTextLengthWithoutEllipses + numCharsToAdd + ellipses.length
					) {
					newText = fullText;
				} else {
					newText = (
						fullText.substring(0, originalTextLengthWithoutEllipses + numCharsToAdd)
						+ ellipses
					);
				}
				resetTextInSpanAnchorOrDropDown(newText, spanOrAnchor, dropDown);
				numCharsToAdd++;
			}
			if (getSpanAnchorOrDropDownWidth(spanOrAnchor, dropDown) > widthToUse && lastNewText) {
				resetTextInSpanAnchorOrDropDown(lastNewText, spanOrAnchor, dropDown);
			}
		};
		//
		// **** Widening Functions ****
		//
		var widenInheritsFromArtifactName = function(numPixelsToAdd) {
			var newWidth = dojo.coords(self._inheritsFromAnchor).w + numPixelsToAdd;
			addTextToAccommodateWidthInSpanAnchorOrDropDown(
				newWidth, self._inheritsFromArtifactName, self._inheritsFromAnchor, null);
		};
		var widenTitleAndDescription = function(titleTDWidth) {
			if (isTitleTruncated()) {
				var dropDownWidth = dojo.coords(self._artifactTitleDropDown.domNode).w;
				var growAmount = titleTDWidth - getTitleLineWidth();
				var newDropDownWidth = dropDownWidth + growAmount;
				addTextToAccommodateWidthInSpanAnchorOrDropDown(
					newDropDownWidth, self._artifactTitleWithoutRRCId, null, self._artifactTitleDropDown);
			}
			var newDescriptionWidth = titleTDWidth;
			if (isDescriptionTruncated()) {
				addTextToAccommodateWidthInSpanAnchorOrDropDown(
					newDescriptionWidth, self._description, self._descriptionOnSpan, null);
			}
		};
		var widenSpansEvenly = function(numPixelsToAdd, spanInfos) {
			// Add 'numPixelsToAdd' pixels to the total width represented by the
			// summation of all 'spanNodes' widths.  We make sure to do so evenly,
			// so that we always show the same width for all nodes that are truncated.
			// Plesae note that it may be the case that some of these 'spanNodes' do
			// NOT need truncation in order to satisfy removal of 'numPixelsToAdd'
			// pixels (e.g. it may be the case that we only need to truncate the
			// folders and public tags while leaving the private tags alone [for
			// example, perhaps there's many folders and public tags but only one
			// private tag]).
			var spanWidths = [];
			var newSpanWidths = [];
			var widthOfWidestSpan = null;
			var i;
			for (i = 0; i < spanInfos.length; i++) {
				var spanWidth = dojo.coords(spanInfos[i].spanNode).w;
				spanWidths.push(spanWidth);
				newSpanWidths.push(spanWidth);
				if (!widthOfWidestSpan || (spanWidth > widthOfWidestSpan)) {
					widthOfWidestSpan = spanWidth;
				}
			}
			var pixelIncrementAmount = 1;
			var numPixelsAdded = 0;
			var count = 1;
			var atLeastOneSpanStillNeedsTruncation = true;
			var truncatedSpanWidth = null;
			while (numPixelsAdded < numPixelsToAdd
				   && atLeastOneSpanStillNeedsTruncation
				   ) {
				var truncatedSpanWidth = widthOfWidestSpan + (count * pixelIncrementAmount);
				var isTruncationStillNeeded = false;
				for (i = 0; i < spanInfos.length; i++) {
					var spanInfo = spanInfos[i];
					if (spanInfo.fullTextSpanWidth > truncatedSpanWidth) {
						newSpanWidths[i] = truncatedSpanWidth;
						isTruncationStillNeeded = true;
					}
				}
				numPixelsAdded = 0;
				for (i = 0; i < spanInfos.length; i++) {
					numPixelsAdded += (newSpanWidths[i] - spanWidths[i]);
				}
				if (!isTruncationStillNeeded) {
					atLeastOneSpanStillNeedsTruncation = false;
				}
				count++;
			}
			if (truncatedSpanWidth) {
				for (i = 0; i < spanInfos.length; i++) {
					if (spanInfos[i].fullTextSpanWidth > truncatedSpanWidth) {
						addTextToAccommodateWidthInSpanAnchorOrDropDown(
							truncatedSpanWidth, spanInfos[i].fullText, spanInfos[i].spanNode, null);
					}
					else if (spanInfos[i].isTruncatedPredicateFunction()) {
						resetTextInSpanAnchorOrDropDown(
							spanInfos[i].fullText, spanInfos[i].spanNode, null);
					}
				}
			}
		};
		var widenFoldersAndTags = function(numPixelsToAdd) {
			var addSpanInfoIfNotEmpty = function(array, span, fullText, fullTextSpanWidth, isTruncatedPredicateFunction) {
				if (fullText !== "" && span.firstChild) {
					array.push({
						spanNode: span,
						fullText: fullText,
						fullTextSpanWidth: fullTextSpanWidth,
						isTruncatedPredicateFunction: isTruncatedPredicateFunction
					});
				}
			};
			var nonEmptySpanInfos = [];
			addSpanInfoIfNotEmpty(
				nonEmptySpanInfos, self._projectNameAndFoldersSpan,
				self._projectNameAndFolders, self._resizeControlsInfo.projectNameAndFoldersSpanWidth,
				isProjectNameAndFoldersSectionTruncated);
			addSpanInfoIfNotEmpty(
				nonEmptySpanInfos, self._tagOnSpan,
				self._publicTags, self._resizeControlsInfo.publicTagsSpanWidth,
				isPublicTagsTruncated);
			addSpanInfoIfNotEmpty(
				nonEmptySpanInfos, self._privateTagOnSpan,
				self._privateTags, self._resizeControlsInfo.privateTagsSpanWidth,
				isPrivateTagsTruncated);
			if (nonEmptySpanInfos.length > 0) {
				widenSpansEvenly(numPixelsToAdd, nonEmptySpanInfos);
			}
		};
		var widenHeaderControlsToAccommodateBrowserWidth = function(
				availableWidth, headerDivWidth, titleTDWidth) {
			if (isProjectNameAndFoldersSectionTruncated()
				|| isPublicTagsTruncated() || isPrivateTagsTruncated()
				) {
				var growAmount = titleTDWidth - getTitleLineWidth();
				var originalSpanWidth = dojo.coords(self._projectNameAndFoldersSpan).w;
				widenFoldersAndTags(growAmount);
				var newSpanWidth = dojo.coords(self._projectNameAndFoldersSpan).w;
				// Note: We make sure here to NOT expand the title/description area
				// until we know the folders and tags areas are _fully_ expanded.
				if (!isProjectNameAndFoldersSectionTruncated()
					&& !isPublicTagsTruncated() && !isPrivateTagsTruncated()
					) {
					var delta = newSpanWidth - originalSpanWidth;
					widenTitleAndDescription(titleTDWidth - delta);
				}
			}
			else {
				widenTitleAndDescription(titleTDWidth);
			}
			if (isInheritsFromTruncated()
				&& (! (isTitleTruncated() || isDescriptionTruncated()))
				) {
				var growAmount =
					titleTDWidth - Math.max(getTitleLineWidth(), dojo.coords(self._descriptionOnSpan).w);
				widenInheritsFromArtifactName(growAmount);
			}
		};
		//
		// **** Narrowing Functions ****
		//
		var narrowInheritsFromArtifactName = function(numPixelsToRemove, inheritsFromAnchorWidth) {
			var newWidth = inheritsFromAnchorWidth - numPixelsToRemove;
			removeTextToAccommodateWidthInSpanAnchorOrDropDown(
				newWidth, self._inheritsFromAnchor, null);
		};
		var narrowTitleAndDescription = function(numPixelsToRemove, titleTDWidth) {
			var tdWidth = titleTDWidth;
			var newTDWidth = tdWidth - numPixelsToRemove;
			var titleLineWidth = getTitleLineWidth();
			// Shrink the title, if needed.
			if (titleLineWidth > newTDWidth) {
				var dropDownWidth = dojo.coords(self._artifactTitleDropDown.domNode).w;
				var newDropDownWidth = dropDownWidth - numPixelsToRemove;
				removeTextToAccommodateWidthInSpanAnchorOrDropDown(
					newDropDownWidth, null, self._artifactTitleDropDown);
			}
			// Shrink the description, if needed.
			if (self._description && self._description.length > 0) {
				var descriptionWidth = dojo.coords(self._descriptionOnSpan).w;
				if (descriptionWidth > newTDWidth) {
					var newDescriptionWidth = Math.max(getTitleLineWidth(), newTDWidth);
					removeTextToAccommodateWidthInSpanAnchorOrDropDown(
						newDescriptionWidth, self._descriptionOnSpan, null);
				}
			}
		};
		var narrowSpansEvenly = function(numPixelsToRemove, spanNodes) {
			// Remove 'numPixelsToRemove' pixels from the total width represented by
			// the summation of all 'spanNodes' widths.  We make sure to do so evenly,
			// so that we always show the same width for all nodes that are truncated.
			// Plesae note that it may be the case that some of these 'spanNodes' do
			// NOT need truncation in order to satisfy remove of 'numPixelsToRemove'
			// (e.g. it may be the case that we only need to truncate the folders
			// and public tags while leaving the private tags alone [for example,
			// perhaps there's only one private tag but many folders and public tags]).
			var spanWidths = [];
			var newSpanWidths = [];
			var widthOfWidestSpan = null;
			var i;
			for (i = 0; i < spanNodes.length; i++) {
				var spanWidth = dojo.coords(spanNodes[i]).w;
				spanWidths.push(spanWidth);
				if (!widthOfWidestSpan || (spanWidth > widthOfWidestSpan)) {
					widthOfWidestSpan = spanWidth;
				}
			}
			var pixelDecrementAmount = 1;
			var numPixelsAboutToBeRemoved = 0;
			var count = 1;
			while (numPixelsAboutToBeRemoved < numPixelsToRemove) {
				var truncatedSpanWidth = widthOfWidestSpan - (count * pixelDecrementAmount);
				numPixelsAboutToBeRemoved = 0;
				for (i = 0; i < spanNodes.length; i++) {
					if (spanWidths[i] > truncatedSpanWidth) {
						newSpanWidths[i] = truncatedSpanWidth;
						numPixelsAboutToBeRemoved += (spanWidths[i] - truncatedSpanWidth);
					}
				}
				count++;
			}
			for (i = 0; i < spanNodes.length; i++) {
				if (newSpanWidths[i]) {
					removeTextToAccommodateWidthInSpanAnchorOrDropDown(
						newSpanWidths[i], spanNodes[i], null);
				}
			}
		};
		var narrowFoldersAndTags = function(numPixelsToRemove) {
			var addSpanIfNotEmpty = function(array, span, fullText) {
				if (fullText !== "" && span.firstChild) {
					array.push(span);
				}
			};
			var nonEmptySpanNodes = [];
			addSpanIfNotEmpty(nonEmptySpanNodes, self._projectNameAndFoldersSpan, self._projectNameAndFolders);
			addSpanIfNotEmpty(nonEmptySpanNodes, self._tagOnSpan, self._publicTags);
			addSpanIfNotEmpty(nonEmptySpanNodes, self._privateTagOnSpan, self._privateTags);
			if (nonEmptySpanNodes.length > 0) {
				narrowSpansEvenly(numPixelsToRemove, nonEmptySpanNodes);
			}
		};
		var narrowHeaderControlsToAccommodateBrowserWidth = function(
				availableWidth, headerDivWidth, titleTDWidth) {
 			var numPixelsToRemove = headerDivWidth - availableWidth;
			if (self._inheritsFromArtifactName) {
				var originalWidth = dojo.coords(self._inheritsFromAnchor).w;
				narrowInheritsFromArtifactName(numPixelsToRemove, originalWidth);
				var newWidth = dojo.coords(self._inheritsFromAnchor).w;
				numPixelsToRemove -= (originalWidth - newWidth);
			}
			if (numPixelsToRemove > 0) {
				var originalTDWidth = self._artifactTitleDropDown.domNode.parentNode.clientWidth;
				narrowTitleAndDescription(numPixelsToRemove, titleTDWidth);
				var newTDWidth = self._artifactTitleDropDown.domNode.parentNode.clientWidth;
				numPixelsToRemove -= (originalTDWidth - newTDWidth);
				// Note: We make sure here to NOT narrow the folders and tags
				// areas until we know that the title and description are
				// narrowed as much as they possibly can be.
				if (numPixelsToRemove > 0 && newTDWidth === originalTDWidth) {
					narrowFoldersAndTags(numPixelsToRemove);
				}
			}
		};
		// **** The main function implementing the truncation alogithm ****
		var resizeHeaderControlsIfNeeded = function() {
			var pageWidth = document.documentElement.clientWidth;
			var lastPageWidth = self._resizeControlsInfo.lastPageWidth;
			var headerDivSideAffordance = dojo.coords(self.resourcesPage.domNode).x;
			var headerDivWidth = self._resourceHeaderDiv.clientWidth;
			// TODO We're subtracting 10 in the following to avoid the right gutter
			// getting too narrow ... find a better solution ...
			var titleTDWidth = self._artifactTitleDropDown.domNode.parentNode.clientWidth - 10;
			var widthAvailable = pageWidth - (2 * headerDivSideAffordance);
			var minWidth = document.body.clientWidth - (2 * headerDivSideAffordance);
			if (widthAvailable < minWidth) {
				widthAvailable = minWidth;
			}
			if (dojo.isIE) {
				// Note by SPC: This 'isIE' block and the one below are needed for IE in
				// order to workaround the fact that IE for some reason believes that an
				// unnecessarily large gap should exist between the TD holding the title/-
				// description area and the TD to the right that displays the folders and
				// and tags (and the longer the artifact name the wider this gap would become).
				var button = self._artifactTitleDropDown.domNode.firstChild.firstChild.firstChild;
				button.style.width = "";
			}
			if (lastPageWidth) {
				if (pageWidth < lastPageWidth
					|| widthAvailable < headerDivWidth
					) {
					narrowHeaderControlsToAccommodateBrowserWidth(
						widthAvailable, headerDivWidth, titleTDWidth);
				} else {
					widenHeaderControlsToAccommodateBrowserWidth(
						widthAvailable, headerDivWidth, titleTDWidth);
				}
			} else {
				if (widthAvailable < headerDivWidth) {
		 			narrowHeaderControlsToAccommodateBrowserWidth(
						widthAvailable, headerDivWidth, titleTDWidth);
				} else if (isTitleTruncated()) {
					widenHeaderControlsToAccommodateBrowserWidth(
						widthAvailable, headerDivWidth, titleTDWidth);
				}
			}
 			self._resizeControlsInfo.lastPageWidth = pageWidth;
			if (dojo.isIE) {
				// See above SPC note about IE issue ...
				var button = self._artifactTitleDropDown.domNode.firstChild.firstChild.firstChild;
				var coords = dojo.coords(button);
				button.style.width = coords.w + "px";
				button.style.overflow = "";
			}
		};
		// In the following code, we use a timeout-based approach to making sure that
		// the local 'resizeHeaderControlsIfNeeded' function is called appropriately.
		// One wrinkle on this is that there are two cases in which we need to make
		// sure to call 'resizeHeaderControlsIfNeeded' twice:
		// (1) When restoring in the sense of performing the "restore" action after
		//     maximizing the view
		// (2) When restoring in the sense of restoring this resource viewer from
		//     our action view cache
		// If we _don't_ call it twice in these situations we end up with needless
		// whitespace between the vertical scrollbar in the main content area and
		// the left side of the artifact sidebar.
		var doCallResizeTwoTimes = false;
		var pageWidth = document.documentElement.clientWidth;
		if (this._resizeControlsInfo.lastPageWidth == null
			|| this._resizeControlsInfo.lastPageWidth == pageWidth // is true when view is restored from action view cache
			) {
			doCallResizeTwoTimes = true;
		} else {
			var widthDelta = (pageWidth - this._resizeControlsInfo.lastPageWidth);
			// Note by SPC: I don't know of a way to detect when "restore" occurs in
			// maximize/restore situations.  Instead we'll just check to see if the
			// page is now at least 100 pixels narrower than the last time this
			// function was called.
			if (widthDelta < -100) {
 				doCallResizeTwoTimes = true;
			}
		}
		if (this._resizeTimoutHandle) {
			clearTimeout(this._resizeTimoutHandle);
		}
		this._resizeTimoutHandle = setTimeout(
	       	function() {
	       		resizeHeaderControlsIfNeeded();
				if (doCallResizeTwoTimes) {
		 			self._resizeTimoutHandle = setTimeout(
	 			       	function() {
	 			       		resizeHeaderControlsIfNeeded();
	 			   		},
	 			   		250
	 			   	);
				}
	   		},
	   		1
	   	);
	}
});


})();


com.ibm.rdm.web.viewer.ResourceViewer.viewType = {
	ShowArtifact: 0,
	ShowArtifactHistory: 1,
	ShowArtifactInReviewContext: 2
};
